@@ -1,10 +1,11 @@
-/* $Id: about.hpp 10087 2006-02-13 04:14:27Z darthfool $ */
+/* $Id: about.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,8 +16,9 @@
 #define ABOUT_H_INCLUDED
 
 #include "global.hpp"
-#include "config.hpp"
+
 class display;
+class config;
 
 #include <vector>
 #include <string>
@@ -24,8 +26,6 @@ class display;
 namespace about
 {
 
-extern config about_list;
-
 void show_about(display &disp, std::string campaign="");
 void set_about(const config& cfg);
 std::vector<std::string> get_text(std::string campaign="");
