@@ -1,29 +1,36 @@
-/* $Id: ai_attack.cpp 13988 2006-10-16 01:58:47Z rusty $ */
+/* $Id: ai_attack.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version
+   or at your option any later version2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file ai_attack.cpp
+//! Calculate & analyse attacks.
+
 #include "global.hpp"
 
 #include "ai.hpp"
 #include "attack_prediction.hpp"
 #include "game_config.hpp"
 #include "gamestatus.hpp"
-#include "wassert.hpp"
+#include "log.hpp"
+
+#include <cassert>
 
 #define LOG_AI LOG_STREAM(info, ai)
 
 const int max_positions = 10000;
 
-//analyze possibility of attacking target on 'loc'
+//! Analyze possibility of attacking target on 'loc'.
 void ai::do_attack_analysis(
 	                 const location& loc,
 	                 const move_map& srcdst, const move_map& dstsrc,
@@ -35,7 +42,7 @@ void ai::do_attack_analysis(
 					 attack_analysis& cur_analysis
 	                )
 {
-	//this function is called fairly frequently, so interact with the user here.
+	// This function is called fairly frequently, so interact with the user here.
 	raise_user_interact();
 
 	if(cur_analysis.movements.size() >= size_t(attack_depth()))
@@ -60,7 +67,7 @@ void ai::do_attack_analysis(
 	double rating_to_beat = cur_rating;
 
 	if(!cur_analysis.movements.empty()) {
-		wassert(cur_analysis.movements.size() < 6);
+		assert(cur_analysis.movements.size() < 6);
 		double& best_res = best_results[cur_analysis.movements.size()-1];
 		rating_to_beat = best_res = maximum(best_res,cur_rating);
 	}
@@ -69,11 +76,13 @@ void ai::do_attack_analysis(
 		const location current_unit = units[i];
 
 		unit_map::iterator unit_itor = units_.find(current_unit);
-		wassert(unit_itor != units_.end());
+		assert(unit_itor != units_.end());
 
-		//see if the unit has the backstab ability -- units with backstab
-		//will want to try to have a friendly unit opposite the position they move to
-		//see if the unit has the slow ability -- units with slow only attack first
+		// See if the unit has the backstab ability.
+		// Units with backstab will want to try to have a
+		// friendly unit opposite the position they move to.
+		//
+		// See if the unit has the slow ability -- units with slow only attack first.
 		bool backstab = false, slow = false;
 		std::vector<attack_type>& attacks = unit_itor->second.attacks();
 		for(std::vector<attack_type>::iterator a = attacks.begin(); a != attacks.end(); ++a) {
@@ -92,10 +101,11 @@ void ai::do_attack_analysis(
 			continue;
 		}
 
-               //check if the friendly unit is surrounded, a unit is surrounded if
-               //it is flanked by enemy units and at least one other enemy unit is
-               //nearby or if the unit is totaly surrounded by enemies with max. one
-               //tile to escape.
+               // Check if the friendly unit is surrounded,
+			   // A unit is surrounded if it is flanked by enemy units
+			   // and at least one other enemy unit is nearby
+			   // or if the unit is totaly surrounded by enemies
+			   // with max. one tile to escape.
                bool is_surrounded = false;
                bool is_flanked = false;
                int enemy_units_around = 0;
@@ -143,15 +153,15 @@ void ai::do_attack_analysis(
 		int best_rating = 0;
 		int cur_position = -1;
 
-		//iterate over positions adjacent to the unit, finding the best rated one
+		// Iterate over positions adjacent to the unit, finding the best rated one.
 		for(int j = 0; j != 6; ++j) {
 
-			//if in this planned attack, a unit is already in this location
+			// If in this planned attack, a unit is already in this location.
 			if(used_locations[j]) {
 				continue;
 			}
 
-			//see if the current unit can reach that position
+			// See if the current unit can reach that position.
 			if (tiles[j] != current_unit) {
 				typedef std::multimap<location,location>::const_iterator Itor;
 				std::pair<Itor,Itor> its = dstsrc.equal_range(tiles[j]);
@@ -161,26 +171,26 @@ void ai::do_attack_analysis(
 					++its.first;
 				}
 
-				//if the unit can't move to this location
+				// If the unit can't move to this location.
 				if(its.first == its.second || units_.find(tiles[j]) != units_.end()) {
 					continue;
 				}
 			}
 
-			//check to see whether this move would be a backstab
+			// Check to see whether this move would be a backstab.
 			int backstab_bonus = 1;
 			double surround_bonus = 1.0;
 
 			if(tiles[(j+3)%6] != current_unit) {
 				const unit_map::const_iterator itor = units_.find(tiles[(j+3)%6]);
 
-				//note that we *could* also check if a unit plans to move there before we're
-				//at this stage, but we don't because, since the attack calculations don't
-				//actually take backstab into account (too complicated), this could actually
-				//make our analysis look *worse* instead of better.
-				//so we only check for 'concrete' backstab opportunities.
-				//That would also break backstab_check, since it
-				//assumes the defender is in place.
+				// Note that we *could* also check if a unit plans to move there
+				// before we're at this stage, but we don't because, since the
+				// attack calculations don't actually take backstab into account (too complicated),
+				// this could actually make our analysis look *worse* instead of better.
+				// So we only check for 'concrete' backstab opportunities.
+				// That would also break backstab_check, since it assumes
+				// the defender is in place.
 				if(itor != units_.end() &&
 					backstab_check(tiles[j], loc, units_, teams_)) {
 					if(backstab) {
@@ -193,22 +203,22 @@ void ai::do_attack_analysis(
 
 			}
 
-			//see if this position is the best rated we've seen so far
+			// See if this position is the best rated we've seen so far.
 			const int rating = rate_terrain(unit_itor->second,tiles[j]) * backstab_bonus;
 			if(cur_position >= 0 && rating < best_rating) {
 				continue;
 			}
 
-			//find out how vulnerable we are to attack from enemy units in this hex
+			// Find out how vulnerable we are to attack from enemy units in this hex.
 			const double vulnerability = power_projection(tiles[j],enemy_dstsrc);
 
-			//calculate how much support we have on this hex from allies. Support does not
-			//take into account terrain, because we don't want to move into a hex that is
-			//surrounded by good defensive terrain
+			// Calculate how much support we have on this hex from allies.
+			// Support does not take into account terrain, because we don't want
+			// to move into a hex that is surrounded by good defensive terrain.
 			const double support = power_projection(tiles[j],fullmove_dstsrc,false);
 
-			//if this is a position with equal defense to another position, but more vulnerability
-			//then we don't want to use it
+			// If this is a position with equal defense to another position,
+			// but more vulnerability then we don't want to use it.
 			if(cur_position >= 0 && rating == best_rating && vulnerability/surround_bonus - support*surround_bonus >= best_vulnerability - best_support) {
 				continue;
 			}
@@ -221,7 +231,7 @@ void ai::do_attack_analysis(
 
 		if(cur_position != -1) {
 			units.erase(units.begin() + i);
-			
+
 			cur_analysis.movements.push_back(std::pair<location,location>(current_unit,tiles[cur_position]));
 
 			cur_analysis.vulnerability += best_vulnerability;
@@ -253,32 +263,6 @@ void ai::do_attack_analysis(
 }
 
 
-struct battle_type {
-	battle_type(const gamemap::location& a, const gamemap::location& d,
-	            gamemap::TERRAIN t)
-	         : attacker(a),defender(d),terrain(t),weapon(-1)
-	{}
-
-	const gamemap::location attacker;
-	const gamemap::location defender;
-	const gamemap::TERRAIN terrain;
-	int weapon;
-};
-
-bool operator<(const battle_type& a, const battle_type& b)
-{
-	return a.attacker < b.attacker ||
-	       a.attacker == b.attacker && a.defender < b.defender ||
-		   a.attacker == b.attacker && a.defender == b.defender &&
-		   a.terrain < b.terrain;
-}
-
-bool operator==(const battle_type& a, const battle_type& b)
-{
-	return a.attacker == b.attacker && a.defender == b.defender &&
-	       a.terrain == b.terrain;
-}
-
 void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 								  const std::vector<team>& teams,
 								  const gamestatus& status, const game_data& gamedata,
@@ -287,9 +271,9 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
                                   const move_map& enemy_dstsrc, double aggression)
 {
 	const unit_map::const_iterator defend_it = units.find(target);
-	wassert(defend_it != units.end());
+	assert(defend_it != units.end());
 
-	//see if the target is a threat to our leader or an ally's leader
+	// See if the target is a threat to our leader or an ally's leader.
 	gamemap::location adj[6];
 	get_adjacent_tiles(target,adj);
 	size_t tile;
@@ -309,10 +293,10 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 	target_starting_damage = defend_it->second.max_hitpoints() -
 	                         defend_it->second.hitpoints();
 
-	//calculate the 'alternative_terrain_quality' -- the best possible defensive values
-	//the attacking units could hope to achieve if they didn't attack and moved somewhere.
-	//this is could for comparative purposes to see just how vulnerable the AI is
-	//making itself
+	// Calculate the 'alternative_terrain_quality' -- the best possible defensive values
+	// the attacking units could hope to achieve if they didn't attack and moved somewhere.
+	// This is used for comparative purposes, to see just how vulnerable the AI is
+	// making itself.
 	alternative_terrain_quality = 0.0;
 	double cost_sum = 0.0;
 	for(size_t i = 0; i != movements.size(); ++i) {
@@ -334,7 +318,7 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 	double first_chance_kill = 0.0;
 
 	double prob_dead_already = 0.0;
-	wassert(!movements.empty());
+	assert(!movements.empty());
 	std::vector<std::pair<location,location> >::const_iterator m;
 
 	battle_context *prev_bc = NULL;
@@ -358,9 +342,16 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 		// This cache is only about 99% correct, but speeds up evaluation by about 1000 times.
 		// We recalculate when we actually attack.
 		std::map<std::pair<location, const unit_type *>,std::pair<battle_context::unit_stats,battle_context::unit_stats> >::iterator usc;
-		usc = ai_obj.unit_stats_cache_.find(std::pair<location, const unit_type *>(target, &up->second.type()));
+		if(up->second.type()) {
+			usc = ai_obj.unit_stats_cache_.find(std::pair<location, const unit_type *>(target, up->second.type()));
+		} else {
+			usc = ai_obj.unit_stats_cache_.end();
+		}
 		// Just check this attack is valid for this attacking unit (may be modified)
-		if (usc != ai_obj.unit_stats_cache_.end() && usc->second.first.attack_num < (int)up->second.attacks().size()) {
+		if (usc != ai_obj.unit_stats_cache_.end() &&
+				usc->second.first.attack_num <
+				static_cast<int>(up->second.attacks().size())) {
+
 			from_cache = true;
 			bc = new battle_context(usc->second.first, usc->second.second);
 		} else {
@@ -373,17 +364,17 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 		prev_bc = bc;
 		prev_def = &bc->get_defender_combatant(prev_def);
 
-		if (!from_cache) {
+		if (!from_cache && up->second.type()) {
 			ai_obj.unit_stats_cache_.insert(std::pair<std::pair<location, const unit_type *>,std::pair<battle_context::unit_stats,battle_context::unit_stats> >
-											(std::pair<location, const unit_type *>(target, &up->second.type()),
+											(std::pair<location, const unit_type *>(target, up->second.type()),
 											 std::pair<battle_context::unit_stats,battle_context::unit_stats>(bc->get_attacker_stats(),
 																											  bc->get_defender_stats())));
 		}
 
-		// Note we didn't fight at all if defender already dead.
+		// Note we didn't fight at all if defender is already dead.
 		double prob_fought = (1.0 - prob_dead_already);
 
-		// FIXME: add combatant.prob_killed
+		//! @todo FIXME: add combatant.prob_killed
 		double prob_killed = def.hp_dist[0] - prob_dead_already;
 		prob_dead_already = def.hp_dist[0];
 
@@ -392,12 +383,12 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 
 		double cost = up->second.cost();
 		const bool on_village = map.is_village(m->second);
-		//up to double the value of a unit based on experience
+		// Up to double the value of a unit based on experience
 		cost += (double(up->second.experience())/double(up->second.max_experience()))*cost;
 		resources_used += cost;
 		avg_losses += cost * prob_died;
 
-		//double reward to emphasize getting onto villages if they survive
+		// Double reward to emphasize getting onto villages if they survive.
 		if (on_village) {
 			avg_damage_taken -= game_config::poison_amount*2 * prob_survived;
 		}
@@ -405,12 +396,12 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 		terrain_quality += (double(bc->get_defender_stats().chance_to_hit)/100.0)*cost * (on_village ? 0.5 : 1.0);
 
 		double advance_prob = 0.0;
-		//the reward for advancing a unit is to get a 'negative' loss of that unit
+		// The reward for advancing a unit is to get a 'negative' loss of that unit
 		if (!up->second.advances_to().empty()) {
 			int xp_for_advance = up->second.max_experience() - up->second.experience();
 			int kill_xp, fight_xp;
 
-			// See bug #6272... in some cases unit has already got enough xp to advance,
+			// See bug #6272... in some cases, unit already has got enough xp to advance,
 			// but hasn't (bug elsewhere?).  Can cause divide by zero.
 			if (xp_for_advance <= 0)
 				xp_for_advance = 1;
@@ -424,16 +415,17 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 				advance_prob = prob_killed;
 			avg_losses -= up->second.cost() * advance_prob;
 
-			//the reward for getting a unit closer to advancement (if
-			//it didn't advance) is to get the proportion of remaining
-			//experience needed, and multiply it by a quarter of the
-			//unit cost. This will cause the AI to heavily favor
-			//getting xp for close-to-advance units.
+			// The reward for getting a unit closer to advancement
+			// (if it didn't advance) is to get the proportion of
+			// remaining experience needed, and multiply it by
+			// a quarter of the unit cost.
+			// This will cause the AI to heavily favor
+			// getting xp for close-to-advance units.
 			avg_losses -= (up->second.cost()*fight_xp)/(xp_for_advance*4) * (prob_fought - prob_killed);
 			avg_losses -= (up->second.cost()*kill_xp)/(xp_for_advance*4) * prob_killed;
 
-			//the reward for killing with a unit that
-			//plagues is to get a 'negative' loss of that unit
+			// The reward for killing with a unit that plagues
+			// is to get a 'negative' loss of that unit.
 			if (bc->get_attacker_stats().plagues) {
 				avg_losses -= prob_killed * up->second.cost();
 			}
@@ -442,7 +434,7 @@ void ai::attack_analysis::analyze(const gamemap& map, unit_map& units,
 		// If we didn't advance, we took this damage.
 		avg_damage_taken += (up->second.hitpoints() - att.average_hp()) * (1.0 - advance_prob);
 
-		// FIXME: attack_prediction.cpp should understand advancement directly.
+		//! @todo FIXME: attack_prediction.cpp should understand advancement directly.
 		// For each level of attacker def gets 1 xp or kill_experience.
 		def_avg_experience += up->second.level() *
 			(1.0 - att.hp_dist[0] + game_config::kill_experience * att.hp_dist[0]);
@@ -479,8 +471,8 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 		aggression = 1.0;
 	}
 
-	//only use the leader if we do a serious amount of damage
-	//compared to how much they do to us.
+	// Only use the leader if we do a serious amount of damage,
+	// compared to how much they do to us.
 	if(uses_leader && aggression > -4.0) {
 		LOG_AI << "uses leader..\n";
 		aggression = -4.0;
@@ -489,9 +481,10 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 	double value = chance_to_kill*target_value - avg_losses*(1.0-aggression);
 
 	if(terrain_quality > alternative_terrain_quality) {
-		//this situation looks like it might be a bad move: we are moving our attackers out
-		//of their optimal terrain into sub-optimal terrain.
-		//calculate the 'exposure' of our units to risk
+		// This situation looks like it might be a bad move:
+		// we are moving our attackers out of their optimal terrain
+		// into sub-optimal terrain.
+		// Calculate the 'exposure' of our units to risk.
 
 		const double exposure_mod = uses_leader ? 2.0 : ai_obj.current_team().caution();
 		const double exposure = exposure_mod*resources_used*(terrain_quality - alternative_terrain_quality)*vulnerability/maximum<double>(0.01,support);
@@ -504,22 +497,27 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 		value -= exposure*(1.0-aggression);
 	}
 
-	//if this attack uses our leader, and the leader can reach the keep, and has gold to spend, reduce
-	//the value to reflect the leader's lost recruitment opportunity in the case of an attack
+	// If this attack uses our leader, and the leader can reach the keep,
+	// and has gold to spend, reduce the value to reflect the leader's
+	// lost recruitment opportunity in the case of an attack.
 	if(uses_leader && ai_obj.leader_can_reach_keep() && ai_obj.current_team().gold() > 20) {
 		value -= double(ai_obj.current_team().gold())*0.5;
 	}
 
-	//prefer to attack already damaged targets
+	// Prefer to attack already damaged targets.
 	value += ((target_starting_damage/3 + avg_damage_inflicted) - (1.0-aggression)*avg_damage_taken)/10.0;
 
-       //if the unit is surrounded and there is no support or if the unit is surrounded and the average damage
-       //is 0, the unit skips its sanity check and tries to break free as good as possible
+       // If the unit is surrounded and there is no support,
+	   // or if the unit is surrounded and the average damage is 0,
+	   // the unit skips its sanity check and tries to break free as good as possible.
        if(!is_surrounded || (support != 0 && avg_damage_taken != 0))
        {
-               //sanity check: if we're putting ourselves at major risk, and have no chance to kill, and we're not
-               //aiding our allies who are also attacking, then don't do it
-               if(vulnerability > 50.0 && vulnerability > support*2.0 && chance_to_kill < 0.02 && aggression < 0.75 && !ai_obj.attack_close(target)) {
+               // Sanity check: if we're putting ourselves at major risk,
+			   // and have no chance to kill, and we're not aiding our allies
+			   // who are also attacking, then don't do it.
+               if(vulnerability > 50.0 && vulnerability > support*2.0
+			   && chance_to_kill < 0.02 && aggression < 0.75
+			   && !ai_obj.attack_close(target)) {
                        return -1.0;
                }
         }
@@ -535,9 +533,12 @@ double ai::attack_analysis::rating(double aggression, ai& ai_obj) const
 	}
 
 	LOG_AI << "attack on " << target << ": attackers: " << movements.size()
-		<< " value: " << value << " chance to kill: " << chance_to_kill << " damage inflicted: "
-		<< avg_damage_inflicted << " damage taken: " << avg_damage_taken << " vulnerability: "
-		<< vulnerability << " support: " << support << " quality: " << terrain_quality
+		<< " value: " << value << " chance to kill: " << chance_to_kill
+		<< " damage inflicted: " << avg_damage_inflicted
+		<< " damage taken: " << avg_damage_taken
+		<< " vulnerability: " << vulnerability
+		<< " support: " << support
+		<< " quality: " << terrain_quality
 		<< " alternative quality: " << alternative_terrain_quality << "\n";
 
 	return value;
@@ -570,7 +571,8 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 
 	for(unit_map::const_iterator j = units_.begin(); j != units_.end(); ++j) {
 
-		//attack anyone who is on the enemy side, and who is not invisible or turned to stone
+		// Attack anyone who is on the enemy side,
+		// and who is not invisible or turned to stone.
 		if(current_team().is_enemy(j->second.side()) && !j->second.incapacitated() &&
 		   j->second.invisible(j->first,units_,teams_) == false) {
 			location adjacent[6];
@@ -580,15 +582,15 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 			analysis.vulnerability = 0.0;
 			analysis.support = 0.0;
 
-			const int ticks = SDL_GetTicks();
+//			const int ticks = SDL_GetTicks();
 
 			do_attack_analysis(j->first,srcdst,dstsrc,fullmove_srcdst,fullmove_dstsrc,enemy_srcdst,enemy_dstsrc,
 			            adjacent,used_locations,unit_locs,res,analysis);
 
-			const int time_taken = SDL_GetTicks() - ticks;
-			static int max_time = 0;
-			if(time_taken > max_time)
-				max_time = time_taken;
+//			const int time_taken = SDL_GetTicks() - ticks;
+//			static int max_time = 0;
+//			if(time_taken > max_time)
+//				max_time = time_taken;
 		}
 	}
 
@@ -597,23 +599,32 @@ std::vector<ai::attack_analysis> ai::analyze_targets(
 
 double ai::power_projection(const gamemap::location& loc,  const move_map& dstsrc, bool use_terrain) const
 {
-	static gamemap::location used_locs[6];
-	static double ratings[6];
+	gamemap::location used_locs[6];
+	int ratings[6];
 	int num_used_locs = 0;
 
-	static gamemap::location locs[6];
+	gamemap::location locs[6];
 	get_adjacent_tiles(loc,locs);
 
 	const int lawful_bonus = state_.get_time_of_day().lawful_bonus;
 
-	double res = 0.0;
+	int res = 0;
 
-	for(int i = 0; i != 6; ++i) {
-		if(map_.on_board(locs[i]) == false) {
+	bool changed = false;
+	for (int i = 0;; ++i) {
+		if (i == 6) {
+			if (!changed) break;
+			// Loop once again, in case a unit found a better spot
+			// and freed the place for another unit.
+			changed = false;
+			i = 0;
+		}
+
+		if (map_.on_board(locs[i]) == false) {
 			continue;
 		}
 
-		const gamemap::TERRAIN terrain = map_[locs[i].x][locs[i].y];
+		const t_translation::t_terrain terrain = map_[locs[i]];
 
 		typedef move_map::const_iterator Itor;
 		typedef std::pair<Itor,Itor> Range;
@@ -622,95 +633,77 @@ double ai::power_projection(const gamemap::location& loc,  const move_map& dstsr
 		gamemap::location* const beg_used = used_locs;
 		gamemap::location* end_used = used_locs + num_used_locs;
 
-		double best_rating = 0.0;
+		int best_rating = 0;
 		gamemap::location best_unit;
 
-		for(int n = 0; n != 2; ++n) {
-			for(Itor it = its.first; it != its.second; ++it) {
-				if(std::find(beg_used,end_used,it->second) != end_used) {
-					continue;
+		for(Itor it = its.first; it != its.second; ++it) {
+			const unit_map::const_iterator u = units_.find(it->second);
+
+			// Unit might have been killed, and no longer exist
+			if(u == units_.end()) {
+				continue;
+			}
+
+			const unit& un = u->second;
+
+			int tod_modifier = 0;
+			if(un.alignment() == unit_type::LAWFUL) {
+				tod_modifier = lawful_bonus;
+			} else if(un.alignment() == unit_type::CHAOTIC) {
+				tod_modifier = -lawful_bonus;
+			}
+
+			int hp = un.hitpoints() * 1000 / un.max_hitpoints();
+			int most_damage = 0;
+			for(std::vector<attack_type>::const_iterator att =
+			    un.attacks().begin(); att != un.attacks().end(); ++att) {
+				int damage = att->damage() * att->num_attacks() *
+				             (100 + tod_modifier);
+				if(damage > most_damage) {
+					most_damage = damage;
 				}
+			}
 
-				const unit_map::const_iterator u = units_.find(it->second);
-
-				//unit might have been killed, and no longer exist
-				if(u == units_.end()) {
-					continue;
-				}
-
-				const unit& un = u->second;
-
-				int tod_modifier = 0;
-				if(un.alignment() == unit_type::LAWFUL) {
-					tod_modifier = lawful_bonus;
-				} else if(un.alignment() == unit_type::CHAOTIC) {
-					tod_modifier = -lawful_bonus;
-				}
-
-				const double hp = double(un.hitpoints())/
-				                  double(un.max_hitpoints());
-				int most_damage = 0;
-				for(std::vector<attack_type>::const_iterator att =
-				    un.attacks().begin(); att != un.attacks().end(); ++att) {
-					const int damage = (att->damage()*att->num_attacks()*(100+tod_modifier))/100;
-					if(damage > most_damage) {
-						most_damage = damage;
-					}
-				}
-
-				const bool village = map_.is_village(terrain);
-				const double village_bonus = (use_terrain && village) ? 1.5 : 1.0;
-
-				const double defense = use_terrain ? double(100 - un.defense_modifier(terrain))/100.0 : 0.5;
-				const double rating = village_bonus*hp*defense*double(most_damage);
-				if(rating > best_rating) {
+			int village_bonus = use_terrain && map_.is_village(terrain) ? 3 : 2;
+			int defense = use_terrain ? 100 - un.defense_modifier(terrain) : 50;
+			int rating = hp * defense * most_damage * village_bonus / 200;
+			if(rating > best_rating) {
+				gamemap::location *pos = std::find(beg_used, end_used, it->second);
+				// Check if the spot is the same or better than an older one.
+				if (pos == end_used || rating >= ratings[pos - beg_used]) {
 					best_rating = rating;
 					best_unit = it->second;
 				}
 			}
-
-			//if this is the second time through, then we are looking at a unit
-			//that has already been used, but for whom we may have found
-			//a better position to attack from
-			if(n == 1 && best_unit.valid()) {
-				end_used = beg_used + num_used_locs;
-				gamemap::location* const pos = std::find(beg_used,end_used,best_unit);
-				const int index = pos - beg_used;
-				if(best_rating >= ratings[index]) {
-					res -= ratings[index];
-					res += best_rating;
-					used_locs[index] = best_unit;
-					ratings[index] = best_rating;
-				}
-
-				best_unit = gamemap::location();
-				break;
-			}
-
-			if(best_unit.valid()) {
-				break;
-			}
-
-			end_used = beg_used;
 		}
 
-		if(best_unit.valid()) {
-			used_locs[num_used_locs] = best_unit;
-			ratings[num_used_locs] = best_rating;
+		if (!best_unit.valid()) continue;
+		gamemap::location *pos = std::find(beg_used, end_used, best_unit);
+		int index = pos - beg_used;
+		if (index == num_used_locs)
 			++num_used_locs;
-			res += best_rating;
+		else if (best_rating == ratings[index])
+			continue;
+		else {
+			// The unit was in another spot already, so remove its older rating
+			// from the final result, and require a new run to fill its old spot.
+			res -= ratings[index];
+			changed = true;
 		}
+		used_locs[index] = best_unit;
+		ratings[index] = best_rating;
+		res += best_rating;
 	}
 
-	return res;
+	return res / 100000.;
 }
 
-// There is no real hope for us: we should try to do some damage to the enemy.
-// We can spend some cycles here, since it's rare.
+//! There is no real hope for us: we should try to do some damage to the enemy.
+//! We can spend some cycles here, since it's rare.
 bool ai::desperate_attack(const gamemap::location &loc)
 {
 	const unit &u = units_.find(loc)->second;
-	std::cerr << "desperate attack by '" << u.id() << "' " << loc << "\n";
+	LOG_AI << "desperate attack by '" << u.id() << "' " << loc << "\n";
 
 	gamemap::location adj[6];
 	get_adjacent_tiles(loc, adj);
@@ -727,7 +720,7 @@ bool ai::desperate_attack(const gamemap::location &loc)
 			   current_team().is_enemy(enemy->second.side()) && !enemy->second.incapacitated()) {
 				const std::vector<attack_type>& attacks = u.attacks();
 				for (unsigned int i = 0; i != attacks.size(); ++i) {
-					// skip weapons with attack_weight=0
+					// Skip weapons with attack_weight=0
 					if (attacks[i].attack_weight() > 0) {
 						battle_context bc(map_, teams_, units_, state_, gameinfo_, loc, adj[n], i);
 						combatant att(bc.get_attacker_stats());
@@ -761,7 +754,7 @@ bool ai::desperate_attack(const gamemap::location &loc)
 
 				const std::vector<attack_type>& attacks = units_.find(adj[n])->second.attacks();
 				for (unsigned int i = 0; i != attacks.size(); ++i) {
-					// skip weapons with attack_weight=0
+					// SKip weapons with attack_weight=0
 					if (attacks[i].attack_weight() > 0) {
 						battle_context bc(map_, teams_, units_, state_, gameinfo_, adj[n], loc, i);
 						combatant att(bc.get_attacker_stats());
@@ -777,7 +770,7 @@ bool ai::desperate_attack(const gamemap::location &loc)
 		}
 	}
 
-	// It's possible that there were no adjacent units to attack...
+	// It is possible that there were no adjacent units to attack...
 	if (least_hp != u.hitpoints() + 1) {
 		battle_context bc(map_, teams_, units_, state_, gameinfo_, loc, adj[best_dir], -1, -1, 0.5);
 		attack_enemy(loc, adj[best_dir], bc.get_attacker_stats().attack_num,
