@@ -1,10 +1,12 @@
-/* $Id: ai_move.cpp 11677 2006-05-28 11:57:45Z rusty $ */
+/* $Id: ai_move.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version
+   or at your option any later version2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,11 +17,14 @@
 
 #include "ai.hpp"
 #include "game_config.hpp"
+#include "gettext.hpp"
 #include "log.hpp"
 #include "map.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
+#include "variable.hpp"
+#include "wml_exception.hpp"
 
+#include <cassert>
 #include <iostream>
 
 #define LOG_AI LOG_STREAM(info, ai)
@@ -53,9 +58,9 @@ struct move_cost_calculator : cost_calculator
 			++range.first;
 		}
 		*/
-		wassert(map_.on_board(loc));
+		assert(map_.on_board(loc));
 
-		const gamemap::TERRAIN terrain = map_[loc.x][loc.y];
+		const t_translation::t_terrain terrain = map_[loc];
 
 		const double modifier = 1.0; //move_type_.defense_modifier(map_,terrain);
 		const double move_cost = unit_.movement_cost(terrain);//move_type_[terrain];
@@ -68,7 +73,8 @@ struct move_cost_calculator : cost_calculator
 		if (units_.count(loc))
 			res *= 4.0;
 
-		wassert(res > 0);
+		VALIDATE(res > 0, 
+			_("Movement cost is 0, probably a terrain with movement cost of 0."));
 		return res;
 	}
 
@@ -77,7 +83,7 @@ private:
 	const gamemap& map_;
 	const game_data& data_;
 	const unit_map& units_;
-//	mutable std::map<gamemap::TERRAIN,int> move_type_;
+//	mutable std::map<t_translation::t_terrain,int> move_type_;
 	const gamemap::location loc_;
 	const ai::move_map dstsrc_, enemy_dstsrc_;
 	const bool avoid_enemies_;
@@ -111,7 +117,7 @@ std::vector<ai::target> ai::find_targets(unit_map::const_iterator leader, const
 				}
 			}
 
-			wassert(threats.empty() == false);
+			assert(threats.empty() == false);
 
 			const double value = threat/double(threats.size());
 			for(std::set<gamemap::location>::const_iterator i = threats.begin(); i != threats.end(); ++i) {
@@ -122,8 +128,10 @@ std::vector<ai::target> ai::find_targets(unit_map::const_iterator leader, const
 
 	if(has_leader && current_team().village_value() > 0.0) {
 		const std::vector<location>& villages = map_.villages();
-		for(std::vector<location>::const_iterator t = villages.begin(); t != villages.end(); ++t) {
-			wassert(map_.on_board(*t));
+		for(std::vector<location>::const_iterator t = 
+				villages.begin(); t != villages.end(); ++t) {
+
+			assert(map_.on_board(*t));
 			bool get_village = true;
 			for(size_t i = 0; i != teams_.size(); ++i) {
 				if(!current_team().is_enemy(i+1) && teams_[i].owns_village(*t)) {
@@ -145,15 +153,17 @@ std::vector<ai::target> ai::find_targets(unit_map::const_iterator leader, const
 	for(u = units_.begin(); u != units_.end(); ++u) {
 
 		//is an enemy leader
-		if(u->second.can_recruit() && current_team().is_enemy(u->second.side())) {
-			wassert(map_.on_board(u->first));
+		if(u->second.can_recruit() && 
+				current_team().is_enemy(u->second.side())) {
+
+			assert(map_.on_board(u->first));
 			targets.push_back(target(u->first,current_team().leader_value(),target::LEADER));
 		}
 
 		//explicit targets for this team
 		for(std::vector<team::target>::iterator j = team_targets.begin();
 		    j != team_targets.end(); ++j) {
-			if(u->second.matches_filter(j->criteria,u->first)) {
+			if(u->second.matches_filter(&(j->criteria),u->first)) {
 				LOG_AI << "found explicit target..." << j->value << "\n";
 				targets.push_back(target(u->first,j->value,target::EXPLICIT));
 			}
@@ -178,7 +188,7 @@ std::vector<ai::target> ai::find_targets(unit_map::const_iterator leader, const
 		}
 	}
 
-	wassert(new_values.size() == targets.size());
+	assert(new_values.size() == targets.size());
 	for(size_t n = 0; n != new_values.size(); ++n) {
 		LOG_AI << "target value: " << targets[n].value << " -> " << new_values[n] << "\n";
 		targets[n].value = new_values[n];
@@ -386,7 +396,7 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 
 	std::vector<target>::const_iterator ittg;
 	for(ittg = targets.begin(); ittg != targets.end(); ++ittg) {
-		wassert(map_.on_board(ittg->loc));
+		assert(map_.on_board(ittg->loc));
 	}
 
 	paths::route best_route;
@@ -425,10 +435,10 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 
 		raise_user_interact();
 
-		wassert(map_.on_board(tg->loc));
+		assert(map_.on_board(tg->loc));
 
 		const double locStopValue = minimum(tg->value / best_rating, 500.0);
-		paths::route cur_route = a_star_search(u->first, tg->loc, locStopValue, &cost_calc, map_.x(), map_.y());
+		paths::route cur_route = a_star_search(u->first, tg->loc, locStopValue, &cost_calc, map_.w(), map_.h());
 
 		if (cur_route.move_left < locStopValue)
 		{
@@ -513,7 +523,7 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 
 			const move_cost_calculator calc(u->second, map_, gameinfo_, units_, u->first, dstsrc, enemy_dstsrc);
 			const double locStopValue = minimum(best_target->value / best_rating, 100.0);
-			paths::route cur_route = a_star_search(u->first, best_target->loc, locStopValue, &calc, map_.x(), map_.y());
+			paths::route cur_route = a_star_search(u->first, best_target->loc, locStopValue, &calc, map_.w(), map_.h());
 
 			if (cur_route.move_left < locStopValue)
 			{
@@ -566,10 +576,10 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 
 	LOG_AI << "best unit: " << best->first << '\n';
 
-	wassert(best_target >= targets.begin() && best_target < targets.end());
+	assert(best_target >= targets.begin() && best_target < targets.end());
 
 	for(ittg = targets.begin(); ittg != targets.end(); ++ittg) {
-		wassert(map_.on_board(ittg->loc));
+		assert(map_.on_board(ittg->loc));
 	}
 
 	//if our target is a position to support, then we
@@ -710,7 +720,7 @@ std::pair<gamemap::location,gamemap::location> ai::choose_move(std::vector<targe
 	    best_route.steps.rbegin(); ri != best_route.steps.rend(); ++ri) {
 
 		if(game_config::debug) {
-			//display::debug_highlight(*ri,0.2);
+			//game_display::debug_highlight(*ri,static_cast<size_t>(0.2));
 		}
 
 		//this is set to 'true' if we are hesitant to proceed because of enemy units,
@@ -777,7 +787,7 @@ void ai::access_points(const move_map& srcdst, const location& u, const location
 		const location& loc = i->second;
 		if (int(distance_between(loc,dst)) <= u_it->second.total_movement()) {
 			shortest_path_calculator calc(u_it->second, current_team(), units_, teams_, map_);
-			const paths::route& rt = a_star_search(loc, dst, u_it->second.total_movement(), &calc, map_.x(), map_.y());
+			const paths::route& rt = a_star_search(loc, dst, u_it->second.total_movement(), &calc, map_.w(), map_.h());
 			if(rt.steps.empty() == false) {
 				out.push_back(loc);
 			}
