@@ -1,4 +1,4 @@
-/* $Id: ai_python.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: ai_python.cpp 24954 2008-03-21 19:36:43Z brunowolff $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -738,7 +738,7 @@ static long location_internal_hash(wesnoth_location* obj)
 	// e.g. when checking all positions in a certain radius at the map border.
 	unsigned char x = static_cast<unsigned>(obj->location_->x);
 	unsigned char y = static_cast<unsigned>(obj->location_->y);
-	return x << 8 + y;
+	return (x << 8) + y;
 }
 
 static PyGetSetDef location_getseters[] = {
