@@ -1,11 +1,12 @@
-/* $Id: astarnode.hpp 8553 2005-10-11 19:03:59Z isaac $ */
+/* $Id: astarnode.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-Copyright (C) 2003 by David White <davidnwhite@verizon.net>
-Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+Copyright (C) 2003 by David White <dave@whitevine.net>
+Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
 Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
 This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License.
+it under the terms of the GNU General Public License version 2
+or at your option any later version.
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY.
 
@@ -15,10 +16,8 @@ See the COPYING file for more details.
 #ifndef ASTARNODE_H_INCLUDED
 #define ASTARNODE_H_INCLUDED
 
-#include "map.hpp"
 #include "pathutils.hpp"
 #include <set>
-#include <vector>
 
 struct a_star_node
 {
