@@ -1,10 +1,11 @@
-/* $Id$ */
+/* $Id: attack_prediction.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2006 by Rusty Russell <rusty@rustcorp.com.au>
+   Copyright (C) 2006 - 2008 by Rusty Russell <rusty@rustcorp.com.au>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -12,18 +13,23 @@
 
    Full algorithm by Yogin.  Typing and optimization by Rusty.
 
-   This code has lots of debugging.  It is there for a reason: this
-   code is kinda tricky.  Do not remove it.
+   This code has lots of debugging.  It is there for a reason:
+   this code is kinda tricky.  Do not remove it.
 */
+
+//! @file attack_prediction.cpp
+//! Simulate combat to calculate attacks. Standalone program, benchmark.
+
 #include "attack_prediction.hpp"
 
+#include <cassert>
 #include <cstring> // For memset
 #include <vector>
 
-#include "wassert.hpp"
 
-// Compile with -O3 -DBENCHMARK for speed testing, -DCHECK for testing
-// correctness (run tools/wesnoth-attack-sim.c --check on output)
+// Compile with -O3 -DBENCHMARK for speed testing,
+// -DCHECK for testing correctness
+// (run tools/wesnoth-attack-sim.c --check on output)
 #if !defined(BENCHMARK) && !defined(CHECK)
 #include "util.hpp"
 #else
@@ -43,7 +49,7 @@
 
 namespace
 {
-// A matrix of A's hitpoints vs B's hitpoints.
+//! A matrix of A's hitpoints vs B's hitpoints.
 struct prob_matrix
 {
 	// Simple matrix, both known HP.
@@ -77,13 +83,12 @@ struct prob_matrix
 	void dump() const;
 
 	// We need four matrices, or "planes", reflecting the possible
-	// "slowed" states (neither slowed, A slowed, B slowed, both
-	// slowed).
+	// "slowed" states (neither slowed, A slowed, B slowed, both slowed).
 	enum {
 		NEITHER_SLOWED,
 		A_SLOWED,
 		B_SLOWED,
-		BOTH_SLOWED,
+		BOTH_SLOWED
 	};
 
 private:
@@ -107,7 +112,7 @@ private:
 	void shift_rows(unsigned dst, unsigned src,
 					unsigned damage, double prob, bool drain);
 
-	// FIXME: rename using _ at end.
+	//! @todo FIXME: rename using _ at end.
 	unsigned int rows, cols;
 	double *plane[4];
 
@@ -127,7 +132,7 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 		if (!a_summary[1].empty())
 			b_slows = true;
 		// Don't handle both being reused.
-		wassert(b_summary[0].empty());
+		assert(b_summary[0].empty());
 	}
 	if (!b_summary[0].empty()) {
 		// B has fought before.  Do we need a slow plane for it?
@@ -156,7 +161,7 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 
 	// Transfer HP distribution from A?
 	if (!a_summary[0].empty()) {
-		// FIXME: Can optimize here.
+		// @todo FIXME: Can optimize here.
 		min_row[NEITHER_SLOWED] = 0;
 		min_row[A_SLOWED] = 0;
 		min_col[A_SLOWED] = b_hp - 1;
@@ -181,8 +186,8 @@ prob_matrix::prob_matrix(unsigned int a_max_hp, unsigned int b_max_hp,
 		debug(("B has fought before\n"));
 		dump();
 	} else {
-		// if a unit has drain it might end with more HP than before
-		// make sure we don't access the matrix in invalid positions
+		// If a unit has drain it might end with more HP than before.
+		// Make sure we don't access the matrix in invalid positions.
 		a_hp = minimum<unsigned int>(a_hp, rows - 1);
 		b_hp = minimum<unsigned int>(b_hp, cols - 1);
 		val(NEITHER_SLOWED, a_hp, b_hp) = 1.0;
@@ -207,11 +212,15 @@ double *prob_matrix::new_arr(unsigned int size)
 
 double &prob_matrix::val(unsigned p, unsigned row, unsigned col)
 {
+	assert(row < rows);
+	assert(col < cols);
 	return plane[p][row * cols + col];
 }
 
 const double &prob_matrix::val(unsigned p, unsigned row, unsigned col) const
 {
+	assert(row < rows);
+	assert(col < cols);
 	return plane[p][row * cols + col];
 }
 
@@ -311,8 +320,8 @@ void prob_matrix::shift_rows(unsigned dst, unsigned src,
 	}
 }
 
-// Shift prob_matrix to reflect probability 'hit_chance' that damage (up
-// to) 'damage' is done to 'b'.
+// Shift prob_matrix to reflect probability 'hit_chance'
+// that damage (up to) 'damage' is done to 'b'.
 void prob_matrix::receive_blow_b(unsigned damage, unsigned slow_damage, double hit_chance,
 								 bool a_slows, bool a_drains)
 {
@@ -325,7 +334,7 @@ void prob_matrix::receive_blow_b(unsigned damage, unsigned slow_damage, double h
 		if (!plane[src])
 			continue;
 
-		// If a slows us we go from 0=>2, 1=>3, 2=>2 3=>3.
+		// If A slows us, we go from 0=>2, 1=>3, 2=>2 3=>3.
 		if (a_slows)
 			dst = (src|2);
 		else
@@ -437,8 +446,8 @@ double prob_matrix::dead_prob() const
 	return prob;
 }
 
-// Shift matrix to reflect probability 'hit_chance' that damage (up
-// to) 'damage' is done to 'a'.
+// Shift matrix to reflect probability 'hit_chance'
+// that damage (up to) 'damage' is done to 'a'.
 void prob_matrix::receive_blow_a(unsigned damage, unsigned slow_damage, double hit_chance,
 								 bool b_slows, bool b_drains)
 {
@@ -451,7 +460,7 @@ void prob_matrix::receive_blow_a(unsigned damage, unsigned slow_damage, double h
 		if (!plane[src])
 			continue;
 
-		// If b slows us we go from 0=>1, 1=>1, 2=>3 3=>3.
+		// If B slows us, we go from 0=>1, 1=>1, 2=>3 3=>3.
 		if (b_slows)
 			dst = (src|1);
 		else
@@ -473,7 +482,7 @@ void prob_matrix::receive_blow_a(unsigned damage, unsigned slow_damage, double h
 	}
 }
 
-};
+} // end anon namespace
 
 unsigned combatant::hp_dist_size(const battle_context::unit_stats &u, const combatant *prev)
 {
@@ -518,9 +527,9 @@ combatant::combatant(const combatant &that, const battle_context::unit_stats &u)
 
 
 
-// For swarm, whether we get an attack depends on HP distribution from
-// previous combat.  So we roll this into our P(hitting), since no
-// attack is equivalent to missing.
+// For swarm, whether we get an attack depends on HP distribution
+// from previous combat.  So we roll this into our P(hitting),
+// since no attack is equivalent to missing.
 void combatant::adjust_hitchance()
 {
 	if (summary[0].empty() || u_.swarm_min == u_.swarm_max)
@@ -542,7 +551,9 @@ void combatant::adjust_hitchance()
 		}
 		if (!summary[1].empty())
 			prob += summary[1][i];
-		for (unsigned int j = 0; j < u_.swarm_min + (u_.swarm_max - (double)u_.swarm_min) * u_.hp / u_.max_hp; j++)
+		for (unsigned int j = 0; j < u_.swarm_min + (u_.swarm_max -
+                static_cast<double>(u_.swarm_min)) * u_.hp / u_.max_hp; j++)
+
 			hit_chances_[j] += prob * u_.chance_to_hit / 100.0 / alive_prob;
 	}
 
@@ -552,14 +563,14 @@ void combatant::adjust_hitchance()
 	debug(("\n"));
 }
 
-// Minimum hp we could possibly have.
+// Minimum HP we could possibly have.
 unsigned combatant::min_hp() const
 {
 	if (summary[0].empty())
 		return u_.hp;
 
 	// We don't handle this (yet).
-	wassert(summary[1].empty());
+	assert(summary[1].empty());
 
 	unsigned int i;
 	for (i = 0; summary[0][i] == 0; i++);
@@ -623,7 +634,7 @@ void combatant::one_strike_fight(combatant &opp)
 			opp.summary[0][opp.u_.hp] = 1.0 - hit_chances_[0];
 			opp.summary[0][maximum<int>(opp.u_.hp - u_.damage, 0)] = hit_chances_[0];
 		} else {
-			wassert(hit_chances_.size() == 0);
+			assert(hit_chances_.size() == 0);
 			opp.summary[0][opp.u_.hp] = 1.0;
 		}
 	} else {
@@ -644,7 +655,7 @@ void combatant::one_strike_fight(combatant &opp)
 			summary[0][u_.hp] = 1.0 - opp.hit_chances_[0] * opp_alive_prob;
 			summary[0][maximum<int>(u_.hp - opp.u_.damage, 0)] = opp.hit_chances_[0] * opp_alive_prob;
 		} else {
-			wassert(opp.hit_chances_.size() == 0);
+			assert(opp.hit_chances_.size() == 0);
 			summary[0][u_.hp] = 1.0;
 		}
 	} else {
@@ -672,7 +683,7 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 	unsigned int b_damage = opp.u_.damage, b_slow_damage = opp.u_.slow_damage;
 
 	// To simulate stoning, we set to amount which kills, and re-adjust after.
-	// FIXME: This doesn't work for rolling calculations, just first battle.
+	//! @todo FIXME: This doesn't work for rolling calculations, just first battle.
 	if (u_.stones)
 		a_damage = a_slow_damage = opp.u_.max_hp;
 	if (opp.u_.stones)
@@ -708,8 +719,9 @@ void combatant::complex_fight(combatant &opp, unsigned int rounds)
 }
 
 // Two man enter.  One man leave!
-// ... Or maybe two.  But definitely not three.  Of course, one could
-// be a woman.  Or both.  And neither could be human, too.
+// ... Or maybe two.  But definitely not three.
+// Of course, one could be a woman.  Or both.
+// And neither could be human, too.
 // Um, ok, it was a stupid thing to say.
 void combatant::fight(combatant &opp)
 {
@@ -757,8 +769,8 @@ void combatant::fight(combatant &opp)
 	}
 
 #if 0
-	wassert(summary[0].size() == res.size());
-	wassert(opp.summary[0].size() == opp_res.size());
+	assert(summary[0].size() == res.size());
+	assert(opp.summary[0].size() == opp_res.size());
 	for (unsigned int i = 0; i < summary[0].size(); i++) {
 		if (fabs(summary[0][i] - res[i]) > 0.000001) {
 			std::cerr << "Mismatch for " << i << " hp: " << summary[0][i] << " should have been " << res[i] << "\n";
@@ -787,8 +799,9 @@ void combatant::fight(combatant &opp)
 			opp.hp_dist[i] = opp.summary[0][i] + opp.summary[1][i];
 	}
 
-	// make sure we don't try to access the vectors out of bounds, drain increases hps
-	// so we determine the number of hp here and make sure it stays within bounds
+	// Make sure we don't try to access the vectors out of bounds,
+	// drain increases HPs so we determine the number of HP here
+	// and make sure it stays within bounds
 	const unsigned int hp = minimum<unsigned int>(u_.hp, hp_dist.size() - 1);
 	const unsigned int opp_hp = minimum<unsigned int>(opp.u_.hp, opp.hp_dist.size() - 1);
 
@@ -805,7 +818,7 @@ void combatant::fight(combatant &opp)
 	if (u_.slows)
 		opp.slowed += (1 - opp.slowed) * opp_touched;
 
-	// FIXME: This is approximate: we could drain, then get hit.
+	//! @todo FIXME: This is approximate: we could drain, then get hit.
 	untouched = hp_dist[hp];
 	opp.untouched = opp.hp_dist[opp_hp];
 }
@@ -822,8 +835,8 @@ double combatant::average_hp(unsigned int healing) const
 }
 
 #if defined(BENCHMARK) || defined(CHECK)
-// We create a significant number of nasty-to-calculate units, and
-// test each one against the others.
+// We create a significant number of nasty-to-calculate units,
+// and test each one against the others.
 #define NUM_UNITS 50
 
 // Stolen from glibc headers sys/time.h
@@ -893,8 +906,8 @@ static void run(unsigned specific_battle)
 				if (specific_battle && battle != specific_battle)
 					continue;
 				u[j]->fight(*u[i]);
-				// We need this here, because swarm means out num hits
-				// can change.
+				// We need this here, because swarm means
+				// out num hits can change.
 				u[i]->set_effectiveness((i % 7) + 2, 0.3 + (i % 6)*0.1,
 										(i % 8) == 0);
 				u[k]->fight(*u[i]);
@@ -920,6 +933,11 @@ static void run(unsigned specific_battle)
 #else
 	printf("Total combats: %i\n", NUM_UNITS*(NUM_UNITS-1)*(NUM_UNITS-2));
 #endif
+
+	for (i = 0; i < NUM_UNITS; i++) {
+		delete u[i];
+	}
+
 	exit(0);
 }
 
@@ -1024,6 +1042,11 @@ int main(int argc, char *argv[])
 	def->print("Defender", 0);
 	for (i = 0; att[i]; i++)
 		att[i]->print("Attacker", 0);
+
+	delete def;
+	for (i = 0; att[i]; i++)
+		delete att[i];
+
 	return 0;
 }
 #endif // Standalone program
