@@ -1,16 +1,20 @@
-/* $Id: builder.cpp 18614 2007-07-02 16:40:44Z mordante $ */
+/* $Id: builder.cpp 23963 2008-02-20 00:16:11Z esr $ */
 /*
-   Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file builder.cpp
+//! Terrain builder.
+
 #include "global.hpp"
 
 #include "array.hpp"
@@ -20,56 +24,47 @@
 #include "pathutils.hpp"
 #include "terrain.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "serialization/string_utils.hpp"
+
+#include <cassert>
 #include <climits>
 
 #define ERR_NG LOG_STREAM(err, engine)
+#define DEBUG_NG LOG_STREAM(info, engine)
 
-namespace {
+/** The tile width used when using basex and basey. This is not,
+ * necessarily, the tile width in pixels, this is totally
+ * arbitrary. However, it will be set to 72 for convenience.
+ */
+static const int TILEWIDTH = 72;
+/** The position of unit graphics in a tile. Graphics whose y
+ * position is below this value are considered background for
+ * this tile; graphics whose y position is above this value are
+ * considered foreground.
+ */
+static const int UNITPOS = 36 + 18;
+/** The allowed interval for the base-y position. The possible values are from
+ * -BASE_Y_INTERVAL to BASE_Y_INTERVAL-1
+ */
+static const int BASE_Y_INTERVAL = 100000;
 
-class locator_string_initializer : public animated<image::locator>::string_initializer
-{
-public:
-	locator_string_initializer() : no_loc_(true) {}
-	locator_string_initializer(const gamemap::location& loc): no_loc_(false), loc_(loc)  {}
-	image::locator operator()(const std::string &s) const;
-
-private:
-	bool no_loc_;
-	gamemap::location loc_;
-};
-
-image::locator locator_string_initializer::operator()(const std::string &s) const
-{
-	if(no_loc_) {
-		return image::locator("terrain/" + s + ".png");
-	} else {
-		return image::locator("terrain/" + s + ".png", loc_);
-	}
-}
-
-}
-
-const int terrain_builder::rule_image::TILEWIDTH = 72;
-const int terrain_builder::rule_image::UNITPOS = 36 + 18;
-
-terrain_builder::rule_image::rule_image(int layer, bool global_image) :
-	position(HORIZONTAL), layer(layer),
-	basex(0), basey(0), global_image(global_image)
+terrain_builder::rule_image::rule_image(int layer, int x, int y, bool global_image) :
+	layer(layer),
+	basex(x),
+	basey(y),
+	variants(),
+	global_image(global_image)
 {}
 
-terrain_builder::rule_image::rule_image(int x, int y, bool global_image) :
-	position(VERTICAL), layer(0),
-	basex(x), basey(y), global_image(global_image)
+terrain_builder::tile::tile() :
+	flags(),
+	images(),
+	images_foreground(),
+	images_background(),
+	last_tod("invalid_tod")
 {}
 
-terrain_builder::tile::tile() : last_tod("invalid_tod")
-{
-	memset(adjacents, 0, sizeof(adjacents));
-}
-
-void terrain_builder::tile::add_image_to_cache(const std::string &tod, ordered_ri_list::const_iterator itor) const
+void terrain_builder::tile::add_image_to_cache(const std::string &tod, ordered_ri_list::const_iterator itor)
 {
 	rule_image_variantlist::const_iterator tod_variant =
 		itor->second->variants.find(tod);
@@ -78,7 +73,16 @@ void terrain_builder::tile::add_image_to_cache(const std::string &tod, ordered_r
 		tod_variant = itor->second->variants.find("");
 
 	if(tod_variant != itor->second->variants.end()) {
-		if(itor->first < 0) {
+		//calculate original y-value and layer from list index
+		int layer = itor->first / BASE_Y_INTERVAL;
+		int basey = itor->first % BASE_Y_INTERVAL;
+
+		if (basey < 0)
+			basey += BASE_Y_INTERVAL/2;
+		else
+			basey -= BASE_Y_INTERVAL/2;
+
+		if(layer < 0 || (layer == 0 && basey < UNITPOS)) {
 			images_background.push_back(tod_variant->second.image);
 		} else {
 			images_foreground.push_back(tod_variant->second.image);
@@ -86,34 +90,24 @@ void terrain_builder::tile::add_image_to_cache(const std::string &tod, ordered_r
 	}
 }
 
-void terrain_builder::tile::rebuild_cache(const std::string &tod) const
+void terrain_builder::tile::rebuild_cache(const std::string &tod)
 {
 	images_background.clear();
 	images_foreground.clear();
 
 	ordered_ri_list::const_iterator itor;
-	for(itor = horizontal_images.begin(); itor != horizontal_images.end(); ++itor) {
-		if (itor->first <= 0)
-			add_image_to_cache(tod, itor);
-	}
-	for(itor = vertical_images.begin(); itor != vertical_images.end(); ++itor) {
+	for(itor = images.begin(); itor != images.end(); ++itor) {
 		add_image_to_cache(tod, itor);
 	}
-	for(itor = horizontal_images.begin(); itor != horizontal_images.end(); ++itor) {
-		if (itor->first > 0)
-			add_image_to_cache(tod, itor);
-	}
 }
 
 void terrain_builder::tile::clear()
 {
 	flags.clear();
-	horizontal_images.clear();
-	vertical_images.clear();
+	images.clear();
 	images_foreground.clear();
 	images_background.clear();
 	last_tod = "invalid_tod";
-	memset(adjacents, 0, sizeof(adjacents));
 }
 
 void terrain_builder::tilemap::reset()
@@ -124,42 +118,53 @@ void terrain_builder::tilemap::reset()
 
 bool terrain_builder::tilemap::on_map(const gamemap::location &loc) const
 {
-	if(loc.x < -1 || loc.y < -1 || loc.x > x_ || loc.y > y_)
+	if(loc.x < -2 || loc.y < -2 || loc.x > (x_ + 1) || loc.y > (y_ + 1)) {
 		return false;
+	}
 
 	return true;
+	
 }
 
 terrain_builder::tile& terrain_builder::tilemap::operator[](const gamemap::location &loc)
 {
-	wassert(on_map(loc));
+	assert(on_map(loc));
 
-	return map_[(loc.x+1) + (loc.y+1)*(x_+2)];
+	return map_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
 }
 
 const terrain_builder::tile& terrain_builder::tilemap::operator[] (const gamemap::location &loc) const
 {
-	wassert(on_map(loc));
+	assert(on_map(loc));
 
-	return map_[(loc.x+1) + (loc.y+1)*(x_+2)];
+	return map_[(loc.x + 2) + (loc.y + 2) * (x_ + 4)];
 }
 
-terrain_builder::terrain_builder(const config& cfg, const config& level, const gamemap& gmap) :
-	map_(gmap), tile_map_(gmap.x(), gmap.y())
+terrain_builder::terrain_builder(const config& cfg, const config& level,
+		const gamemap& map, const std::string& offmap_image) :
+	map_(map), 
+	tile_map_(map.w(), map.h()),
+	terrain_by_type_(),
+	building_rules_()
 {
+	// Make sure there's nothing left in the cache,
+	// since it might give problems (or not?)
+	//image::flush_cache();
+
 	parse_config(cfg);
 	parse_config(level);
+	add_off_map_rule(offmap_image);
+
 	build_terrains();
-	//rebuild_terrain(gamemap::location(0,0));
 }
 
 const terrain_builder::imagelist *terrain_builder::get_terrain_at(const gamemap::location &loc,
-		const std::string &tod, ADJACENT_TERRAIN_TYPE terrain_type) const
+		const std::string &tod, const ADJACENT_TERRAIN_TYPE terrain_type)
 {
 	if(!tile_map_.on_map(loc))
 		return NULL;
 
-	const tile& tile_at = tile_map_[loc];
+	tile& tile_at = tile_map_[loc];
 
 	if(tod != tile_at.last_tod) {
 		tile_at.rebuild_cache(tod);
@@ -190,22 +195,22 @@ bool terrain_builder::update_animation(const gamemap::location &loc)
 
 	imagelist::iterator itor = bg.begin();
 	for(; itor != bg.end(); ++itor) {
-		itor->update_current_frame();
-		if(itor->frame_changed())
+		if(itor->need_update())
 			changed = true;
+		itor->update_last_draw_time();
 	}
 
 	itor = fg.begin();
 	for(; itor != fg.end(); ++itor) {
-		itor->update_current_frame();
-		if(itor->frame_changed())
+		if(itor->need_update())
 			changed = true;
+		itor->update_last_draw_time();
 	}
 
 	return changed;
 }
 
-// TODO: rename this function
+//! @todo TODO: rename this function
 void terrain_builder::rebuild_terrain(const gamemap::location &loc)
 {
 	if (tile_map_.on_map(loc)) {
@@ -214,9 +219,10 @@ void terrain_builder::rebuild_terrain(const gamemap::location &loc)
 		btile.images_foreground.clear();
 		btile.images_background.clear();
 		const std::string filename =
-			map_.get_terrain_info(map_.get_terrain(loc)).symbol_image();
-		animated<image::locator> img_loc("terrain/" + filename + ".png");
-		img_loc.start_animation(0, animated<image::locator>::INFINITE_CYCLES);
+			map_.get_terrain_info(map_.get_terrain(loc)).minimap_image();
+		animated<image::locator> img_loc;
+		img_loc.add_frame(100,image::locator("terrain/" + filename + ".png"));
+		img_loc.start_animation(0, true);
 		btile.images_background.push_back(img_loc);
 	}
 }
@@ -225,18 +231,17 @@ void terrain_builder::rebuild_all()
 {
 	tile_map_.reset();
 	terrain_by_type_.clear();
-	terrain_by_type_border_.clear();
 	build_terrains();
 }
 
-bool terrain_builder::rule_valid(const building_rule &rule)
+bool terrain_builder::rule_valid(const building_rule &rule) const
 {
-	//if the rule has no constraints, it is invalid
+	// If the rule has no constraints, it is invalid
 	if(rule.constraints.empty())
 		return false;
 
-	//checks if all the images referenced by the current rule are valid.
-	//if not, this rule will not match.
+	// Checks if all the images referenced by the current rule are valid.
+	// If not, this rule will not match.
 	rule_imagelist::const_iterator image;
 	constraint_set::const_iterator constraint;
 	rule_image_variantlist::const_iterator variant;
@@ -275,18 +280,33 @@ bool terrain_builder::start_animation(building_rule &rule)
 
 			for(variant = image->variants.begin(); variant != image->variants.end(); ++variant) {
 
-				locator_string_initializer initializer;
+				animated<image::locator>::anim_description image_vector;
+				std::vector<std::string> items = utils::split(variant->second.image_string);
+				std::vector<std::string>::const_iterator itor = items.begin();
+				for(; itor != items.end(); ++itor) {
+					const std::vector<std::string>& items = utils::split(*itor, ':');
+					std::string str;
+					int time;
+
+					if(items.size() > 1) {
+						str = items.front();
+						time = atoi(items.back().c_str());
+					} else {
+						str = *itor;
+						time = 100;
+					}
+					if(image->global_image) {
+						image_vector.push_back(animated<image::locator>::frame_description(time,image::locator("terrain/" + str + ".png",constraint->second.loc)));
+					} else {
+						image_vector.push_back(animated<image::locator>::frame_description(time,image::locator("terrain/" + str + ".png")));
+					}
 
-				if(image->global_image) {
-					initializer = locator_string_initializer(constraint->second.loc);
 				}
 
-				animated<image::locator> th(variant->second.image_string,
-						initializer);
+				animated<image::locator> th(image_vector);
 
 				variant->second.image = th;
-				variant->second.image.start_animation(0, animated<image::locator>::INFINITE_CYCLES);
-				variant->second.image.update_current_frame();
+				variant->second.image.start_animation(0, true);
 			}
 		}
 	}
@@ -325,8 +345,8 @@ terrain_builder::terrain_constraint terrain_builder::rotate(const terrain_builde
 	// r = [[ 1/2  -3/4 ]
 	//      [ 1    1/2  ]]
 	//
-	// And the following array contains I(2), r, r^2, r^3, r^4, r^5 (with
-	// r^3 == -I(2)), which are the successive rotations.
+	// And the following array contains I(2), r, r^2, r^3, r^4, r^5
+	// (with r^3 == -I(2)), which are the successive rotations.
 	static const struct {
 		double xx;
 		double xy;
@@ -341,7 +361,7 @@ terrain_builder::terrain_constraint terrain_builder::rotate(const terrain_builde
 		{ 1./2. ,  3./4., -1., 1./2. },
 	};
 
-	wassert(angle >= 0);
+	assert(angle >= 0);
 
 	angle %= 6;
 	terrain_constraint ret = constraint;
@@ -359,19 +379,16 @@ terrain_builder::terrain_constraint terrain_builder::rotate(const terrain_builde
 	for (rule_imagelist::iterator itor = ret.images.begin();
 			itor != ret.images.end(); ++itor) {
 
-		if (itor->position == rule_image::HORIZONTAL)
-			continue;
-
 		double vx, vy, rx, ry;
 
-		vx = double(itor->basex) - double(rule_image::TILEWIDTH)/2;
-		vy = double(itor->basey) - double(rule_image::TILEWIDTH)/2;
+		vx = double(itor->basex) - double(TILEWIDTH)/2;
+		vy = double(itor->basey) - double(TILEWIDTH)/2;
 
 		rx = xyrotations[angle].xx * vx + xyrotations[angle].xy * vy;
 		ry = xyrotations[angle].yx * vx + xyrotations[angle].yy * vy;
 
-		itor->basex = int(rx + rule_image::TILEWIDTH/2);
-		itor->basey = int(ry + rule_image::TILEWIDTH/2);
+		itor->basex = int(rx + TILEWIDTH/2);
+		itor->basey = int(ry + TILEWIDTH/2);
 
 		//std::cerr << "Rotation: from " << vx << ", " << vy << " to " << itor->basex <<
 		//	", " << itor->basey << "\n";
@@ -393,11 +410,6 @@ void terrain_builder::replace_token(std::string &s, const std::string &token, co
 	}
 }
 
-void terrain_builder::replace_token(terrain_builder::rule_image_variant &variant, const std::string &token, const std::string &replacement)
-{
-	replace_token(variant.image_string, token, replacement);
-}
-
 void terrain_builder::replace_token(terrain_builder::rule_image &image, const std::string &token, const std::string &replacement)
 {
 	rule_image_variantlist::iterator itor;
@@ -421,7 +433,7 @@ void terrain_builder::replace_token(terrain_builder::building_rule &rule, const
 	constraint_set::iterator cons;
 
 	for(cons = rule.constraints.begin(); cons != rule.constraints.end(); ++cons) {
-		//Transforms attributes
+		// Transforms attributes
 		std::vector<std::string>::iterator flag;
 
 		for(flag = cons->second.set_flag.begin(); flag != cons->second.set_flag.end(); flag++) {
@@ -475,7 +487,7 @@ terrain_builder::building_rule terrain_builder::rotate_rule(const terrain_builde
 		miny -= 2;
 
 	for(cons2 = tmp_cons.begin(); cons2 != tmp_cons.end(); ++cons2) {
-		//Adjusts positions
+		// Adjusts positions
 		cons2->second.loc += gamemap::location(-minx, -((miny-1)/2));
 		ret.constraints[cons2->second.loc] = cons2->second;
 	}
@@ -498,26 +510,24 @@ void terrain_builder::add_images_from_config(rule_imagelist& images, const confi
 	for(config::child_list::const_iterator img = cimages.begin(); img != cimages.end(); ++img) {
 
 		const std::string &name = (**img)["name"];
+		const int layer = lexical_cast_default<int>((**img)["layer"], 0);
 
-		if((**img)["position"].empty() ||
-				(**img)["position"] == "horizontal") {
-
-			const int layer = atoi((**img)["layer"].c_str());
-			images.push_back(rule_image(layer, global));
-
-		} else if((**img)["position"] == "vertical") {
-
+		int basex = 0, basey = 0;
+		if((**img)["base"].empty()) {
+			basex = TILEWIDTH / 2 + dx;
+			basey = TILEWIDTH / 2 + dy;
+		} else {
 			std::vector<std::string> base = utils::split((**img)["base"]);
-			int basex = 0, basey = 0;
 
 			if(base.size() >= 2) {
 				basex = atoi(base[0].c_str());
 				basey = atoi(base[1].c_str());
 			}
-			images.push_back(rule_image(basex - dx, basey - dy, global));
-
 		}
 
+		images.push_back(rule_image(layer, basex - dx, basey - dy, global));
+
+
 		// Adds the main (default) variant of the image, if present
 		images.back().variants.insert(std::pair<std::string, rule_image_variant>("", rule_image_variant(name,"")));
 
@@ -538,26 +548,27 @@ void terrain_builder::add_images_from_config(rule_imagelist& images, const confi
 void terrain_builder::add_constraints(
 		terrain_builder::constraint_set& constraints,
 		const gamemap::location& loc,
-		const std::string& type, const config& global_images)
+		const t_translation::t_match& type, const config& global_images)
 {
 	if(constraints.find(loc) == constraints.end()) {
-		//the terrain at the current location did not exist, so create it
+		// The terrain at the current location did not exist, so create it
 		constraints[loc] = terrain_constraint(loc);
 	}
 
-	if(!type.empty())
-		constraints[loc].terrain_types = type;
+	if(!type.terrain.empty()) {
+		constraints[loc].terrain_types_match = type;
+	}
 
-	int x = loc.x * rule_image::TILEWIDTH * 3 / 4;
-	int y = loc.y * rule_image::TILEWIDTH + (loc.x % 2) *
-		rule_image::TILEWIDTH / 2;
+	int x = loc.x * TILEWIDTH * 3 / 4;
+	int y = loc.y * TILEWIDTH + (loc.x % 2) * TILEWIDTH / 2;
 	add_images_from_config(constraints[loc].images, global_images, true, x, y);
-
 }
 
-void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints, const gamemap::location& loc, const config& cfg, const config& global_images)
+void terrain_builder::add_constraints(terrain_builder::constraint_set &constraints,
+		const gamemap::location& loc, const config& cfg, const config& global_images)
+
 {
-	add_constraints(constraints, loc, cfg["type"], global_images);
+	add_constraints(constraints, loc, t_translation::t_match(cfg["type"], t_translation::WILDCARD), global_images);
 
 	terrain_constraint& constraint = constraints[loc];
 
@@ -576,61 +587,50 @@ void terrain_builder::add_constraints(terrain_builder::constraint_set &constrain
 	add_images_from_config(constraint.images, cfg, false);
 }
 
-
 void terrain_builder::parse_mapstring(const std::string &mapstring,
 		struct building_rule &br, anchormap& anchors,
 		const config& global_images)
 {
-	int lineno = 0;
-	int x = 0;
 
-	const std::vector<std::string> &lines = utils::split(mapstring, '\n', 0);
-	std::vector<std::string>::const_iterator line = lines.begin();
+	const t_translation::t_map map = t_translation::read_builder_map(mapstring);
 
-	//Strips trailing empty lines
-	while (line != lines.end() && std::find_if(line->begin(), line->end(), utils::notspace) == line->end()) {
-		line++;
-	}
-	//Break if there only are blank lines
-	if(line == lines.end())
+	// If there is an empty map leave directly.
+	// Determine after conversion, since a
+	// non-empty string can return an empty map.
+	if(map.empty()) {
 		return;
+	}
 
-	//If the strings starts with a space, the first line is an odd line,
-	//else it is an even one
-	if((*line)[0] == ' ')
-		lineno = 1;
+	int lineno = (map[0][0] == t_translation::NONE_TERRAIN) ? 1 : 0;
+	int x = lineno;
+	int y = 0;
+	for(size_t y_off = 0; y_off < map.size(); ++y_off) {
+		for(size_t x_off = x; x_off < map[y_off].size(); ++x_off) {
 
-	for(; line != lines.end(); ++line) {
-		//cuts each line into chunks of 4 characters, ignoring the 2
-		//first ones if the line is odd
+			const t_translation::t_terrain terrain = map[y_off][x_off];
 
-		x = 0;
-		std::string::size_type lpos = 0;
-		if(lineno % 2) {
-			lpos = 2;
+			if(terrain.base == t_translation::TB_DOT) {
+				// Dots are simple placeholders,
+				// which do not represent actual terrains.
+			} else if (terrain.overlay != 0 ) {
+				anchors.insert(std::pair<int, gamemap::location>(terrain.overlay, gamemap::location(x, y)));
+			} else if (terrain.base == t_translation::TB_STAR) {
+				add_constraints(br.constraints, gamemap::location(x, y), t_translation::STAR, global_images);
+			} else {
+					ERR_NG << "Invalid terrain (" << t_translation::write_terrain_code(terrain) << ") in builder map\n";
+					assert(false); 
+					return;
+			}
+		x += 2;
+		}
+
+		if(lineno % 2 == 1) {
+			++y;
+			x = 0;
+		} else {
 			x = 1;
 		}
-
-		while(lpos < line->size()) {
-			std::string types = line->substr(lpos, 4);
-			utils::strip(types);
-
-			//If there are numbers in the types string, consider it
-			//is an anchor
-			if(types[0] == '.') {
-				// Dots are simple placeholders, which do not
-				// represent actual terrains.
-			} else if(types.find_first_of("0123456789") != std::string::npos) {
-				int anchor = atoi(types.c_str());
-				anchors.insert(std::pair<int, gamemap::location>(anchor, gamemap::location(x, lineno / 2)));
-			} else {
-				const gamemap::location loc(x, lineno / 2);
-				add_constraints(br.constraints, loc, types, global_images);
-			}
-			lpos += 4;
-			x += 2;
-		}
-		lineno++;
+		++lineno;
 	}
 }
 
@@ -640,7 +640,6 @@ void terrain_builder::add_rule(building_ruleset& rules, building_rule &rule)
 		start_animation(rule);
 		rules.insert(std::pair<int, building_rule>(rule.precedence, rule));
 	}
-
 }
 
 void terrain_builder::add_rotated_rules(building_ruleset& rules, building_rule& tpl, const std::string &rotations)
@@ -663,7 +662,7 @@ void terrain_builder::parse_config(const config &cfg)
 {
 	log_scope("terrain_builder::parse_config");
 
-	//Parses the list of building rules (BRs)
+	// Parses the list of building rules (BRs)
 	const config::child_list& brs = cfg.get_children("terrain_graphics");
 
 	for(config::child_list::const_iterator br = brs.begin(); br != brs.end(); ++br) {
@@ -677,7 +676,7 @@ void terrain_builder::parse_config(const config &cfg)
 		pbr.probability = (**br)["probability"].empty() ? -1 : atoi((**br)["probability"].c_str());
 		pbr.precedence = (**br)["precedence"].empty() ? 0 : atoi((**br)["precedence"].c_str());
 
-		//Mapping anchor indices to anchor locations.
+		// Mapping anchor indices to anchor locations.
 		anchormap anchors;
 
 		// Parse the map= , if there is one (and fill the anchors list)
@@ -687,9 +686,8 @@ void terrain_builder::parse_config(const config &cfg)
 		config::child_list tcs((*br)->get_children("tile"));
 
 		for(config::child_list::const_iterator tc = tcs.begin(); tc != tcs.end(); tc++) {
-			//Adds the terrain constraint to the current built
-			//terrain's list of terrain constraints, if it does not
-			//exist.
+			// Adds the terrain constraint to the current built terrain's list
+			// of terrain constraints, if it does not exist.
 			gamemap::location loc;
 			if((**tc)["x"].size()) {
 				loc.x = atoi((**tc)["x"].c_str());
@@ -749,17 +747,22 @@ void terrain_builder::parse_config(const config &cfg)
 
 	}
 
+// Debug output for the terrain rules
 #if 0
 	std::cerr << "Built terrain rules: \n";
 
 	building_ruleset::const_iterator rule;
 	for(rule = building_rules_.begin(); rule != building_rules_.end(); ++rule) {
-		std::cerr << ">> New rule: image_background = " /* << rule->second.image_background << " , image_foreground = "<< rule->second.image_foreground */ << "\n";
+		std::cerr << ">> New rule: image_background = " 
+			<< "\n>> Location " << rule->second.location_constraints
+			<< "\n>> Probability " << rule->second.probability 
+			<< "\n>> Precedence " << rule->second.precedence << '\n';
+
 		for(constraint_set::const_iterator constraint = rule->second.constraints.begin();
 		    constraint != rule->second.constraints.end(); ++constraint) {
 
 			std::cerr << ">>>> New constraint: location = (" << constraint->second.loc
-			          << "), terrain types = " << constraint->second.terrain_types << "\n";
+			          << "), terrain types = '" << t_translation::write_list(constraint->second.terrain_types_match.terrain) << "'\n";
 
 			std::vector<std::string>::const_iterator flag;
 
@@ -777,82 +780,86 @@ void terrain_builder::parse_config(const config &cfg)
 
 }
 
-bool terrain_builder::terrain_matches(gamemap::TERRAIN letter, const std::string &terrains)
+void terrain_builder::add_off_map_rule(const std::string& image)
 {
-	bool negative = false;
-	std::string::const_iterator itor;
+	// Build a config object
+	config cfg;
 
-	if(terrains.empty())
-		return true;
-	for(itor = terrains.begin(); itor != terrains.end(); ++itor) {
-		if(*itor == '*')
-			return true;
-		if(*itor == '!') {
-			negative = true;
-			continue;
-		}
-		if(*itor == letter)
-			break;
-	}
+	cfg.add_child("terrain_graphics");
+	config *item = cfg.child("terrain_graphics");
 
-	if(itor == terrains.end())
-		return negative;
-	return !negative;
+	(*item).add_child("tile");
+	config *tile = (*item).child("tile");
+	(*tile)["x"] = "0";
+	(*tile)["y"] = "0";
+	(*tile)["type"] = t_translation::write_terrain_code(t_translation::OFF_MAP_USER);
+
+	(*tile).add_child("image");
+	config *tile_image = (*tile).child("image");
+	(*tile_image)["layer"] = "-1000";
+	(*tile_image)["name"] = image;
+
+	(*item)["probability"] = "100";
+	(*item)["no_flag"] = "base";
+	(*item)["set_flag"] = "base";
+
+	// Parse the object
+	parse_config(cfg);
 }
 
-bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule, const gamemap::location &loc, int rule_index, bool check_loc)
+bool terrain_builder::rule_matches(const terrain_builder::building_rule &rule,
+		const gamemap::location &loc, const int rule_index, const constraint_set::const_iterator type_checked) const
 {
-	if(rule.location_constraints.valid() && rule.location_constraints != loc)
+	if(rule.location_constraints.valid() && rule.location_constraints != loc) {
 		return false;
-
-
-	if(check_loc) {
-		for(constraint_set::const_iterator cons = rule.constraints.begin();
-				cons != rule.constraints.end(); ++cons) {
-
-			// translated location
-			const gamemap::location tloc = loc + cons->second.loc;
-
-			if(!tile_map_.on_map(tloc))
-				return false;
-
-			if(!terrain_matches(map_.get_terrain(tloc), cons->second.terrain_types))
-				return false;
-		}
 	}
 
 	if(rule.probability != -1) {
 		unsigned int a = (loc.x + 92872973) ^ 918273;
 		unsigned int b = (loc.y + 1672517) ^ 128123;
 		unsigned int c = (rule_index + 127390) ^ 13923787;
-		unsigned int random = a*b*c + a*b + b*c + a*c + a + b + c;
+		unsigned int abc = a*b*c + a*b + b*c + a*c + a + b + c;
+		unsigned int random = (abc*abc) % 100;
 
-		random %= 100;
-
-		if(random > (unsigned int)rule.probability)
+		if(random > static_cast<unsigned int>(rule.probability)) {
 			return false;
+		}
 	}
 
 	for(constraint_set::const_iterator cons = rule.constraints.begin();
 			cons != rule.constraints.end(); ++cons) {
 
+		// Translated location
 		const gamemap::location tloc = loc + cons->second.loc;
-		if(!tile_map_.on_map(tloc))
+
+		if(!tile_map_.on_map(tloc)) {
 			return false;
+		}
+
+		//std::cout << "testing..." << builder_letter(map_.get_terrain(tloc))
+
+		// check if terrain matches except if we already know that it does
+		if(cons != type_checked &&
+				!terrain_matches(map_.get_terrain(tloc), cons->second.terrain_types_match)) {
+			return false;
+		}
+
 		const tile& btile = tile_map_[tloc];
 
 		std::vector<std::string>::const_iterator itor;
 		for(itor = cons->second.no_flag.begin(); itor != cons->second.no_flag.end(); ++itor) {
 
-			//If a flag listed in "no_flag" is present, the rule does not match
-			if(btile.flags.find(*itor) != btile.flags.end())
+			// If a flag listed in "no_flag" is present, the rule does not match
+			if(btile.flags.find(*itor) != btile.flags.end()) {
 				return false;
+			}
 		}
 		for(itor = cons->second.has_flag.begin(); itor != cons->second.has_flag.end(); ++itor) {
 
-			//If a flag listed in "has_flag" is not present, this rule does not match
-			if(btile.flags.find(*itor) == btile.flags.end())
+			// If a flag listed in "has_flag" is not present, this rule does not match
+			if(btile.flags.find(*itor) == btile.flags.end()) {
 				return false;
+			}
 		}
 	}
 
@@ -866,20 +873,18 @@ void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, con
 
 		rule_imagelist::const_iterator img;
 		const gamemap::location tloc = loc + constraint->second.loc;
-		if(!tile_map_.on_map(tloc))
+		if(!tile_map_.on_map(tloc)) {
 			return;
+		}
 
 		tile& btile = tile_map_[tloc];
 
+		// We want to order the images by layer first and base-y second,
+		// so we sort by layer*BASE_Y_INTERVAL + BASE_Y_INTERVAL/2 + basey
+		// Thus, allowed values for basey are from -50000 to 49999
 		for(img = constraint->second.images.begin(); img != constraint->second.images.end(); ++img) {
-
-			if(img->position == rule_image::HORIZONTAL) {
-				btile.horizontal_images.insert(std::pair<int, const rule_image*>(
-							img->layer, &*img));
-			} else if(img->position == rule_image::VERTICAL) {
-				btile.vertical_images.insert(std::pair<int, const rule_image*>(
-						img->basey - rule_image::UNITPOS, &*img));
-			}
+			btile.images.insert(std::pair<int, const rule_image*>(
+									img->layer*BASE_Y_INTERVAL + BASE_Y_INTERVAL/2 + img->basey, &*img));
 		}
 
 		// Sets flags
@@ -891,204 +896,84 @@ void terrain_builder::apply_rule(const terrain_builder::building_rule &rule, con
 	}
 }
 
-int terrain_builder::get_constraint_adjacents(const building_rule& rule, const gamemap::location& loc)
-{
-	int res = 0;
-
-	gamemap::location adj[6];
-	int i;
-	get_adjacent_tiles(loc, adj);
-
-	for(i = 0; i < 6; ++i) {
-		if(rule.constraints.find(adj[i]) != rule.constraints.end()) {
-			res++;
-		}
-	}
-	return res;
-}
-
-//returns the "size" of a constraint: that is, the number of map tiles on which
-//this constraint may possibly match. INT_MAX means "I don't know / all of them".
-int terrain_builder::get_constraint_size(const building_rule& rule, const terrain_constraint& constraint, bool& border)
-{
-	const std::string &types = constraint.terrain_types;
-
-	if(types.empty())
-		return INT_MAX;
-	if(types[0] == '!')
-		return INT_MAX;
-	if(types.find('*') != std::string::npos)
-		return INT_MAX;
-
-	gamemap::location adj[6];
-	int i;
-	get_adjacent_tiles(constraint.loc, adj);
-
-	border = false;
-
-	//if the current constraint only applies to a non-isolated tile,
-	//the "border" flag can be set.
-	for(i = 0; i < 6; ++i) {
-		if(rule.constraints.find(adj[i]) != rule.constraints.end()) {
-			const std::string& atypes = rule.constraints.find(adj[i])->second.terrain_types;
-			for(std::string::const_iterator itor = types.begin();
-					itor != types.end(); ++itor) {
-				if(!terrain_matches(*itor, atypes)) {
-					border = true;
-					break;
-				}
-
-			}
-		}
-		if(border == true)
-			break;
-	}
-
-	int constraint_size = 0;
-
-	for(std::string::const_iterator itor = types.begin();
-			itor != types.end(); ++itor) {
-		if(border) {
-			constraint_size += terrain_by_type_border_[*itor].size();
-		} else {
-			constraint_size += terrain_by_type_[*itor].size();
-		}
-	}
-
-	return constraint_size;
-}
-
 void terrain_builder::build_terrains()
 {
 	log_scope("terrain_builder::build_terrains");
 
-	//builds the terrain_by_type_ cache
-	for(int x = -1; x <= map_.x(); ++x) {
-		for(int y = -1; y <= map_.y(); ++y) {
+	// Builds the terrain_by_type_ cache
+	for(int x = -2; x <= map_.w(); ++x) {
+		for(int y = -2; y <= map_.h(); ++y) {
 			const gamemap::location loc(x,y);
-			const gamemap::TERRAIN t = map_.get_terrain(loc);
+			const t_translation::t_terrain t = map_.get_terrain(loc);
 
 			terrain_by_type_[t].push_back(loc);
-
-			gamemap::location adj[6];
-			int i;
-			bool border = false;
-
-			get_adjacent_tiles(loc, adj);
-
-			tile_map_[loc].adjacents[0] = t;
-			for(i = 0; i < 6; ++i) {
-				//updates the list of adjacents for this tile
-				tile_map_[loc].adjacents[i+1] = map_.get_terrain(adj[i]);
-
-				//determines if this tile is a border tile
-				if(map_.get_terrain(adj[i]) != t)
-					border = true;
-			}
-			if(border)
-				terrain_by_type_border_[t].push_back(loc);
 		}
 	}
 
 	int rule_index = 0;
-	building_ruleset::const_iterator rule;
+	building_ruleset::const_iterator r;
 
-	for(rule = building_rules_.begin(); rule != building_rules_.end(); ++rule) {
+	for(r = building_rules_.begin(); r != building_rules_.end(); ++r) {
 
-		if (rule->second.location_constraints.valid()) {
-			apply_rule(rule->second, rule->second.location_constraints);
+		const building_rule& rule = r->second;
+
+		if (rule.location_constraints.valid()) {
+			apply_rule(rule, rule.location_constraints);
 			continue;
 		}
 
-		constraint_set::const_iterator constraint;
+		// Find the constraint that contains the less terrain of all terrain rules.
+		// We will keep a track of the matching terrains of this constraint
+		// and later try to apply the rule only on them
+		size_t min_size = INT_MAX;
+		t_translation::t_list min_types;
+		constraint_set::const_iterator min_constraint = rule.constraints.end();
 
-		//find the constraint that contains the less terrain of all terrain rules.
-		constraint_set::const_iterator smallest_constraint;
-		constraint_set::const_iterator constraint_most_adjacents;
-		int smallest_constraint_size = INT_MAX;
-		int biggest_constraint_adjacent = -1;
-		bool smallest_constraint_border = false;
+		for(constraint_set::const_iterator constraint = rule.constraints.begin();
+		    	constraint != rule.constraints.end(); ++constraint) {
 
-		for(constraint = rule->second.constraints.begin();
-		    constraint != rule->second.constraints.end(); ++constraint) {
+		    const t_translation::t_match& match = constraint->second.terrain_types_match;
+			t_translation::t_list matching_types;
+			size_t constraint_size = 0;
 
-			bool border;
+			for (terrain_by_type_map::iterator type_it = terrain_by_type_.begin();
+					 type_it != terrain_by_type_.end(); type_it++) {
 
-			int size = get_constraint_size(rule->second, constraint->second, border);
-			if(size < smallest_constraint_size) {
-				smallest_constraint_size = size;
-				smallest_constraint = constraint;
-				smallest_constraint_border = border;
+				const t_translation::t_terrain t = type_it->first;
+				if (terrain_matches(t, match)) {
+					const size_t match_size = type_it->second.size();
+					constraint_size += match_size;
+					if (constraint_size >= min_size) {
+						break; // not a minimum, bail out
+					}
+					matching_types.push_back(t);
+				}
 			}
 
-			int nadjacents = get_constraint_adjacents(rule->second, constraint->second.loc);
-			if(nadjacents > biggest_constraint_adjacent) {
-				biggest_constraint_adjacent = nadjacents;
-				constraint_most_adjacents = constraint;
+			if (constraint_size < min_size) {
+				min_size = constraint_size;
+				min_types = matching_types;
+				min_constraint = constraint;
+				if (min_size == 0) {
+				 	// a constraint is never matched on this map
+				 	// we break with a empty type list
+					break;
+				}
 			}
 		}
 
-		util::array<std::string,7> adjacent_types;
+		//NOTE: if min_types is not empty, we have found a valid min_constraint;
+		for(t_translation::t_list::const_iterator t = min_types.begin();
+				t != min_types.end(); ++t) {
 
-		if(biggest_constraint_adjacent > 0) {
-			gamemap::location loc[7];
-			loc[0] = constraint_most_adjacents->second.loc;
-			get_adjacent_tiles(loc[0], loc+1);
-			for(int i = 0; i < 7; ++i) {
-				constraint_set::const_iterator cons = rule->second.constraints.find(loc[i]) ;
-				if(cons != rule->second.constraints.end()) {
-					adjacent_types[i] = cons->second.terrain_types;
-				} else {
-					adjacent_types[i] = "";
-				}
-			}
+			const std::vector<gamemap::location>* locations = &terrain_by_type_[*t];
 
-		}
-		if(smallest_constraint_size != INT_MAX) {
-			const std::string &types = smallest_constraint->second.terrain_types;
-			const gamemap::location loc = smallest_constraint->second.loc;
-			const gamemap::location aloc = constraint_most_adjacents->second.loc;
+			for(std::vector<gamemap::location>::const_iterator itor = locations->begin();
+					itor != locations->end(); ++itor) {
+				const gamemap::location loc = *itor - min_constraint->second.loc;
 
-			for(std::string::const_iterator c = types.begin(); c != types.end(); ++c) {
-				const std::vector<gamemap::location>* locations;
-				if(smallest_constraint_border) {
-					locations = &terrain_by_type_border_[*c];
-				} else {
-					locations = &terrain_by_type_[*c];
-				}
-
-				for(std::vector<gamemap::location>::const_iterator itor = locations->begin();
-						itor != locations->end(); ++itor) {
-
-					if(biggest_constraint_adjacent > 0) {
-						const gamemap::location pos = (*itor - loc) + aloc;
-						if(!tile_map_.on_map(pos))
-							continue;
-
-						const gamemap::TERRAIN *adjacents = tile_map_[pos].adjacents;
-						int i;
-
-						for(i = 0; i < 7; ++i) {
-							if(!terrain_matches(adjacents[i], adjacent_types[i])) {
-								break;
-							}
-						}
-						// propagates the break
-						if (i < 7)
-							continue;
-					}
-
-					if(rule_matches(rule->second, *itor - loc, rule_index, (size_t)(biggest_constraint_adjacent + 1) != rule->second.constraints.size())) {
-						apply_rule(rule->second, *itor - loc);
-					}
-				}
-			}
-		} else {
-			for(int x = -1; x <= map_.x(); ++x) {
-				for(int y = -1; y <= map_.y(); ++y) {
-					const gamemap::location loc(x,y);
-					if(rule_matches(rule->second, loc, rule_index, true))
-						apply_rule(rule->second, loc);
+				if(rule_matches(rule, loc, rule_index, min_constraint)) {
+					apply_rule(rule, loc);
 				}
 			}
 		}
