@@ -1,30 +1,37 @@
-/* $Id: cavegen.cpp 15917 2007-02-19 15:12:03Z torangan $ */
+/* $Id: cavegen.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file cavegen.cpp 
+//! Map-generator for caves.
+
 #include "global.hpp"
 
 #include "cavegen.hpp"
 #include "log.hpp"
 #include "pathfind.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "serialization/string_utils.hpp"
 
+#include <cassert>
+
 #define LOG_NG LOG_STREAM(info, engine)
 
-cave_map_generator::cave_map_generator(const config* cfg) : wall_('W'), clear_('u'), village_('D'), castle_('o'),
-                                                            cfg_(cfg), width_(50), height_(50), village_density_(0),
-							    flipx_(false), flipy_(false)
+cave_map_generator::cave_map_generator(const config* cfg) : wall_(t_translation::CAVE_WALL),
+	clear_(t_translation::CAVE), village_(t_translation::UNDERGROUND_VILLAGE),
+    castle_(t_translation::DWARVEN_CASTLE), keep_(t_translation::DWARVEN_KEEP),
+	cfg_(cfg), width_(50), height_(50),
+	village_density_(0), flipx_(false), flipy_(false)
 {
 	if(cfg_ == NULL) {
 		static const config default_cfg;
@@ -33,6 +40,9 @@ cave_map_generator::cave_map_generator(const config* cfg) : wall_('W'), clear_('
 
 	width_ = atoi((*cfg_)["map_width"].c_str());
 	height_ = atoi((*cfg_)["map_height"].c_str());
+	width_ += 2 * gamemap::default_border;
+	height_ += 2 * gamemap::default_border;
+
 	village_density_ = atoi((*cfg_)["village_density"].c_str());
 
 	const int r = rand()%100;
@@ -43,7 +53,6 @@ cave_map_generator::cave_map_generator(const config* cfg) : wall_('W'), clear_('
 	LOG_NG << "flipx: " << r << " < " << chance << " = " << (flipx_ ? "true" : "false") << "\n";
 	flipy_ = (rand()%100) < atoi((*cfg_)["flipy_chance"].c_str());
 
-
 }
 
 size_t cave_map_generator::translate_x(size_t x) const
@@ -72,7 +81,7 @@ std::string cave_map_generator::create_map(const std::vector<std::string>& args)
 
 config cave_map_generator::create_scenario(const std::vector<std::string>& /*args*/)
 {
-	map_ = std::vector<std::vector<gamemap::TERRAIN> >(width_,std::vector<gamemap::TERRAIN>(height_,wall_));
+	map_ = t_translation::t_map(width_, t_translation::t_list(height_, wall_));
 	chambers_.clear();
 	passages_.clear();
 
@@ -97,16 +106,9 @@ config cave_map_generator::create_scenario(const std::vector<std::string>& /*arg
 	}
 
 	LOG_NG << "outputting map....\n";
-	std::stringstream out;
-	for(size_t y = 0; y != height_; ++y) {
-		for(size_t x = 0; x != width_; ++x) {
-			out << map_[x][y];
-		}
 
-		out << "\n";
-	}
-
-	res_["map_data"] = out.str();
+	res_["map_data"] = gamemap::default_map_header + 
+		t_translation::write_game_map(map_, starting_positions_);
 
 	LOG_NG << "returning result...\n";
 
@@ -123,7 +125,7 @@ void cave_map_generator::build_chamber(gamemap::location loc, std::set<gamemap::
 	gamemap::location adj[6];
 	get_adjacent_tiles(loc,adj);
 	for(size_t n = 0; n != 6; ++n) {
-		if((rand()%100) < (100l-(long)jagged)) {
+		if((rand() % 100) < (100l - static_cast<long>(jagged))) {
 			build_chamber(adj[n],locs,size-1,jagged);
 		}
 	}
@@ -133,7 +135,7 @@ void cave_map_generator::generate_chambers()
 {
 	const config::child_list& chambers = cfg_->get_children("chamber");
 	for(config::child_list::const_iterator i = chambers.begin(); i != chambers.end(); ++i) {
-		//if there is only a chance of the chamber appearing, deal with that here.
+		// If there is only a chance of the chamber appearing, deal with that here.
 		const std::string& chance = (**i)["chance"];
 		if(chance != "" && (rand()%100) < atoi(chance.c_str())) {
 			continue;
@@ -192,12 +194,12 @@ void cave_map_generator::generate_chambers()
 		for(config::child_list::const_iterator p = passages.begin(); p != passages.end(); ++p) {
 			const std::string& dst = (**p)["destination"];
 
-			//find the destination of this passage
+			// Find the destination of this passage
 			const std::map<std::string,size_t>::const_iterator itor = chamber_ids_.find(dst);
 			if(itor == chamber_ids_.end())
 				continue;
 
-			wassert(itor->second < chambers_.size());
+			assert(itor->second < chambers_.size());
 
 			passages_.push_back(passage(new_chamber.center,chambers_[itor->second].center,**p));
 		}
@@ -235,59 +237,73 @@ void cave_map_generator::place_items(const chamber& c, config::all_children_iter
 		if(!utils::string_bool(cfg["same_location_as_previous"])) {
 			index = rand()%c.locs.size();
 		}
+		const std::string loc_var = cfg["store_location_as"];
 
 		std::set<gamemap::location>::const_iterator loc = c.locs.begin();
 		std::advance(loc,index);
 
-		char buf[50];
-		snprintf(buf,sizeof(buf),"%d",loc->x+1);
-		cfg.values["x"] = buf;
+		char xbuf[50];
+		snprintf(xbuf,sizeof(xbuf),"%d",loc->x+1);
+		cfg.values["x"] = xbuf;
 		if(filter != NULL) {
-			(*filter)["x"] = buf;
+			(*filter)["x"] = xbuf;
 		}
 
 		if(object_filter != NULL) {
-			(*object_filter)["x"] = buf;
+			(*object_filter)["x"] = xbuf;
 		}
 
-		snprintf(buf,sizeof(buf),"%d",loc->y+1);
-		cfg.values["y"] = buf;
+		char ybuf[50];
+		snprintf(ybuf,sizeof(ybuf),"%d",loc->y+1);
+		cfg.values["y"] = ybuf;
 		if(filter != NULL) {
-			(*filter)["y"] = buf;
+			(*filter)["y"] = ybuf;
 		}
 
 		if(object_filter != NULL) {
-			(*object_filter)["y"] = buf;
+			(*object_filter)["y"] = ybuf;
 		}
 
-		//if this is a side, place a castle for the side
+		// If this is a side, place a castle for the side
 		if(key == "side" && !utils::string_bool(cfg["no_castle"])) {
 			place_castle(cfg["side"],*loc);
 		}
 
 		res_.add_child(key,cfg);
 
+		if(!loc_var.empty()) {
+			config &temp = res_.add_child("event");
+			temp["name"] = "prestart";
+			config &xcfg = temp.add_child("set_variable");
+			xcfg["name"] = loc_var + "_x";
+			xcfg["value"] = xbuf;
+			config &ycfg = temp.add_child("set_variable");
+			ycfg["name"] = loc_var + "_y";
+			ycfg["value"] = ybuf;
+		}
+
 		++i1;
 	}
 }
 
 struct passage_path_calculator : cost_calculator
 {
-	passage_path_calculator(const std::vector<std::vector<gamemap::TERRAIN> >& mapdata, gamemap::TERRAIN wall, double laziness, size_t windiness)
-		        : map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness)
+	passage_path_calculator(const t_translation::t_map& mapdata,
+	t_translation::t_terrain wall, double laziness, size_t windiness):
+		map_(mapdata), wall_(wall), laziness_(laziness), windiness_(windiness)
 	{}
 
 	virtual double cost(const gamemap::location& src,const gamemap::location& loc, const double so_far, const bool is_dest) const;
 private:
-	const std::vector<std::vector<gamemap::TERRAIN> >& map_;
-	gamemap::TERRAIN wall_;
+	const t_translation::t_map& map_;
+	t_translation::t_terrain wall_;
 	double laziness_;
 	size_t windiness_;
 };
 
 double passage_path_calculator::cost(const gamemap::location& /*src*/,const gamemap::location& loc, const double, const bool) const
 {
-	wassert(loc.x >= 0 && loc.y >= 0 && size_t(loc.x) < map_.size() &&
+	assert(loc.x >= 0 && loc.y >= 0 && size_t(loc.x) < map_.size() &&
 	        !map_.empty() && size_t(loc.y) < map_.front().size());
 
 	double res = 1.0;
@@ -330,14 +346,16 @@ void cave_map_generator::place_passage(const passage& p)
 	}
 }
 
-void cave_map_generator::set_terrain(gamemap::location loc, gamemap::TERRAIN t)
+void cave_map_generator::set_terrain(gamemap::location loc, t_translation::t_terrain t)
 {
 	if(on_board(loc)) {
-		if(t == clear_ && (rand()%1000) < (long)village_density_) {
+		if(t == clear_ &&
+                (rand() % 1000) < static_cast<long>(village_density_)) {
+
 			t = village_;
 		}
 
-		gamemap::TERRAIN& c = map_[loc.x][loc.y];
+		t_translation::t_terrain& c = map_[loc.x][loc.y];
 		if(c == clear_ || c == wall_ || c == village_) {
 			c = t;
 		}
@@ -346,8 +364,12 @@ void cave_map_generator::set_terrain(gamemap::location loc, gamemap::TERRAIN t)
 
 void cave_map_generator::place_castle(const std::string& side, gamemap::location loc)
 {
-	if(side != "") {
-		set_terrain(loc,side[0]);
+	const int starting_position = lexical_cast_default<int>(side, -1);
+	if(starting_position != -1) {
+		set_terrain(loc, keep_);
+
+		const struct t_translation::coordinate coord = {loc.x, loc.y};
+		starting_positions_[starting_position] = coord;
 	}
 
 	gamemap::location adj[6];
@@ -356,3 +378,4 @@ void cave_map_generator::place_castle(const std::string& side, gamemap::location
 		set_terrain(adj[n],castle_);
 	}
 }
+
