@@ -1,16 +1,20 @@
-/* $Id: clipboard.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: clipboard.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file clipboard.cpp 
+//!
+
 #include "global.hpp"
 
 #include "clipboard.hpp"
@@ -26,9 +30,9 @@
 #include "SDL_syswm.h"
 
 /**
- The following are two classes which wrap the SDL's interface to X, including
- locking/unlocking, and which manage the atom internment. They exist mainly to make
- the actual clipboard code somewhat readable
+ The following are two classes which wrap the SDL's interface to X, 
+ including locking/unlocking, and which manage the atom internment. 
+ They exist mainly to make the actual clipboard code somewhat readable.
 */
 class XHelper
 {
@@ -38,7 +42,7 @@ private:
 		acquireCount_ = 0;
 		acquire();
 
-		//Intern some atoms;
+		// Intern some atoms;
 		const char* atoms[] = {
 			"CLIPBOARD",
 			"TEXT",
@@ -48,7 +52,7 @@ private:
 			"TARGETS"
 		};
 
-		XInternAtoms(dpy(), (char**)atoms, 6, false, atomTable_);
+		XInternAtoms(dpy(), const_cast<char**>(reinterpret_cast<const char**>(atoms)), 6, false, atomTable_);
 
 		release();
 	}
@@ -150,25 +154,27 @@ public:
 
 /**
  Note: unfortunately, SDL does not keep track of event timestamps.
- This means we are forced to use CurrentTime in many spots and are
- unable to perform many safety checks. Hence, the code below is
- not compliant to the ICCCM, and may ocassionally suffer from
- race conditions if an X client is connected to the server over
- a slow/high-latency link. This implementation is also very minimal.
- The text is assumed to be reasonably small as INCR transactions are not
- supported. MULTIPLE is not supported either.
+ This means we are forced to use CurrentTime in many spots and 
+ are unable to perform many safety checks. 
+ Hence, the code below is not compliant to the ICCCM, and 
+ may ocassionally suffer from race conditions if an X client 
+ is connected to the server over a slow/high-latency link. 
+ This implementation is also very minimal.
+ The text is assumed to be reasonably small, as INCR transactions 
+ are not supported. 
+ MULTIPLE is not supported either.
 
- We provide UTF8_STRING, COMPOUND_TEXT, and TEXT, and
- try to grab all of them, plus STRING (which is latin1).
+ We provide UTF8_STRING, COMPOUND_TEXT, and TEXT, 
+ and try to grab all of them, plus STRING (which is latin1).
 */
 
 
 /**
  We primarily. keep a copy of the string to response to data requests,
  but it also has an another function: in case we're both the source
- and destination, we just copy it accross; this is so that we
- don't have to handle SelectionRequest events while waiting for SelectionNotify.
- To make this work, however, this gets cleared when we loose CLIPBOARD
+ and destination, we just copy it across; this is so that we don't 
+ have to handle SelectionRequest events while waiting for SelectionNotify.
+ To make this work, however, this gets cleared when we loose CLIPBOARD.
 */
 static std::string clipboard_string;
 
@@ -179,8 +185,8 @@ void handle_system_event(const SDL_Event& event)
 	if (xev.type == SelectionRequest) {
 		UseX x11;
 
-		//Since wesnoth does not notify us of selections,
-		//we set both selection + clipboard when copying.
+		// Since wesnoth does not notify us of selections,
+		// we set both selection + clipboard when copying.
 		if ((xev.xselectionrequest.owner     == x11->window()) &&
 		    ((xev.xselectionrequest.selection == XA_PRIMARY) ||
 		     (xev.xselectionrequest.selection == x11->XA_CLIPBOARD()))) {
@@ -209,12 +215,12 @@ void handle_system_event(const SDL_Event& event)
 
 				XChangeProperty(x11->dpy(), responseEvent.xselection.requestor,
 					xev.xselectionrequest.property, XA_ATOM, 32, PropModeReplace,
-					(unsigned char*)supported, 4);
+					const_cast<unsigned char*>(reinterpret_cast<const unsigned char*>(supported)), 4);
 			}
 
-			//The encoding of XA_TEXT and XA_COMPOUND_TEXT is not specified
-			//by the ICCCM... So we assume wesnoth native/utf-8 for simplicity.
-			//modern apps are going to use UTF8_STRING anyway
+			// The encoding of XA_TEXT and XA_COMPOUND_TEXT is not specified
+			// by the ICCCM... So we assume wesnoth native/utf-8 for simplicity.
+			// Modern apps are going to use UTF8_STRING anyway.
 			if (xev.xselectionrequest.target == x11->XA_TEXT() ||
 			    xev.xselectionrequest.target == x11->XA_COMPOUND_TEXT() ||
 			    xev.xselectionrequest.target == x11->UTF8_STRING()) {
@@ -223,7 +229,7 @@ void handle_system_event(const SDL_Event& event)
 				XChangeProperty(x11->dpy(), responseEvent.xselection.requestor,
 					xev.xselectionrequest.property,
 					xev.xselectionrequest.target, 8, PropModeReplace,
-					(const unsigned char*) clipboard_string.c_str(), clipboard_string.length());
+					reinterpret_cast<const unsigned char*>(clipboard_string.c_str()), clipboard_string.length());
 			}
 
 			XSendEvent(x11->dpy(), xev.xselectionrequest.requestor, False, NoEventMask,
@@ -239,16 +245,6 @@ void handle_system_event(const SDL_Event& event)
 	}
 }
 
-void copy_ucs2_to_clipboard(const ucs2_string& text)
-{
-	if(text.empty())
-		return;
-	clipboard_string = utils::ucs2_string_to_utf8_string(text);
-	UseX x11;
-	XSetSelectionOwner(x11->dpy(), XA_PRIMARY, x11->window(), CurrentTime);
-	XSetSelectionOwner(x11->dpy(), x11->XA_CLIPBOARD(), x11->window(), CurrentTime);
-}
-
 void copy_to_clipboard(const std::string& text)
 {
 	if (text.empty()) {
@@ -264,22 +260,23 @@ void copy_to_clipboard(const std::string& text)
 }
 
 
-//Tries to grab a given target. Returns true if successful, false otherwise
+//! Tries to grab a given target. 
+//! Returns true if successful, false otherwise.
 static bool try_grab_target(Atom target, std::string& ret)
 {
 	UseX x11;
 
-	//Cleanup previous data
+	// Cleanup previous data
 	XDeleteProperty(x11->dpy(), x11->window(), x11->WES_PASTE());
 	XSync          (x11->dpy(), False);
 
 	//std::cout<<"We request target:"<<XGetAtomName(x11->dpy(), target)<<"\n";
 
-	//Request information
+	// Request information
 	XConvertSelection(x11->dpy(), x11->XA_CLIPBOARD(), target,
 	                  x11->WES_PASTE(), x11->window(), CurrentTime);
 
-	//Wait (with timeout) for a response SelectionNotify
+	// Wait (with timeout) for a response SelectionNotify
 	for (int attempt = 0; attempt < 15; attempt++) {
 		if (XPending(x11->dpy())) {
 			XEvent selectNotify;
@@ -289,25 +286,25 @@ static bool try_grab_target(Atom target, std::string& ret)
 					return false;
 				else if (selectNotify.xselection.property == x11->WES_PASTE() &&
 				         selectNotify.xselection.target   == target) {
-					//The size
+					// The size
 					unsigned long length = 0;
 					unsigned char* data;
 
-					//these 3 XGetWindowProperty returns but we don't use
+					// These 3 XGetWindowProperty returns but we don't use
 					Atom         typeRet;
 					int          formatRet;
 					unsigned long remaining;
 
 //					std::cout<<"Grab:"<<XGetAtomName(x11->dpy(), target)<<"\n";
 
-					//Grab the text out of the property
+					// Grab the text out of the property
 					XGetWindowProperty(x11->dpy(), x11->window(),
 					                   selectNotify.xselection.property,
 					                   0, 65535/4, True, target,
 					                   &typeRet, &formatRet, &length, &remaining, &data);
 
 					if (data && length) {
-						ret = (char*)data;
+						ret = reinterpret_cast<char*>(data);
 						XFree(data);
 						return true;
 					} else {
@@ -320,32 +317,14 @@ static bool try_grab_target(Atom target, std::string& ret)
 		usleep(10000);
 	}
 
-	//Timed out -- return empty string
+	// Timed out -- return empty string
 	return false;
 }
 
-ucs2_string copy_ucs2_from_clipboard()
-{
-	if(!clipboard_string.empty())
-		return utils::utf8_string_to_ucs2_string(clipboard_string);
-	utf8_string text;
-
-	UseX x11;
-	if(try_grab_target(x11->UTF8_STRING(), text))
-		return utils::utf8_string_to_ucs2_string(text);
-	if(try_grab_target(x11->XA_COMPOUND_TEXT(), text))
-		return utils::utf8_string_to_ucs2_string(text);
-	if(try_grab_target(x11->XA_TEXT(), text))
-		return utils::utf8_string_to_ucs2_string(text);
-	if(try_grab_target(XA_STRING, text))
-		return utils::utf8_string_to_ucs2_string(text);
-	return ucs2_string();
-}
-
 std::string copy_from_clipboard()
 {
 	if (!clipboard_string.empty())
-		return clipboard_string; //in-wesnoth copy-paste
+		return clipboard_string; 	// in-wesnoth copy-paste
 
 	std::string ret;
 
@@ -360,7 +339,7 @@ std::string copy_from_clipboard()
 	if (try_grab_target(x11->XA_TEXT(), ret))
 		return ret;
 
-	if (try_grab_target(XA_STRING, ret)) //acroread only provides this
+	if (try_grab_target(XA_STRING, ret)) 	// acroread only provides this
 		return ret;
 
 
@@ -376,43 +355,6 @@ std::string copy_from_clipboard()
 void handle_system_event(const SDL_Event& )
 {}
 
-void copy_ucs2_to_clipboard(const ucs2_string& text)
-{
-	if(text.empty())
-		return;
-	if(!OpenClipboard(NULL))
-		return;
-	EmptyClipboard();
-
-	//convert newlines
-	ucs2_string str;
-	str.reserve(text.size() + 1);
-	ucs2_string::const_iterator first = text.begin();
-	ucs2_string::const_iterator last = text.begin();
-	do {
-		if(*last != '\n') {
-			++last;
-			continue;
-		}
-		str.insert(str.end(), first, last);
-		str.push_back('\r');
-		str.push_back('\n');
-		first = ++last;
-	} while(last != text.end());
-	str.push_back('\0');
-
-	HGLOBAL hglb = GlobalAlloc(GMEM_MOVEABLE, str.size() * sizeof(Uint16));
-	if(hglb == NULL) {
-		CloseClipboard();
-		return;
-	}
-	Uint16* const buffer = reinterpret_cast<Uint16* const>(GlobalLock(hglb));
-	memcpy(buffer, (Uint16 const *)&str.front(), str.size() * sizeof(Uint16));
-	GlobalUnlock(hglb);
-	SetClipboardData(CF_UNICODETEXT, hglb);
-	CloseClipboard();
-}
-
 void copy_to_clipboard(const std::string& text)
 {
 	if(text.empty())
@@ -421,7 +363,7 @@ void copy_to_clipboard(const std::string& text)
 		return;
 	EmptyClipboard();
 
-	//convert newlines
+	// Convert newlines
 	std::string str;
 	str.reserve(text.size());
 	std::string::const_iterator first = text.begin();
@@ -448,36 +390,6 @@ void copy_to_clipboard(const std::string& text)
 	CloseClipboard();
 }
 
-ucs2_string copy_ucs2_from_clipboard()
-{
-	if(!IsClipboardFormatAvailable(CF_UNICODETEXT))
-		return ucs2_string();
-	if(!OpenClipboard(NULL))
-		return ucs2_string();
-
-	HGLOBAL hglb = GetClipboardData(CF_UNICODETEXT);
-	if(hglb == NULL) {
-		CloseClipboard();
-		return ucs2_string();
-	}
-	Uint16 const * buffer = reinterpret_cast<Uint16 const *>(GlobalLock(hglb));
-	if(buffer == NULL) {
-		CloseClipboard();
-		return ucs2_string();
-	}
-
-	//convert newlines
-	ucs2_string str;
-	while(*buffer != '\0') {
-		if(*buffer != '\r')
-			str.push_back(*buffer);
-		++buffer;
-	}
-	GlobalUnlock(hglb);
-	CloseClipboard();
-	return str;
-}
-
 std::string copy_from_clipboard()
 {
 	if(!IsClipboardFormatAvailable(CF_TEXT))
@@ -496,7 +408,7 @@ std::string copy_from_clipboard()
 		return "";
 	}
 
-	//convert newlines
+	// Convert newlines
 	std::string str(buffer);
 	str.erase(std::remove(str.begin(),str.end(),'\r'),str.end());
 
@@ -548,51 +460,6 @@ std::string copy_from_clipboard()
 #define CLIPBOARD_FUNCS_DEFINED
 
 #include <Carbon/Carbon.h>
-void copy_ucs2_to_clipboard(const ucs2_string& text) {
-	ucs2_string str;
-	str.reserve(text.size() + 1);
-	for(int i = 0; i < text.size(); ++i) {
-		if(text[i] == '\n')
-			str.push_back('\r');
-		else
-			str.push_back(text[i]);
-	}
-	str.push_back(0);
-	OSStatus err = noErr;
-	ScrapRef scrap = kScrapRefNone;
-	err = ClearCurrentScrap();
-	if(err != noErr)
-		return;
-	err = GetCurrentScrap(&scrap);
-	if(err != noErr)
-		return;
-	PutScrapFlavor(scrap, kScrapFlavorTypeUnicode, kScrapFlavorMaskNone, str.size(), &str.front());
-}
-
-ucs2_string copy_ucs2_from_clipboard() {
-	ucs2_string str;
-	OSStatus err = noErr;
-	ScrapRef scrap = kScrapRefNone;
-	err = GetCurrentScrap(&scrap);
-	if(err != noErr)
-		return str;
-	Size scrapsize;
-	err = GetScrapFlavorSize(scrap, kScrapFlavorTypeUnicode, &scrapsize);
-	if(err != noErr)
-		return str;
-	str.reserve(scrapsize);
-	str.resize(scrapsize);
-	err = GetScrapFlavorData(scrap, kScrapFlavorTypeUnicode, &scrapsize, const_cast<Uint16*>(&str.front()));
-	if(err != noErr) {
-		str.clear();
-		return str;
-	}
-	for(int i = 0; i < str.size(); ++i) {
-		if(str[i] == '\r')
-			str[i] = '\n';
-	}
-	return str;
-}
 
 void copy_to_clipboard(const std::string& text)
 {
