@@ -1,4 +1,4 @@
-/* $Id: config.cpp 23936 2008-02-18 19:17:49Z suokko $ */
+/* $Id: config.cpp 26448 2008-05-07 17:35:33Z mordante $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -20,6 +20,8 @@
 
 #include <algorithm>
 #include <sstream>
+#include <string.h>
+#include <ostream>
 #include "config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
@@ -428,7 +430,12 @@ config::all_children_iterator config::ordered_end() const
 config config::get_diff(const config& c) const
 {
 	config res;
+	get_diff(c, res);
+	return res;
+}
 
+void config::get_diff(const config& c, config& res) const
+{
 	config* inserts = NULL;
 
 	string_map::const_iterator i;
@@ -529,9 +536,6 @@ config config::get_diff(const config& c) const
 			}
 		}
 	}
-
-
-	return res;
 }
 
 void config::apply_diff(const config& diff)
@@ -781,6 +785,13 @@ std::string config::hash() const
 	return std::string(hash_str);
 }
 
+void config::swap(config& cfg)
+{
+	values.swap(cfg.values);
+	children.swap(cfg.children);
+	ordered_children.swap(cfg.ordered_children);
+}
+
 bool operator==(const config& a, const config& b)
 {
 	if (a.values != b.values)
