@@ -1,16 +1,20 @@
-/* $Id: cursor.cpp 9702 2006-01-15 13:03:14Z j_daniel $ */
+/* $Id: cursor.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file cursor.cpp
+//! Support for different cursors-shapes.
+
 #include "global.hpp"
 
 #include "cursor.hpp"
@@ -26,22 +30,19 @@
 #include <iostream>
 #include <vector>
 
-namespace
-{
-
-bool use_colour_cursors()
+static bool use_colour_cursors()
 {
 	return game_config::editor == false && preferences::use_colour_cursors();
 }
 
-SDL_Cursor* create_cursor(surface surf)
+static SDL_Cursor* create_cursor(surface surf)
 {
 	const surface nsurf(make_neutral_surface(surf));
 	if(nsurf == NULL) {
 		return NULL;
 	}
 
-	//the width must be a multiple of 8 (SDL requirement)
+	// The width must be a multiple of 8 (SDL requirement)
 
 #ifdef __APPLE__
 	size_t cursor_width = 16;
@@ -54,8 +55,8 @@ SDL_Cursor* create_cursor(surface surf)
 	std::vector<Uint8> data((cursor_width*nsurf->h)/8,0);
 	std::vector<Uint8> mask(data.size(),0);
 
-	//see http://sdldoc.csn.ul.ie/sdlcreatecursor.php for documentation on
-	//the format that data has to be in to pass to SDL_CreateCursor
+	// See http://sdldoc.csn.ul.ie/sdlcreatecursor.php for documentation
+	// on the format that data has to be in to pass to SDL_CreateCursor
 	surface_lock lock(nsurf);
 	const Uint32* const pixels = reinterpret_cast<Uint32*>(lock.pixels());
 	for(int y = 0; y != nsurf->h; ++y) {
@@ -66,13 +67,13 @@ SDL_Cursor* create_cursor(surface surf)
 
 			const size_t index = y*cursor_width + x;
 
-			if ((size_t)x < cursor_width) {
+			if (static_cast<size_t>(x) < cursor_width) {
 				SDL_GetRGBA(pixels[y*nsurf->w + x],nsurf->format,&r,&g,&b,&a);
 
-				const size_t shift = 7 - (index%8);
+				const size_t shift = 7 - (index % 8);
 
 				trans = (a < 128 ? 0 : 1) << shift;
-				black = (trans == 0 || (r+g+b)/3 > 128 ? 0 : 1) << shift;
+				black = (trans == 0 || (r+g + b) / 3 > 128 ? 0 : 1) << shift;
 
 				data[index/8] |= black;
 				mask[index/8] |= trans;
@@ -85,33 +86,45 @@ SDL_Cursor* create_cursor(surface surf)
 	return SDL_CreateCursor(&data[0],&mask[0],cursor_width,nsurf->h,0,0);
 }
 
-SDL_Cursor* cache[cursor::NUM_CURSORS] = { NULL, NULL, NULL, NULL };
+namespace {
 
-//this array must have members corresponding to cursor::CURSOR_TYPE enum members
+SDL_Cursor* cache[cursor::NUM_CURSORS] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL};
+
+// This array must have members corresponding to cursor::CURSOR_TYPE enum members
+// Apple need 16x16 b&w cursors
 #ifdef __APPLE__
-const std::string images[cursor::NUM_CURSORS] = { "normal.png", "wait-alt.png", "move.png", "attack.png", "select.png" };
+const std::string bw_images[cursor::NUM_CURSORS] = { "normal.png", "wait-alt.png", "move.png", "attack.png", "select.png", "move_drag_alt.png" , "attack_drag_alt.png", "no_cursor.png"};
 #else
-const std::string images[cursor::NUM_CURSORS] = { "normal.png", "wait.png", "move.png", "attack.png", "select.png" };
+const std::string bw_images[cursor::NUM_CURSORS] = { "normal.png", "wait.png", "move.png", "attack.png", "select.png", "move_drag.png", "attack_drag.png", "no_cursor.png"};
 #endif
 
-cursor::CURSOR_TYPE current_cursor = cursor::NUM_CURSORS;
+const std::string colour_images[cursor::NUM_CURSORS] = { "normal.png", "wait.png", "move.png", "attack.png", "select.png", "move_drag.png", "attack_drag.png", ""};
+
+// Position of the hotspot of the cursor, from the normal topleft
+const int shift_x[cursor::NUM_CURSORS] = {0, 0, 0, 0, 0, 2, 3, 0};
+const int shift_y[cursor::NUM_CURSORS] = {0, 0, 0, 0, 0, 20, 22, 0};
+
+cursor::CURSOR_TYPE current_cursor = cursor::NORMAL;
 
 int cursor_x = -1, cursor_y = -1;
 surface cursor_buf = NULL;
 bool have_focus = true;
+bool colour_ready = false;
 
-SDL_Cursor* get_cursor(cursor::CURSOR_TYPE type)
+}
+
+static SDL_Cursor* get_cursor(cursor::CURSOR_TYPE type)
 {
 	if(cache[type] == NULL) {
 		static const std::string prefix = "cursors-bw/";
-		const surface surf(image::get_image(prefix + images[type],image::UNSCALED));
+		const surface surf(image::get_image(prefix + bw_images[type]));
 		cache[type] = create_cursor(surf);
 	}
 
 	return cache[type];
 }
 
-void clear_cache()
+static void clear_cache()
 {
 	for(size_t n = 0; n != cursor::NUM_CURSORS; ++n) {
 		if(cache[n] != NULL) {
@@ -125,14 +138,13 @@ void clear_cache()
 	}
 }
 
-}
-
 namespace cursor
 {
 
 manager::manager()
 {
-	use_colour(use_colour_cursors());
+	SDL_ShowCursor(SDL_ENABLE);
+	set();
 }
 
 manager::~manager()
@@ -141,30 +153,65 @@ manager::~manager()
 	SDL_ShowCursor(SDL_ENABLE);
 }
 
-void use_colour(bool value)
+void temporary_use_bw()
 {
-	if(game_config::editor == false) {
-		SDL_ShowCursor(value ? SDL_DISABLE : SDL_ENABLE);
-	}
+	colour_ready = false;
+	set();
 }
 
 void set(CURSOR_TYPE type)
 {
-	current_cursor = type;
-
-	if(type == NUM_CURSORS) {
-		return;
+	// Change only if it's a valid cursor
+	if (type != NUM_CURSORS) {
+		current_cursor = type;
+	} else if (current_cursor == NUM_CURSORS) {
+		// Except if the current one is also invalid.
+		// In this case, change to a valid one.
+		current_cursor = NORMAL;
 	}
 
-	SDL_Cursor* const cursor = get_cursor(type);
-	if(cursor != NULL) {
-		SDL_SetCursor(cursor);
+	const CURSOR_TYPE new_cursor = use_colour_cursors() && colour_ready ? cursor::NO_CURSOR : current_cursor;
+
+	SDL_Cursor * cursor_image = get_cursor(new_cursor);
+
+	// Causes problem on Mac:
+	//if (cursor_image != NULL && cursor_image != SDL_GetCursor())
+		SDL_SetCursor(cursor_image);
+
+}
+
+void set_dragging(bool drag)
+{
+	switch(current_cursor) {
+		case MOVE:
+			if (drag) cursor::set(MOVE_DRAG);
+			break;
+		case ATTACK:
+			if (drag) cursor::set(ATTACK_DRAG);
+			break;
+		case MOVE_DRAG:
+			if (!drag) cursor::set(MOVE);
+			break;
+		case ATTACK_DRAG:
+			if (!drag) cursor::set(ATTACK);
+			break;
+		default:
+			break;
 	}
 }
 
+CURSOR_TYPE get()
+{
+	return current_cursor;
+}
+
 void set_focus(bool focus)
 {
 	have_focus = focus;
+	if (focus==false) {
+		colour_ready = false;
+		set();
+	}
 }
 
 setter::setter(CURSOR_TYPE type) : old_(current_cursor)
@@ -184,7 +231,7 @@ void draw(surface screen)
 	}
 
 	if(current_cursor == NUM_CURSORS) {
-		return;
+		current_cursor = NORMAL;
 	}
 
 	if(have_focus == false) {
@@ -192,16 +239,18 @@ void draw(surface screen)
 		return;
 	}
 
-	int new_cursor_x, new_cursor_y;
-	SDL_GetMouseState(&new_cursor_x,&new_cursor_y);
+	if (!colour_ready) {
+		// Display start to draw cursor
+		// so it can now display colour cursor
+		colour_ready = true;
+		// Reset the cursor to be sure that we hide the b&w
+		set();
+	}
 
-	const bool must_update = new_cursor_x != cursor_x || new_cursor_y != cursor_y;
-	cursor_x = new_cursor_x;
-	cursor_y = new_cursor_y;
-
-	const surface surf(image::get_image("cursors/" + images[current_cursor],image::UNSCALED));
+	//! @todo FIXME: don't parse the file path every time
+	const surface surf(image::get_image("cursors/" + colour_images[current_cursor]));
 	if(surf == NULL) {
-		//fall back to b&w cursors
+		// Fall back to b&w cursors
 		std::cerr << "could not load colour cursors. Falling back to hardware cursors\n";
 		preferences::set_colour_cursors(false);
 		return;
@@ -219,11 +268,17 @@ void draw(surface screen)
 		}
 	}
 
-	//save the screen area where the cursor is being drawn onto the back buffer
-	SDL_Rect area = {cursor_x,cursor_y,surf->w,surf->h};
+	int new_cursor_x, new_cursor_y;
+	SDL_GetMouseState(&new_cursor_x,&new_cursor_y);
+	const bool must_update = new_cursor_x != cursor_x || new_cursor_y != cursor_y;
+	cursor_x = new_cursor_x;
+	cursor_y = new_cursor_y;
+
+	// Save the screen area where the cursor is being drawn onto the back buffer
+	SDL_Rect area = {cursor_x - shift_x[current_cursor], cursor_y - shift_y[current_cursor],surf->w,surf->h};
 	SDL_BlitSurface(screen,&area,cursor_buf,NULL);
 
-	//blit the surface
+	// Blit the surface
 	SDL_BlitSurface(surf,NULL,screen,&area);
 
 	if(must_update) {
@@ -241,9 +296,10 @@ void undraw(surface screen)
 		return;
 	}
 
-	SDL_Rect area = {cursor_x,cursor_y,cursor_buf->w,cursor_buf->h};
+	SDL_Rect area = {cursor_x - shift_x[current_cursor], cursor_y - shift_y[current_cursor],cursor_buf->w,cursor_buf->h};
 	SDL_BlitSurface(cursor_buf,NULL,screen,&area);
 	update_rect(area);
 }
 
-}
+} // end namespace cursor
+
