@@ -1,4 +1,4 @@
-/* $Id: dialogs.cpp 24187 2008-03-01 22:57:55Z mordante $ */
+/* $Id: dialogs.cpp 26766 2008-05-22 09:56:53Z baufo $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -223,9 +223,14 @@ void show_objectives(game_display& disp, const config& level, const std::string&
 
 bool is_illegal_file_char(char c)
 {
-	return c == '/' || c == '\\' || c == ':';
+	return c == '/' || c == '\\' || c == ':'
+ 	#ifdef WIN32
+	|| c == '?' || c == '|' || c == '<' || c == '>' || c == '*' || c == '"'
+	#endif
+	;
 }
 
+
 int get_save_name(display & disp,const std::string& message, const std::string& txt_label,
 				  std::string* fname, gui::DIALOG_TYPE dialog_type, const std::string& title,
 				  const bool has_exit_button, const bool ask_for_filename)
@@ -550,9 +555,21 @@ void save_preview_pane::draw_contents()
 		str << "\n";
 
 		if(campaign_type == "scenario") {
+			const std::string campaign_id = summary["campaign"];
+			const config* campaign = campaign_id.empty() ? NULL : game_config_->find_child("campaign", "id", campaign_id);
 			utils::string_map symbols;
-			symbols["campaign_name"] = summary["campaign"];
+			if (campaign != NULL) {
+				symbols["campaign_name"] = (*campaign)["name"];
+			} else {
+				// Fallback to nontranslatable campaign id.
+				symbols["campaign_name"] = "(" + campaign_id + ")";
+			}
 			str << vgettext("Campaign: $campaign_name", symbols);
+
+			// Display internal id for debug purposes if we didn't above
+			if (game_config::debug && (campaign != NULL)) {
+				str << '\n' << "(" << campaign_id << ")";
+			}
 		} else if(campaign_type == "multiplayer") {
 			str << _("Multiplayer");
 		} else if(campaign_type == "tutorial") {
