@@ -1,9 +1,11 @@
+/* $Id: editor_dialogs.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-  Copyright (C) 2003 by David White <davidnwhite@comcast.net>
+  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License.
+  it under the terms of the GNU General Public License version 2
+  or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
@@ -33,8 +35,8 @@ bool confirm_modification_disposal(display &disp);
 /// map if a new one is created. Return the string representation of the
 /// new map, or the empty string if the operation failed or was
 /// cancelled.
-std::string new_map_dialog(display &disp, gamemap::TERRAIN fill_terrain,
-						   bool confirmation_needed, const config &gconfig);
+std::string new_map_dialog(display &disp, const t_translation::t_terrain fill_terrain,
+   const bool confirmation_needed, const config &gconfig);
 
 /// Show a dialog where the user may set the preferences used in the
 /// editor.
@@ -43,8 +45,8 @@ void preferences_dialog(display &disp, config &prefs);
 /// Show a dialog asking for the new size of the map. Return the chosen
 /// width and height. Checks are made to see that the desired values
 /// will result in a feasible map.
-std::pair<unsigned, unsigned>
-resize_dialog(display &disp, const unsigned curr_w, const unsigned curr_h);
+bool resize_dialog(display &disp, unsigned& width, unsigned& height,
+	int& x_offset, int& y_offset, bool& do_expand);
 
 FLIP_AXIS flip_dialog(display &disp);
 
