@@ -1,15 +1,20 @@
+/* $Id: editor_layout.cpp 23886 2008-02-17 16:38:40Z ivanovic $ */
 /*
-  Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License.
+  it under the terms of the GNU General Public License version 2
+  or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
   See the COPYING file for more details.
 */
 
+//! @file editor/editor_layout.cpp 
+//! Set various sizes for the screen-layout of the map editor.
+
 #include "editor_layout.hpp"
 #include "util.hpp"
 
@@ -18,6 +23,7 @@ namespace {
 	const size_t default_terrain_size = 36;
 	const size_t default_palette_width = 3;
 }
+
 namespace map_editor {
 
 size_specs::size_specs() {
@@ -35,12 +41,17 @@ size_specs::size_specs() {
 }
 
 void adjust_sizes(const display &disp, size_specs &sizes) {
-	sizes.brush_x = disp.mapx() + 23;
-	sizes.brush_y = 190;
-	sizes.palette_x = disp.mapx() + 13;
-	sizes.palette_y = sizes.brush_y + 160 + 10;
+	//hardcoded coordinates for brush selection, TODO: make it themeable
+	sizes.brush_x = disp.map_outside_area().w + 22;
+	sizes.brush_y = 165;
+	//hardcoded coordinates for terrain palette, TODO: make it themeable
+	sizes.palette_x = disp.map_outside_area().w + 16;
+	sizes.palette_y = sizes.brush_y + 132 + 10;
 	sizes.palette_w = sizes.terrain_space * default_palette_width;
-	sizes.palette_h = disp.y() - sizes.palette_y - 60;
+	//the '-5' make sure that the scroll down button can not be outside of
+	//the window, this is probably a bug in placement anyway...
+	sizes.palette_h = disp.h() - sizes.palette_y - 5;
 }
 
-}
+} // end namespace map_editor
+
