@@ -1,9 +1,11 @@
+/* $Id: editor_main.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-  Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License.
+  it under the terms of the GNU General Public License version 2
+  or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
@@ -17,30 +19,42 @@
 #include "../filesystem.hpp"
 #include "../font.hpp"
 #include "../game_config.hpp"
-#include "../gamestatus.hpp"
 #include "../gettext.hpp"
 #include "../image.hpp"
 #include "../language.hpp"
+#include "../log.hpp"
 #include "../map.hpp"
 #include "../minimap.hpp"
 #include "../preferences.hpp"
 #include "../random.hpp"
-#include "../team.hpp"
 #include "../util.hpp"
 #include "../video.hpp"
 #include "../wesconfig.h"
+#include "../wml_exception.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
 #include "serialization/string_utils.hpp"
 
 #include <cctype>
+#include <clocale>
 #include <cstdlib>
 #include <iostream>
 #include <map>
 #include <string>
 
+namespace {
+	static std::string wm_title_string;
+}
+
 int main(int argc, char** argv)
 {
+	const std::string rev = game_config::svnrev.empty() ? "" :
+		" (" + game_config::svnrev + ")";
+
+	std::cerr << "Battle for Wesnoth Map Editor v" << VERSION << rev << '\n';
+	time_t t = time(NULL);
+	std::cerr << "Started on " << ctime(&t) << "\n";
+
 	game_config::editor = true;
 	rng generator;
 	const set_random_generator generator_setter(&generator);
@@ -54,34 +68,51 @@ int main(int argc, char** argv)
 
 		if(val == "--help" || val == "-h") {
 			std::cout << "usage: " << argv[0]
-		    << " [options] [map]\n"
-			<< "  -f, --fullscreen  Runs the game in full-screen\n"
-			<< "  -h, --help        Prints this message and exits\n"
-			<< "  --path            Prints the name of the game data directory and exits\n"
-			<< "  -w, --windowed    Runs the game in windowed mode\n"
-			<< "  -v, --version     Prints the game's version number and exits\n"
-			<< "  --resolution      Set the resolution of the window\n"
-			<< "  --bpp             Set the bits per pixel\n"
-		    << "  --datadir         Select the data directory to use\n";
+			<< " [options] [map]\n"
+                	<< "  --bpp                     Set the bits per pixel.\n"
+                	<< "  --datadir                 Select the data directory to use.\n"
+                	<< "  -f, --fullscreen          Runs the editor in full screen mode.\n"
+                	<< "  -h, --help                Prints this message and exits.\n"
+                	<< "  --log-<level>=<domain1>,<domain2>,...\n"
+                	<< "                            Sets the severity level of the log domains.\n"
+                	<< "                            'all' can be used to match any log domain.\n"
+                	<< "                            Available levels: error, warning, info, debug.\n"
+                	<< "                            By default the 'error' level is used.\n"
+                	<< "  --logdomains              List defined log domains and exit.\n"
+                	<< "  --path                    Prints the name of the game data directory and exits.\n"
+			<< "  -r, --resolution XxY      Sets the screen resolution. Example: -r 800x600.\n"
+                	<< "  -v, --version             Prints the game's version number and exits.\n"
+                	<< "  -w, --windowed            Runs the editor in windowed mode.\n"
+			;
 			return 0;
 		} else if(val == "--version" || val == "-v") {
-			std::cout << "Battle for Wesnoth " << game_config::version
+			std::cout << "Battle for Wesnoth "
+				  << game_config::version
+#if defined(SVNREV) && defined(DO_DISPLAY_REVISION)
+				  << " (" << game_config::svnrev << ")"
+#endif /* defined(SVNREV) and defined(DO_DISPLAY_REVISION) */
 			          << "\n";
 			return 0;
 		} else if(val == "--path") {
 			std::cout <<  game_config::path
 			          << "\n";
 			return 0;
+		} else if(val == "--logdomains") {
+		  std::cout << lg::list_logdomains() << "\n";
+			return 0;
 		}
 	}
 
 	CVideo video;
 
 	const font::manager font_manager;
-	const preferences::manager prefs_manager;
+	const preferences::base_manager prefs_manager;
 	const image::manager image_manager;
+	resize_monitor resize_monitor_;
+	binary_paths_manager paths_manager;
 	std::string filename = "";
 	std::string mapdata;
+	bool from_scenario;
 
 	int choosen_bpp = 0;
 
@@ -133,6 +164,30 @@ int main(int argc, char** argv)
 					return 0;
 				}
 			}
+		} else if (val.substr(0, 6) == "--log-") {
+			size_t p = val.find('=');
+			if (p == std::string::npos) {
+				std::cerr << "unknown option: " << val << '\n';
+				return 0;
+			}
+			std::string s = val.substr(6, p - 6);
+			int severity;
+			if (s == "error") severity = 0;
+			else if (s == "warning") severity = 1;
+			else if (s == "info") severity = 2;
+			else {
+				std::cerr << "unknown debug level: " << s << '\n';
+				return 0;
+			}
+			while (p != std::string::npos) {
+				size_t q = val.find(',', p + 1);
+				s = val.substr(p + 1, q == std::string::npos ? q : q - (p + 1));
+				if (!lg::set_log_domain_severity(s, severity)) {
+					std::cerr << "unknown debug domain: " << s << '\n';
+					return 0;
+				}
+				p = q;
+			}
 		} else if(val[0] == '-') {
 			std::cerr << "unknown option: " << val << "\n";
 			return 0;
@@ -156,11 +211,6 @@ int main(int argc, char** argv)
 	bind_textdomain_codeset (PACKAGE "-lib", "UTF-8");
 	textdomain (PACKAGE "-editor");
 
-        if (!filesystem_init()) {
-		std::cerr << "cannot init filesystem code\n";
-		return 1;
-	}
-
 	// Blatant cut and paste from game.cpp
 	image::set_wm_icon();
 	int video_flags = preferences::fullscreen() ? FULL_SCREEN : 0;
@@ -222,6 +272,9 @@ int main(int argc, char** argv)
 		return 0;
 	}
 	preproc_map defines_map;
+	// define editor to do conditionnal loading in the main cfg
+	defines_map["EDITOR"] = preproc_define();
+
 	defines_map["MEDIUM"] = preproc_define();
 	defines_map["NORMAL"] = preproc_define();
 	// Multiplayer define is needed to access the random map generator
@@ -249,32 +302,31 @@ int main(int argc, char** argv)
 	}
 
 	// Set the caption of the window
-	SDL_WM_SetCaption(_("Battle for Wesnoth Map Editor"), NULL);
+	wm_title_string = _("Battle for Wesnoth Map Editor");
+	wm_title_string += " - " + game_config::version
+			+ (game_config::svnrev.empty() ? "" :
+			" (" + game_config::svnrev + ")");
+	SDL_WM_SetCaption(wm_title_string.c_str(), NULL);
 
 	//Read the configuration af
 	config cfg;
 	try {
-		scoped_istream stream = preprocess_file("data/game.cfg", &defines_map);
+		scoped_istream stream = preprocess_file("data/", &defines_map);
 		read(cfg, *stream);
 	}
 	catch (config::error e) {
 		std::cerr << "Error when reading game config: '" << e.message << "'" << std::endl;
 	}
 	font::load_font_config();
+	paths_manager.set_paths(cfg);
 	::init_textdomains(cfg);
 
 	if(mapdata.empty()) {
-		for(int i = 0; i != 20; ++i) {
-			mapdata += "gggggggggggggggggggg\n";
-		}
+		mapdata = map_editor::new_map(22, 22, t_translation::GRASS_LAND);
 	}
 
 	srand(time(NULL));
 	bool done = false;
-	gamestatus status(cfg, 0);
-	std::vector<team> teams;
-	// Add a dummy team so the reports will be handled properly.
-	teams.push_back(team(cfg));
 	config* theme_cfg = cfg.find_child("theme", "name", "editor");
 	config dummy_theme;
 	if (!theme_cfg) {
@@ -283,32 +335,38 @@ int main(int argc, char** argv)
 	}
 
 	std::cerr << "entering while...\n";
-	unit_map units;
 	events::event_context ec;
-	while (! done) {
+	map_editor::check_data(mapdata, filename, from_scenario, cfg);
+	while (!done) {
 		try {
 			std::cerr << "creating map...\n";
-			gamemap map(cfg, mapdata);
+			//! @todo allow the editor to also create mask maps
+			editormap map(cfg, mapdata);
 
 			const config dummy_cfg;
-			display gui(units, video, map, status, teams,
-				    *theme_cfg, cfg, dummy_cfg);
+			editor_display gui(video, map, *theme_cfg, cfg, dummy_cfg);
 
 			map_editor::map_editor editor(gui, map, *theme_cfg, cfg);
-			editor.set_file_to_save_as(filename);
+			editor.set_file_to_save_as(filename, from_scenario);
 			editor.main_loop();
 			done = true;
 		}
 		catch (map_editor::map_editor::new_map_exception &e) {
 			mapdata = e.new_map;
 			filename = e.new_filename;
+			from_scenario = e.from_scenario;
 		}
 		catch (gamemap::incorrect_format_exception) {
 			std::cerr << "The map is not in a correct format, sorry." << std::endl;
 			return 1;
+		} catch(twml_exception& e) {
+			std::cerr << "WML exception:\nUser message: " 
+				<< e.user_message << "\nDev message: " << e.dev_message << '\n';
+			return 1;
 		} catch (CVideo::quit&) {
 			return 0;
 		}
 	}
+
 	return 0;
 }
