@@ -1,9 +1,11 @@
+/* $Id: editor_undo.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-  Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+  Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
   Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License.
+  it under the terms of the GNU General Public License version 2
+  or at your option any later version.
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY.
 
@@ -31,8 +33,8 @@ class map_undo_action {
 public:
 	map_undo_action();
 
-	const std::map<gamemap::location,gamemap::TERRAIN>& undo_terrains() const;
-	const std::map<gamemap::location,gamemap::TERRAIN>& redo_terrains() const;
+	const std::map<gamemap::location, t_translation::t_terrain>& undo_terrains() const;
+	const std::map<gamemap::location, t_translation::t_terrain>& redo_terrains() const;
 
 	const std::set<gamemap::location> undo_selection() const;
 	const std::set<gamemap::location> redo_selection() const;
@@ -43,8 +45,8 @@ public:
 	const std::map<gamemap::location, int>& undo_starting_locations() const;
 	const std::map<gamemap::location, int>& redo_starting_locations() const;
 
-	void add_terrain(const gamemap::TERRAIN& old_tr,
-					 const gamemap::TERRAIN& new_tr,
+	void add_terrain(const t_translation::t_terrain& old_tr,
+					 const t_translation::t_terrain& new_tr,
 					 const gamemap::location& lc);
 
 	/// Return true if a terrain change has been saved in this undo
@@ -74,8 +76,8 @@ public:
 	bool starting_location_set() const;
 
 private:
-	std::map<gamemap::location,gamemap::TERRAIN> old_terrain_;
-	std::map<gamemap::location,gamemap::TERRAIN> new_terrain_;
+	std::map<gamemap::location, t_translation::t_terrain> old_terrain_;
+	std::map<gamemap::location, t_translation::t_terrain> new_terrain_;
 	bool terrain_set_;
 	std::set<gamemap::location> old_selection_;
 	std::set<gamemap::location> new_selection_;
