@@ -1,10 +1,11 @@
-/* $Id: events.cpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: events.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,35 +17,28 @@
 #include "clipboard.hpp"
 #include "cursor.hpp"
 #include "events.hpp"
-#include "mouse.hpp"
+#include "log.hpp"
 #include "preferences_display.hpp"
 #include "sound.hpp"
 #include "video.hpp"
-#include "wassert.hpp"
 
 #include "SDL.h"
 
 #include <algorithm>
+#include <cassert>
 #include <deque>
 #include <utility>
 #include <vector>
 
+#define ERR_GEN LOG_STREAM(err, general)
+#define INFO_GEN LOG_STREAM(info, general)
+
+unsigned input_blocker::instance_count = 0; //static initialization
+
 namespace events
 {
 
-namespace {
-	int disallow_resize = 0;
-}
-
-resize_lock::resize_lock()
-{
-	++disallow_resize;
-}
-
-resize_lock::~resize_lock()
-{
-	--disallow_resize;
-}
+void raise_help_string_event(int mousex, int mousey);
 
 namespace {
 
@@ -59,7 +53,6 @@ struct context
 	std::vector<handler*> handlers;
 	int focused_handler;
 
-private:
 	void delete_handler_index(size_t handler);
 };
 
@@ -70,9 +63,9 @@ void context::add_handler(handler* ptr)
 
 void context::delete_handler_index(size_t handler)
 {
-	if(focused_handler == int(handler)) {
+	if(focused_handler == static_cast<int>(handler)) {
 		focused_handler = -1;
-	} else if(focused_handler > int(handler)) {
+	} else if(focused_handler > static_cast<int>(handler)) {
 		--focused_handler;
 	}
 
@@ -143,8 +136,20 @@ void context::set_focus(const handler* ptr)
 //in that context. The current context is the one on the top of the stack
 std::deque<context> event_contexts;
 
+std::vector<pump_monitor*> pump_monitors;
+
 } //end anon namespace
 
+pump_monitor::pump_monitor() {
+	pump_monitors.push_back(this);
+}
+
+pump_monitor::~pump_monitor() {
+	pump_monitors.erase(
+		std::remove(pump_monitors.begin(), pump_monitors.end(), this),
+		pump_monitors.end());
+}
+
 event_context::event_context()
 {
 	event_contexts.push_back(context());
@@ -152,15 +157,16 @@ event_context::event_context()
 
 event_context::~event_context()
 {
-	wassert(event_contexts.empty() == false);
+	assert(event_contexts.empty() == false);
 	event_contexts.pop_back();
 }
 
-handler::handler(bool auto_join) : unicode_(SDL_EnableUNICODE(1)), has_joined_(false)
+handler::handler(const bool auto_join) : unicode_(SDL_EnableUNICODE(1)), has_joined_(false)
 {
 	SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
 	if(auto_join) {
-		join();
+		event_contexts.back().add_handler(this);
+		has_joined_ = true;
 	}
 }
 
@@ -175,19 +181,35 @@ void handler::join()
 	if(has_joined_) {
 		leave(); // should not be in multiple event contexts
 	}
+	//join self
 	event_contexts.back().add_handler(this);
 	has_joined_ = true;
+
+	//instruct members to join
+	handler_vector members = handler_members();
+	if(!members.empty()) {
+		for(handler_vector::iterator i = members.begin(); i != members.end(); i++) {
+			(*i)->join();
+		}
+	}
 }
 
 void handler::leave()
 {
-	wassert(event_contexts.empty() == false);
+	handler_vector members = handler_members();
+	if(!members.empty()) {
+		for(handler_vector::iterator i = members.begin(); i != members.end(); i++) {
+			(*i)->leave();
+		}
+	} else {
+		assert(event_contexts.empty() == false);
+	}
 	for(std::deque<context>::reverse_iterator i = event_contexts.rbegin(); i != event_contexts.rend(); ++i) {
 		if(i->remove_handler(this)) {
 			break;
 		}
 	}
-	//has_joined_ = false;
+	has_joined_ = false;
 }
 
 void focus_handler(const handler* ptr)
@@ -197,39 +219,43 @@ void focus_handler(const handler* ptr)
 	}
 }
 
-void cycle_focus()
-{
-	if(event_contexts.empty() == false) {
-		event_contexts.back().cycle_focus();
-	}
-}
-
-bool has_focus(const handler* ptr, const SDL_Event* event)
+bool has_focus(const handler* hand, const SDL_Event* event)
 {
 	if(event_contexts.empty()) {
 		return true;
 	}
 
-	if(ptr->requires_event_focus(event) == false) {
+	if(hand->requires_event_focus(event) == false) {
 		return true;
 	}
 
-	const int index = event_contexts.back().focused_handler;
+	const int foc_i = event_contexts.back().focused_handler;
 
 	//if no-one has focus at the moment, this handler obviously wants
 	//focus, so give it to it.
-	if(index == -1) {
-		focus_handler(ptr);
+	if(foc_i == -1) {
+		focus_handler(hand);
 		return true;
-	} else if(event_contexts.back().handlers[index] == ptr){
+	}
+
+	handler *const foc_hand = event_contexts.back().handlers[foc_i];
+	if(foc_hand == hand){
 		return true;
-	} else if(!event_contexts.back().handlers[index]->requires_event_focus(event)) {
+	} else if(!foc_hand->requires_event_focus(event)) {
 		//if the currently focused handler doesn't need focus for this event
-		//allow the first-in interested handler to take care of it
-		for(int i=0; i<event_contexts.back().handlers.size(); i++) {
-			if(i != index && event_contexts.back().handlers[i]->requires_event_focus(event)) {
-				//focus_handler(event_contexts.back().handlers[i]); //steal focus?
-				return event_contexts.back().handlers[i] == ptr;
+		//allow the most recent interested handler to take care of it
+		int back_i = event_contexts.back().handlers.size() - 1;
+		for(int i=back_i; i>=0; --i) {
+			handler *const thief_hand = event_contexts.back().handlers[i];
+			if(i != foc_i && thief_hand->requires_event_focus(event)) {
+				//steal focus
+				focus_handler(thief_hand);
+				if(foc_i < back_i) {
+					//position the previously focused handler to allow stealing back
+					event_contexts.back().delete_handler_index(foc_i);
+					event_contexts.back().add_handler(foc_hand);
+				}
+				return thief_hand == hand;
 			}
 		}
 	}
@@ -240,21 +266,44 @@ void pump()
 {
 	SDL_PumpEvents();
 
-	static std::pair<int,int> resize_dimensions(0,0);
+	pump_info info;
 
 	//used to keep track of double click events
 	static int last_mouse_down = -1;
 	static int last_click_x = -1, last_click_y = -1;
 
-	SDL_Event event;
-	while(SDL_PollEvent(&event)) {
-
+	SDL_Event temp_event;
+	int poll_count = 0;
+	int begin_ignoring = 0;
+	std::vector< SDL_Event > events;
+	while(SDL_PollEvent(&temp_event)) {
+		++poll_count;
+		if(!begin_ignoring && temp_event.type == SDL_ACTIVEEVENT) {
+			begin_ignoring = poll_count;
+		} else if(begin_ignoring > 0 && SDL_EVENTMASK(temp_event.type)&INPUT_MASK) {
+			//ignore user input events that occurred after the window was activated
+			continue;
+		}
+		events.push_back(temp_event);
+	}
+	std::vector<SDL_Event>::iterator ev_it = events.begin();
+	for(int i=1; i < begin_ignoring; ++i){
+		if(SDL_EVENTMASK(ev_it->type)&INPUT_MASK) {
+			//ignore user input events that occurred before the window was activated
+			ev_it = events.erase(ev_it);
+		} else {
+			++ev_it;
+		}
+	}
+	std::vector<SDL_Event>::iterator ev_end = events.end();
+	for(ev_it = events.begin(); ev_it != ev_end; ++ev_it){
+		SDL_Event &event = *ev_it;
 		switch(event.type) {
 
-			case SDL_APPMOUSEFOCUS: {
+			case SDL_ACTIVEEVENT: {
 				SDL_ActiveEvent& ae = reinterpret_cast<SDL_ActiveEvent&>(event);
-				if(ae.state == SDL_APPMOUSEFOCUS || ae.state == SDL_APPINPUTFOCUS) {
-					cursor::set_focus(ae.gain == 1);
+				if((ae.state & SDL_APPMOUSEFOCUS) != 0 || (ae.state & SDL_APPINPUTFOCUS) != 0) {
+					cursor::set_focus(ae.gain != 0);
 				}
 				break;
 			}
@@ -267,15 +316,8 @@ void pump()
 
 			case SDL_VIDEORESIZE: {
 				const SDL_ResizeEvent* const resize = reinterpret_cast<SDL_ResizeEvent*>(&event);
-
-				if(resize->w < min_allowed_width || resize->h < min_allowed_height) {
-					resize_dimensions.first = 0;
-					resize_dimensions.second = 0;
-				} else {
-					resize_dimensions.first = resize->w;
-					resize_dimensions.second = resize->h;
-				}
-
+				info.resize_dimensions.first = resize->w;
+				info.resize_dimensions.second = resize->h;
 				break;
 			}
 
@@ -293,10 +335,8 @@ void pump()
 				cursor::set_focus(true);
 				if(event.button.button == SDL_BUTTON_LEFT) {
 					static const int DoubleClickTime = 500;
-
 					static const int DoubleClickMaxMove = 3;
-					const int current_ticks = ::SDL_GetTicks();
-					if(last_mouse_down >= 0 && current_ticks - last_mouse_down < DoubleClickTime &&
+					if(last_mouse_down >= 0 && info.ticks() - last_mouse_down < DoubleClickTime &&
 					   abs(event.button.x - last_click_x) < DoubleClickMaxMove &&
 					   abs(event.button.y - last_click_y) < DoubleClickMaxMove) {
 						SDL_UserEvent user_event;
@@ -306,19 +346,10 @@ void pump()
 						user_event.data2 = reinterpret_cast<void*>(event.button.y);
 						::SDL_PushEvent(reinterpret_cast<SDL_Event*>(&user_event));
 					}
-
-					last_mouse_down = current_ticks;
+					last_mouse_down = info.ticks();
 					last_click_x = event.button.x;
 					last_click_y = event.button.y;
 				}
-
-				//mouse wheel support
-				else if(event.button.button == 4) {
-					gui::scroll_dec();
-				} else if(event.button.button == 5) {
-					gui::scroll_inc();
-				}
-
 				break;
 			}
 
@@ -347,14 +378,10 @@ void pump()
 		}
 	}
 
-	if(resize_dimensions.first > 0 && disallow_resize == 0) {
-		preferences::set_resolution(resize_dimensions);
-		resize_dimensions.first = 0;
-		resize_dimensions.second = 0;
+	//inform the pump monitors that an events::pump() has occurred
+	for(size_t i1 = 0, i2 = pump_monitors.size(); i1 != i2 && i1 < pump_monitors.size(); ++i1) {
+		pump_monitors[i1]->process(info);
 	}
-
-	if (preferences::music_on())
-		sound::think_about_music();
 }
 
 void raise_process_event()
@@ -425,4 +452,61 @@ void raise_help_string_event(int mousex, int mousey)
 	}
 }
 
+int discard(Uint32 event_mask)
+{
+	int discard_count = 0;
+	SDL_Event temp_event;
+	std::vector< SDL_Event > keepers;
+	SDL_Delay(10);
+	while(SDL_PollEvent(&temp_event) > 0) {
+		if((SDL_EVENTMASK(temp_event.type) & event_mask) == 0) {
+			keepers.push_back( temp_event );
+		} else {
+			++discard_count;
+		}
+	}
+
+	//FIXME: there is a chance new events are added before kept events are replaced
+	for (unsigned int i=0; i < keepers.size(); ++i)
+	{
+		if(SDL_PushEvent(&keepers[i]) != 0) {
+			ERR_GEN << "failed to return an event to the queue.";
+		}
+	}
+
+	return discard_count;
+}
+
+int pump_info::ticks(unsigned *refresh_counter, unsigned refresh_rate) {
+	if(!ticks_ && !(refresh_counter && ++*refresh_counter % refresh_rate)) {
+		ticks_ = ::SDL_GetTicks();
+	}
+	return ticks_;
+}
+
+} //end events namespace
+
+input_blocker::input_blocker()
+{
+	SDL_EventState(SDL_KEYDOWN, SDL_IGNORE);
+	SDL_EventState(SDL_KEYUP, SDL_IGNORE);
+	SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_IGNORE);
+	SDL_EventState(SDL_MOUSEBUTTONUP, SDL_IGNORE);
+	SDL_EventState(SDL_JOYBUTTONDOWN, SDL_IGNORE);
+	SDL_EventState(SDL_JOYBUTTONUP, SDL_IGNORE);
+	instance_count++;
+}
+
+input_blocker::~input_blocker()
+{
+	instance_count--;
+	if(instance_count == 0) {
+		events::discard(INPUT_MASK);
+		SDL_EventState(SDL_KEYDOWN, SDL_ENABLE);
+		SDL_EventState(SDL_KEYUP, SDL_ENABLE);
+		SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE);
+		SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE);
+		SDL_EventState(SDL_JOYBUTTONDOWN, SDL_ENABLE);
+		SDL_EventState(SDL_JOYBUTTONUP, SDL_ENABLE);
+	}
 }
