@@ -1,10 +1,11 @@
-/* $Id: events.hpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: events.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,6 +16,7 @@
 #define EVENTS_HPP_INCLUDED
 
 #include "SDL.h"
+#include <vector>
 
 //our user-defined double-click event type
 #define DOUBLE_CLICK_EVENT SDL_USEREVENT
@@ -22,13 +24,6 @@
 namespace events
 {
 
-//an object which prevents resizing of the screen occuring during
-//its lifetime.
-struct resize_lock {
-	resize_lock();
-	~resize_lock();
-};
-
 //any classes that derive from this class will automatically
 //receive sdl events through the handle function for their lifetime,
 //while the event context they were created in is active.
@@ -46,19 +41,19 @@ public:
 	virtual void volatile_draw() {}
 	virtual void volatile_undraw() {}
 
-	virtual bool requires_event_focus(const SDL_Event *event=NULL) const { return false; }
+	virtual bool requires_event_focus(const SDL_Event * = NULL) const { return false; }
 
 	virtual void process_help_string(int /*mousex*/, int /*mousey*/) {}
 
 	virtual void join(); /*joins the current event context*/
+	virtual void leave(); /*leave the event context*/
 
 protected:
-	handler(bool auto_join=true);
+	handler(const bool auto_join=true);
 	virtual ~handler();
+	virtual std::vector<handler*> handler_members() {std::vector<handler*> h; return h;}
 
 private:
-	void leave(); /*leave the event context*/
-
 	int unicode_;
 	bool has_joined_;
 };
@@ -86,6 +81,24 @@ struct event_context
 //causes events to be dispatched to all handler objects.
 void pump();
 
+struct pump_info {
+	pump_info() : resize_dimensions(), ticks_(0) {}
+	std::pair<int,int> resize_dimensions;
+	int ticks(unsigned *refresh_counter=NULL, unsigned refresh_rate=1);
+private:
+	int ticks_; //0 if not calculated
+};
+
+class pump_monitor {
+//pump_monitors receive notifcation after an events::pump() occurs
+public:
+	pump_monitor();
+	virtual ~pump_monitor();
+	virtual void process(pump_info& info) = 0;
+};
+
+int discard(Uint32 event_mask=SDL_ALLEVENTS);
+
 void raise_process_event();
 void raise_draw_event();
 void raise_volatile_draw_event();
@@ -93,4 +106,22 @@ void raise_volatile_undraw_event();
 void raise_help_string_event(int mousex, int mousey);
 }
 
+typedef std::vector<events::handler*> handler_vector;
+
+#define INPUT_MASK (SDL_EVENTMASK(SDL_KEYDOWN)|\
+			               SDL_EVENTMASK(SDL_KEYUP)|\
+			               SDL_EVENTMASK(SDL_MOUSEBUTTONDOWN)|\
+			               SDL_EVENTMASK(SDL_MOUSEBUTTONUP)|\
+			               SDL_EVENTMASK(SDL_JOYBUTTONDOWN)|\
+			               SDL_EVENTMASK(SDL_JOYBUTTONUP))
+
+class input_blocker
+{
+public:
+	input_blocker();
+	~input_blocker();
+private:
+	static unsigned instance_count;
+};
+
 #endif
