@@ -1,15 +1,20 @@
-/* $Id: filesystem.hpp 11861 2006-06-05 11:26:38Z rusty $ */
+/* $Id: filesystem.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file filesystem.hpp 
+//! Declarations for File-IO.
+
 #ifndef FILESYSTEM_HPP_INCLUDED
 #define FILESYSTEM_HPP_INCLUDED
 
@@ -19,9 +24,9 @@
 #include <string>
 #include <vector>
 
-//an exception object used when an IO error occurs
+//! An exception object used when an IO error occurs
 struct io_exception : public std::exception {
-	io_exception() {}
+	io_exception() : message("") {}
 	io_exception(const std::string& msg) : message(msg) {}
 	virtual ~io_exception() throw() {}
 
@@ -31,20 +36,24 @@ private:
 };
 
 enum FILE_NAME_MODE { ENTIRE_FILE_PATH, FILE_NAME_ONLY };
+enum FILE_FILTER { NO_FILTER, SKIP_MEDIA_DIR};
+enum FILE_REORDER_OPTION { DONT_REORDER, DO_REORDER };
 
-//function which populates files with all the files and dirs
-//with all the directories in dir. If files or dirs are NULL
-//they will not be used.
-//
-//mode determines whether the entire path or just the filename
-//is retrieved.
-void get_files_in_dir(const std::string& dir,
+//! Populates 'files' with all the files and 
+//! 'dirs' with all the directories in dir. 
+//! If files or dirs are NULL they will not be used.
+//!
+//! Mode determines whether the entire path or just the filename is retrieved.
+void get_files_in_dir(const std::string dir,
                       std::vector<std::string>* files,
                       std::vector<std::string>* dirs=NULL,
-                      FILE_NAME_MODE mode=FILE_NAME_ONLY);
+                      FILE_NAME_MODE mode=FILE_NAME_ONLY,
+                      FILE_FILTER = NO_FILTER,
+                      FILE_REORDER_OPTION reorder=DONT_REORDER);
 
 std::string get_dir(const std::string &dir);
-//the location of various important files
+
+// The location of various important files:
 std::string get_prefs_file();
 std::string get_save_index_file();
 std::string get_saves_dir();
@@ -56,32 +65,35 @@ std::string get_user_data_dir();
 
 std::string get_cwd();
 
-void make_directory(const std::string& dirname);
+bool make_directory(const std::string& dirname);
 bool delete_directory(const std::string& dirname);
 
-//basic disk I/O
-bool filesystem_init();
+// Basic disk I/O:
+
+//! Basic disk I/O - read file.
 std::string read_file(const std::string& fname);
 std::istream *istream_file(std::string const &fname);
 std::ostream *ostream_file(std::string const &fname);
-//throws io_exception if an error occurs
+//! Throws io_exception if an error occurs.
 void write_file(const std::string& fname, const std::string& data);
-std::string read_stdin();
 
 std::string read_map(const std::string& name);
 
-//function which returns true iff the given file is a directory
+//! Returns true if the given file is a directory.
 bool is_directory(const std::string& fname);
 
-//function which returns true iff file with name already exists
+//! Returns true if file with name already exists.
 bool file_exists(const std::string& name);
 
-//function to get the creation time of a file
+//! Get the creation time of a file.
 time_t file_create_time(const std::string& fname);
 
-//return the next ordered full filename within this directory
+//! Return the next ordered full filename within this directory.
 std::string next_filename(const std::string &dirname, unsigned int max = 0);
 
+//! Returns true if the file ends with '.gz'.
+bool is_gzip_file(const std::string& filename);
+
 struct file_tree_checksum
 {
 	file_tree_checksum();
@@ -95,31 +107,33 @@ bool operator==(const file_tree_checksum& lhs, const file_tree_checksum& rhs);
 bool operator!=(const file_tree_checksum& lhs, const file_tree_checksum& rhs);
 
 
-//function to get the time at which the most recently modified file
-//in a directory tree was modified at
-file_tree_checksum get_file_tree_checksum(const std::string& path);
-
-//function to get the time at which the data/ tree was last modified at
+//! Get the time at which the data/ tree was last modified at.
 const file_tree_checksum& data_tree_checksum();
 
-//returns the size of a file, or -1 if the file doesn't exist
+//! Returns the size of a file, or -1 if the file doesn't exist.
 int file_size(const std::string& fname);
 
-//returns the base filename of a file, with directory name stripped. Equivalent
-//to a portable basename() function
+bool ends_with(const std::string& str, const std::string& suffix);
+
+//! Returns the base filename of a file, with directory name stripped. 
+//! Equivalent to a portable basename() function.
 std::string file_name(const std::string& file);
 
-//returns the directory name of a file, with filename stripped. Equivalent to a
-//portable dirname()
+//! Returns the directory name of a file, with filename stripped. 
+//! Equivalent to a portable dirname()
 std::string directory_name(const std::string& file);
 
-///the paths manager is responsible for recording the various paths that
-///binary files may be located at. It should be passed a config object
-///which holds binary path information. This is in the format
-///[binary_path]
-///path=<path>
-///[/binary_path]
-///Binaries will be searched for in [wesnoth-path]/data/<path>/images/
+/*! The paths manager is responsible for recording the various paths 
+ *  that binary files may be located at. 
+ *  It should be passed a config object which holds binary path information. 
+ *  This is in the format
+ *@verbatim
+ *    [binary_path]
+ *      path=<path>
+ *    [/binary_path]
+ *  Binaries will be searched for in [wesnoth-path]/data/<path>/images/
+ *@endverbatim
+ */
 struct binary_paths_manager
 {
 	binary_paths_manager();
@@ -139,20 +153,19 @@ private:
 
 void clear_binary_paths_cache();
 
-//function which, given a type of binary, e.g. 'images', 'sounds', etc,
-//will return a vector with all possible paths to that type of binary
+//! Returns a vector with all possible paths to a given type of binary,
+//! e.g. 'images', 'sounds', etc,
 const std::vector<std::string>& get_binary_paths(const std::string& type);
 
-//function which, given a type of binary, and the name of the binary file,
-//will return a complete path to the actual file, or an empty string if
-//the file isn't present
+//! Returns a complete path to the actual file of a given a type of binary, 
+//! or an empty string if the file isn't present.
 std::string get_binary_file_location(const std::string& type, const std::string& filename);
 
 class scoped_istream {
 	std::istream *stream;
 public:
 	scoped_istream(std::istream *s): stream(s) {}
-	void operator=(std::istream *);
+	scoped_istream& operator=(std::istream *);
 	std::istream &operator*() { return *stream; }
 	std::istream *operator->() { return stream; }
 	~scoped_istream();
@@ -162,7 +175,7 @@ class scoped_ostream {
 	std::ostream *stream;
 public:
 	scoped_ostream(std::ostream *s): stream(s) {}
-	void operator=(std::ostream *);
+	scoped_ostream& operator=(std::ostream *);
 	std::ostream &operator*() { return *stream; }
 	std::ostream *operator->() { return stream; }
 	~scoped_ostream();
