@@ -1,4 +1,4 @@
-/* $Id: font.cpp 23985 2008-02-21 08:15:14Z mythological $ */
+/* $Id: font.cpp 24636 2008-03-14 11:13:37Z shadowmaster $ */
 /* vim:set encoding=utf-8: */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
@@ -346,7 +346,7 @@ const SDL_Color NORMAL_COLOUR = {0xDD,0xDD,0xDD,0},
                 STONED_COLOUR = {0xA0,0xA0,0xA0,0},
                 TITLE_COLOUR  = {0xBC,0xB0,0x88,0},
 				LABEL_COLOUR  = {0x6B,0x8C,0xFF,0},
-				BIGMAP_COLOUR = {0x00,0xAA,0x00,0};
+				BIGMAP_COLOUR = {0xFF,0xFF,0xFF,0};
 const SDL_Color DISABLED_COLOUR = inverse(STONED_COLOUR);
 
 namespace {
