@@ -1,4 +1,4 @@
-/* $Id: game.cpp 24055 2008-02-25 20:01:41Z mordante $ */
+/* $Id: game.cpp 26838 2008-05-25 07:58:36Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -55,6 +55,7 @@
 #include "serialization/binary_wml.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
+#include "serialization/string_utils.hpp"
 #include "sha1.hpp"
 
 #ifdef HAVE_PYTHON
@@ -767,7 +768,7 @@ bool game_controller::load_game()
 		}
 
 		const std::string version = cfg["version"];
-		if(version != game_config::version) {
+		if (!utils::wildcard_string_match(version, "1.4*")) {
 			// do not load if too old, if either the savegame or the current game
 			// has the version 'test' allow loading
 			if(version < game_config::min_savegame_version &&
@@ -895,10 +896,12 @@ bool game_controller::new_campaign()
 		campaign_desc.push_back(std::pair<std::string,std::string>(desc,image));
 	}
 
-	dialogs::campaign_preview_pane campaign_preview(disp().video(),&campaign_desc);
 	if(campaign_names.size() <= 0) {
+	  gui::show_error_message(disp(),
+				  _("No campaigns are available.\n"));
 		return false;
 	}
+	dialogs::campaign_preview_pane campaign_preview(disp().video(),&campaign_desc);
 	gui::dialog cmenu(disp(), _("Play a campaign"), " ", gui::OK_CANCEL);
 	cmenu.set_menu(campaign_names);
 	cmenu.add_pane(&campaign_preview);
@@ -1153,8 +1156,8 @@ void game_controller::download_campaigns(std::string host)
 
 		config cfg;
 		cfg.add_child("request_campaign_list");
-		// @todo Should be enabled once the campaign server can be recompiled.
-		network::send_data(cfg, sock, false);
+
+		network::send_data(cfg, sock, true);
 
 		network::connection res = dialogs::network_receive_dialog(disp(),_("Asking for list of add-ons"),cfg,sock);
 		if(!res) {
@@ -1300,8 +1303,8 @@ void game_controller::download_campaigns(std::string host)
 
 		config request;
 		request.add_child("request_campaign")["name"] = campaigns[index];
-		// @todo Should be enabled once the campaign server can be recompiled.
-		network::send_data(request, sock, false);
+
+		network::send_data(request, sock, true);
 
 		res = dialogs::network_receive_dialog(disp(),_("Downloading add-on..."),cfg,sock);
 		if(!res) {
@@ -1378,8 +1381,8 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 {
 	config request_terms;
 	request_terms.add_child("request_terms");
-	// @todo Should be enabled once the campaign server can be recompiled.
-	network::send_data(request_terms, sock, false);
+
+	network::send_data(request_terms, sock, true);
 	config data;
 	sock = network::receive_data(data,sock,5000);
 	if(!sock) {
@@ -1421,8 +1424,8 @@ void game_controller::upload_campaign(const std::string& campaign, network::conn
 	data.add_child("upload",cfg).add_child("data",campaign_data);
 
 	LOG_NET << "uploading campaign...\n";
-	// @todo Should be enabled once the campaign server can be recompiled.
-	network::send_data(data, sock, false);
+
+	network::send_data(data, sock, true);
 
 	sock = dialogs::network_send_dialog(disp(),_("Sending add-on"),data,sock);
 	if(!sock) {
@@ -1449,8 +1452,8 @@ void game_controller::delete_campaign(const std::string& campaign, network::conn
 	config data;
 	data.add_child("delete",msg);
 
-	// @todo Should be enabled once the campaign server can be recompiled.
-	network::send_data(data, sock, false);
+
+	network::send_data(data, sock, true);
 
 	sock = network::receive_data(data,sock,5000);
 	if(!sock) {
@@ -1487,8 +1490,8 @@ bool game_controller::play_multiplayer()
 		+ sep1 + _("Join Official Server")
 		+ sep2 + _("Log on to the official Wesnoth multiplayer server"));
 	host_or_join.push_back(pre + "serverother.png"
-		+ sep1 + _("Connect to Host/Server")
-		+ sep2 + _("Join a server or hosted game"));
+		+ sep1 + _("Connect to Server")
+		+ sep2 + _("Join a different server"));
 	host_or_join.push_back(pre + "hotseat.png"
 		+ sep1 + _("Local Game")
 		+ sep2 + _("Play a multiplayer game with the AI or humans sharing the same machine"));
@@ -1752,7 +1755,9 @@ void game_controller::read_game_cfg(const preproc_map& defines, config& cfg, boo
 
 			if(error_campaigns.empty() == false) {
 				std::stringstream msg;
-				msg << _("The following add-on(s) had errors and could not be loaded:");
+				msg << _n("The following add-on had errors and could not be loaded:",
+						"The following add-ons had errors and could not be loaded:",
+						error_campaigns.size());
 				for(std::vector<std::string>::const_iterator i = error_campaigns.begin(); i != error_campaigns.end(); ++i) {
 					msg << "\n" << *i;
 				}
