@@ -1,4 +1,4 @@
-/* $Id: game_config.cpp 24124 2008-02-27 18:10:46Z soliton $ */
+/* $Id: game_config.cpp 26448 2008-05-07 17:35:33Z mordante $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -23,7 +23,9 @@
 #include "wesconfig.h"
 
 #include <cstdlib>
+#include <string>
 #include <sstream>
+#include <ostream>
 
 #define DBG_NG LOG_STREAM(debug, engine)
 #define ERR_NG LOG_STREAM(err, engine)
