@@ -1,4 +1,4 @@
-/* $Id: game_preferences.cpp 24035 2008-02-24 23:11:09Z sapient $ */
+/* $Id: game_preferences.cpp 24653 2008-03-14 22:26:46Z noyga $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -724,6 +724,10 @@ void encounter_map_terrain(gamemap& map){
 		for (int map_y = 0; map_y < map.h(); map_y++) {
 			const t_translation::t_terrain t = map.get_terrain(gamemap::location(map_x, map_y));
 			preferences::encountered_terrains().insert(t);
+			const t_translation::t_list& underlaying_list = map.underlying_union_terrain(gamemap::location(map_x, map_y));
+			for (std::vector<t_translation::t_terrain>::const_iterator ut = underlaying_list.begin(); ut != underlaying_list.end(); ut++) {
+				preferences::encountered_terrains().insert(*ut);
+			};
 		}
 	}
 }
