@@ -1,4 +1,4 @@
-/* $Id: gamestatus.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: gamestatus.cpp 26453 2008-05-07 23:31:24Z shadowmaster $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -920,6 +920,7 @@ void extract_summary_data_from_save(const game_state& gamestate, config& out)
 	out["snapshot"] = has_snapshot ? "yes" : "no";
 
 	out["label"] = gamestate.label;
+	out["campaign"] = gamestate.campaign;
 	out["campaign_type"] = gamestate.campaign_type;
 	out["scenario"] = gamestate.scenario;
 	out["difficulty"] = gamestate.difficulty;
@@ -964,7 +965,7 @@ void extract_summary_data_from_save(const game_state& gamestate, config& out)
 
 			const config::child_list& units = (**s).get_children("unit");
 			for(config::child_list::const_iterator u = units.begin(); u != units.end(); ++u) {
-				if((**u)["canrecruit"] == "1") {
+				if(utils::string_bool( (**u)["canrecruit"], false) == true) {
 					leader = (**u)["id"];
 					break;
 				}
@@ -1022,7 +1023,7 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 	const config::child_list& players = cfg_save.get_children("player");
 
 	for(config::child_list::const_iterator i = players.begin(); i != players.end(); ++i) {
-		if ((**i)["canrecruit"] == "1"){
+		if (utils::string_bool( (**i)["canrecruit"], false) == true){
 			leader = (**i)["save_id"];
 		}
 	}
@@ -1045,7 +1046,7 @@ void extract_summary_from_config(config& cfg_save, config& cfg_summary)
 
 				const config::child_list& units = (**s).get_children("unit");
 				for(config::child_list::const_iterator u = units.begin(); u != units.end(); ++u) {
-					if((**u)["canrecruit"] == "1") {
+					if(utils::string_bool( (**u)["canrecruit"], false) == true) {
 						leader = (**u)["id"];
 						break;
 					}
