@@ -1,10 +1,11 @@
-/* $Id: hotkeys.cpp 13893 2006-10-10 19:59:46Z jhinrichs $ */
+/* $Id: hotkeys.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,12 +17,14 @@
 #define GETTEXT_DOMAIN "wesnoth-lib"
 
 #include "config.hpp"
+#include "construct_dialog.hpp"
 #include "display.hpp"
 #include "events.hpp"
 #include "hotkeys.hpp"
 #include "game_config.hpp"
 #include "game_errors.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
 #include "menu_events.hpp"
 #include "preferences_display.hpp"
 #include "show_dialog.hpp"
@@ -36,7 +39,9 @@
 #include <map>
 
 #define ERR_G LOG_STREAM(err, general)
-#define INFO_C LOG_STREAM(info, config)
+#define LOG_G LOG_STREAM(info, general)
+#define DBG_G LOG_STREAM(debug, general)
+#define ERR_CONFIG LOG_STREAM(err, config)
 
 namespace {
 
@@ -62,6 +67,8 @@ const struct {
 	{ hotkey::HOTKEY_UNIT_DESCRIPTION, "describeunit", N_("Unit Description"), false },
 	{ hotkey::HOTKEY_RENAME_UNIT, "renameunit", N_("Rename Unit"), false },
 	{ hotkey::HOTKEY_SAVE_GAME, "save", N_("Save Game"), false },
+	{ hotkey::HOTKEY_SAVE_REPLAY, "savereplay", N_("Save Replay"), false },
+	{ hotkey::HOTKEY_SAVE_MAP, "savemap", N_("Save The Map"), false },
 	{ hotkey::HOTKEY_LOAD_GAME, "load", N_("Load Game"), false },
 	{ hotkey::HOTKEY_RECRUIT, "recruit", N_("Recruit"), false },
 	{ hotkey::HOTKEY_REPEAT_RECRUIT, "repeatrecruit", N_("Repeat Recruit"), false },
@@ -79,6 +86,7 @@ const struct {
 	{ hotkey::HOTKEY_UNIT_LIST, "unitlist", N_("Unit List"), false },
 	{ hotkey::HOTKEY_STATISTICS, "statistics", N_("Statistics"), false },
 	{ hotkey::HOTKEY_QUIT_GAME, "quit", N_("Quit Game"), false },
+	{ hotkey::HOTKEY_LABEL_TEAM_TERRAIN, "labelteamterrain", N_("Set Team Label"), false },
 	{ hotkey::HOTKEY_LABEL_TERRAIN, "labelterrain", N_("Set Label"), false },
 	{ hotkey::HOTKEY_CLEAR_LABELS, "clearlabels", N_("Clear Labels"), false },
 	{ hotkey::HOTKEY_SHOW_ENEMY_MOVES, "showenemymoves", N_("Show Enemy Moves"), false },
@@ -88,8 +96,12 @@ const struct {
 	{ hotkey::HOTKEY_STOP_REPLAY, "stopreplay", N_("Stop"), false },
 	{ hotkey::HOTKEY_REPLAY_NEXT_TURN, "replaynextturn", N_("Next Turn"), false },
 	{ hotkey::HOTKEY_REPLAY_NEXT_SIDE, "replaynextside", N_("Next Side"), false },
-	{ hotkey::HOTKEY_REPLAY_SHROUD, "replayswitchshroud", N_("Shroud"), false },
-	{ hotkey::HOTKEY_REPLAY_FOG, "replayswitchfog", N_("Fog"), false },
+	{ hotkey::HOTKEY_REPLAY_SHOW_EVERYTHING, "replayshoweverything",
+	  N_("Full map"), false },
+	{ hotkey::HOTKEY_REPLAY_SHOW_EACH, "replayshoweach",
+	  N_("Each team"), false },
+	{ hotkey::HOTKEY_REPLAY_SHOW_TEAM1, "replayshowteam1",
+	  N_("Team 1"), false },
 	{ hotkey::HOTKEY_REPLAY_SKIP_ANIMATION, "replayskipanimation", N_("Skip animation"), false },
 
 	{ hotkey::HOTKEY_EDIT_SET_TERRAIN, "editsetterrain", N_("Set Terrain"),true },
@@ -101,6 +113,7 @@ const struct {
 	{ hotkey::HOTKEY_EDIT_SET_START_POS, "editsetstartpos", N_("Set Player's keep"),true },
 	{ hotkey::HOTKEY_EDIT_FLOOD_FILL, "editfloodfill", N_("Flood Fill"),true },
 	{ hotkey::HOTKEY_EDIT_FILL_SELECTION, "editfillselection", N_("Fill Selection"),true },
+	{ hotkey::HOTKEY_EDIT_ROTATE_SELECTION, "editrotateselection", N_("Rotate Selection"),true },
 	{ hotkey::HOTKEY_EDIT_CUT, "editcut", N_("Cut"),true },
 	{ hotkey::HOTKEY_EDIT_COPY, "editcopy", N_("Copy"),true },
 	{ hotkey::HOTKEY_EDIT_PASTE, "editpaste", N_("Paste"),true },
@@ -110,6 +123,8 @@ const struct {
 	{ hotkey::HOTKEY_EDIT_SELECT_ALL, "editselectall", N_("Select All"),true },
 	{ hotkey::HOTKEY_EDIT_DRAW, "editdraw", N_("Draw Terrain"),true },
 	{ hotkey::HOTKEY_EDIT_REFRESH, "editrefresh", N_("Refresh Image Cache"), true },
+	{ hotkey::HOTKEY_EDIT_AUTO_UPDATE, "editautoupdate", N_("Delay transition updates"), true },
+	{ hotkey::HOTKEY_EDIT_UPDATE, "editupdate", N_("Update transitions"), true },
 
 	{ hotkey::HOTKEY_DELAY_SHROUD, "delayshroud", N_("Delay Shroud Updates"), false },
 	{ hotkey::HOTKEY_UPDATE_SHROUD, "updateshroud", N_("Update Shroud Now"), false },
@@ -119,10 +134,14 @@ const struct {
 	{ hotkey::HOTKEY_SPEAK_ALL, "speaktoall", N_("Speak to All"), false },
 	{ hotkey::HOTKEY_HELP, "help", N_("Help"), false },
 	{ hotkey::HOTKEY_CHAT_LOG, "chatlog", N_("View Chat Log"), false },
-	{ hotkey::HOTKEY_USER_CMD, "command", N_("Enter user command"), false },
-
 	{ hotkey::HOTKEY_LANGUAGE, "changelanguage", N_("Change the language"), true },
 
+	{ hotkey::HOTKEY_USER_CMD, "command", N_("Enter user command"), false },
+	{ hotkey::HOTKEY_CLEAR_MSG, "clearmessages", N_("Clear messages"), false },
+#ifdef USRCMD2
+	{ hotkey::HOTKEY_USER_CMD_2, "usercommand#2", N_("User-Command#2"), false },
+	{ hotkey::HOTKEY_USER_CMD_3, "usercommand#3", N_("User-Command#3"), false },
+#endif
 	{ hotkey::HOTKEY_NULL, NULL, NULL, true }
 };
 
@@ -133,9 +152,24 @@ hotkey::hotkey_item null_hotkey_;
 
 namespace hotkey {
 
-hotkey_item::hotkey_item(HOTKEY_COMMAND id, const std::string& command, const std::string& description, bool hidden)
-	: id_(id), command_(command), description_(description), type_(UNBOUND),
-	  ctrl_(false), alt_(false), cmd_(false), shift_(false), hidden_(hidden)
+const std::string CLEARED_TEXT = "__none__";
+
+static void key_event_execute(display& disp, const SDL_KeyboardEvent& event, command_executor* executor);
+
+
+hotkey_item::hotkey_item(HOTKEY_COMMAND id, 
+		const std::string& command, const std::string& description, bool hidden) :
+	id_(id), 
+	command_(command), 
+	description_(description), 
+	type_(UNBOUND),
+	character_(0),
+	ctrl_(false), 
+	alt_(false), 
+	cmd_(false),
+	keycode_(0),
+	shift_(false), 
+	hidden_(hidden)
 {
 }
 
@@ -158,14 +192,17 @@ void hotkey_item::load_from_config(const config& cfg)
 	if (!key.empty()) {
 		// They may really want a specific key on the keyboard: we assume
 		// that any single character keyname is a character.
-		if (key.size() > 1) {
+		if (key == CLEARED_TEXT)
+		{
+			type_ = hotkey_item::CLEARED;
+		}
+		else if (key.size() > 1) {
 			type_ = BY_KEYCODE;
 
 			keycode_ = sdl_keysym_from_name(key);
 			if (keycode_ == SDLK_UNKNOWN) {
 				if (tolower(key[0]) != 'f') {
-					LOG_STREAM(err, config)
-						<< "hotkey key '" << key << "' invalid\n";
+					ERR_CONFIG << "hotkey key '" << key << "' invalid\n";
 				} else {
 					int num = lexical_cast_default<int>
 						(std::string(key.begin()+1,	key.end()), 1);
@@ -202,7 +239,7 @@ std::string hotkey_item::get_name() const
 			str << "cmd+";
 		if (ctrl_)
 			str << "ctrl+";
-		str << (char)character_;
+		str << static_cast<char>(character_);
 	} else if (type_ == BY_KEYCODE) {
 		if (alt_)
 			str << "alt+";
@@ -221,12 +258,16 @@ void hotkey_item::set_description(const std::string& description)
 {
 	description_ = description;
 }
+void hotkey_item::clear_hotkey()
+{
+	type_ = CLEARED;
+}
 
 void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, bool alt, bool cmd)
 {
 	const std::string keyname = SDL_GetKeyName(SDLKey(keycode));
 
-	INFO_C << "setting hotkey: char=" << lexical_cast<std::string>(character)
+	LOG_G << "setting hotkey: char=" << lexical_cast<std::string>(character)
 		   << " keycode="  << lexical_cast<std::string>(keycode) << " "
 		   << (shift ? "shift," : "")
 		   << (ctrl ? "ctrl," : "")
@@ -240,9 +281,13 @@ void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, boo
 			character += 64;
 		else
 			character += 96;
-		INFO_C << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
+		LOG_G << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
 	}
 
+	// For some reason on Mac OS, if cmd and shift are down, the character doesn't get upper-cased
+	if (cmd && character > 96 && character < 123 && shift)
+		character -= 32;
+
 	// We handle simple cases by character, others by the actual key.
 	if (isprint(character) && !isspace(character)) {
 		type_ = BY_CHARACTER;
@@ -250,7 +295,7 @@ void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, boo
 		ctrl_ = ctrl;
 		alt_ = alt;
 		cmd_ = cmd;
-		INFO_C << "type = BY_CHARACTER\n";
+		LOG_G << "type = BY_CHARACTER\n";
 	} else {
 		type_ = BY_KEYCODE;
 		keycode_ = keycode;
@@ -258,7 +303,7 @@ void hotkey_item::set_key(int character, int keycode, bool shift, bool ctrl, boo
 		ctrl_ = ctrl;
 		alt_ = alt;
 		cmd_ = cmd;
-		INFO_C << "type = BY_KEYCODE\n";
+		LOG_G << "type = BY_KEYCODE\n";
 	}
 }
 
@@ -305,15 +350,22 @@ void save_hotkeys(config& cfg)
 	for(std::vector<hotkey_item>::iterator i = hotkeys_.begin(); i != hotkeys_.end(); ++i) {
 		if (i->hidden() || i->get_type() == hotkey_item::UNBOUND)
 			continue;
-
 		config& item = cfg.add_child("hotkey");
+
 		item["command"] = i->get_command();
 
+		if (i->get_type() == hotkey_item::CLEARED)
+		{
+			item["key"] = CLEARED_TEXT;
+			continue;
+		}
+
+
 		if (i->get_type() == hotkey_item::BY_KEYCODE) {
 			item["key"] = SDL_GetKeyName(SDLKey(i->get_keycode()));
 			item["shift"] = i->get_shift() ? "yes" : "no";
 		} else if (i->get_type() == hotkey_item::BY_CHARACTER) {
-			item["key"] = std::string(1, (char)i->get_character());
+			item["key"] = std::string(1, static_cast<char>(i->get_character()));
 		}
 		item["alt"] = i->get_alt() ? "yes" : "no";
 		item["ctrl"] = i->get_ctrl() ? "yes" : "no";
@@ -355,7 +407,7 @@ hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool
 {
 	std::vector<hotkey_item>::iterator itor;
 
-	INFO_C << "getting hotkey: char=" << lexical_cast<std::string>(character)
+	DBG_G << "getting hotkey: char=" << lexical_cast<std::string>(character)
 		   << " keycode="  << lexical_cast<std::string>(keycode) << " "
 		   << (shift ? "shift," : "")
 		   << (ctrl ? "ctrl," : "")
@@ -369,19 +421,23 @@ hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool
 			character += 64;
 		else
 			character += 96;
-		INFO_C << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
+		DBG_G << "Mapped to character " << lexical_cast<std::string>(character) << "\n";
 	}
 
+	// For some reason on Mac OS, if cmd and shift are down, the character doesn't get upper-cased
+	if (cmd && character > 96 && character < 123 && shift)
+		character -= 32;
+
 	for (itor = hotkeys_.begin(); itor != hotkeys_.end(); ++itor) {
 		if (itor->get_type() == hotkey_item::BY_CHARACTER) {
 			if (character == itor->get_character()) {
 				if (ctrl == itor->get_ctrl()
 					&& alt == itor->get_alt()
 					&& cmd == itor->get_cmd()) {
-					INFO_C << "Could match by character..." << "yes\n";
+					DBG_G << "Could match by character..." << "yes\n";
 					break;
 				}
-				INFO_C << "Could match by character..." << "but modifiers different\n";
+				DBG_G << "Could match by character..." << "but modifiers different\n";
 			}
 		} else if (itor->get_type() == hotkey_item::BY_KEYCODE) {
 			if (keycode == itor->get_keycode()) {
@@ -389,10 +445,10 @@ hotkey_item& get_hotkey(int character, int keycode, bool shift, bool ctrl, bool
 					&& ctrl == itor->get_ctrl()
 					&& alt == itor->get_alt()
 					&& cmd == itor->get_cmd()) {
-					INFO_C << "Could match by keycode..." << "yes\n";
+					DBG_G << "Could match by keycode..." << "yes\n";
 					break;
 				}
-				INFO_C << "Could match by keycode..." << "but modifiers different\n";
+				DBG_G << "Could match by keycode..." << "but modifiers different\n";
 			}
 		}
 	}
@@ -415,12 +471,9 @@ hotkey_item& get_hotkey(const SDL_KeyboardEvent& event)
 #endif
 			);
 }
-
-hotkey_item& get_visible_hotkey(int index)
+void _get_visible_hotkey_itor(int index, std::vector<hotkey_item>::iterator& itor)
 {
 	int counter = 0;
-
-	std::vector<hotkey_item>::iterator itor;
 	for (itor = hotkeys_.begin(); itor != hotkeys_.end(); ++itor) {
 		if (itor->hidden())
 			continue;
@@ -430,7 +483,13 @@ hotkey_item& get_visible_hotkey(int index)
 
 		counter++;
 	}
+}
 
+hotkey_item& get_visible_hotkey(int index)
+{
+
+	std::vector<hotkey_item>::iterator itor;
+	_get_visible_hotkey_itor(index, itor);
 	if (itor == hotkeys_.end())
 		return null_hotkey_;
 
@@ -462,8 +521,8 @@ void basic_handler::handle_event(const SDL_Event& event)
 void key_event(display& disp, const SDL_KeyboardEvent& event, command_executor* executor)
 {
 	if(event.keysym.sym == SDLK_ESCAPE && disp.in_game()) {
-		std::cerr << "escape pressed..showing quit\n";
-		const int res = gui::show_dialog(disp,NULL,_("Quit"),_("Do you really want to quit?"),gui::YES_NO);
+		ERR_G << "escape pressed..showing quit\n";
+		const int res = gui::dialog(disp,_("Quit"),_("Do you really want to quit?"),gui::YES_NO).show();
 		if(res == 0) {
 			throw end_level_exception(QUIT);
 		} else {
@@ -492,26 +551,267 @@ void key_event_execute(display& disp, const SDL_KeyboardEvent& event, command_ex
 	execute_command(disp,hk->get_id(),executor);
 }
 
-void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* executor)
+bool command_executor::execute_command(HOTKEY_COMMAND command, int /*index*/)
+{
+	switch(command) {
+		case HOTKEY_CYCLE_UNITS:
+			cycle_units();
+			break;
+		case HOTKEY_CYCLE_BACK_UNITS:
+			cycle_back_units();
+			break;
+		case HOTKEY_ENDTURN:
+			end_turn();
+			break;
+		case HOTKEY_UNIT_HOLD_POSITION:
+			unit_hold_position();
+			break;
+		case HOTKEY_END_UNIT_TURN:
+			end_unit_turn();
+			break;
+		case HOTKEY_LEADER:
+			goto_leader();
+			break;
+		case HOTKEY_UNDO:
+			undo();
+			break;
+		case HOTKEY_REDO:
+			redo();
+			break;
+		case HOTKEY_UNIT_DESCRIPTION:
+			unit_description();
+			break;
+		case HOTKEY_RENAME_UNIT:
+			rename_unit();
+			break;
+		case HOTKEY_SAVE_GAME:
+			save_game();
+			break;
+		case HOTKEY_SAVE_REPLAY:
+			save_replay();
+			break;
+		case HOTKEY_SAVE_MAP:
+			save_map();
+			break;
+		case HOTKEY_LOAD_GAME:
+			load_game();
+			break;
+		case HOTKEY_TOGGLE_GRID:
+			toggle_grid();
+			break;
+		case HOTKEY_STATUS_TABLE:
+			status_table();
+			break;
+		case HOTKEY_RECALL:
+			recall();
+			break;
+		case HOTKEY_RECRUIT:
+			recruit();
+			break;
+		case hotkey::HOTKEY_REPEAT_RECRUIT:
+			repeat_recruit();
+			break;
+		case HOTKEY_SPEAK:
+			speak();
+			break;
+		case HOTKEY_SPEAK_ALLY:
+			whisper();
+			break;
+		case HOTKEY_SPEAK_ALL:
+			shout();
+			break;
+		case HOTKEY_CREATE_UNIT:
+			create_unit();
+			break;
+		case HOTKEY_CHANGE_UNIT_SIDE:
+			change_unit_side();
+			break;
+		case HOTKEY_PREFERENCES:
+			preferences();
+			break;
+		case HOTKEY_OBJECTIVES:
+			objectives();
+			break;
+		case HOTKEY_UNIT_LIST:
+			unit_list();
+			break;
+		case HOTKEY_STATISTICS:
+			show_statistics();
+			break;
+		case HOTKEY_LABEL_TEAM_TERRAIN:
+			label_terrain(true);
+			break;
+		case HOTKEY_LABEL_TERRAIN:
+			label_terrain(false);
+			break;
+		case HOTKEY_CLEAR_LABELS:
+			clear_labels();
+			break;
+		case HOTKEY_SHOW_ENEMY_MOVES:
+			show_enemy_moves(false);
+			break;
+		case HOTKEY_BEST_ENEMY_MOVES:
+			show_enemy_moves(true);
+			break;
+		case HOTKEY_DELAY_SHROUD:
+			toggle_shroud_updates();
+			break;
+		case HOTKEY_UPDATE_SHROUD:
+			update_shroud_now();
+			break;
+		case HOTKEY_CONTINUE_MOVE:
+			continue_move();
+			break;
+		case HOTKEY_SEARCH:
+			search();
+			break;
+		case HOTKEY_HELP:
+			show_help();
+			break;
+		case HOTKEY_CHAT_LOG:
+			show_chat_log();
+			break;
+		case HOTKEY_USER_CMD:
+			user_command();
+			break;
+//%%
+		case HOTKEY_CLEAR_MSG:
+			clear_messages();
+			break;
+#ifdef USRCMD2
+		case HOTKEY_USER_CMD_2:
+			//user_command();
+			user_command_2();
+			break;
+		case HOTKEY_USER_CMD_3:
+			user_command_3();
+			break;
+#endif
+
+		case HOTKEY_EDIT_SET_TERRAIN:
+			edit_set_terrain();
+			break;
+		case HOTKEY_EDIT_QUIT:
+			edit_quit();
+			break;
+		 case HOTKEY_EDIT_NEW_MAP:
+			edit_new_map();
+			break;
+		 case HOTKEY_EDIT_LOAD_MAP:
+			edit_load_map();
+			break;
+		 case HOTKEY_EDIT_SAVE_MAP:
+			edit_save_map();
+			break;
+		 case HOTKEY_EDIT_SAVE_AS:
+			edit_save_as();
+			break;
+		 case HOTKEY_EDIT_SET_START_POS:
+			edit_set_start_pos();
+			break;
+		 case HOTKEY_EDIT_FLOOD_FILL:
+			edit_flood_fill();
+			break;
+		 case HOTKEY_EDIT_FILL_SELECTION:
+			edit_fill_selection();
+			break;
+		 case HOTKEY_EDIT_ROTATE_SELECTION:
+			edit_rotate_selection();
+			break;
+		 case HOTKEY_EDIT_CUT:
+			edit_cut();
+			break;
+		 case HOTKEY_EDIT_PASTE:
+			edit_paste();
+			break;
+		 case HOTKEY_EDIT_COPY:
+			edit_copy();
+			break;
+		 case HOTKEY_EDIT_REVERT:
+			edit_revert();
+			break;
+		 case HOTKEY_EDIT_RESIZE:
+			edit_resize();
+			break;
+		 case HOTKEY_EDIT_FLIP:
+			edit_flip();
+			break;
+		 case HOTKEY_EDIT_SELECT_ALL:
+			edit_select_all();
+			break;
+		 case HOTKEY_EDIT_DRAW:
+			edit_draw();
+			break;
+		 case HOTKEY_EDIT_REFRESH:
+			edit_refresh();
+			break;
+		 case HOTKEY_EDIT_UPDATE:
+			edit_update();
+			break;
+		 case HOTKEY_EDIT_AUTO_UPDATE:
+			edit_auto_update();
+			break;
+
+		 case HOTKEY_LANGUAGE:
+			change_language();
+			break;
+		 case HOTKEY_PLAY_REPLAY:
+			play_replay();
+			 break;
+		 case HOTKEY_RESET_REPLAY:
+			reset_replay();
+			 break;
+		 case HOTKEY_STOP_REPLAY:
+			 stop_replay();
+			 break;
+		 case HOTKEY_REPLAY_NEXT_TURN:
+			replay_next_turn();
+			 break;
+		 case HOTKEY_REPLAY_NEXT_SIDE:
+			replay_next_side();
+			 break;
+		 case HOTKEY_REPLAY_SHOW_EVERYTHING:
+			replay_show_everything();
+			 break;
+		 case HOTKEY_REPLAY_SHOW_EACH:
+			replay_show_each();
+			 break;
+		 case HOTKEY_REPLAY_SHOW_TEAM1:
+			replay_show_team1();
+			 break;
+		 case HOTKEY_REPLAY_SKIP_ANIMATION:
+			replay_skip_animation();
+			 break;
+		 default:
+			 return false;
+	}
+	return true;
+}
+
+void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* executor, int index)
 {
 	const int zoom_amount = 4;
 
-	if(executor != NULL && executor->can_execute_command(command) == false)
+	if(executor != NULL) {
+		if(!executor->can_execute_command(command, index) || executor->execute_command(command, index))
 		return;
-
+	}
 	switch(command) {
 		case HOTKEY_ZOOM_IN:
-			disp.zoom(zoom_amount);
+			disp.set_zoom(zoom_amount);
 			break;
 		case HOTKEY_ZOOM_OUT:
-			disp.zoom(-zoom_amount);
+			disp.set_zoom(-zoom_amount);
 			break;
 		case HOTKEY_ZOOM_DEFAULT:
-			disp.default_zoom();
+			disp.set_default_zoom();
 			break;
 		case HOTKEY_FULLSCREEN:
 			preferences::set_fullscreen(!preferences::fullscreen());
 			break;
+		case HOTKEY_MOUSE_SCROLL:
+			preferences::enable_mouse_scroll(!preferences::mouse_scroll_enabled());
+			break;
 		case HOTKEY_SCREENSHOT:
 			disp.screenshot();
 			break;
@@ -536,159 +836,16 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 				}
 				else
 				{
-					//then set setings before mute
+					//then set settings before mute
 					preferences::set_sound(before_muted.playing_sound);
 					preferences::set_music(before_muted.playing_music);
 				}
 			}
 			break;
-		case HOTKEY_CYCLE_UNITS:
-			if(executor)
-				executor->cycle_units();
-			break;
-		case HOTKEY_CYCLE_BACK_UNITS:
-			if(executor)
-				executor->cycle_back_units();
-			break;
-		case HOTKEY_ENDTURN:
-			if(executor)
-				executor->end_turn();
-			break;
-		case HOTKEY_UNIT_HOLD_POSITION:
-			if(executor)
-                executor->unit_hold_position();
-			break;
-		case HOTKEY_END_UNIT_TURN:
-			if(executor)
-				executor->end_unit_turn();
-			break;
-		case HOTKEY_LEADER:
-			if(executor)
-				executor->goto_leader();
-			break;
-		case HOTKEY_UNDO:
-			if(executor)
-				executor->undo();
-			break;
-		case HOTKEY_REDO:
-			if(executor)
-				executor->redo();
-			break;
-		case HOTKEY_UNIT_DESCRIPTION:
-			if(executor)
-				executor->unit_description();
-			break;
-		case HOTKEY_RENAME_UNIT:
-			if(executor)
-				executor->rename_unit();
-			break;
-		case HOTKEY_SAVE_GAME:
-			if(executor)
-				executor->save_game();
-			break;
-		case HOTKEY_LOAD_GAME:
-			if(executor)
-				executor->load_game();
-			break;
-		case HOTKEY_TOGGLE_GRID:
-			if(executor)
-				executor->toggle_grid();
-			break;
-		case HOTKEY_MOUSE_SCROLL:
-			preferences::enable_mouse_scroll(!preferences::mouse_scroll_enabled());
-			break;
-		case HOTKEY_STATUS_TABLE:
-			if(executor)
-				executor->status_table();
-			break;
-		case HOTKEY_RECALL:
-			if(executor)
-				executor->recall();
-			break;
-		case HOTKEY_RECRUIT:
-			if(executor)
-				executor->recruit();
-			break;
-		case hotkey::HOTKEY_REPEAT_RECRUIT:
-			if(executor)
-				executor->repeat_recruit();
-			break;
-		case HOTKEY_SPEAK:
-			if(executor)
-				executor->speak();
-			break;
-		case HOTKEY_SPEAK_ALLY:
-			if(executor) {
-				preferences::set_message_private(true);
-				executor->speak();
-			}
-			break;
-		case HOTKEY_SPEAK_ALL:
-			if(executor) {
-				preferences::set_message_private(false);
-				executor->speak();
-			}
-			break;
-		case HOTKEY_CREATE_UNIT:
-			if(executor)
-				executor->create_unit();
-			break;
-		case HOTKEY_CHANGE_UNIT_SIDE:
-			if(executor)
-				executor->change_unit_side();
-			break;
-		case HOTKEY_PREFERENCES:
-			if(executor)
-				executor->preferences();
-			break;
-		case HOTKEY_OBJECTIVES:
-			if(executor)
-				executor->objectives();
-			break;
-		case HOTKEY_UNIT_LIST:
-			if(executor)
-				executor->unit_list();
-			break;
-		case HOTKEY_STATISTICS:
-			if(executor)
-				executor->show_statistics();
-			break;
-		case HOTKEY_LABEL_TERRAIN:
-			if(executor)
-				executor->label_terrain();
-			break;
-		case HOTKEY_CLEAR_LABELS:
-			if(executor)
-				executor->clear_labels();
-			break;
-		case HOTKEY_SHOW_ENEMY_MOVES:
-			if(executor)
-				executor->show_enemy_moves(false);
-			break;
-		case HOTKEY_BEST_ENEMY_MOVES:
-			if(executor)
-				executor->show_enemy_moves(true);
-			break;
-		case HOTKEY_DELAY_SHROUD:
-			if(executor)
-				executor->toggle_shroud_updates();
-			break;
-		case HOTKEY_UPDATE_SHROUD:
-			if(executor)
-				executor->update_shroud_now();
-			break;
-		case HOTKEY_CONTINUE_MOVE:
-			if(executor)
-				executor->continue_move();
-			break;
-		case HOTKEY_SEARCH:
-			if(executor)
-				executor->search();
-			break;
 		case HOTKEY_QUIT_GAME: {
 			if(disp.in_game()) {
-				std::cerr << "is in game -- showing quit message\n";
-				const int res = gui::show_dialog(disp,NULL,_("Quit"),_("Do you really want to quit?"),gui::YES_NO);
+				DBG_G << "is in game -- showing quit message\n";
+				const int res = gui::dialog(disp,_("Quit"),_("Do you really want to quit?"),gui::YES_NO).show();
 				if(res == 0) {
 					throw end_level_exception(QUIT);
 				}
@@ -696,169 +853,44 @@ void execute_command(display& disp, HOTKEY_COMMAND command, command_executor* ex
 
 			break;
 		}
-		case HOTKEY_HELP:
-			if(executor) {
-				executor->show_help();
-			}
-			break;
-		case HOTKEY_CHAT_LOG:
-			if(executor) {
-				executor->show_chat_log();
-			}
-			break;
-		case HOTKEY_USER_CMD:
-			if(executor) {
-				executor->user_command();
-			}
-			break;
-		case HOTKEY_EDIT_SET_TERRAIN:
-			if(executor)
-				executor->edit_set_terrain();
-			break;
-		case HOTKEY_EDIT_QUIT:
-			if(executor)
-				executor->edit_quit();
-			break;
-		 case HOTKEY_EDIT_NEW_MAP:
-			if(executor)
-				executor->edit_new_map();
-			break;
-		 case HOTKEY_EDIT_LOAD_MAP:
-			if(executor)
-				executor->edit_load_map();
-			break;
-		 case HOTKEY_EDIT_SAVE_MAP:
-			if(executor)
-				executor->edit_save_map();
-			break;
-		 case HOTKEY_EDIT_SAVE_AS:
-			if(executor)
-				executor->edit_save_as();
-			break;
-		 case HOTKEY_EDIT_SET_START_POS:
-			if(executor)
-				executor->edit_set_start_pos();
-			break;
-		 case HOTKEY_EDIT_FLOOD_FILL:
-			if(executor)
-				executor->edit_flood_fill();
-			break;
-
-		 case HOTKEY_EDIT_FILL_SELECTION:
-			if(executor)
-				executor->edit_fill_selection();
-			break;
-		 case HOTKEY_EDIT_CUT:
-			if(executor)
-				executor->edit_cut();
-			break;
-		 case HOTKEY_EDIT_PASTE:
-			if(executor)
-				executor->edit_paste();
-			break;
-		 case HOTKEY_EDIT_COPY:
-			if(executor)
-				executor->edit_copy();
-			break;
-		 case HOTKEY_EDIT_REVERT:
-			if(executor)
-				executor->edit_revert();
-			break;
-		 case HOTKEY_EDIT_RESIZE:
-			if(executor)
-				executor->edit_resize();
-			break;
-		 case HOTKEY_EDIT_FLIP:
-			if(executor)
-				executor->edit_flip();
-			break;
-		 case HOTKEY_EDIT_SELECT_ALL:
-			if(executor)
-				executor->edit_select_all();
-			break;
-		 case HOTKEY_EDIT_DRAW:
-			if(executor)
-				executor->edit_draw();
-			break;
-		 case HOTKEY_EDIT_REFRESH:
-			if(executor)
-				executor->edit_refresh();
-			break;
-		 case HOTKEY_LANGUAGE:
-			if(executor)
-				executor->change_language();
-			break;
-		 case HOTKEY_PLAY_REPLAY:
-			 if (executor)
-				 executor->play_replay();
-			 break;
-		 case HOTKEY_RESET_REPLAY:
-			 if (executor)
-				 executor->reset_replay();
-			 break;
-		 case HOTKEY_STOP_REPLAY:
-			 if (executor)
-				executor->stop_replay();
-			 break;
-		 case HOTKEY_REPLAY_NEXT_TURN:
-			 if (executor)
-				 executor->replay_next_turn();
-			 break;
-		 case HOTKEY_REPLAY_NEXT_SIDE:
-			 if (executor)
-				 executor->replay_next_side();
-			 break;
-		 case HOTKEY_REPLAY_SHROUD:
-			 if (executor)
-				 executor->replay_switch_shroud();
-			 break;
-		 case HOTKEY_REPLAY_FOG:
-			 if (executor)
-				 executor->replay_switch_fog();
-			 break;
-		 case HOTKEY_REPLAY_SKIP_ANIMATION:
-			 if (executor)
-				 executor->replay_skip_animation();
-			 break;
 		default:
-			std::cerr << "command_executor: unknown command number " << command << ", ignoring.\n";
+			DBG_G << "command_executor: unknown command number " << command << ", ignoring.\n";
 			break;
 	}
 }
 
-void command_executor::show_menu(const std::vector<std::string>& items_arg, int xloc, int yloc, bool /*context_menu*/, display& gui, const std::vector<std::string>& savenames)
+void command_executor::show_menu(const std::vector<std::string>& items_arg, int xloc, int yloc, bool context_menu, display& gui)
 {
 	std::vector<std::string> items = items_arg;
-	if (can_execute_command(hotkey::get_hotkey(items.front()).get_id())){
+	if (can_execute_command(hotkey::get_hotkey(items.front()).get_id(), 0)){
 		//if just one item is passed in, that means we should execute that item
-		if(items.size() == 1 && items_arg.size() == 1) {
+		if(!context_menu && items.size() == 1 && items_arg.size() == 1) {
 			hotkey::execute_command(gui,hotkey::get_hotkey(items.front()).get_id(),this);
 			return;
 		}
 
 		std::vector<std::string> menu = get_menu_images(items);
 
-		static const std::string style = "menu2";
-		const int res = gui::show_dialog(gui,NULL,"","",
-				gui::MESSAGE,&menu,NULL,"",NULL,-1,NULL,NULL,xloc,yloc,&style);
-		if (size_t(res) >= items.size())
+		int res = 0;
+		{
+			gui::dialog mmenu = gui::dialog(gui,"","",
+			gui::MESSAGE, gui::dialog::hotkeys_style);
+			mmenu.set_menu(menu);
+			res = mmenu.show(xloc, yloc);
+		} // this will kill the dialog
+		if (res < 0 || size_t(res) >= items.size())
 			return;
 
 		const hotkey::HOTKEY_COMMAND cmd = hotkey::get_hotkey(items[res]).get_id();
-		// Fake entries: they want us to load a specific autosave.
-		if (cmd == hotkey::HOTKEY_NULL && (unsigned)res < savenames.size() && !savenames[(unsigned)res].empty()) {
-			throw game::load_game_exception(savenames[(unsigned)res],false);
-		}
-
-		hotkey::execute_command(gui,cmd,this);
+		hotkey::execute_command(gui,cmd,this,res);
 	}
 }
 
-std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command) const {
+std::string command_executor::get_menu_image(hotkey::HOTKEY_COMMAND command, int index) const {
 	switch(get_action_state(command)) {
 		case ACTION_ON: return game_config::checked_menu_image;
 		case ACTION_OFF: return game_config::unchecked_menu_image;
-		default: return get_action_image(command);
+		default: return get_action_image(command, index);
 	}
 }
 
@@ -866,19 +898,20 @@ std::vector<std::string> command_executor::get_menu_images(const std::vector<std
 	std::vector<std::string> result;
 	bool has_image = false;
 
-	for(std::vector<std::string>::const_iterator i = items.begin(); i != items.end(); ++i) {
-		const hotkey::hotkey_item hk = hotkey::get_hotkey(*i);
+	for(size_t i = 0; i < items.size(); ++i) {
+		std::string const& item = items[i];
+		const hotkey::hotkey_item hk = hotkey::get_hotkey(item);
 
 		std::stringstream str;
 		//see if this menu item has an associated image
-		std::string img(get_menu_image(hk.get_id()));
+		std::string img(get_menu_image(hk.get_id(), i));
 		if(img.empty() == false) {
 			has_image = true;
 			str << IMAGE_PREFIX << img << COLUMN_SEPARATOR;
 		}
 
 		if (hk.get_id() == hotkey::HOTKEY_NULL) {
-			str << *i << COLUMN_SEPARATOR;
+			str << item.substr(0, item.find_last_not_of(' ') + 1) << COLUMN_SEPARATOR;
 		} else {
 			str << hk.get_description() << COLUMN_SEPARATOR << hk.get_name();
 		}
