@@ -1,10 +1,11 @@
-/* $Id: image.hpp 12655 2006-07-19 22:12:08Z xan $ */
+/* $Id: image.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -26,11 +27,18 @@
 //
 ///images come in a number of varieties:
 /// - unscaled: no modifications have been done on the image.
-/// - scaled: images are scaled to the size of a tile
+/// - scaled_to_hex: images are scaled to the size of a tile
+/// - scaled_to_zoom: images are scaled to the zoom factor, factor 1 is same as unscaled no other modifications are done
 /// - unmasked: images are scaled, but have no time of day masking applied to them
-/// - greyed: images are scaled and in greyscale
 /// - brightened: images are scaled and brighter than normal.
 namespace image {
+#ifdef USE_TINY_GUI
+	// images in tiny-gui will be scaled at install time
+	const int tile_size = 36;
+#else
+	const int tile_size = 72;
+#endif
+
 	template<typename T>
 	struct cache_item {
 		cache_item() : loaded(false), item() {}
@@ -47,7 +55,6 @@ namespace image {
 		// Called by each constructor after actual construction to
 		// initialize the index_ field
 		void init_index();
-		void get_tc_info(const std::string& field);
 		void parse_arguments();
 	public:
 		enum type { NONE, FILE, SUB_FILE };
@@ -56,10 +63,10 @@ namespace image {
 			value();
 			value(const value &a);
 			value(const char *filename);
-			value(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb);
+			value(const char *filename, const std::string& modifications);
 			value(const std::string& filename);
-		        value(const std::string& filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb);
-			value(const std::string& filename, const gamemap::location& loc, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb);
+		     value(const std::string& filename, const std::string& modifications);
+			value(const std::string& filename, const gamemap::location& loc, const std::string& modifications);
 
 			bool operator==(const value& a) const;
 			bool operator<(const value& a) const;
@@ -67,8 +74,7 @@ namespace image {
 			type type_;
 			std::string filename_;
 			gamemap::location loc_;
-		        color_range new_color;
-		std::vector<Uint32> swap_colors;
+			std::string modifications_;
 		};
 
 		// Constructing locators is somewhat slow, accessing image
@@ -76,12 +82,12 @@ namespace image {
 		// should store locators, and not strings to construct locators
 		// (the second will work, of course, but will be slower)
 	        locator();
-		locator(const locator &a);
+		locator(const locator &a, const std::string &mods ="");
 		locator(const char *filename);
-		locator(const char *filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb);
+		locator(const char *filename, const std::string& modifications);
 		locator(const std::string& filename);
-		locator(const std::string& filename, const color_range& new_rgb, const std::vector<Uint32>& swap_rgb);
-		locator(const std::string& filename, const gamemap::location& loc, const color_range& new_rgb = std::vector<Uint32>(), const std::vector<Uint32>& swap_rgb = std::vector<Uint32>());
+		locator(const std::string& filename, const std::string& modifications);
+		locator(const std::string& filename, const gamemap::location& loc, const std::string& modifications="");
 
 		locator& operator=(const locator &a);
 		bool operator==(const locator &a) const { return index_ == a.index_; }
@@ -90,8 +96,7 @@ namespace image {
 
 		const std::string &get_filename() const { return val_.filename_; }
 		const gamemap::location& get_loc() const { return val_.loc_ ; }
-		const color_range& get_new_color() const { return val_.new_color ; }
-		const std::vector<Uint32>& get_swap_colors() const { return val_.swap_colors ; }
+		const std::string& get_modifications() const {return val_.modifications_;}
 		const type get_type() const { return val_.type_; };
 		// const int get_index() const { return index_; };
 
@@ -114,12 +119,18 @@ namespace image {
 		template<typename T>
 		void add_to_cache(std::vector<cache_item<T> >& cache, const T &image) const;
 #endif
-		bool in_cache(const std::vector<cache_item<surface> >& cache) const;
-		surface locate_in_cache(const std::vector<cache_item<surface> >& cache) const;
-		void add_to_cache(std::vector<cache_item<surface> >& cache, const surface &image) const;
-		bool in_cache(const std::vector<cache_item<locator> >& cache) const;
-		locator locate_in_cache(const std::vector<cache_item<locator> >& cache) const;
-		void add_to_cache(std::vector<cache_item<locator> >& cache, const locator &image) const;
+		bool in_cache(const std::vector<cache_item<surface> >& cache) const
+			{ return index_ == -1 ? false : cache[index_].loaded; }
+		surface locate_in_cache(const std::vector<cache_item<surface> >& cache) const
+			{ return index_ == -1 ? surface() : cache[index_].item; }
+		void add_to_cache(std::vector<cache_item<surface> >& cache, const surface &image) const
+			{ if(index_ != -1 ) cache[index_] = cache_item<surface>(image); }
+		bool in_cache(const std::vector<cache_item<locator> >& cache) const
+			{ return index_ == -1 ? false : cache[index_].loaded; }
+		locator locate_in_cache(const std::vector<cache_item<locator> >& cache) const
+			{ return index_ == -1 ? locator() : cache[index_].item; }
+		void add_to_cache(std::vector<cache_item<locator> >& cache, const locator &image) const
+			{ if(index_ != -1) cache[index_] = cache_item<locator>(image); }
 	protected:
 		static int last_index_;
 	private:
@@ -134,8 +145,9 @@ namespace image {
 
 	typedef std::vector<cache_item<surface> > image_cache;
 	typedef std::vector<cache_item<locator> > locator_cache;
-	typedef std::map<gamemap::TERRAIN, surface> mini_terrain_cache_map;
+	typedef std::map<t_translation::t_terrain, surface> mini_terrain_cache_map;
 	extern mini_terrain_cache_map mini_terrain_cache;
+	extern mini_terrain_cache_map mini_fogged_terrain_cache;
 
 	void flush_cache();
 
@@ -158,8 +170,10 @@ namespace image {
 	///for representing day/night. Invalidates all scaled images.
 	void set_colour_adjustment(int r, int g, int b);
 
-	///function to get back the current colour adjustment values
-	void get_colour_adjustment(int *r, int *g, int *b);
+	///set the team colors used by the TC image modification
+	///use a vector with one string for each team
+	///using NULL will reset to default TC
+	void set_team_colors(const std::vector<std::string>* colors = NULL);
 
 	///function which sets a certain image as a 'mask' for all scaled images.
 	///the 'mask' is blitted onto all scaled images.
@@ -175,36 +189,23 @@ namespace image {
 	///scaled images.
 	void set_zoom(int zoom);
 
-	enum TYPE { UNSCALED, SCALED, UNMASKED, GREYED, DARKENED, BRIGHTENED, SEMI_BRIGHTENED };
+	// unscaled : image will be drawn "as is" without changing size, even in case of redraw
+	// hexed : the hex mask is applied on the image
+	// unmasked : image will be scaled to fit into a hex, taking zoom into account
+	// scaled_to_hex : same but ToD coloring is also applied
+	enum TYPE { UNSCALED, HEXED, UNMASKED, SCALED_TO_HEX, SCALED_TO_ZOOM, BRIGHTENED, SEMI_BRIGHTENED };
 
-	enum COLOUR_ADJUSTMENT { ADJUST_COLOUR, NO_ADJUST_COLOUR };
 
 	///function to get the surface corresponding to an image.
 	///note that this surface must be freed by the user by calling
 	///SDL_FreeSurface()
-	surface get_image(const locator& i_locator,TYPE type=SCALED, COLOUR_ADJUSTMENT adj=ADJUST_COLOUR);
-
-	///function to get a scaled image, but scale it to specific dimensions.
-	///if you later try to get the same image using get_image() the image will
-	///have the dimensions specified here.
-	///Note that this surface must be freed by the user by calling SDL_FreeSurface
-	surface get_image_dim(const locator& i_locator, size_t x, size_t y);
+	surface get_image(const locator& i_locator, TYPE type=UNSCALED, bool add_to_cache = true);
 
 	///function to reverse an image. The image MUST have originally been returned from
 	///an image:: function. Returned images have the same semantics as for get_image()
 	///and must be freed using SDL_FreeSurface()
 	surface reverse_image(const surface &surf);
 
-
-	locator get_alternative(const locator &i_locator, const std::string &alt);
-
-	///function to register an image with the given id. Calls to get_image(id,UNSCALED) will
-	///return this image. register_image() will take ownership of this image and free
-	///it when the cache is cleared (change of video mode or colour adjustment).
-	///If there is already an image registered with this id, that image will be freed
-	///and replaced with this image.
-	void register_image(const locator& i_locator, const surface& surf);
-
 	//returns true if the given image actually exists, without loading it.
 	bool exists(const locator& i_locator);
 }
