@@ -1,11 +1,12 @@
-/* $Id: language.cpp 16783 2007-04-12 15:45:19Z elias $ */
+/* $Id: language.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /* vim:set encoding=utf-8: */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -19,16 +20,18 @@
 #include "game_config.hpp"
 #include "gettext.hpp"
 #include "language.hpp"
+#include "log.hpp"
 #include "preferences.hpp"
 #include "util.hpp"
 #include "serialization/parser.hpp"
 #include "serialization/preprocessor.hpp"
-#include "wassert.hpp"
 #include "wesconfig.h"
 
 #include <algorithm>
+#include <cassert>
 #include <cctype>
 #include <cerrno>
+#include <clocale>
 #include <cstdlib>
 #include <cstring>
 #include <iostream>
@@ -38,28 +41,18 @@ namespace {
 	string_map strings_;
 }
 
-std::vector<language_def> known_languages;
+static std::vector<language_def> known_languages;
 
 std::string languagedef_name (const language_def& def)
 {
 	return def.language;
 }
 
-bool languagedef_rtl (const language_def& def)
-{
-	return def.rtl;
-}
-
 bool current_language_rtl()
 {
 	return get_language().rtl;
 }
 
-bool languagedef_lessthan_p (const language_def& def1, const language_def& def2)
-{
-	return (def1.language < def2.language);
-}
-
 bool language_def::operator== (const language_def& a) const
 {
 	return ((language == a.language) /* && (localename == a.localename) */ );
@@ -92,7 +85,7 @@ bool load_language_list()
 {
 	config cfg;
 	try {
-		scoped_istream stream = preprocess_file("data/language.cfg");
+		scoped_istream stream = preprocess_file("data/hardwired/language.cfg");
 		read(cfg, *stream);
 	} catch(config::error &) {
 		return false;
@@ -127,12 +120,10 @@ static void wesnoth_setlocale(int category, std::string const &slocale,
 	// LANGUAGE overrides other settings, so for now just get rid of it
 	// FIXME: add configure check for unsetenv
 #ifndef _WIN32
-#ifndef SOLARIS
 #ifndef __AMIGAOS4__
 	unsetenv ("LANGUAGE"); // void so no return value to check
 #endif
 #endif
-#endif
 
 #ifdef __BEOS__
 	if(setenv ("LANG", locale, 1) == -1)
@@ -179,12 +170,17 @@ static void wesnoth_setlocale(int category, std::string const &slocale,
 	std::vector<std::string>::const_iterator i;
 	if (alternates) i = alternates->begin();
 	while (true) {
-		res = setlocale(category, try_loc);
+		res = std::setlocale(category, try_loc);
 		if (res) break;
-		std::string utf8 = std::string(try_loc) + std::string(".utf8");
-		try_loc = utf8.c_str();
-		res = setlocale(category, try_loc);
+
+		std::string utf8 = std::string(try_loc) + std::string(".utf-8");
+		res = std::setlocale(category, utf8.c_str());
 		if (res) break;
+
+		utf8 = std::string(try_loc) + std::string(".UTF-8");
+		res = std::setlocale(category, utf8.c_str());
+		if (res) break;
+
 		if (!alternates) break;
 		if (i == alternates->end()) break;
 		try_loc = i->c_str();
@@ -192,10 +188,10 @@ static void wesnoth_setlocale(int category, std::string const &slocale,
 	}
 
 	if (res == NULL)
-		std::cerr << "WARNING: setlocale() failed for "
-			  << locale << ".\n";
+		std::cerr << "WARNING: setlocale() failed for '"
+			  << locale << "'.\n";
 	else
-		std::cerr << "set locale to " << try_loc << "\n";
+		std::cerr << "set locale to '" << try_loc << "'\n";
 }
 
 bool set_language(const language_def& locale)
@@ -209,13 +205,12 @@ bool set_language(const language_def& locale)
 	config cfg;
 
 	current_language = locale;
-
 	wesnoth_setlocale(LC_MESSAGES, locale.localename, &locale.alternates);
 	wesnoth_setlocale(LC_COLLATE, locale.localename, &locale.alternates);
 
 	// fill string_table (should be moved somwhere else some day)
 	try {
-		scoped_istream stream = preprocess_file("data/english.cfg");
+		scoped_istream stream = preprocess_file("data/hardwired/english.cfg");
 		read(cfg, *stream);
 	} catch(config::error& e) {
 		std::cerr << "Could not read english.cfg\n";
@@ -249,7 +244,7 @@ const language_def& get_locale()
 {
 	//TODO: Add in support for querying the locale on Windows
 
-	wassert(known_languages.size() != 0);
+	assert(known_languages.size() != 0);
 
 	const std::string& prefs_locale = preferences::language();
 	if(prefs_locale.empty() == false) {
@@ -259,7 +254,7 @@ const language_def& get_locale()
 			if (prefs_locale == i->localename)
 				return *i;
 		}
-		std::cerr << "locale not found in known array; defaulting to system locale\n";
+		LOG_STREAM(info, general) << "'" << prefs_locale << "' locale not found in known array; defaulting to system locale\n";
 		return known_languages[0];
 	}
 
@@ -276,7 +271,7 @@ const language_def& get_locale()
 	}
 #endif
 
-	std::cerr << "locale could not be determined; defaulting to system locale\n";
+	LOG_STREAM(info, general) << "locale could not be determined; defaulting to system locale\n";
 	return known_languages[0];
 }
 
