@@ -1,10 +1,11 @@
-/* $Id: language.hpp 16783 2007-04-12 15:45:19Z elias $ */
+/* $Id: language.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -28,10 +29,19 @@ class config;
 
 struct language_def
 {
-	language_def() {}
+	language_def() :
+		localename(),
+		alternates(),
+		language(),
+		rtl(false)
+		{}
+
 	language_def(const std::string& name, const t_string& lang, const std::string& dir,
 	    const std::string &salternates = "") :
-		localename(name), language(lang), rtl(dir == "rtl")
+		localename(name), 
+		alternates(),
+		language(lang), 
+		rtl(dir == "rtl")
 	{
             alternates = utils::split(salternates);
         }
@@ -43,8 +53,6 @@ struct language_def
 };
 
 std::string languagedef_name (const language_def& def);
-bool languagedef_rtl (const language_def& def);
-bool languagedef_lessthan_p (const language_def& def1, const language_def& def2);
 
 struct symbol_table
 {
