@@ -1,10 +1,11 @@
-/* $Id: mapgen_dialog.cpp 11647 2006-05-26 01:03:08Z rusty $ */
+/* $Id: mapgen_dialog.cpp 24154 2008-02-29 18:17:30Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -20,6 +21,7 @@
 #include "display.hpp"
 #include "events.hpp"
 #include "gettext.hpp"
+#include "log.hpp"
 #include "mapgen.hpp"
 #include "marked-up_text.hpp"
 #include "show_dialog.hpp"
@@ -29,6 +31,8 @@
 #include "widgets/button.hpp"
 #include "widgets/slider.hpp"
 
+#define DBG_NG LOG_STREAM(debug, engine)
+
 namespace {
 	const size_t max_island = 10;
 	const size_t max_coastal = 5;
@@ -36,7 +40,7 @@ namespace {
 
 default_map_generator::default_map_generator(const config* cfg)
 : width_(40), height_(40), island_size_(0), iterations_(1000), hill_size_(10), max_lakes_(20),
-  nvillages_(25), nplayers_(2), link_castles_(true)
+  nvillages_(25), castle_size_(9), nplayers_(2), link_castles_(true)
 {
 	if(cfg != NULL) {
 		cfg_ = *cfg;
@@ -68,6 +72,10 @@ default_map_generator::default_map_generator(const config* cfg)
 		if(nvillages > 0)
 			nvillages_ = nvillages;
 
+		const int castle_size = ::atoi((*cfg)["castle_size"].c_str());
+		if(castle_size > 0)
+			castle_size_ = castle_size;
+
 		const int nplayers = ::atoi((*cfg)["players"].c_str());
 		if(nplayers > 0)
 			nplayers_ = nplayers;
@@ -82,7 +90,7 @@ bool default_map_generator::allow_user_config() const { return true; }
 
 void default_map_generator::user_config(display& disp)
 {
-	const events::resize_lock prevent_resizing;
+	const resize_lock prevent_resizing;
 	const events::event_context dialog_events_context;
 
 	CVideo& screen = disp.video();
@@ -92,12 +100,12 @@ void default_map_generator::user_config(display& disp)
 	const int xpos = screen.getx()/2 - width/2;
 	int ypos = screen.gety()/2 - height/2;
 
-	surface_restorer restorer;
-
 	gui::button close_button(screen,_("Close Window"));
 	std::vector<gui::button*> buttons(1,&close_button);
 
-	gui::draw_dialog(xpos,ypos,width,height,screen,_("Map Generator"),NULL,&buttons,&restorer);
+	gui::dialog_frame f(screen,_("Map Generator"),gui::dialog_frame::default_style,true,&buttons);
+	f.layout(xpos,ypos,width,height);
+	f.draw();
 
 	SDL_Rect dialog_rect = {xpos,ypos,width,height};
 	surface_restorer dialog_restorer(&screen,dialog_rect);
@@ -108,6 +116,7 @@ void default_map_generator::user_config(display& disp)
 	const std::string& iterations_label = _("Number of Hills:");
 	const std::string& hillsize_label = _("Max Hill Size:");
 	const std::string& villages_label = _("Villages:");
+	const std::string& castlesize_label = _("Castle Size:");
 	const std::string& landform_label = _("Landform:");
 
 	SDL_Rect players_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,players_label,0,0);
@@ -116,12 +125,13 @@ void default_map_generator::user_config(display& disp)
 	SDL_Rect iterations_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,iterations_label,0,0);
 	SDL_Rect hillsize_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,hillsize_label,0,0);
 	SDL_Rect villages_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,villages_label,0,0);
+	SDL_Rect castlesize_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,castlesize_label,0,0);
 	SDL_Rect landform_rect = font::draw_text(NULL,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,landform_label,0,0);
 
 	const int horz_margin = 15;
 	const int text_right = xpos + horz_margin +
-	        maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(
-		         players_rect.w,width_rect.w),height_rect.w),iterations_rect.w),hillsize_rect.w),villages_rect.w);
+	        maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(maximum<int>(
+		         players_rect.w,width_rect.w),height_rect.w),iterations_rect.w),hillsize_rect.w),villages_rect.w),castlesize_rect.w);
 
 	players_rect.x = text_right - players_rect.w;
 	width_rect.x = text_right - width_rect.w;
@@ -129,6 +139,7 @@ void default_map_generator::user_config(display& disp)
 	iterations_rect.x = text_right - iterations_rect.w;
 	hillsize_rect.x = text_right - hillsize_rect.w;
 	villages_rect.x = text_right - villages_rect.w;
+	castlesize_rect.x = text_right - castlesize_rect.w;
 	landform_rect.x = text_right - landform_rect.w;
 
 	const int vertical_margin = 20;
@@ -138,9 +149,8 @@ void default_map_generator::user_config(display& disp)
 	iterations_rect.y = height_rect.y + height_rect.h + vertical_margin;
 	hillsize_rect.y = iterations_rect.y + iterations_rect.h + vertical_margin;
 	villages_rect.y = hillsize_rect.y + hillsize_rect.h + vertical_margin;
-	landform_rect.y = villages_rect.y + villages_rect.h + vertical_margin;
-
-	const int max_players = 9;
+	castlesize_rect.y = villages_rect.y + iterations_rect.h + vertical_margin;
+	landform_rect.y = castlesize_rect.y + villages_rect.h + vertical_margin;
 
 	const int right_space = 150;
 
@@ -150,7 +160,7 @@ void default_map_generator::user_config(display& disp)
 	gui::slider players_slider(screen);
 	players_slider.set_location(slider_rect);
 	players_slider.set_min(2);
-	players_slider.set_max(max_players);
+	players_slider.set_max(gamemap::MAX_PLAYERS);
 	players_slider.set_value(nplayers_);
 
 	const int min_width = 20;
@@ -202,6 +212,17 @@ void default_map_generator::user_config(display& disp)
 	villages_slider.set_max(max_villages);
 	villages_slider.set_value(nvillages_);
 
+	const int min_castlesize = 2;
+	const int max_castlesize = 14;
+
+	slider_rect.y = castlesize_rect.y;
+	gui::slider castlesize_slider(screen);
+	castlesize_slider.set_location(slider_rect);
+	castlesize_slider.set_min(min_castlesize);
+	castlesize_slider.set_max(max_castlesize);
+	castlesize_slider.set_value(castle_size_);
+
+
 	const int min_landform = 0;
 	const int max_landform = int(max_island);
 	slider_rect.y = landform_rect.y;
@@ -225,6 +246,7 @@ void default_map_generator::user_config(display& disp)
 		iterations_ = iterations_slider.value();
 		hill_size_ = hillsize_slider.value();
 		nvillages_ = villages_slider.value();
+		castle_size_ = castlesize_slider.value();
 		island_size_ = landform_slider.value();
 
 		dialog_restorer.restore();
@@ -238,6 +260,7 @@ void default_map_generator::user_config(display& disp)
 		iterations_slider.set_dirty();
 		hillsize_slider.set_dirty();
 		villages_slider.set_dirty();
+		castlesize_slider.set_dirty();
 		landform_slider.set_dirty();
 		link_castles.set_dirty();
 
@@ -253,6 +276,7 @@ void default_map_generator::user_config(display& disp)
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,iterations_label,iterations_rect.x,iterations_rect.y);
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,hillsize_label,hillsize_rect.x,hillsize_rect.y);
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,villages_label,villages_rect.x,villages_rect.y);
+		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,castlesize_label,castlesize_rect.x,castlesize_rect.y);
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,landform_label,landform_rect.x,landform_rect.y);
 
 		std::stringstream players_str;
@@ -275,6 +299,11 @@ void default_map_generator::user_config(display& disp)
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,villages_str.str(),
 		                slider_right+horz_margin,villages_rect.y);
 
+		std::stringstream castlesize_str;
+		castlesize_str << castle_size_;
+		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,castlesize_str.str(),
+		                slider_right+horz_margin,castlesize_rect.y);
+
 		std::stringstream landform_str;
 		landform_str << gettext(island_size_ == 0 ? N_("Inland") : (island_size_ < max_coastal ? N_("Coastal") : N_("Island")));
 		font::draw_text(&screen,screen_area(),font::SIZE_NORMAL,font::NORMAL_COLOUR,landform_str.str(),
@@ -283,7 +312,7 @@ void default_map_generator::user_config(display& disp)
 		update_rect(xpos,ypos,width,height);
 
 		disp.update_display();
-		disp.delay(10);
+		disp.delay(100);
 		events::pump();
 	}
 
@@ -318,20 +347,30 @@ std::string default_map_generator::generate_map(const std::vector<std::string>&
 		const size_t island_radius = 50 + ((max_island - island_size_)*50)/(max_island - max_coastal);
 		island_size = (island_radius*(width_/2))/100;
 	} else if(island_size_ > 0) {
-		std::cerr << "coastal...\n";
+		DBG_NG << "coastal...\n";
 		//the radius of the island should be up to twice the width of the map
 		const size_t island_radius = 40 + ((max_coastal - island_size_)*40)/max_coastal;
 		island_size = (island_radius*width_*2)/100;
 		island_off_center = minimum<size_t>(width_,height_);
-		std::cerr << "calculated coastal params...\n";
+		DBG_NG << "calculated coastal params...\n";
 	}
 
-	return default_generate_map(width_,height_,island_size,island_off_center,iterations,hill_size_,max_lakes,(nvillages_*width_*height_)/1000,nplayers_,link_castles_,labels,cfg_);
+	// A map generator can fail so try a few times to get a map before aborting.
+	std::string map;
+	int tries = 10;
+	do {
+		map = default_generate_map(width_, height_, island_size, island_off_center,
+			iterations, hill_size_, max_lakes, (nvillages_ * width_ * height_) / 1000,
+			castle_size_, nplayers_, link_castles_, labels, cfg_);
+		--tries; 
+	} while (tries && map.empty());
+
+	return map;
 }
 
 config default_map_generator::create_scenario(const std::vector<std::string>& args)
 {
-	std::cerr << "creating scenario...\n";
+	DBG_NG << "creating scenario...\n";
 	config res;
 
 	const config* const scenario = cfg_.child("scenario");
@@ -339,15 +378,20 @@ config default_map_generator::create_scenario(const std::vector<std::string>& ar
 		res = *scenario;
 	}
 
-	std::cerr << "got scenario data...\n";
+	DBG_NG << "got scenario data...\n";
 
 	std::map<gamemap::location,std::string> labels;
-	std::cerr << "generating map...\n";
+	DBG_NG << "generating map...\n";
 	res["map_data"] = generate_map(args,&labels);
-	std::cerr << "done generating map..\n";
+	DBG_NG << "done generating map..\n";
+
+	for(std::map<gamemap::location,std::string>::const_iterator i =
+			labels.begin(); i != labels.end(); ++i) {
+
+		if(i->first.x >= 0 && i->first.y >= 0 &&
+				i->first.x < static_cast<long>(width_) &&
+				i->first.y < static_cast<long>(height_)) {
 
-	for(std::map<gamemap::location,std::string>::const_iterator i = labels.begin(); i != labels.end(); ++i) {
-		if(i->first.x >= 0 && i->first.y >= 0 && i->first.x < (long)width_ && i->first.y < (long)height_) {
 			config& label = res.add_child("label");
 			label["text"] = i->second;
 			i->first.write(label);
