@@ -1,34 +1,45 @@
-/* $Id: marked-up_text.cpp 13726 2006-09-28 10:49:42Z edb $ */
+/* $Id: marked-up_text.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file marked-up_text.cpp
+//! Support for simple markup in text (fonts, colors, images).
+//! E.g. "@Victory" will be shown in green.
+
 #include "global.hpp"
 
 #include "font.hpp"
+#include "gettext.hpp"
 #include "marked-up_text.hpp"
 #include "team.hpp"
 #include "video.hpp"
-#include "wassert.hpp"
+#include "wml_exception.hpp"
 
 namespace font {
 
-const char LARGE_TEXT='*', SMALL_TEXT='`', GOOD_TEXT='@', BAD_TEXT='#',
-           NORMAL_TEXT='{', BLACK_TEXT='}', BOLD_TEXT='~', IMAGE='&',
-           COLOR_TEXT='<', NULL_MARKUP='^';
+const char LARGE_TEXT='*', SMALL_TEXT='`',
+		   BOLD_TEXT='~',  NORMAL_TEXT='{',
+		   NULL_MARKUP='^',
+		   BLACK_TEXT='}', GRAY_TEXT='|',
+           GOOD_TEXT='@',  BAD_TEXT='#',
+           GREEN_TEXT='@', RED_TEXT='#',
+           COLOR_TEXT='<', IMAGE='&';
 
-namespace {
-
-//function which will parse the markup tags at the front of a string
-std::string::const_iterator parse_markup(std::string::const_iterator i1, std::string::const_iterator i2,
-					 int* font_size, SDL_Color* colour, int* style)
+//! Parses the markup-tags at the front of a string.
+static std::string::const_iterator parse_markup(std::string::const_iterator i1,
+												std::string::const_iterator i2,
+												int* font_size,
+												SDL_Color* colour, int* style)
 {
 	if(font_size == NULL || colour == NULL) {
 		return i1;
@@ -38,7 +49,7 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 	while(i1 != i2) {
 		switch(*i1) {
 		case '\\':
-			// this must either be a quoted special character or a
+			// This must either be a quoted special character or a
 			// quoted backslash - either way, remove leading backslash
 			break;
 		case BAD_TEXT:
@@ -53,6 +64,9 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 		case BLACK_TEXT:
 			*colour = BLACK_COLOUR;
 			break;
+		case GRAY_TEXT:
+			*colour = GRAY_COLOUR;
+			break;
 		case LARGE_TEXT:
 			*font_size += 2;
 			break;
@@ -64,27 +78,9 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 			break;
 		case NULL_MARKUP:
 			return i1+1;
-		// semi ANSI colour escape sequences at the start of the line for now only
-		case '\033':
-			if(i2 - i1 >= 4) {
-				++i1;
-				if(*i1 == '[') {
-					++i1;
-					if(*i1 == '3') {
-						++i1;
-						if(*i1 >= '0' && *i1 <= '9' && *(i1 + 1) == 'm')
-						{
-							if(*i1 != '0')
-								*colour = team::get_side_colour(lexical_cast<int, char>(*i1));
-							++i1;
-						}
-					}
-				}
-			}
-			break;
 		case COLOR_TEXT:
 		  {
-		    //very primitive parsing for rgb value
+		    //Very primitive parsing for rgb value
 		    //should look like <213,14,151>
 		    //but no checking on commas or end '>',
 		    //could be any non-# char
@@ -116,12 +112,11 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 		      }
 		    }
 		    blue=temp;
-		    if(i1 != i2 && i1+1 != i2 && '>'==(*i1)){
+		    if(i1 != i2 && '>'==(*i1)){
 		      SDL_Color temp_color = {red,green,blue,0};
 		      (*colour) = temp_color;
-		      return i1+1;
-		    }else
-		      return i_start;
+		    }
+		    if(i1 == i2) return i1;
 		    break;
 		  }
 		default:
@@ -132,21 +127,77 @@ std::string::const_iterator parse_markup(std::string::const_iterator i1, std::st
 	return i1;
 }
 
+
+//! Copy string, but without tags at the beginning
+std::string del_tags(const std::string& text){
+	int ignore_int;
+	SDL_Color ignore_color;
+	std::vector<std::string> lines = utils::split(text, '\n', 0);
+	std::vector<std::string>::iterator line;
+	for(line = lines.begin(); line != lines.end(); ++line) {
+		std::string::const_iterator i1 = line->begin(),
+			i2 = line->end();
+		*line = std::string(parse_markup(i1,i2,&ignore_int,&ignore_color,&ignore_int),i2); 
+	}
+	return utils::join(lines, '\n');
+}
+
+//! Copy string, but with NULL MARKUP tag at the beginning of each line
+std::string nullify_markup(const std::string& text) {
+	std::vector<std::string> lines = utils::split(text, '\n', 0);
+	std::vector<std::string>::iterator line;
+	for(line = lines.begin(); line != lines.end(); ++line) {
+		*line = std::string() + NULL_MARKUP + *line; 
+	}
+	return utils::join(lines, '\n');
 }
 
 
+//! Create string of color-markup, such as "<255,255,0>" for yellow.
+std::string color2markup(const SDL_Color color) {
+	std::stringstream markup;
+	// The RGB of SDL_Color are Uint8, we need to cast them to int.
+	// Without cast, it gives their char equivalent.
+	markup << "<" 
+		   << static_cast<int>(color.r) << "," 
+		   << static_cast<int>(color.g) << "," 
+		   << static_cast<int>(color.b) << ">";
+	return markup.str();
+}
+
+//! Calculate the size of a text (in pixels) if it were to be drawn.
 SDL_Rect text_area(const std::string& text, int size, int style)
 {
 	const SDL_Rect area = {0,0,10000,10000};
 	return draw_text(NULL, area, size, font::NORMAL_COLOUR, text, 0, 0, false, style);
 }
 
+//! Draw text on the screen, clip text to area. Supports simple markup.
+//!
+//! If the text runs outside of area horizontally, 
+//! an ellipsis will be displayed at the end of it.
+//! If use_tooltips is true, then text with an ellipsis will have a 
+//! tooltip set for it equivalent to the entire contents of the text.
+//!
+//! Some very basic 'markup' will be done on the text:
+//! - any line beginning in # will be displayed in BAD_COLOUR  (red)
+//! - any line beginning in @ will be displayed in GOOD_COLOUR (green)
+//! - any line beginning in + will be displayed with size increased by 2
+//! - any line beginning in - will be displayed with size decreased by 2
+//! - any line beginning with 0x0n will be displayed in the colour of side n
+//!
+//! The above special characters can be quoted using a C-style backslash.
+//!
+//! A bounding rectangle of the text is returned. 
+//! If gui is NULL, then the text will not be drawn, 
+//! and only a bounding rectangle will be returned.
+//! 
 SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
                    const SDL_Color& colour, const std::string& txt,
                    int x, int y, bool use_tooltips, int style)
 {
-	//make sure there's always at least a space, so we can ensure
-	//that we can return a rectangle for height
+	// Make sure there's always at least a space,
+	// so we can ensure that we can return a rectangle for height
 	static const std::string blank_text(" ");
 	const std::string& text = txt.empty() ? blank_text : txt;
 
@@ -190,13 +241,11 @@ SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
 	return res;
 }
 
+//! Determine if char is one of the special chars used as markup.
+//! @retval true	input-char is a markup-char
+//! @retval false	input-char is a normal char
 bool is_format_char(char c)
 {
-	//side coloring
-	if(c > 0 && c <= 10) {
-		return true;
-	}
-
 	switch(c) {
 	case LARGE_TEXT:
 	case SMALL_TEXT:
@@ -204,6 +253,7 @@ bool is_format_char(char c)
 	case BAD_TEXT:
 	case NORMAL_TEXT:
 	case BLACK_TEXT:
+	case GRAY_TEXT:
 	case BOLD_TEXT:
 	case NULL_MARKUP:
 		return true;
@@ -212,9 +262,7 @@ bool is_format_char(char c)
 	}
 }
 
-namespace {
-
-void cut_word(std::string& line, std::string& word, int size, int max_width)
+static void cut_word(std::string& line, std::string& word, int font_size, int style, int max_width)
 {
 	std::string tmp = line;
 	utils::utf8_iterator tc(word);
@@ -222,7 +270,7 @@ void cut_word(std::string& line, std::string& word, int size, int max_width)
 
 	for(;tc != utils::utf8_iterator::end(word); ++tc) {
 		tmp.append(tc.substr().first, tc.substr().second);
-		SDL_Rect tsize = line_size(tmp, size);
+		SDL_Rect tsize = line_size(tmp, font_size, style);
 		if(tsize.w > max_width) {
 			const std::string& w = word;
 			if(line.empty() && first) {
@@ -238,6 +286,7 @@ void cut_word(std::string& line, std::string& word, int size, int max_width)
 	}
 }
 
+namespace {
 
 /*
  * According to Kinsoku-Shori, Japanese rules about line-breaking:
@@ -295,11 +344,17 @@ inline bool break_after(wchar_t ch)
 		(ch >= 0xf900 && ch < 0xfb00) ||
 		(ch >= 0xff00 && ch < 0xfff0);
 }
-}
 
+} // end of anon namespace
+
+//! Wrap text.
+//!
+//! If the text exceedes the specified max width, wrap it one a word basis.
+//! If this is not possible, e.g. the word is too big to fit, 
+//! wrap it on a char basis.
 std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int max_width, int max_height, int max_lines)
 {
-	wassert(max_width > 0);
+	VALIDATE(max_width > 0, _("The maximum text width is less than 1."));
 
 	utils::utf8_iterator ch(unwrapped_text);
 	std::string current_word;
@@ -311,16 +366,26 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 	bool start_of_line = true;
 	std::string wrapped_text;
 	std::string format_string;
+	SDL_Color color;
+	int font_sz = font_size;
+	int style = TTF_STYLE_NORMAL;
 	utils::utf8_iterator end = utils::utf8_iterator::end(unwrapped_text);
 
 	while(1) {
 		if(start_of_line) {
 			line_width = 0;
 			format_string = "";
-			while(ch != end && *ch < (wchar_t)0x100 && is_format_char(*ch) && !ch.next_is_end()) {
+			while(ch != end && *ch < static_cast<wchar_t>(0x100)
+					&& is_format_char(*ch) && !ch.next_is_end()) {
+
 				format_string.append(ch.substr().first, ch.substr().second);
 				++ch;
 			}
+			// We need to parse the special format characters
+			// to give the proper font_size and style to line_size()
+			font_sz = font_size;
+			style = TTF_STYLE_NORMAL;
+			parse_markup(format_string.begin(),format_string.end(),&font_sz,&color,&style);
 			current_line = format_string;
 			start_of_line = false;
 		}
@@ -360,15 +425,14 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 			start_of_line = true;
 		} else {
 
-			const std::string word = format_string + current_word;
-
-			const size_t word_width = line_size(word,font_size).w;
+			const size_t word_width = line_size(current_word, font_sz, style).w;
 
 			line_width += word_width;
 
-			if((long)line_width > max_width) {
-				if((long)word_width > max_width) {
-					cut_word(current_line, current_word, font_size, max_width);
+			if(static_cast<long>(line_width) > max_width) {
+				if(static_cast<long>(word_width) > max_width) {
+					cut_word(current_line,
+						current_word, font_sz, style, max_width);
 				}
 				if(current_word == " ")
 					current_word = "";
@@ -380,7 +444,7 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 		}
 
 		if(line_break || current_word.empty() && ch == end) {
-			SDL_Rect size = line_size(current_line, font_size);
+			SDL_Rect size = line_size(current_line, font_sz, style);
 			if(max_height > 0 && current_height + size.h >= size_t(max_height)) {
 				return wrapped_text;
 			}
@@ -404,6 +468,16 @@ std::string word_wrap_text(const std::string& unwrapped_text, int font_size, int
 	return wrapped_text;
 }
 
+//! Draw text on the screen, fit text to maximum width, no markup, no tooltips.
+//!
+//! This method makes sure that the text fits within a given maximum width. 
+//! If a line exceedes this width, it will be wrapped 
+//! on a word basis if possible, otherwise on a char basis.
+//! This method is otherwise similar to the draw_text method,
+//! but it doesn't support special markup or tooltips.
+//!
+//! @return 	a bounding rectangle of the text.
+//!
 SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 		     const SDL_Color& colour, const std::string& text,
 		     int x, int y, int max_width)
@@ -413,6 +487,7 @@ SDL_Rect draw_wrapped_text(CVideo* gui, const SDL_Rect& area, int font_size,
 }
 
 
+//! Chop up one long string of text into lines.
 size_t text_to_lines(std::string& message, size_t max_length)
 {
 	std::string starting_markup;
@@ -460,5 +535,5 @@ size_t text_to_lines(std::string& message, size_t max_length)
 }
 
 
+} // end namespace font
 
-}
