@@ -1,4 +1,4 @@
-/* $Id: menu_events.cpp 24205 2008-03-02 11:29:03Z mordante $ */
+/* $Id: menu_events.cpp 24951 2008-03-21 19:28:14Z brunowolff $ */
 /*
    Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playturn Copyright (C) 2003 by David White <dave@whitevine.net>
@@ -2181,7 +2181,7 @@ private:
 			change_logging(data);
 		} else if (cmd == "theme") {
 			preferences::show_theme_dialog(*gui_);
-		} else if (cmd == "muteall" || cmd == "ping"
+		} else if ((cmd == "muteall" || cmd == "ping")
 				&& network::nconnections() != 0)
 		{
 			send_command(cmd);
