@@ -1,10 +1,11 @@
-/* $Id: minimap.hpp 8663 2005-10-29 08:56:16Z ydirson $ */
+/* $Id: minimap.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -22,7 +23,7 @@ class team;
 namespace image {
 	///function to create the minimap for a given map
 	///the surface returned must be freed by the user
-	surface getMinimap(int w, int h, const gamemap& map_, const team* tm=NULL);
+	surface getMinimap(int w, int h, const gamemap& map_, const viewpoint* vm=NULL);
 }
 
 #endif
