@@ -1,10 +1,11 @@
-/* $Id: multiplayer.cpp 18055 2007-06-03 03:34:13Z sapient $ */
+/* $Id: multiplayer.cpp 24038 2008-02-25 00:06:47Z sapient $ */
 /*
-   Copyright (C)
+   Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -12,6 +13,9 @@
 */
 
 #include "global.hpp"
+
+#include "construct_dialog.hpp"
+#include "dialogs.hpp"
 #include "game_config.hpp"
 #include "gettext.hpp"
 #include "log.hpp"
@@ -21,10 +25,10 @@
 #include "multiplayer_wait.hpp"
 #include "multiplayer_lobby.hpp"
 #include "multiplayer_create.hpp"
+#include "playmp_controller.hpp"
 #include "network.hpp"
 #include "playcampaign.hpp"
-#include "playmp_controller.hpp"
-#include "preferences.hpp"
+#include "game_preferences.hpp"
 #include "preferences_display.hpp"
 #include "random.hpp"
 #include "replay.hpp"
@@ -50,14 +54,15 @@ public:
 			LOG_NW << "sending leave_game\n";
 			config cfg;
 			cfg.add_child("leave_game");
-			network::send_data(cfg);
+			network::send_data(cfg, 0, true);
 			LOG_NW << "sent leave_game\n";
 		}
 	};
 };
 
+}
 
-void run_lobby_loop(display& disp, mp::ui& ui)
+static void run_lobby_loop(display& disp, mp::ui& ui)
 {
 	disp.video().modeChanged();
 	bool first = true;
@@ -81,74 +86,61 @@ void run_lobby_loop(display& disp, mp::ui& ui)
 	font::cache_mode(font::CACHE_GAME);
 }
 
+namespace {
+
 enum server_type {
 	ABORT_SERVER,
 	WESNOTHD_SERVER,
 	SIMPLE_SERVER
 };
 
-class server_list_action : public gui::dialog_button_action
+class server_button : public gui::dialog_button
 {
 public:
-	server_list_action(display& disp, std::string& result) : disp_(disp), result_(result)
+	server_button(CVideo &vid): dialog_button(vid, _("View List"))
 	{}
-private:
-	gui::dialog_button_action::RESULT button_pressed(int /*menu_selection*/)
+protected:
+	int action(gui::dialog_process_info &dp_info)
 	{
+		//display a dialog with a list of known servers
+		gui::dialog server_dialog(dialog()->get_display(), _("List of Servers"),
+			_("Choose a known server from the list"), gui::OK_CANCEL);
 		std::vector<std::string> servers;
-		std::ostringstream menu_heading; 
-		menu_heading << HEADING_PREFIX << _("Name") << COLUMN_SEPARATOR << _("Address"); 
-		servers.push_back(menu_heading.str()); 
-
+		std::ostringstream menu_heading;
+		menu_heading << HEADING_PREFIX << _("Name") << COLUMN_SEPARATOR << _("Address");
+		servers.push_back(menu_heading.str());
 		const std::vector<game_config::server_info>& pref_servers = preferences::server_list();
 		std::vector<game_config::server_info>::const_iterator server;
 		for(server = pref_servers.begin(); server != pref_servers.end(); ++server) {
 			servers.push_back(server->name + COLUMN_SEPARATOR + server->address);
 		}
-
+		server_dialog.set_menu(servers);
 		gui::menu::basic_sorter server_sorter;
 		server_sorter.set_alpha_sort(0).set_id_sort(1);
-
-		int res = gui::show_dialog(disp_, NULL, _("List of Servers"), 
-			_("Choose a known server from the list"), gui::OK_CANCEL, &servers,
-			NULL, "", NULL, -1, NULL, NULL, -1, -1, NULL, NULL, "", &server_sorter);
-		if(res >= 0) {
-			result_ = pref_servers[res].address;
-		} else {
-			result_ = "invalid";
+		server_dialog.get_menu().set_sorter(&server_sorter);
+		if(server_dialog.show() >= 0) {
+			//now save the result back to the parent dialog
+			dialog()->get_textbox().set_text(preferences::server_list()[server_dialog.result()].address);
 		}
-		return gui::dialog_button_action::SAVE_TEXT;
+		//the button state should be cleared after popping up an intermediate dialog
+		dp_info.clear_buttons();
+		return gui::CONTINUE_DIALOG;
 	}
-
-	display& disp_;
-	std::string& result_;
 };
+}
 
-server_type open_connection(display& disp, const std::string& original_host)
+static server_type open_connection(game_display& disp, const std::string& original_host)
 {
 	std::string h = original_host;
 
 	if(h.empty()) {
-		h = preferences::network_host();
-		std::string list_return;
-		do {
-			if(!(list_return.empty() || list_return == "invalid")) {
-				h = list_return;
-			}
-			list_return = "";
-			server_list_action sla(disp, list_return);
-			gui::dialog_button sla_button(&sla,_("View List"));
-			std::vector<gui::dialog_button> buttons;
-			buttons.push_back(sla_button);
-			const int res = gui::show_dialog(disp, NULL, _("Connect to Host"), "",
-				gui::OK_CANCEL, NULL, NULL,
-				_("Choose host to connect to: "), &h,
-				-1, NULL, NULL, -1, -1, NULL, &buttons);
-
-			if(list_return.empty() && (res != 0 || h.empty())) {
-				return ABORT_SERVER;
-			}
-		} while(!list_return.empty());
+		gui::dialog d(disp, _("Connect to Host"), "", gui::OK_CANCEL);
+		d.set_textbox(_("Choose host to connect to: "), preferences::network_host());
+		d.add_button( new server_button(disp.video()), gui::dialog::BUTTON_EXTRA);
+		if(d.show() || d.textbox_text().empty()) {
+			return ABORT_SERVER;
+		}
+		h = d.textbox_text();
 	}
 
 	network::connection sock;
@@ -173,7 +165,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 	config::child_list redirects;
 	config data;
-	sock = gui::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
+	sock = dialogs::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
 
 	do {
 
@@ -182,8 +174,9 @@ server_type open_connection(display& disp, const std::string& original_host)
 		}
 
 		data.clear();
-		network::connection data_res = gui::network_receive_dialog(
+		network::connection data_res = dialogs::network_receive_dialog(
 				disp,_("Reading from Server..."),data);
+		if (!data_res) return ABORT_SERVER;
 		mp::check_response(data_res, data);
 
 		// Backwards-compatibility "version" attribute
@@ -210,7 +203,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 			if(network::nconnections() > 0)
 				network::disconnect();
-			sock = gui::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
+			sock = dialogs::network_connect_dialog(disp,_("Connecting to Server..."),host,port);
 			continue;
 		}
 
@@ -219,7 +212,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 			config res;
 			cfg["version"] = game_config::version;
 			res.add_child("version", cfg);
-			network::send_data(res);
+			network::send_data(res, 0, true);
 		}
 
 		//if we got a direction to login
@@ -229,7 +222,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 			do {
 				if(error != NULL) {
-					gui::show_dialog(disp,NULL,"",(*error)["message"],gui::OK_ONLY);
+					gui::dialog(disp,"",(*error)["message"],gui::OK_ONLY).show();
 				}
 
 				std::string login = preferences::login();
@@ -237,15 +230,10 @@ server_type open_connection(display& disp, const std::string& original_host)
 				if(!first_time) {
 					const int res = gui::show_dialog(disp, NULL, "",
 							_("You must log in to this server"), gui::OK_CANCEL,
-							NULL, NULL, _("Login: "), &login);
+							NULL, NULL, _("Login: "), &login, mp::max_login_size);
 					if(res != 0 || login.empty()) {
 						return ABORT_SERVER;
 					}
-					if(login.size() > 18) {
-						gui::show_error_message(disp, _("The login name you chose is too long, please use a login with less than 18 characters"));
-						return ABORT_SERVER;
-					}
-
 					preferences::set_login(login);
 				}
 
@@ -253,7 +241,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 
 				config response;
 				response.add_child("login")["username"] = login;
-				network::send_data(response);
+				network::send_data(response, 0, true);
 
 				network::connection data_res = network::receive_data(data, 0, 3000);
 				if(!data_res) {
@@ -289,7 +277,7 @@ server_type open_connection(display& disp, const std::string& original_host)
 // creating the dialogs, then, according to the dialog result, of calling other
 // of those screen functions.
 
-void enter_wait_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, bool observe)
+static void enter_wait_mode(game_display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, bool observe)
 {
 	mp::ui::result res;
 	game_state state;
@@ -309,11 +297,11 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
-			//FIXME commented a pointeless if since the else does exactly the same thing
+			// FIXME commented a pointeless if since the else does exactly the same thing
 			//if (preferences::skip_mp_replay()){
 				//FIXME implement true skip replay
 				//state = ui.request_snapshot();
-			//	state = ui.get_state();
+				//state = ui.get_state();
 			//}
 			//else{
 				state = ui.get_state();
@@ -323,7 +311,8 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, data, disp.video(), nolog, IO_CLIENT, preferences::skip_mp_replay());
+		play_game(disp, state, game_config, data, nolog, IO_CLIENT, 
+			preferences::skip_mp_replay() && observe);
 		recorder.clear();
 
 		break;
@@ -333,13 +322,15 @@ void enter_wait_mode(display& disp, const config& game_config, game_data& data,
 	}
 }
 
-void enter_connect_mode(display& disp, const config& game_config, game_data& data,
+static void enter_create_mode(game_display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, mp::controller default_controller, bool is_server);
+
+static void enter_connect_mode(game_display& disp, const config& game_config, game_data& data,
 		mp::chat& chat, config& gamelist, const mp::create::parameters& params,
 		mp::controller default_controller, bool is_server)
 {
 	mp::ui::result res;
 	game_state state;
-	const network::manager net_manager;
+	const network::manager net_manager(1,1);
 	const network::server_manager serv_manager(15000, is_server ?
 			network::server_manager::TRY_CREATE_SERVER :
 			network::server_manager::NO_SERVER);
@@ -360,24 +351,26 @@ void enter_connect_mode(display& disp, const config& game_config, game_data& dat
 		if (res == mp::ui::PLAY) {
 			ui.start_game();
 			state = ui.get_state();
-			const config* const era_cfg = game_config.find_child("era","id",params.era);
-			game_events::add_events(era_cfg->get_children("event"),"all");
 		}
 	}
 
 	switch (res) {
 	case mp::ui::PLAY:
-		play_game(disp, state, game_config, data, disp.video(), nolog, IO_SERVER);
+		play_game(disp, state, game_config, data, nolog, IO_SERVER);
 		recorder.clear();
 
+		break;
+	case mp::ui::CREATE:
+		enter_create_mode(disp, game_config, data, chat, gamelist, default_controller, is_server);
 		break;
 	case mp::ui::QUIT:
 	default:
+		network::send_data(config("refresh_lobby"), 0, true);
 		break;
 	}
 }
 
-void enter_create_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, mp::controller default_controller, bool is_server)
+static void enter_create_mode(game_display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist, mp::controller default_controller, bool is_server)
 {
 	mp::ui::result res;
 	mp::create::parameters params;
@@ -396,14 +389,12 @@ void enter_create_mode(display& disp, const config& game_config, game_data& data
 	case mp::ui::QUIT:
 	default:
 		//update lobby content
-		config request;
-		request.add_child("refresh_lobby");
-		network::send_data(request);
+		network::send_data(config("refresh_lobby"), 0, true);
 		break;
 	}
 }
 
-void enter_lobby_mode(display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist)
+static void enter_lobby_mode(game_display& disp, const config& game_config, game_data& data, mp::chat& chat, config& gamelist)
 {
 	mp::ui::result res;
 
@@ -418,41 +409,33 @@ void enter_lobby_mode(display& disp, const config& game_config, game_data& data,
 		case mp::ui::JOIN:
 			try {
 				enter_wait_mode(disp, game_config, data, chat, gamelist, false);
-			} catch(network::error& error) {
+			} catch(config::error& error) {
 				if(!error.message.empty()) {
-					if(error.message == _("No multiplayer sides available in this game") ||
-					   error.message == _("Era not available") ||
-					   error.message == _("No multiplayer sides found")) {
-						gui::show_error_message(disp, error.message);
-						break;
-					}
+					gui::show_error_message(disp, error.message);
 				}
-				throw error;
+				//update lobby content
+				network::send_data(config("refresh_lobby"), 0, true);
 			}
 			break;
 		case mp::ui::OBSERVE:
 			try {
 				enter_wait_mode(disp, game_config, data, chat, gamelist, true);
-			} catch(network::error& error) {
+			} catch(config::error& error) {
 				if(!error.message.empty()) {
-					if(error.message == _("No multiplayer sides available in this game") ||
-					   error.message == _("Era not available") ||
-					   error.message == _("No multiplayer sides found")) {
-						gui::show_error_message(disp, error.message);
-						break;
-					}
+					gui::show_error_message(disp, error.message);
 				}
-				throw error;
 			}
+			//update lobby content
+			network::send_data(config("refresh_lobby"), 0, true);
 			break;
 		case mp::ui::CREATE:
 			try {
 				enter_create_mode(disp, game_config, data, chat, gamelist, mp::CNTR_NETWORK, false);
-			} catch(network::error& error) {
+			} catch(config::error& error) {
 				if (!error.message.empty())
 					gui::show_error_message(disp, error.message);
-				// Exit the lobby on network errors
-				return;
+				//update lobby content
+				network::send_data(config("refresh_lobby"), 0, true);
 			}
 			break;
 		case mp::ui::QUIT:
@@ -462,9 +445,7 @@ void enter_lobby_mode(display& disp, const config& game_config, game_data& data,
 				const preferences::display_manager disp_manager(&disp);
 				preferences::show_preferences_dialog(disp,game_config);
 				//update lobby content
-				config request;
-				request.add_child("refresh_lobby");
-				network::send_data(request);
+				network::send_data(config("refresh_lobby"), 0, true);
 			}
 			break;
 		default:
@@ -473,26 +454,24 @@ void enter_lobby_mode(display& disp, const config& game_config, game_data& data,
 	}
 }
 
-}
-
 namespace mp {
 
-void start_server(display& disp, const config& game_config, game_data& data,
+void start_server(game_display& disp, const config& game_config, game_data& data,
 		mp::controller default_controller, bool is_server)
 {
 	const set_random_generator generator_setter(&recorder);
 	mp::chat chat;
 	config gamelist;
 	playmp_controller::set_replay_last_turn(0);
-
+	preferences::set_message_private(false);
 	enter_create_mode(disp, game_config, data, chat, gamelist, default_controller, is_server);
 }
 
-void start_client(display& disp, const config& game_config, game_data& data,
+void start_client(game_display& disp, const config& game_config, game_data& data,
 		const std::string host)
 {
 	const set_random_generator generator_setter(&recorder);
-	const network::manager net_manager;
+	const network::manager net_manager(1,1);
 
 	mp::chat chat;
 	config gamelist;
@@ -504,6 +483,7 @@ void start_client(display& disp, const config& game_config, game_data& data,
 		break;
 	case SIMPLE_SERVER:
 		playmp_controller::set_replay_last_turn(0);
+		preferences::set_message_private(false);
 		enter_wait_mode(disp, game_config, data, chat, gamelist, false);
 		break;
 	case ABORT_SERVER:
