@@ -1,10 +1,11 @@
-/* $Id: multiplayer.hpp 12375 2006-07-03 18:52:20Z jhinrichs $ */
+/* $Id: multiplayer.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2005 Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2008 Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -16,11 +17,14 @@
 #include "multiplayer_ui.hpp"
 
 class config;
-class display;
-struct game_data;
+class game_display;
+class game_data;
 
 namespace mp {
 
+// max. length of a player name
+const size_t max_login_size = 18;
+
 /*
  * This is the main entry points of multiplayer mode.
  */
@@ -33,7 +37,7 @@ namespace mp {
  * @param default_controller The default controller type
  * @param is_server   Whether to open a port to the outside, or not.
  */
-void start_server(display& disp, const config& game_config, game_data& data,
+void start_server(game_display& disp, const config& game_config, game_data& data,
 		mp::controller default_controller, bool is_server);
 
 /** Starts a multiplayer game in client mode.
@@ -43,7 +47,7 @@ void start_server(display& disp, const config& game_config, game_data& data,
  * @param data        The global game data (unit types, etc)
  * @param host        The host to connect to.
  */
-void start_client(display& disp, const config& game_config, game_data& data,
+void start_client(game_display& disp, const config& game_config, game_data& data,
 		const std::string host);
 
 }
