@@ -1,4 +1,4 @@
-/* $Id: multiplayer_connect.cpp 23988 2008-02-21 19:34:02Z suokko $ */
+/* $Id: multiplayer_connect.cpp 26477 2008-05-09 08:58:47Z suokko $ */
 /*
    Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -1004,7 +1004,7 @@ void connect::process_event()
 		if(network::nconnections() > 0) {
 			config cfg;
 			cfg.add_child("leave_game");
-			network::send_data(cfg, 0, false);
+			network::send_data(cfg, 0, true);
 		}
 
 		set_result(QUIT);
@@ -1082,6 +1082,11 @@ void connect::process_network_data(const config& data, const network::connection
 {
 	ui::process_network_data(data, sock);
 
+	if(data.child("leave_game")) {
+		set_result(QUIT);
+		return;
+	}
+
 	if (!data["side_drop"].empty()) {
 		const int side_drop = lexical_cast_default<int>(data["side_drop"], 0) - 1;
 		if(side_drop >= 0 && side_drop < int(sides_.size())) {
@@ -1165,7 +1170,7 @@ void connect::process_network_data(const config& data, const network::connection
 
 			// sides_[side_taken].set_connection(sock);
 			sides_[side_taken].import_network_user(data);
-			update_playerlist_state();
+			update_playerlist_state(false);
 			update_and_send_diff();
 
 			LOG_NW << "sent player data\n";
