@@ -1,10 +1,11 @@
-/* $Id: multiplayer_ui.hpp 15228 2007-01-14 00:56:57Z jhinrichs $ */
+/* $Id: multiplayer_ui.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C)
+   Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -13,12 +14,10 @@
 #ifndef MULTIPLAYER_UI_HPP_INCLUDED
 #define MULTIPLAYER_UI_HPP_INCLUDED
 
-#include "config.hpp"
 #include "hotkeys.hpp"
 #include "network.hpp"
 #include "preferences_display.hpp"
 #include "widgets/label.hpp"
-#include "widgets/button.hpp"
 #include "widgets/menu.hpp"
 #include "widgets/textbox.hpp"
 #include "menu_events.hpp"
@@ -27,11 +26,8 @@
 #include <string>
 
 class display;
-struct game_state;
-
-namespace {
-static std::vector<std::string> empty_string_vector;
-}
+class config;
+class game_state;
 
 namespace mp {
 
@@ -39,7 +35,7 @@ enum controller { CNTR_NETWORK = 0, CNTR_LOCAL, CNTR_COMPUTER, CNTR_EMPTY, CNTR_
 
 void check_response(network::connection res, const config& data);
 
-void level_to_gamestate(config& level, game_state& state, bool saved_game = false);
+void level_to_gamestate(config& level, game_state& state, bool saved_game=false);
 
 std::string get_colour_string(int id);
 
@@ -49,15 +45,17 @@ class chat
 public:
 	chat();
 
-	void add_message(const std::string& user, const std::string& message);
+	void add_message(const time_t& time, const std::string& user,
+			const std::string& message);
 
 	void init_textbox(gui::textbox& textbox);
 	void update_textbox(gui::textbox& textbox);
 
 private:
 	struct msg {
-		msg(const std::string& user, const std::string& message) :
-			user(user), message(message) {};
+		msg(const time_t& time, const std::string& user, const std::string& message)
+			: time(time), user(user), message(message) {};
+		time_t time;
 		std::string user;
 		std::string message;
 	};
@@ -66,18 +64,17 @@ private:
 	std::string format_message(const msg& message);
 
 	msg_hist message_history_;
-
 	msg_hist::size_type last_update_;
 };
 
 //a base class for the different multiplayer base dialogs: game list, create
 //game, wait game, game setup
-class ui : public gui::widget, private events::chat_handler
+class ui : public gui::widget, private events::chat_handler, private font::floating_label_context
 {
 public:
 	enum result { CONTINUE, JOIN, OBSERVE, CREATE, PREFERENCES, PLAY, QUIT };
 
-	ui(display& d, const std::string& title,
+	ui(game_display& d, const std::string& title,
 			const config& cfg, chat& c, config& gamelist);
 
 	// Asks the multiplayer_ui to pump some data from the network, and then
@@ -98,11 +95,13 @@ public:
 protected:
 	int xscale(int x) const;
 	int yscale(int y) const;
+	static const int xscale_base;
+	static const int yscale_base;
 
 	SDL_Rect client_area() const;
 
-	display& disp_;
-	display& disp() { return disp_; };
+	game_display& disp_;
+	game_display& disp() { return disp_; };
 
 	// Returns the main game config, as defined by loading the preprocessed
 	// WML files. Children of this class may need this to obtain, for
@@ -117,10 +116,13 @@ protected:
 	virtual void handle_key_event(const SDL_KeyboardEvent& event);
 
 	// Override chat_handler
-	void send_chat_query(const std::string& args);
-	void add_chat_message(const std::string& speaker, int side, const std::string& message, display::MESSAGE_TYPE type=display::MESSAGE_PRIVATE);
+	void add_chat_message(const time_t& time, const std::string& speaker,
+			int side, const std::string& message,
+			game_display::MESSAGE_TYPE type=game_display::MESSAGE_PRIVATE);
 	void send_chat_message(const std::string& message, bool allies_only=false);
 
+	//! Process chat messages.
+	void process_message(const config& msg, const bool whisper=false);
 
 	// Processes any pending network data. Called by the public
 	// process_network() method. Overridden by subclasses who add more
@@ -149,17 +151,23 @@ protected:
 	// Sets the result of this dialog, to be checked by get_result()
 	result set_result(result res);
 
+	// Sets the name of the selected game which is used to highlight
+	// the names of the players which have joined this game
+	void set_selected_game(const std::string game_name);
+
 	// Called each time the gamelist_ variable is updated. May be
 	// overridden by child classes to add custom gamelist behaviour.
 	virtual void gamelist_updated(bool silent=true);
 
 	// Sets the user list
 	void set_user_list(const std::vector<std::string>&, bool silent);
+	void set_user_menu_items(const std::vector<std::string>& list);
 
 	// Returns the current gamelist
 	config& gamelist() { return gamelist_; };
 
-	const gui::widget& title() const;
+	void append_to_title(const std::string& name);
+	const gui::label& title() const;
 
 private:
 	/** Set to true when the widgets are intialized. Allows delayed
@@ -181,14 +189,37 @@ private:
 	config& gamelist_;
 
 	gui::label title_;
-	gui::textbox chat_textbox_;
+#ifndef USE_TINY_GUI
 	gui::textbox entry_textbox_;
+#endif
+	gui::textbox chat_textbox_;
 
 	gui::menu users_menu_;
 
 	std::vector<std::string> user_list_;
 
+	std::string selected_game_;
+
 	result result_;
+
+	bool gamelist_refresh_;
+
+	Uint32 lobby_clock_;
+
+public:
+	enum user_relation { ME, FRIEND, NEUTRAL, IGNORED };
+	enum user_state    { LOBBY, GAME, SEL_GAME };
+
+private:
+	struct user_info
+	{
+		std::string    name;
+		std::string    game_id;
+		std::string    location;
+		user_relation  relation;
+		user_state     state;
+		bool operator> (const user_info& b) const;
+	};
 };
 
 }
