@@ -1,4 +1,4 @@
-/* $Id: multiplayer_wait.cpp 23988 2008-02-21 19:34:02Z suokko $ */
+/* $Id: multiplayer_wait.cpp 24329 2008-03-05 17:22:35Z shadowmaster $ */
 /*
    Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -455,7 +455,7 @@ void wait::generate_menu()
 		// saves.
 		config::const_child_itors side_units = sd.child_range("unit");
 		for(;side_units.first != side_units.second; ++side_units.first) {
-			if((**side_units.first)["canrecruit"] == "1") {
+			if(utils::string_bool((**side_units.first)["canrecruit"], false)) {
 				leader_type = (**side_units.first)["type"];
 				break;
 			}
