@@ -1,4 +1,4 @@
-/* $Id: network.cpp 23927 2008-02-18 08:15:09Z suokko $ */
+/* $Id: network.cpp 27539 2008-06-27 19:58:18Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -25,6 +25,8 @@
 #include "network_worker.hpp"
 #include "thread.hpp"
 
+#include "filesystem.hpp"
+
 #include "SDL_net.h"
 
 #include <algorithm>
@@ -32,11 +34,13 @@
 #include <cerrno>
 #include <queue>
 #include <iostream>
+#include <iomanip>
 #include <set>
 #include <vector>
 #include <ctime>
 
 #include <signal.h>
+#include <string.h>
 #if defined(_WIN32) || defined(__WIN32__) || defined (WIN32)
 #undef INADDR_ANY
 #undef INADDR_BROADCAST
@@ -45,6 +49,8 @@
 #else
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
+#include <netinet/tcp.h>  // for TCP_NODELAY
 #ifdef __BEOS__
 #include <socket.h>
 #else
@@ -157,6 +163,7 @@ static void check_timeout()
 	// Reset last_ping if we didn't check for the last 10s.
 	if (last_ping_check + 10 <= now) last_ping = now;
 	if (static_cast<time_t>(last_ping + network::ping_timeout) <= now) {
+
 		int timeout = now - last_ping;
 		ERR_NW << "No server ping since " << timeout
 				<< " seconds. Connection timed out.\n";
@@ -222,6 +229,11 @@ void error::disconnect()
 	if(socket) network::disconnect(socket);
 }
 
+pending_statistics get_pending_stats()
+{
+	return network_worker_pool::get_pending_stats();
+}
+
 manager::manager(size_t min_threads, size_t max_threads) : free_(true)
 {
 	// If the network is already being managed
@@ -259,6 +271,11 @@ manager::~manager()
 	}
 }
 
+void set_raw_data_only()
+{
+	network_worker_pool::set_raw_data_only();
+}
+
 server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(false)
 {
 	if(create_server != NO_SERVER && !server_socket) {
@@ -278,10 +295,16 @@ server_manager::server_manager(int port, CREATE_SERVER create_server) : free_(fa
 }
 
 server_manager::~server_manager()
+{
+	stop();
+}
+
+void server_manager::stop()
 {
 	if(free_) {
 		SDLNet_TCP_Close(server_socket);
 		server_socket = 0;
+		free_ = false;
 	}
 }
 
@@ -353,6 +376,16 @@ void connect_operation::run()
 		return;
 	}
 
+#ifdef TCP_NODELAY
+	//set TCP_NODELAY to 0 because SDL_Net turns it off, causing packet
+	//flooding!
+	{
+	_TCPsocket* raw_sock = reinterpret_cast<_TCPsocket*>(sock);
+	int no = 0;
+	setsockopt(raw_sock->channel, IPPROTO_TCP, TCP_NODELAY, (char*)&no, sizeof(no));
+	}
+#endif
+
 // Use non blocking IO
 #if defined(_WIN32) || defined(__WIN32__) || defined(WIN32)
 	{
@@ -425,8 +458,10 @@ void connect_operation::run()
 	while(!notify_finished());
 }
 
+
 } // end namespace
 
+
 connection connect(const std::string& host, int port)
 {
 	connect_operation op(host,port);
@@ -593,11 +628,19 @@ void queue_disconnect(network::connection sock)
 	disconnection_queue.push_back(sock);
 }
 
-connection receive_data(config& cfg, connection connection_num, unsigned int timeout)
+connection receive_data(config& cfg, connection connection_num, unsigned int timeout
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* bandwidth_in
+#endif
+		)
 {
 	unsigned int start_ticks = SDL_GetTicks();
 	while(true) {
-		const connection res = receive_data(cfg,connection_num);
+		const connection res = receive_data(cfg,connection_num
+#ifdef BANDWIDTH_MONITOR
+		,(bool*)0 , bandwidth_in
+#endif
+		);
 		if(res != 0) {
 			return res;
 		}
@@ -615,7 +658,11 @@ connection receive_data(config& cfg, connection connection_num, unsigned int tim
 	return 0;
 }
 
-connection receive_data(config& cfg, connection connection_num)
+connection receive_data(config& cfg, connection connection_num, bool* gzipped
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* bandwidth_in
+#endif
+		)
 {
 	if(!socket_set) {
 		return 0;
@@ -670,7 +717,18 @@ connection receive_data(config& cfg, connection connection_num)
 
 	TCPsocket sock = connection_num == 0 ? 0 : get_socket(connection_num);
 	TCPsocket s = sock;
-	sock = network_worker_pool::get_received_data(sock,cfg);
+#ifdef BANDWIDTH_MONITOR
+	bandwidth_in_ptr temp;
+	if (!bandwidth_in)
+	{
+		bandwidth_in = &temp;
+	}
+#endif
+	sock = network_worker_pool::get_received_data(sock,cfg, gzipped
+#ifdef BANDWIDTH_MONITOR
+			, *bandwidth_in
+#endif
+			);
 	if (sock == NULL) {
 		if (!is_server() && last_ping != 0 && ping_timeout != 0)
 		{
@@ -714,10 +772,252 @@ connection receive_data(config& cfg, connection connection_num)
 	return result;
 }
 
+connection receive_data(std::vector<char>& buf
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* bandwidth_in
+#endif
+		)
+{
+	if(!socket_set) {
+		return 0;
+	}
+
+	check_error();
+
+	if(disconnection_queue.empty() == false) {
+		const network::connection sock = disconnection_queue.front();
+		disconnection_queue.pop_front();
+		throw error("",sock);
+	}
+
+	if(bad_sockets.count(0)) {
+		return 0;
+	}
+
+	if(sockets.empty()) {
+		return 0;
+	}
+
+	const int res = SDLNet_CheckSockets(socket_set,0);
+
+	for(std::set<network::connection>::iterator i = waiting_sockets.begin(); res != 0 && i != waiting_sockets.end(); ) {
+		connection_details& details = get_connection_details(*i);
+		const TCPsocket sock = details.sock;
+		if(SDLNet_SocketReady(sock)) {
+
+			// See if this socket is still waiting for it to be assigned its remote handle.
+			// If it is, then the first 4 bytes must be the remote handle.
+			if(is_pending_remote_handle(*i)) {
+				char buf[4];
+				int len = SDLNet_TCP_Recv(sock,buf,4);
+				if(len != 4) {
+					throw error("Remote host disconnected",*i);
+				}
+
+				const int remote_handle = SDLNet_Read32(buf);
+				set_remote_handle(*i,remote_handle);
+
+				continue;
+			}
+
+			waiting_sockets.erase(i++);
+			SDLNet_TCP_DelSocket(socket_set,sock);
+			network_worker_pool::receive_data(sock);
+		} else {
+			++i;
+		}
+	}
+
+
+	TCPsocket sock = network_worker_pool::get_received_data(buf);
+	if (sock == NULL) {
+		return 0;
+	}
+
+#ifdef BANDWIDTH_MONITOR
+	{
+		bandwidth_in_ptr temp;
+		if (!bandwidth_in)
+		{
+			bandwidth_in = &temp;
+		}
+		const int headers = 5;
+		bandwidth_in->reset(new network::bandwidth_in(buf.size() + headers));
+	}
+#endif
+
+	SDLNet_TCP_AddSocket(socket_set,sock);
+
+	connection result = 0;
+	for(connection_map::const_iterator j = connections.begin(); j != connections.end(); ++j) {
+		if(j->second.sock == sock) {
+			result = j->first;
+			break;
+		}
+	}
+
+	assert(result != 0);
+	waiting_sockets.insert(result);
+	return result;
+}
+#ifdef BANDWIDTH_MONITOR
+struct bandwidth_stats {
+	int out_packets;
+	int out_bytes;
+	int in_packets;
+	int in_bytes;
+	int day;
+	const static size_t type_width = 16;
+	const static size_t packet_width = 7;
+	const static size_t bytes_width = 10;
+	bandwidth_stats& operator+=(const bandwidth_stats& a)
+	{
+		out_packets += a.out_packets;
+		out_bytes += a.out_bytes;
+		in_packets += a.in_packets;
+		in_bytes += a.in_bytes;
+
+		return *this;
+	}
+};
+typedef std::map<const std::string, bandwidth_stats> bandwidth_map;
+typedef std::vector<bandwidth_map> hour_stats_vector;
+hour_stats_vector hour_stats(24);
+
+
+
+static bandwidth_map::iterator add_bandwidth_entry(const std::string packet_type)
+{
+	time_t now = time(0);
+	struct tm * timeinfo = localtime(&now);
+	int hour = timeinfo->tm_hour;
+	int day = timeinfo->tm_mday;
+	assert(hour < 24 && hour >= 0);
+	std::pair<bandwidth_map::iterator,bool> insertion = hour_stats[hour].insert(std::make_pair(packet_type, bandwidth_stats()));
+	bandwidth_map::iterator& inserted = insertion.first;
+	if (!insertion.second && day != inserted->second.day)
+	{
+		// clear previuos day stats
+		hour_stats[hour].clear();
+		//insert again to cleared map
+		insertion = hour_stats[hour].insert(std::make_pair(packet_type, bandwidth_stats()));
+		inserted = insertion.first;
+	}
+	inserted->second.day = day;
+	return inserted;
+}
+
+typedef boost::shared_ptr<bandwidth_stats> bandwidth_stats_ptr;
+
+
+struct bandwidth_stats_output {
+	bandwidth_stats_output(std::stringstream& ss) : ss_(ss), totals_(new bandwidth_stats())
+	{}
+	void operator()(const bandwidth_map::value_type& stats)
+	{
+		// name 
+		ss_	<< " " << std::setw(bandwidth_stats::type_width) <<  stats.first << "| "
+			<< std::setw(bandwidth_stats::packet_width)<< stats.second.out_packets << "| "
+			<< std::setw(bandwidth_stats::bytes_width) << stats.second.out_bytes/1024 << "| "
+			<< std::setw(bandwidth_stats::packet_width)<< stats.second.in_packets << "| "
+			<< std::setw(bandwidth_stats::bytes_width) << stats.second.in_bytes/1024 << "\n";
+		*totals_ += stats.second;
+	}
+	void output_totals()
+	{
+		(*this)(std::make_pair(std::string("total"), *totals_));
+	}
+	private:
+	std::stringstream& ss_;
+	bandwidth_stats_ptr totals_;
+};
+
+std::string get_bandwidth_stats_all()
+{
+	std::string result;
+	for (int hour = 0; hour < 24; ++hour)
+	{
+		result += get_bandwidth_stats(hour);
+	}
+	return result;
+}
+
+std::string get_bandwidth_stats()
+{
+	time_t now = time(0);
+	struct tm * timeinfo = localtime(&now);
+	int hour = timeinfo->tm_hour - 1;
+	if (hour < 0)
+		hour = 23;
+	return get_bandwidth_stats(hour);
+}
+
+std::string get_bandwidth_stats(int hour)
+{
+	assert(hour < 24 && hour >= 0);
+	std::stringstream ss;
+
+	ss << "Hour stat starting from " << hour << "\n " << std::left << std::setw(bandwidth_stats::type_width) <<  "Type of packet" << "| " 
+		<< std::setw(bandwidth_stats::packet_width)<< "out #"  << "| "
+		<< std::setw(bandwidth_stats::bytes_width) << "out kb" << "| "
+		<< std::setw(bandwidth_stats::packet_width)<< "in #"  << "| "
+		<< std::setw(bandwidth_stats::bytes_width) << "in kb" << "\n";
+
+	bandwidth_stats_output outputer(ss);
+	std::for_each(hour_stats[hour].begin(), hour_stats[hour].end(), outputer);
+
+	outputer.output_totals();
+	return ss.str();
+}
+
+void add_bandwidth_out(const std::string packet_type, size_t len)
+{
+	bandwidth_map::iterator itor = add_bandwidth_entry(packet_type);
+	itor->second.out_bytes += len;
+	++(itor->second.out_packets);
+}
+
+void add_bandwidth_in(const std::string packet_type, size_t len)
+{
+	bandwidth_map::iterator itor = add_bandwidth_entry(packet_type);
+	itor->second.in_bytes += len;
+	++(itor->second.in_packets);
+}
+
+	bandwidth_in::~bandwidth_in()
+	{
+		add_bandwidth_in(type_, len_);
+	}
+
+#endif
+void send_file(const std::string& filename, connection connection_num, const std::string packet_type
+		)
+{
+	assert(connection_num > 0);
+	if(bad_sockets.count(connection_num) || bad_sockets.count(0)) {
+		return;
+	}
+
+	const connection_map::iterator info = connections.find(connection_num);
+	if (info == connections.end()) {
+		ERR_NW << "Error: socket: " << connection_num
+			<< "\tnot found in connection_map. Not sending...\n";
+		return;
+	}
+
+#ifdef BANDWIDTH_MONITOR
+	const int packet_headers = 5;
+	add_bandwidth_out(packet_type, file_size(filename) + packet_headers);
+#endif
+	network_worker_pool::queue_file(info->second.sock, filename);
+	
+}
+
 //! @todo Note the gzipped parameter should be removed later, we want to send
 //! all data gzipped. This can be done once the campaign server is also updated
 //! to work with gzipped data.
-void send_data(const config& cfg, connection connection_num, const bool gzipped)
+void send_data(const config& cfg, connection connection_num, const bool gzipped, const std::string packet_type
+		)
 {
 	DBG_NW << "in send_data()...\n";
 	
@@ -735,7 +1035,11 @@ void send_data(const config& cfg, connection connection_num, const bool gzipped)
 		for(sockets_list::const_iterator i = sockets.begin();
 		    i != sockets.end(); ++i) {
 			DBG_NW << "server socket: " << server_socket << "\ncurrent socket: " << *i << "\n";
-			send_data(cfg,*i, gzipped);
+			send_data(cfg,*i, gzipped
+#ifdef BANDWIDTH_MONITOR
+					, packet_type
+#endif
+					);
 		}
 		return;
 	}
@@ -748,7 +1052,45 @@ void send_data(const config& cfg, connection connection_num, const bool gzipped)
 	}
 
 	LOG_NW << "SENDING to: " << connection_num << ": " << cfg;
-	network_worker_pool::queue_data(info->second.sock, cfg, gzipped);
+	network_worker_pool::queue_data(info->second.sock, cfg, gzipped
+#ifdef BANDWIDTH_MONITOR
+			, packet_type
+#endif 
+			);
+}
+
+void send_raw_data(const char* buf, int len, connection connection_num, const std::string packet_type
+		)
+{
+	if(len == 0) {
+		return;
+	}
+
+	if(bad_sockets.count(connection_num) || bad_sockets.count(0)) {
+		return;
+	}
+
+	if(!connection_num) {
+		for(sockets_list::const_iterator i = sockets.begin();
+		    i != sockets.end(); ++i) {
+			send_raw_data(buf, len, connection_num, packet_type
+					);
+		}
+		return;
+	}
+
+	const connection_map::iterator info = connections.find(connection_num);
+	if (info == connections.end()) {
+		ERR_NW << "Error: socket: " << connection_num
+			<< "\tnot found in connection_map. Not sending...\n";
+		return;
+	}
+#ifdef BANDWIDTH_MONITOR
+	const int packet_headers = 5;
+	add_bandwidth_out(packet_type, len + packet_headers);
+#endif
+
+	network_worker_pool::queue_raw_data(info->second.sock, buf, len);
 }
 
 void process_send_queue(connection, size_t)
@@ -757,14 +1099,19 @@ void process_send_queue(connection, size_t)
 }
 
 //! @todo Note the gzipped parameter should be removed later.
-void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped)
+void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped, const std::string packet_type
+		)
 {
 	for(sockets_list::const_iterator i = sockets.begin(); i != sockets.end(); ++i) {
 		if(*i == connection_num) {
 			continue;
 		}
 
-		send_data(cfg,*i, gzipped);
+		send_data(cfg,*i, gzipped
+#ifdef BANDWIDTH_MONITOR
+				, packet_type
+#endif
+				);
 	}
 }
 
@@ -786,12 +1133,6 @@ std::string ip_address(connection connection_num)
 	return str.str();
 }
 
-
-std::pair<unsigned int,size_t> get_thread_state()
-{
-	return network_worker_pool::thread_state();
-}
-
 statistics get_send_stats(connection handle)
 {
 	return network_worker_pool::get_current_transfer_stats(handle == 0 ? get_socket(sockets.back()) : get_socket(handle)).first;
