@@ -1,4 +1,4 @@
-/* $Id: network.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: network.hpp 27541 2008-06-27 22:46:37Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -23,6 +23,9 @@ class config;
 #include "SDL_net.h"
 
 #include <string>
+#include <vector>
+
+#include <boost/shared_ptr.hpp>
 
 namespace threading
 {
@@ -33,6 +36,13 @@ namespace threading
 
 namespace network {
 
+struct pending_statistics {
+	int npending_sends;
+	int nbytes_pending_sends;
+};
+
+pending_statistics get_pending_stats();
+
 // A network manager must be created before networking can be used.
 // It must be destroyed only after all networking activity stops.
 
@@ -55,6 +65,8 @@ private:
 	void operator=(const manager&);
 };
 
+void set_raw_data_only();
+
 //! A server manager causes listening on a given port
 //! to occur for the duration of its lifetime.
 struct server_manager {
@@ -69,6 +81,7 @@ struct server_manager {
 	~server_manager();
 
 	bool is_running() const;
+	void stop();
 
 private:
 	bool free_;
@@ -110,6 +123,35 @@ bool disconnect(connection connection_num=0, bool force=false);
 //! on the given connection (and presumably then the handling of the error
 //! will include closing the connection).
 void queue_disconnect(connection connection_num);
+#ifdef BANDWIDTH_MONITOR
+
+std::string get_bandwidth_stats();
+std::string get_bandwidth_stats_all();
+std::string get_bandwidth_stats(int hour);
+
+void add_bandwidth_out(const std::string packet_type, size_t len);
+void add_bandwidth_in(const std::string packet_type, size_t len);
+struct bandwidth_in {
+	bandwidth_in(int len) : len_(len), type_("unknown") {}
+	~bandwidth_in();
+//	{
+//		network::add_bandwidth_in(type_, len_);
+//	}
+
+	void set_type(const std::string& type)
+	{
+		type_ = type;
+	}
+	
+	private:
+	int len_;
+	std::string type_;
+};
+
+typedef boost::shared_ptr<bandwidth_in> bandwidth_in_ptr;
+
+#endif
+
 
 //! Function to receive data from either a certain connection,
 //! or all connections if connection_num is 0.
@@ -118,13 +160,30 @@ void queue_disconnect(connection connection_num);
 //! Returns the connection that data was received from,
 //! or 0 if timeout occurred.
 //! Throws error if an error occurred.
-connection receive_data(config& cfg, connection connection_num=0);
-connection receive_data(config& cfg, connection connection_num, unsigned int timeout);
+connection receive_data(config& cfg, connection connection_num=0, bool* gzipped = 0
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* b = 0
+#endif
+		);
+connection receive_data(config& cfg, connection connection_num, unsigned int timeout
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* b = 0
+#endif
+		);
+connection receive_data(std::vector<char>& buf
+#ifdef BANDWIDTH_MONITOR
+		, bandwidth_in_ptr* = 0
+#endif
+		);
+
+void send_file(const std::string&, connection, const std::string packet_type = "unknown");
 
 //! Function to send data down a given connection,
 //! or broadcast to all peers if connection_num is 0.
 //! Throws error.
-void send_data(const config& cfg, connection connection_num /*= 0*/, const bool gzipped);
+void send_data(const config& cfg, connection connection_num /*= 0*/, const bool gzipped, const std::string packet_type = "unknown");
+
+void send_raw_data(const char* buf, int len, connection connection_num, const std::string packet_type = "unknown");
 
 //! Function to send any data that is in a connection's send_queue,
 //! up to a maximum of 'max_size' bytes --
@@ -132,15 +191,12 @@ void send_data(const config& cfg, connection connection_num /*= 0*/, const bool
 void process_send_queue(connection connection_num=0, size_t max_size=0);
 
 //! Function to send data to all peers except 'connection_num'.
-void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped);
+void send_data_all_except(const config& cfg, connection connection_num, const bool gzipped, const std::string packet_type = "unknown"
+		);
 
 //! Function to get the remote ip address of a socket.
 std::string ip_address(connection connection_num);
 
-
-//! Function to know the total number of threads and the number of idle threads.
-std::pair<unsigned int,size_t> get_thread_state();
-
 struct connection_stats
 {
 	connection_stats(int sent, int received, int connected_at);
