@@ -1,16 +1,20 @@
-/* $Id: pathutils.cpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: pathutils.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file pathutils.cpp
+//! Various pathfinding functions and utilities.
+
 #include "global.hpp"
 
 #include "pathutils.hpp"
@@ -23,7 +27,7 @@ size_t distance_between(const gamemap::location& a, const gamemap::location& b)
 	const size_t vpenalty = ( (is_even(a.x) && is_odd(b.x) && (a.y < b.y))
 		|| (is_even(b.x) && is_odd(a.x) && (b.y < a.y)) ) ? 1 : 0;
 
-	// for any non-negative integer i, i - i/2 - i%2 == i/2
+	// For any non-negative integer i, i - i/2 - i%2 == i/2
 	// previously returned (hdistance + vdistance - vsavings)
 	// = hdistance + vdistance - minimum(vdistance,hdistance/2+hdistance%2)
 	// = maximum(hdistance, vdistance+hdistance-hdistance/2-hdistance%2)
@@ -53,24 +57,13 @@ void get_adjacent_tiles(const gamemap::location& a, gamemap::location* res)
 	res->y = a.y - (is_even(a.x) ? 1:0);
 }
 
-gamemap::location::DIRECTION get_adjacent_direction(const gamemap::location& from, const gamemap::location& to)
-{
-	gamemap::location adj[6];
-	get_adjacent_tiles(from,adj);
-	for(size_t n = 0; n != 6; ++n) {
-		if(adj[n] == to) {
-			return static_cast<gamemap::location::DIRECTION>(n);
-		}
-	}
-
-	return gamemap::location::NDIRECTIONS;
-}
-
 bool tiles_adjacent(const gamemap::location& a, const gamemap::location& b)
 {
-	//two tiles are adjacent if y is different by 1, and x by 0, or if
-	//x is different by 1 and y by 0, or if x and y are each different by 1,
-	//and the x value of the hex with the greater y value is even
+	// Two tiles are adjacent:
+	// if y is different by 1, and x by 0,
+	// or if x is different by 1 and y by 0,
+	// or if x and y are each different by 1,
+	// and the x value of the hex with the greater y value is even.
 
 	const int xdiff = abs(a.x - b.x);
 	const int ydiff = abs(a.y - b.y);
