@@ -1,4 +1,4 @@
-/* $Id: play_controller.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: play_controller.cpp 27256 2008-06-17 14:35:07Z alink $ */
 /*
    Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
    wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
@@ -556,15 +556,15 @@ bool play_controller::can_execute_command(hotkey::HOTKEY_COMMAND command, int in
 		return network::nconnections() > 0;
 
 	case hotkey::HOTKEY_REDO:
-		return !linger_ && !redo_stack_.empty() && !events::commands_disabled;
+		return !linger_ && !redo_stack_.empty() && !events::commands_disabled && !browse_;
 	case hotkey::HOTKEY_UNDO:
-		return !linger_ && !undo_stack_.empty() && !events::commands_disabled;
+		return !linger_ && !undo_stack_.empty() && !events::commands_disabled && !browse_;
 
 	case hotkey::HOTKEY_UNIT_DESCRIPTION:
 		return menu_handler_.current_unit(mouse_handler_) != units_.end();
 
 	case hotkey::HOTKEY_RENAME_UNIT:
-		return !events::commands_disabled &&
+		return !events::commands_disabled && !browse_ &&
 			menu_handler_.current_unit(mouse_handler_) != units_.end() &&
 			!(menu_handler_.current_unit(mouse_handler_)->second.unrenamable()) &&
 			menu_handler_.current_unit(mouse_handler_)->second.side() == gui_->viewing_team()+1 &&
