@@ -1,10 +1,12 @@
+/* $Id: playsingle_controller.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2006 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
-   wesnoth playlevel Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2006 - 2008 by Joerg Hinrichs <joerg.hinrichs@alice-dsl.de>
+   wesnoth playlevel Copyright (C) 2003 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -17,12 +19,10 @@
 #include "global.hpp"
 
 #include "cursor.hpp"
-#include "hotkeys.hpp"
 #include "menu_events.hpp"
 #include "play_controller.hpp"
-#include "playturn.hpp"
-#include "random.hpp"
-#include "upload_log.hpp"
+
+struct upload_log;
 
 #include <vector>
 
@@ -39,18 +39,24 @@ public:
 	virtual void recruit();
 	virtual void repeat_recruit();
 	virtual void recall();
-	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command) const;
+	virtual bool can_execute_command(hotkey::HOTKEY_COMMAND command, int index=-1) const;
 	virtual void toggle_shroud_updates();
 	virtual void update_shroud_now();
 	virtual void end_turn();
 	virtual void rename_unit();
 	virtual void create_unit();
 	virtual void change_unit_side();
-	virtual void label_terrain();
+	virtual void label_terrain(bool);
 	virtual void continue_move();
 	virtual void unit_hold_position();
 	virtual void end_unit_turn();
 	virtual void user_command();
+	virtual void clear_messages();
+#ifdef USRCMD2
+	virtual void user_command_2();
+	virtual void user_command_3();
+#endif
+	void linger(upload_log& log);
 
 protected:
 	virtual void play_turn(bool no_save);
@@ -58,6 +64,8 @@ protected:
 	virtual void before_human_turn(bool save);
 	virtual void play_human_turn();
 	virtual void after_human_turn();
+	void end_turn_record();
+	void end_turn_record_unlock();
 	void play_ai_turn();
 	virtual void init_gui();
 	void check_time_over();
@@ -71,11 +79,13 @@ protected:
 	bool player_type_changed_;
 	bool replaying_;
 private:
+	bool turn_over_;
+	void report_victory(std::stringstream& report,
+		    end_level_exception& end_level,
+		    int player_gold,
+		    int remaining_gold, int finishing_bonus_per_turn, 
+		    int turns_left, int finishing_bonus);
 };
 
 
-LEVEL_RESULT playsingle_scenario(const game_data& gameinfo, const config& terrain_config,
-		const config* level, CVideo& video,	game_state& state_of_game,
-		const std::vector<config*>& story, upload_log& loo, bool skip_replay);
-
 #endif
