@@ -1,19 +1,24 @@
-/* $Id: preferences_display.hpp 8891 2005-11-20 02:00:57Z darthfool $ */
+/* $Id: preferences_display.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file preferences_display.hpp 
+//!
+
 #ifndef PREFERENCES_DISPLAY_HPP_INCLUDED
 #define PREFERENCES_DISPLAY_HPP_INCLUDED
 
-#include "preferences.hpp"
+#include "game_preferences.hpp"
 
 namespace preferences {
 
@@ -26,18 +31,23 @@ namespace preferences {
 	void set_fullscreen(bool ison);
 	void set_resolution(const std::pair<int,int>& res);
 	void set_turbo(bool ison);
-	void set_adjust_gamma(bool val);
-	void set_gamma(int gamma);
 	void set_grid(bool ison);
+	void set_gamma(int gamma);
+	void set_adjust_gamma(bool val);
+	void set_turbo_speed(double speed);
 	void set_colour_cursors(bool value);
 
+	// Control unit idle animations
+	void set_idle_anim(bool ison);
+	void set_idle_anim_rate(int rate);
+
 	void show_preferences_dialog(display& disp, const config& game_cfg);
 	bool show_video_mode_dialog(display& disp);
 	bool show_theme_dialog(display& disp);
 
-	// If prefs is non-null, save the hotkeys in that config instead of
-	// the default.
+	// If prefs is non-null, save the hotkeys in that config 
+	// instead of the default.
 	void show_hotkeys_dialog (display & disp, config *prefs=NULL);
-}
+} // end namespace preferences
 
 #endif
