@@ -1,10 +1,11 @@
-/* $Id: race.hpp 19010 2007-07-22 13:09:05Z martinxyz $ */
+/* $Id: race.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -21,6 +22,8 @@
 #include "serialization/string_utils.hpp"
 #include "config.hpp"
 
+class game_state;
+
 typedef std::map<wide_string, std::vector<wchar_t> > markov_prefix_map;
 
 class unit_race
@@ -31,32 +34,34 @@ public:
 	unit_race();
 	unit_race(const config& cfg);
 
-	const t_string& name() const;
+	const std::string& id() const { return id_; };
+	const t_string& name(GENDER gender=MALE) const { return name_[gender]; };
+	const t_string& plural_name() const { return plural_name_; };
+	const t_string& description() const { return description_; };
 
-	std::string generate_name(GENDER gender) const;
+	std::string generate_name(GENDER gender, game_state* state = 0) const;
 
 	bool uses_global_traits() const;
 
 	const config::child_list& additional_traits() const;
 	unsigned int num_traits() const;
 
-	bool not_living() const;
-
 private:
-	t_string name_;
+	std::string id_;
+	t_string name_[NUM_GENDERS];
+	t_string plural_name_;
+	t_string description_;
 	unsigned int ntraits_;
-	std::vector<std::string> names_[NUM_GENDERS];
-	std::vector<std::string> names_untranslated_[NUM_GENDERS];
 	markov_prefix_map next_[NUM_GENDERS];
-	markov_prefix_map next_untranslated_[NUM_GENDERS];
 	int chain_size_;
 
-	bool not_living_;
-
 	const config::child_list* traits_;
 	bool global_traits_;
 };
 
+unit_race::GENDER string_gender(const std::string& str,unit_race::GENDER def=unit_race::MALE);
+std::string const& gender_string(unit_race::GENDER gender);
+
 typedef std::map<std::string,unit_race> race_map;
 
 #endif
