@@ -1,21 +1,24 @@
-/* $Id: sdl_utils.hpp 15316 2007-01-21 02:59:29Z sapient $ */
+/* $Id: sdl_utils.hpp 23876 2008-02-17 12:18:58Z alink $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file sdl_utils.hpp
+
 #ifndef SDL_UTILS_INCLUDED
 #define SDL_UTILS_INCLUDED
 
 #include "scoped_resource.hpp"
 #include "util.hpp"
-#include "color_range.hpp"
 
 #include "SDL.h"
 
@@ -35,6 +38,18 @@
 #define SDL_BUTTON_WHEELDOWN 5
 #endif
 
+#ifndef SDL_BUTTON_WHEELLEFT
+#define SDL_BUTTON_WHEELLEFT 6
+#endif
+
+#ifndef SDL_BUTTON_WHEELRIGHT
+#define SDL_BUTTON_WHEELRIGHT 7
+#endif
+
+namespace {
+const SDL_Rect empty_rect = { 0, 0, 0, 0 };
+}
+
 SDLKey sdl_keysym_from_name(std::string const &keyname);
 
 bool point_in_rect(int x, int y, const SDL_Rect& rect);
@@ -107,14 +122,16 @@ surface scale_surface_blended(surface const &surf, int w, int h);
 surface adjust_surface_colour(surface const &surf, int r, int g, int b);
 surface greyscale_image(surface const &surf);
 surface darken_image(surface const &surf);
-surface recolor_image(surface surf, const color_range& new_rgb, const std::vector<Uint32>& old_rgb);
+surface recolor_image(surface surf, const std::map<Uint32, Uint32>& map_rgb);
 
 surface brighten_image(surface const &surf, fixed_t amount);
+// send NULL if the portion is outside of the surface
 surface get_surface_portion(surface const &surf, SDL_Rect &rect);
-surface adjust_surface_alpha(surface const &surf, fixed_t amount);
-surface adjust_surface_alpha_add(surface const &surf, int amount);
+surface adjust_surface_alpha(surface const &surf, fixed_t amount, bool optimize=true);
+surface adjust_surface_alpha_add(surface const &surf, int amount, bool optimize=true);
 surface mask_surface(surface const &surf, surface const &mask);
 surface blur_surface(surface const &surf, int depth = 1);
+surface blur_alpha_surface(surface const &surf, int depth = 1);
 surface cut_surface(surface const &surf, SDL_Rect const &r);
 surface blend_surface(surface const &surf, double amount, Uint32 colour);
 surface flip_surface(surface const &surf);
@@ -123,13 +140,15 @@ surface create_compatible_surface(surface const &surf, int width = -1, int heigh
 
 void fill_rect_alpha(SDL_Rect &rect, Uint32 colour, Uint8 alpha, surface const &target);
 
-SDL_Rect get_non_transperant_portion(surface const &surf);
+SDL_Rect get_non_transparent_portion(surface const &surf);
 
 bool operator==(const SDL_Rect& a, const SDL_Rect& b);
 bool operator!=(const SDL_Rect& a, const SDL_Rect& b);
 bool operator==(const SDL_Color& a, const SDL_Color& b);
 bool operator!=(const SDL_Color& a, const SDL_Color& b);
 SDL_Color inverse(const SDL_Color& colour);
+SDL_Color int_to_color(const Uint32 rgb);
+Uint32 color_to_int(const SDL_Color& rgb);
 
 class config; // no need to include config.hpp
 
@@ -141,11 +160,11 @@ struct pixel_data
 	pixel_data(int red, int green, int blue) : r(red), g(green), b(blue)
 	{}
 
-	pixel_data(int pixel, SDL_PixelFormat* fmt) {
+	pixel_data(int pixel, SDL_PixelFormat* fmt) : r(0), g(0), b(0) {
 		unformat(pixel, fmt);
 	}
 
-	pixel_data(config& cfg) {
+	pixel_data(config& cfg) : r(0), g(0), b(0) {
 		read(cfg);
 	}
 
@@ -164,35 +183,6 @@ struct pixel_data
 	int r, g, b;
 };
 
-struct input_blocker 
-{ 
-	static unsigned instance_count; 
-	input_blocker() 
-	{ 
-		SDL_EventState(SDL_KEYDOWN, SDL_IGNORE); 
-		SDL_EventState(SDL_KEYUP, SDL_IGNORE); 
-		SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_IGNORE); 
-		SDL_EventState(SDL_MOUSEBUTTONUP, SDL_IGNORE); 
-		SDL_EventState(SDL_JOYBUTTONDOWN, SDL_IGNORE); 
-		SDL_EventState(SDL_JOYBUTTONUP, SDL_IGNORE); 
-		instance_count++; 
-	} 
-	
-	~input_blocker() 
-	{ 
-		instance_count--; 
-		if(instance_count == 0) { 
-			SDL_EventState(SDL_KEYDOWN, SDL_ENABLE); 
-			SDL_EventState(SDL_KEYUP, SDL_ENABLE); 
-			SDL_EventState(SDL_MOUSEBUTTONDOWN, SDL_ENABLE); 
-			SDL_EventState(SDL_MOUSEBUTTONUP, SDL_ENABLE); 
-			SDL_EventState(SDL_JOYBUTTONDOWN, SDL_ENABLE); 
-			SDL_EventState(SDL_JOYBUTTONUP, SDL_ENABLE); 
-		} 
-	} 
-}; 
-
-
 struct surface_lock
 {
 	surface_lock(surface const &surf) : surface_(surf), locked_(false)
@@ -239,7 +229,7 @@ private:
 
 struct clip_rect_setter
 {
-	clip_rect_setter(surface const &surf, const SDL_Rect& r) : surface_(surf)
+	clip_rect_setter(surface const &surf, const SDL_Rect& r) : surface_(surf), rect()
 	{
 		SDL_GetClipRect(surface_,&rect);
 		SDL_SetClipRect(surface_,&r);
@@ -258,4 +248,10 @@ void draw_rectangle(int x, int y, int w, int h, Uint32 colour, surface tg);
 void draw_solid_tinted_rectangle(int x, int y, int w, int h,
                                  int r, int g, int b,
 				 double alpha, surface target);
+
+// blit the image on the center of the rectangle
+// and a add a colored background
+void draw_centered_on_background(surface surf, const SDL_Rect& rect,
+	const SDL_Color& color, surface target);
+
 #endif
