@@ -1,4 +1,4 @@
-/* $Id: preprocessor.cpp 24128 2008-02-27 20:28:51Z mordante $ */
+/* $Id: preprocessor.cpp 26582 2008-05-13 11:11:38Z suokko $ */
 /*
    Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
@@ -916,13 +916,16 @@ struct preprocessor_deleter: std::basic_istream<char>
 {
 	preprocessor_streambuf *buf_;
 	preproc_map *defines_;
+	std::vector<std::string> *callstack_;
 	std::string *error_log;
-	preprocessor_deleter(preprocessor_streambuf *buf, preproc_map *defines);
+	preprocessor_deleter(preprocessor_streambuf *buf, preproc_map *defines, std::vector<std::string>*);
 	~preprocessor_deleter();
 };
 
-preprocessor_deleter::preprocessor_deleter(preprocessor_streambuf *buf, preproc_map *defines)
-	: std::basic_istream<char>(buf), buf_(buf), defines_(defines), error_log(buf->error_log)
+preprocessor_deleter::preprocessor_deleter(preprocessor_streambuf *buf, 
+		preproc_map *defines, 
+		std::vector<std::string> *callstack)
+	: std::basic_istream<char>(buf), buf_(buf), defines_(defines), callstack_(callstack), error_log(buf->error_log)
 {
 }
 
@@ -931,6 +934,7 @@ preprocessor_deleter::~preprocessor_deleter()
 	rdbuf(NULL);
 	delete buf_;
 	delete defines_;
+	delete callstack_;
 }
 
 
@@ -952,6 +956,6 @@ std::istream *preprocess_file(std::string const &fname,
 	new preprocessor_file(*buf, callstack, fname);
 	if(error_log!=NULL&&error_log->empty()==false)
 		throw preproc_config::error("Error preprocessing files.");
-	return new preprocessor_deleter(buf, owned_defines);
+	return new preprocessor_deleter(buf, owned_defines,callstack);
 }
 
