@@ -1,18 +1,22 @@
-/* $Id: string_utils.cpp 20297 2007-09-11 06:04:09Z soliton $ */
+/* $Id: string_utils.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
    Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
-   Copyright (C) 2005 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2005 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file serialization/string_utils.cpp
+//! Various string-routines.
+
 #include "global.hpp"
 
 #include <cctype>
@@ -29,37 +33,35 @@ variable_set::~variable_set()
 {
 }
 
-namespace {
+static bool two_dots(char a, char b) { return a == '.' && b == '.'; }
 
-bool two_dots(char a, char b) { return a == '.' && b == '.'; }
-
-std::string do_interpolation(const std::string &str, variable_set& set)
+static std::string do_interpolation(const std::string &str, const variable_set& set)
 {
 	std::string res = str;
-	//this needs to be able to store negative numbers to check for the while's condition
-	//(which is only false when the previous '$' was at index 0)
+	// This needs to be able to store negative numbers to check for the while's condition
+	// (which is only false when the previous '$' was at index 0)
 	int rfind_dollars_sign_from = res.size();
 	while(rfind_dollars_sign_from >= 0) {
-		//Going in a backwards order allows nested variable-retrieval, e.g. in arrays.
-		//For example, "I am $creatures[$i].user_description!"
+		// Going in a backwards order allows nested variable-retrieval, e.g. in arrays.
+		// For example, "I am $creatures[$i].user_description!"
 		const std::string::size_type var_begin_loc = res.rfind('$', rfind_dollars_sign_from);
 
-		//If there are no '$' left then we're done.
+		// If there are no '$' left then we're done.
 		if(var_begin_loc == std::string::npos) {
 			break;
 		}
 
-		//For the next iteration of the loop, search for more '$'
-		//(not from the same place because sometimes the '$' is not replaced)
+		// For the next iteration of the loop, search for more '$'
+		// (not from the same place because sometimes the '$' is not replaced)
 		rfind_dollars_sign_from = int(var_begin_loc) - 1;
 
 
 		const std::string::iterator var_begin = res.begin() + var_begin_loc;
 
-		//The '$' is not part of the variable name.
+		// The '$' is not part of the variable name.
 		const std::string::iterator var_name_begin = var_begin + 1;
 
-		//Find the maximum extent of the variable name (it may be shortened later).
+		// Find the maximum extent of the variable name (it may be shortened later).
 		std::string::iterator var_end = var_name_begin;
 		for(int bracket_nesting_level = 0; var_end != res.end(); ++var_end) {
 			const char c = *var_end;
@@ -76,22 +78,22 @@ std::string do_interpolation(const std::string &str, variable_set& set)
 			}
 		}
 
-		//Two dots in a row cannot be part of a valid variable name.
-		//That matters for random=, e.g. $x..$y
+		// Two dots in a row cannot be part of a valid variable name.
+		// That matters for random=, e.g. $x..$y
 		var_end = std::adjacent_find(var_name_begin, var_end, two_dots);
 
-		//If the last character is '.', then it can't be a sub-variable.
-		//It's probably meant to be a period instead. Don't include it.
-		//Would need to do it repetitively if there are multiple '.'s at the end,
-		//but don't actually need to do so because the previous check for adjacent '.'s would catch that.
-		//For example, "My score is $score." or "My score is $score..."
+		// If the last character is '.', then it can't be a sub-variable.
+		// It's probably meant to be a period instead. Don't include it.
+		// Would need to do it repetitively if there are multiple '.'s at the end,
+		// but don't actually need to do so because the previous check for adjacent '.'s would catch that.
+		// For example, "My score is $score." or "My score is $score..."
 		if(*(var_end-1) == '.'
-		//However, "$array[$i]" by itself does not name a variable,
-		//so if "$array[$i]." is encountered, then best to include the '.',
-		//so that it more closely follows the syntax of a variable (if only to get rid of all of it).
-		//(If it's the script writer's error, they'll have to fix it in either case.)
-		//For example in "$array[$i].$field_name", if field_name does not exist as a variable,
-		//then the result of the expansion should be "", not "." (which it would be if this exception did not exist).
+		// However, "$array[$i]" by itself does not name a variable,
+		// so if "$array[$i]." is encountered, then best to include the '.',
+		// so that it more closely follows the syntax of a variable (if only to get rid of all of it).
+		// (If it's the script writer's error, they'll have to fix it in either case.)
+		// For example in "$array[$i].$field_name", if field_name does not exist as a variable,
+		// then the result of the expansion should be "", not "." (which it would be if this exception did not exist).
 		&& *(var_end-2) != ']') {
 			--var_end;
 		}
@@ -99,11 +101,11 @@ std::string do_interpolation(const std::string &str, variable_set& set)
 		const std::string var_name(var_name_begin, var_end);
 
 		if(*var_end == '|') {
-			//It's been used to end this variable name; now it has no more effect.
-			//This can allow use of things like "$$composite_var_name|.x"
-			//(Yes, that's a WML 'pointer' of sorts. They are sometimes useful.)
-			//If there should still be a '|' there afterwards to affect other variable names (unlikely),
-			//just put another '|' there, one matching each '$', e.g. "$$var_containing_var_name||blah"
+			// It's been used to end this variable name; now it has no more effect.
+			// This can allow use of things like "$$composite_var_name|.x"
+			// (Yes, that's a WML 'pointer' of sorts. They are sometimes useful.)
+			// If there should still be a '|' there afterwards to affect other variable names (unlikely),
+			// just put another '|' there, one matching each '$', e.g. "$$var_containing_var_name||blah"
 			var_end++;
 		}
 
@@ -114,7 +116,7 @@ std::string do_interpolation(const std::string &str, variable_set& set)
 			res.replace(var_begin, var_end, "$");
 		}
 		else {
-			//The variable is replaced with its value.
+			// The variable is replaced with its value.
 			res.replace(var_begin, var_end,
 			    set.get_variable_const(var_name));
 		}
@@ -123,36 +125,35 @@ std::string do_interpolation(const std::string &str, variable_set& set)
 	return res;
 }
 
-}
-
 namespace utils {
 
-bool isnewline(char c)
+bool isnewline(const char c)
 {
 	return c == '\r' || c == '\n';
 }
 
-//make sure that we can use Mac, DOS, or Unix style text files on any system
-//and they will work, by making sure the definition of whitespace is consistent
-bool portable_isspace(char c)
+// Make sure that we can use Mac, DOS, or Unix style text files on any system
+// and they will work, by making sure the definition of whitespace is consistent
+bool portable_isspace(const char c)
 {
 	// returns true only on ASCII spaces
-	if ((unsigned char)c >= 128)
+	if (static_cast<unsigned char>(c) >= 128)
 		return false;
 	return isnewline(c) || isspace(c);
 }
 
-//make sure we regard '\r' and '\n' as a space, since Mac, Unix, and DOS
-//all consider these differently.
-bool notspace(char c)
+// Make sure we regard '\r' and '\n' as a space, since Mac, Unix, and DOS
+// all consider these differently.
+bool notspace(const char c)
 {
 	return !portable_isspace(c);
 }
 
+//! Remove whitespace from the front and back of the string 'str'.
 std::string &strip(std::string &str)
 {
-	//if all the string contains is whitespace, then the whitespace may
-	//have meaning, so don't strip it
+	// If all the string contains is whitespace,
+	// then the whitespace may have meaning, so don't strip it
 	std::string::iterator it = std::find_if(str.begin(), str.end(), notspace);
 	if (it == str.end())
 		return str;
@@ -163,6 +164,15 @@ std::string &strip(std::string &str)
 	return str;
 }
 
+//! Removes character 'c' from the first and last position of the string 'str'.
+std::string& strip_char(std::string &str, const char c) {
+	if (*str.begin() == c)
+		str.erase(str.begin(), str.begin() + 1);
+	if (*(str.end() - 1) == c)
+		str.erase(str.end() - 1, str.end());
+	return str;
+}
+
 std::vector< std::string > split(std::string const &val, char c, int flags)
 {
 	std::vector< std::string > res;
@@ -198,16 +208,119 @@ std::vector< std::string > split(std::string const &val, char c, int flags)
 	return res;
 }
 
+//! Splits a string based either on a separator where text within paranthesis
+//! is protected from splitting (Note that one can use the same character for
+//! both the left and right paranthesis. In this mode it usually makes only
+//! sense to have one character for the left and right paranthesis.)
+//! or if the separator == 0 it splits a string into an odd number of parts:
+//! - The part before the first '(',
+//! - the part between the first '('
+//! - and the matching right ')', etc ...
+//! and the remainder of the string.
+//! Note that this will find the first matching char in the left string
+//! and match against the corresponding char in the right string.
+//! In this mode, a correctly processed string should return with
+//! an odd number of elements to the vector and
+//! an empty elements are never removed as they are placeholders.
+//! hence REMOVE EMPTY only works for the separator split.
+//!
+//! parenthetical_split("a(b)c{d}e(f{g})h",0,"({",")}") should return
+//! a vector of <"a","b","c","d","e","f{g}","h">
+std::vector< std::string > paranthetical_split(std::string const &val, const char separator, std::string const &left, std::string const &right,int flags)
+{
+	std::vector< std::string > res;
+	std::vector<char> part;
+	bool in_paranthesis = false;
+
+	std::string::const_iterator i1 = val.begin();
+	std::string::const_iterator i2 = val.begin();
+
+	std::string lp=left;
+	std::string rp=right;
+
+	if(left.size()!=right.size()){
+		ERR_GENERAL << "Left and Right Parenthesis lists not same length\n";
+		return res;
+	}
+
+	while (i2 != val.end()) {
+		if(!in_paranthesis && separator && *i2 == separator){
+			std::string new_val(i1, i2);
+			if (flags & STRIP_SPACES)
+				strip(new_val);
+			if (!(flags & REMOVE_EMPTY) || !new_val.empty())
+				res.push_back(new_val);
+			++i2;
+			if (flags & STRIP_SPACES) {
+				while (i2 != val.end() && *i2 == ' ')
+					++i2;
+			}
+			i1=i2;
+			continue;
+		}
+		if(part.size() && *i2 == part.back()){
+			part.pop_back();
+			if(!separator && part.size() == 0){
+				std::string new_val(i1, i2);
+				if (flags & STRIP_SPACES)
+					strip(new_val);
+				res.push_back(new_val);
+				++i2;
+				i1=i2;
+			}else{
+				if (part.size() == 0)
+					in_paranthesis = false;
+				++i2;
+			}
+			continue;
+		}
+		bool found=false;
+		for(size_t i=0; i < lp.size(); i++){
+			if (*i2 == lp[i]){
+				if (!separator && part.size()==0){
+					std::string new_val(i1, i2);
+					if (flags & STRIP_SPACES)
+						strip(new_val);
+					res.push_back(new_val);
+					++i2;
+					i1=i2;
+				}else{
+					++i2;
+				}
+				part.push_back(rp[i]);
+				found=true;
+				break;
+			}
+		}
+		if(!found){
+			++i2;
+		} else
+			in_paranthesis = true;
+	}
+
+	std::string new_val(i1, i2);
+	if (flags & STRIP_SPACES)
+		strip(new_val);
+	if (!(flags & REMOVE_EMPTY) || !new_val.empty())
+		res.push_back(new_val);
+
+	if(part.size()){
+			ERR_GENERAL << "Mismatched paranthesis:\n"<<val<<"\n";;
+	}
+
+	return res;
+}
+
 class string_map_variable_set : public variable_set
 {
 public:
 	string_map_variable_set(const string_map& map) : map_(map) {};
 
-	virtual const t_string& get_variable_const(const std::string& key)
+	virtual const t_string& get_variable_const (const std::string& key) const
 	{
 		static const t_string empty_string = "";
 
-		string_map::const_iterator itor = map_.find(key);
+		const string_map::const_iterator itor = map_.find(key);
 		if(itor == map_.end()) {
 			return empty_string;
 		} else {
@@ -225,12 +338,12 @@ std::string interpolate_variables_into_string(const std::string &str, const stri
 	return do_interpolation(str, set);
 }
 
-std::string interpolate_variables_into_string(const std::string &str, variable_set& variables)
+std::string interpolate_variables_into_string(const std::string &str, const variable_set& variables)
 {
 	return do_interpolation(str, variables);
 }
 
-// modify a number by string representing integer difference, or optionally %
+// Modify a number by string representing integer difference, or optionally %
 int apply_modifier( const int number, const std::string &amount, const int minimum ) {
 	// wassert( amount.empty() == false );
 	int value = atoi(amount.c_str());
@@ -243,7 +356,7 @@ int apply_modifier( const int number, const std::string &amount, const int minim
 	return value;
 }
 
-//Prepends a configurable set of characters with a backslash
+//! Prepends a configurable set of characters with a backslash
 std::string &escape(std::string &str, const std::string& special_chars)
 {
 	std::string::size_type pos = 0;
@@ -257,16 +370,16 @@ std::string &escape(std::string &str, const std::string& special_chars)
 	return str;
 }
 
-//prepend all special characters with a backslash
-//special characters are:
-//#@{}+-,\*=
+//! Prepend all special characters with a backslash.
+// Special characters are:
+// #@{}+-,\*=
 std::string& escape(std::string& str)
 {
 	static const std::string special_chars("#@{}+-,\\*=");
 	return escape(str, special_chars);
 }
 
-// remove all escape characters (backslash)
+//! Remove all escape characters (backslash)
 std::string &unescape(std::string &str)
 {
 	std::string::size_type pos = 0;
@@ -280,30 +393,144 @@ std::string &unescape(std::string &str)
 	return str;
 }
 
-bool string_bool(const std::string& str,bool def)
-{
-	if(str != "") {
-		if(str == "yes" || str == "on" || str == "true" || lexical_cast_default<int>(str)) {
-			return true;
-		}
-		if(str == "no" || str == "off" || str == "false" || !lexical_cast_default<int>(str)) {
-			return false;
-		}
+bool string_bool(const std::string& str, bool def) {
+	if (str.empty()) return def;
+	if (str == "yes" || str == "on" || str == "true"
+	|| lexical_cast_default<int>(str, 0)) {
+		return true;
+	}
+	if (str == "no" || str == "off" || str == "false"
+	|| !lexical_cast_default<int>(str, 1)) {
+		return false;
 	}
 	return def;
 }
 
-bool isvalid_username(const std::string& username)
-{
-	//check if the username is valid (all alpha-numeric plus underscore)
-	const size_t alnum = std::count_if(username.begin(),username.end(),isalnum);
-	const size_t underscore = std::count(username.begin(),username.end(),'_');
-	if((alnum + underscore != username.size()) || underscore == username.size() || username.empty() ) {
+bool is_username_char(char c) {
+	return ((c == '_') || (c == '-'));
+}
+
+//! Check if the username is valid.
+//! (all alpha-numeric characters plus underscore and hyphen)
+bool isvalid_username(const std::string& username) {
+	const size_t alnum = std::count_if(username.begin(), username.end(), isalnum);
+	const size_t valid_char =
+			std::count_if(username.begin(), username.end(), is_username_char);
+	if ((alnum + valid_char != username.size()) 
+			|| valid_char == username.size() || username.empty() )
+	{
 		return false;
 	}
 	return true;
 }
 
+//! Try to complete the last word of 'text' with the 'wordlist'.
+//! @param[in]  'text'     Text where we try to complete the last word of.
+//! @param[out] 'text'     Text with completed last word.
+//! @param[in]  'wordlist' A vector of strings to complete against.
+//! @param[out] 'wordlist' A vector of strings that matched 'text'.
+//! @return 'true' iff text is just one word (no spaces)
+bool word_completion(std::string& text, std::vector<std::string>& wordlist) {
+	std::vector<std::string> matches;
+	const size_t last_space = text.rfind(" ");
+	// If last character is a space return.
+	if (last_space == text.size() -1) {
+		wordlist = matches;
+		return false;
+	}
+
+	bool text_start;
+	std::string semiword;
+	if (last_space == std::string::npos) {
+		text_start = true;
+		semiword = text;
+	} else {
+		text_start = false;
+		semiword.assign(text, last_space + 1, text.size());
+	}
+
+	std::string best_match = semiword;
+	for (std::vector<std::string>::const_iterator word = wordlist.begin();
+			word != wordlist.end(); ++word)
+	{
+		if (word->size() < semiword.size()
+		|| !std::equal(semiword.begin(), semiword.end(), word->begin(),
+				chars_equal_insensitive))
+		{
+			continue;
+		}
+		if (matches.empty()) {
+			best_match = *word;
+		} else {
+			int j = 0;
+			while (toupper(best_match[j]) == toupper((*word)[j])) j++;
+			if (best_match.begin() + j < best_match.end()) {
+				best_match.erase(best_match.begin() + j, best_match.end());
+			}
+		}
+		matches.push_back(*word);
+	}
+	if(!matches.empty()) {
+		text.replace(last_space + 1, best_match.size(), best_match);
+	}
+	wordlist = matches;
+	return text_start;
+}
+
+bool is_word_boundary(char c) {
+	return (c == ' ' || c == ',' || c == ':' || c == '\'' || c == '"' || c == '-');
+}
+
+//! Check if a string contains a word.
+bool word_match(const std::string& message, const std::string& word) {
+	size_t first = message.find(word);
+	if (first == std::string::npos) return false;
+	if (first == 0 || is_word_boundary(message[first - 1])) {
+		size_t next = first + word.size();
+		if (next == message.size() || is_word_boundary(message[next])) {
+			return true;
+		}
+	}
+	return false;
+}
+
+//! Match using '*' as any number of characters (including none), 
+//! and '?' as any one character.
+bool wildcard_string_match(const std::string& str, const std::string& match) {
+	const bool wild_matching = (!match.empty() && match[0] == '*');
+	const std::string::size_type solid_begin = match.find_first_not_of('*');
+	const bool have_solids = (solid_begin != std::string::npos);
+	// Check the simple case first
+	if(str.empty() || !have_solids) {
+		return wild_matching || str == match;
+	}
+	const std::string::size_type solid_end = match.find_first_of('*', solid_begin);
+	const std::string::size_type solid_len = (solid_end == std::string::npos)
+		? match.length() - solid_begin : solid_end - solid_begin;
+	std::string::size_type current = 0;
+	bool matches;
+	do {
+		matches = true;
+		// Now try to place the str into the solid space
+		const std::string::size_type test_len = str.length() - current;
+		for(std::string::size_type i=0; i < solid_len && matches; ++i) {
+			char solid_c = match[solid_begin + i];
+			if(i > test_len || !(solid_c == '?' || solid_c == str[current+i])) {
+				matches = false;
+			}
+		}
+		if(matches) {
+			// The solid space matched, now consume it and attempt to find more
+			const std::string consumed_match = (solid_begin+solid_len < match.length())
+				? match.substr(solid_end) : "";
+			const std::string consumed_str = (solid_len < test_len)
+				? str.substr(current+solid_len) : "";
+			matches = wildcard_string_match(consumed_str, consumed_match);
+		}
+	} while(wild_matching && !matches && ++current < str.length());
+	return matches;
+}
+
 std::string join(std::vector< std::string > const &v, char c)
 {
 	std::stringstream str;
@@ -316,12 +543,13 @@ std::string join(std::vector< std::string > const &v, char c)
 	return str.str();
 }
 
-//identical to split(), except it does not split when it otherwise
-//would if the previous character was identical to the parameter 'quote'.
-//i.e. it does not split quoted commas.
-//this method was added to make it possible to quote user input,
-//particularly so commas in user input will not cause visual problems in menus.
-//why not change split()? that would change the methods post condition.
+// This function is identical to split(), except it does not split
+// when it otherwise would if the previous character was identical to the parameter 'quote'.
+// i.e. it does not split quoted commas.
+// This method was added to make it possible to quote user input,
+// particularly so commas in user input will not cause visual problems in menus.
+//
+//! @todo Why not change split()? That would change the methods post condition.
 std::vector< std::string > quoted_split(std::string const &val, char c, int flags, char quote)
 {
 	std::vector<std::string> res;
@@ -331,7 +559,7 @@ std::vector< std::string > quoted_split(std::string const &val, char c, int flag
 
 	while (i2 != val.end()) {
 		if (*i2 == quote) {
-			// ignore quoted character
+			// Ignore quoted character
 			++i2;
 			if (i2 != val.end()) ++i2;
 		} else if (*i2 == c) {
@@ -373,6 +601,17 @@ std::pair< int, int > parse_range(std::string const &str)
 	return res;
 }
 
+std::vector< std::pair< int, int > > parse_ranges(std::string const &str)
+{
+	std::vector< std::pair< int, int > > to_return;
+	std::vector<std::string> strs = utils::split(str);
+	std::vector<std::string>::const_iterator i, i_end=strs.end();
+	for(i = strs.begin(); i != i_end; ++i) {
+		to_return.push_back(parse_range(*i));
+	}
+	return to_return;
+}
+
 int byte_size_from_utf8_first(unsigned char ch)
 {
 	int count;
@@ -390,22 +629,25 @@ int byte_size_from_utf8_first(unsigned char ch)
 	else if ((ch & 0xFE) == 0xFC)
 		count = 6;
 	else
-		throw invalid_utf8_exception(); /* stop on invalid characters */
+		throw invalid_utf8_exception(); // Stop on invalid characters
 
 	return count;
 }
 
-utf8_iterator::utf8_iterator(const std::string& str)
+utf8_iterator::utf8_iterator(const std::string& str) : 
+	current_char(0),
+	string_end(str.end()),
+	current_substr(std::make_pair(str.begin(), str.begin()))
 {
-	current_substr.first = str.begin();
-	string_end = str.end();
 	update();
 }
 
-utf8_iterator::utf8_iterator(std::string::const_iterator const &beg, std::string::const_iterator const &end)
+utf8_iterator::utf8_iterator(std::string::const_iterator const &beg, 
+		std::string::const_iterator const &end) :
+	current_char(0),
+	string_end(end),
+	current_substr(std::make_pair(beg, beg))
 {
-	current_substr.first = beg;
-	string_end = end;
 	update();
 }
 
@@ -457,25 +699,24 @@ void utf8_iterator::update()
 	size_t size = byte_size_from_utf8_first(*current_substr.first);
 	current_substr.second = current_substr.first + size;
 
-	current_char = (unsigned char)(*current_substr.first);
+	current_char = static_cast<unsigned char>(*current_substr.first);
 
-	/* Convert the first character */
+	// Convert the first character
 	if(size != 1) {
 		current_char &= 0xFF >> (size + 1);
 	}
 
-	/* Convert the continuation bytes */
+	// Convert the continuation bytes
 	for(std::string::const_iterator c = current_substr.first+1;
 			c != current_substr.second; ++c) {
-		// If the string ends occurs within an UTF8-sequence, this is
-		// bad.
+		// If the string ends occurs within an UTF8-sequence, this is bad.
 		if (c == string_end)
 			throw invalid_utf8_exception();
 
 		if ((*c & 0xC0) != 0x80)
 			throw invalid_utf8_exception();
 
-		current_char = (current_char << 6) | ((unsigned char)*c & 0x3F);
+		current_char = (current_char << 6) | (static_cast<unsigned char>(*c) & 0x3F);
 	}
 }
 
@@ -493,7 +734,7 @@ std::string wstring_to_string(const wide_string &src)
 			unsigned int count;
 			ch = *i;
 
-			/* Determine the bytes required */
+			// Determine the bytes required
 			count = 1;
 			if(ch >= 0x80)
 				count++;
@@ -514,12 +755,13 @@ std::string wstring_to_string(const wide_string &src)
 			if(count == 1) {
 				push_back(ret,static_cast<char>(ch));
 			} else {
-				for(int j = (int)count-1; j >= 0; --j) {
-					unsigned char c = (ch >> (6*j)) & 0x3f;
+				for(int j = static_cast<int>(count) - 1; j >= 0; --j) {
+					unsigned char c = (ch >> (6 * j)) & 0x3f;
 					c |= 0x80;
-					if(j == (int)count-1)
+					if(j == static_cast<int>(count) - 1) {
 						c |= 0xff << (8 - count);
-					push_back(ret,c);
+					}
+					push_back(ret, c);
 				}
 			}
 
@@ -548,7 +790,7 @@ wide_string string_to_wstring(const std::string &src)
 		utf8_iterator i1(src);
 		const utf8_iterator i2(utf8_iterator::end(src));
 
-		//equivalent to res.insert(res.end(),i1,i2) which doesn't work on VC++6.
+		// Equivalent to res.insert(res.end(),i1,i2) which doesn't work on VC++6.
 		while(i1 != i2) {
 			push_back(res,*i1);
 			++i1;
@@ -562,54 +804,12 @@ wide_string string_to_wstring(const std::string &src)
 	return res;
 }
 
-ucs2_string utf8_string_to_ucs2_string(const utf8_string& src)
-{
-	ucs2_string res;
-	try {
-		utf8_iterator i1(src);
-		const utf8_iterator i2(utf8_iterator::end(src));
-		while(i1 != i2) {
-			push_back(res, *i1);
-			++i1;
-		}
-	}
-	catch(invalid_utf8_exception e) {
-		ERR_GENERAL << "Invalid UTF-8 string: \"" << src << "\"\n";
-		return res;
-	}
-	return res;
-}
-
-utf8_string ucs2_string_to_utf8_string(const ucs2_string& src)
-{
-	utf8_string dst;
-	dst.reserve(src.size());
-	ucs2_string::const_iterator itor = src.begin();
-	for(;itor != src.end(); ++itor) {
-		if(*itor < 0x0080) {
-			push_back(dst,*itor);
-			continue;
-		}
-		if(0x0080 <= *itor && *itor < 0x0800) {
-			push_back(dst,(*itor >> 6) | 0xC0);
-			push_back(dst,(*itor & 0x003F) | 0x80);
-			continue;
-		}
-		if((0x0800 <= *itor && *itor < 0xD800) || (0xDFFF < *itor && *itor < 0xFFFE)) {
-			push_back(dst,(*itor >> 12) | 0xE0);
-			push_back(dst,((*itor >> 6) & 0x003F) | 0x80);
-			push_back(dst,(*itor & 0x003F) | 0x80);
-		}
-	}
-	return dst;
-}
-
 utf8_string capitalize(const utf8_string& s)
 {
 	if(s.size() > 0) {
 		utf8_iterator itor(s);
 #if defined(__APPLE__) || defined(__AMIGAOS4__)
-		// FIXME: Should we support towupper on recent OSX platforms?
+		//! @todo FIXME: Should we support towupper on recent OSX platforms?
 		wchar_t uchar = *itor;
 		if(uchar >= 0 && uchar < 0x100)
 			uchar = toupper(uchar);
@@ -631,7 +831,7 @@ utf8_string uppercase(const utf8_string& s)
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
 #if defined(__APPLE__) || defined(__AMIGAOS4__)
-			// FIXME: Should we support towupper on recent OSX platforms?
+			//! @todo FIXME: Should we support towupper on recent OSX platforms?
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
 				uchar = toupper(uchar);
@@ -654,7 +854,7 @@ utf8_string lowercase(const utf8_string& s)
 
 		for(;itor != utf8_iterator::end(s); ++itor) {
 #if defined(__APPLE__) || defined(__OpenBSD__) || defined(__AMIGAOS4__)
-			// FIXME: Should we support towupper on recent OSX platforms?
+			//! @todo FIXME: Should we support towupper on recent OSX platforms?
 			wchar_t uchar = *itor;
 			if(uchar >= 0 && uchar < 0x100)
 				uchar = tolower(uchar);
@@ -670,45 +870,25 @@ utf8_string lowercase(const utf8_string& s)
 	return s;
 }
 
-// match using '*' as any number of characters (including none), and '?' as any one character
-bool wildcard_string_match(const std::string& str, const std::string& match)
+//! Truncates a string.
+//!
+//! If the string send has more than size utf-8 characters it will be truncated
+//! to this size. 
+//! No assumptions can be made about the actual size of the string.
+//! 
+//! @param[in]  str     String which can be converted to utf-8.
+//! @param[out] str     String which contains maximal size utf-8 characters.
+//! @param size         The size to truncate at.
+void truncate_as_wstring(std::string& str, const size_t size)
 {
-	const bool wild_matching = (!match.empty() && match[0] == '*');
-	const std::string::size_type solid_begin = match.find_first_not_of('*');
-	const bool have_solids = (solid_begin != std::string::npos);
-	//check the simple case first
-	if(str.empty() || !have_solids) {
-		return wild_matching || str == match;
+	wide_string utf8_str = utils::string_to_wstring(str);
+	if(utf8_str.size() > size) {
+		utf8_str.resize(size);
+		str = utils::wstring_to_string(utf8_str);
 	}
-	const std::string::size_type solid_end = match.find_first_of('*', solid_begin);
-	const std::string::size_type solid_len = (solid_end == std::string::npos)
-		? match.length() - solid_begin : solid_end - solid_begin;
-	std::string::size_type current = 0;
-	bool matches;
-	do {
-		matches = true;
-		//now try to place the str into the solid space
-		const std::string::size_type test_len = str.length() - current;
-		for(std::string::size_type i=0; i < solid_len && matches; ++i) {
-			char solid_c = match[solid_begin + i];
-			if(i > test_len || !(solid_c == '?' || solid_c == str[current+i])) {
-				matches = false;
-			}
-		}
-		if(matches) {
-			//the solid space matched, now consume it and attempt to find more
-			const std::string consumed_match = (solid_begin+solid_len < match.length())
-				? match.substr(solid_end) : "";
-			const std::string consumed_str = (solid_len < test_len)
-				? str.substr(current+solid_len) : "";
-			matches = wildcard_string_match(consumed_str, consumed_match);
-		}
-	} while(wild_matching && !matches && ++current < str.length());
-	return matches;
 }
 
-
-}
+} // end namespace utils
 
 std::string vgettext(const char *msgid, const utils::string_map& symbols)
 {
