@@ -1,4 +1,4 @@
-/* $Id: tokenizer.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: tokenizer.cpp 26897 2008-05-28 09:46:02Z suokko $ */
 /*
    Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
@@ -146,7 +146,10 @@ const token& tokenizer::next_token()
 	case '"':
 		token_.type = token::QSTRING;
 		while (1) {
-			next_char();
+			/** Have to use next_char_no_strip 
+			 * because we will break image ifwe do striping
+			 **/
+			next_char_no_strip();
 
 			if(current_ == EOF) {
 				token_.type = token::UNTERMINATED_QSTRING;
@@ -155,11 +158,20 @@ const token& tokenizer::next_token()
 			if(current_ == '"' && peek_char() != '"')
 				break;
 			if(current_ == '"' && peek_char() == '"')
-				next_char_fast();
-			if (current_ == 254) {
-				skip_comment();
-				--lineno_;
-				continue;
+				next_char_fast_no_strip();
+			if (current_ == 254 && 
+					(peek_char() == 'l' || peek_char() == 't')) {
+				next_char_fast_no_strip();
+				if ((current_ == 'l' && peek_char() == 'i') || (current_ == 't' && peek_char() == 'e'))
+				{
+					skip_comment();
+					--lineno_;
+					continue;
+				}
+				else
+				{
+					token_.value += 254;
+				}
 			}
 
 			token_.value += current_;
