@@ -1,4 +1,4 @@
-/* $Id: input_stream.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: input_stream.cpp 26477 2008-05-09 08:58:47Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -51,10 +51,18 @@ input_stream::input_stream(const std::string& path) : fd_(-1), path_(path)
 
 input_stream::~input_stream()
 {
+#ifndef _WIN32
+	stop();
+#endif
+}
+
+void input_stream::stop()
+{
 #ifndef _WIN32
 	if(fd_ != -1) {
 		close(fd_);
 		unlink(path_.c_str());
+		fd_ = -1;
 	}
 #endif
 }
