@@ -1,4 +1,4 @@
-/* $Id: metrics.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: metrics.hpp 26582 2008-05-13 11:11:38Z suokko $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -23,19 +23,33 @@
 #include <map>
 #include <string>
 
+#include "simple_wml.hpp"
+
 class metrics
 {
 public:
 	metrics();
+	~metrics();
 
 	void service_request();
 	void no_requests();
 
+	void record_sample(const simple_wml::string_span& name,
+	                   clock_t parsing_time, clock_t processing_time);
+
 	void game_terminated(const std::string& reason);
 
 	friend std::ostream& operator<<(std::ostream& out, metrics& met);
 
+	struct sample {
+		simple_wml::string_span name;
+		int nsamples;
+		clock_t parsing_time, processing_time;
+	};
+
 private:
+	std::vector<sample> samples_;
+
 	int most_consecutive_requests_;
 	int current_requests_;
 	int nrequests_;
