@@ -1,4 +1,4 @@
-/* $Id: player.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: player.hpp 24794 2008-03-19 04:11:40Z soliton $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -18,13 +18,14 @@
 #include <ctime>
 
 #include "../config.hpp"
+#include "simple_wml.hpp"
 
 #include <string>
 
 class player
 {
 public:
-	player(const std::string& n, config& cfg, const size_t max_messages=4, const size_t time_period=10);
+	player(const std::string& n, simple_wml::node& cfg, const size_t max_messages=4, const size_t time_period=10);
 
 	// mark a player as member of the game 'game_id' or as located in the lobby
 	void mark_available(const int game_id=0, const std::string location="");
@@ -32,14 +33,14 @@ public:
 
 	const std::string& name() const { return name_; }
 
-	config* config_address() const { return &cfg_; }
+	simple_wml::node* config_address() const { return &cfg_; }
 
 	bool silenced() const { return messages_since_flood_start_ > MaxMessages; }
 	bool is_message_flooding();
 
 private:
 	const std::string name_;
-	config& cfg_;
+	simple_wml::node& cfg_;
 
 	time_t flood_start_;
 	unsigned int messages_since_flood_start_;
