@@ -1,4 +1,4 @@
-/* $Id: proxy.cpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: proxy.cpp 24794 2008-03-19 04:11:40Z soliton $ */
 /*
    Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -75,14 +75,15 @@ void disconnect(network::connection sock)
 	network::disconnect(peer);
 }
 
-void received_data(network::connection sock, const config& data)
+void received_data(network::connection sock, simple_wml::document& data)
 {
 	const network::connection peer = find_peer(sock);
 	if(!peer) {
 		return;
 	}
 
-	network::send_data(data, peer, true);
+	const simple_wml::string_span& output = data.output_compressed();
+	network::send_raw_data(output.begin(), output.size(), peer);
 }
 
 }
