@@ -1,4 +1,4 @@
-/* $Id: proxy.hpp 23842 2008-02-16 08:47:16Z mordante $ */
+/* $Id: proxy.hpp 24794 2008-03-19 04:11:40Z soliton $ */
 /*
    Copyright (C) 2007 - 2008
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -15,8 +15,8 @@
 #ifndef PROXY_HPP_INCLUDED
 #define PROXY_HPP_INCLUDED
 
-#include "config.hpp"
 #include "network.hpp"
+#include "simple_wml.hpp"
 
 #include <string>
 
@@ -27,7 +27,7 @@ void create_proxy(network::connection sock, const std::string& host, int port);
 bool is_proxy(network::connection sock);
 void disconnect(network::connection sock);
 
-void received_data(network::connection sock, const config& data);
+void received_data(network::connection sock, simple_wml::document& data);
 
 }
 
