@@ -1,21 +1,30 @@
-/* $Id: statistics.hpp 12583 2006-07-15 04:24:57Z rusty $ */
+/* $Id: statistics.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file statistics.hpp
+//!
+
 #ifndef STATISTICS_HPP_INCLUDED
 #define STATISTICS_HPP_INCLUDED
 
+class config;
+class config_writer;
 class unit;
-#include "actions.hpp"
+//#include "actions.hpp"
+#include <string>
+#include <map>
+#include <vector>
 
 namespace statistics
 {
@@ -25,29 +34,33 @@ namespace statistics
 		explicit stats(const config& cfg);
 
 		config write() const;
+		void write(config_writer &out) const;
 		void read(const config& cfg);
 
 		typedef std::map<std::string,int> str_int_map;
 		str_int_map recruits, recalls, advanced_to, deaths, killed;
 		int recruit_cost, recall_cost;
 
-		//a type that will map a string of hit/miss to the number of times
-		//that sequence has occurred
+		//! A type that will map a string of hit/miss
+		//! to the number of times that sequence has occurred.
 		typedef str_int_map battle_sequence_frequency_map;
 
-		//a type that will map different % chances to hit to different results
+		//! A type that will map different % chances to hit to different results.
 		typedef std::map<int,battle_sequence_frequency_map> battle_result_map;
 
 		battle_result_map attacks, defends;
 
 		int damage_inflicted, damage_taken;
+		int turn_damage_inflicted, turn_damage_taken;
 
-		//expected value for damage inflicted/taken * 100, based on probability
-		//to hit, Use this long term to see how lucky a side is
-		// FIXME: Since integers are used, rounding errors accumulate. Also,
-		// slow isn't accounted for properly. Rusty's simulator could be used
-		// obtain valid values.
+		// Expected value for damage inflicted/taken * 100, based on
+		// probability to hit,
+		// Use this long term to see how lucky a side is.
+		//! @todo FIXME: Since integers are used, rounding errors accumulate.
+		// Also, slow isn't accounted for properly.
+		// Rusty's simulator could be used obtain valid values.
 		int expected_damage_inflicted, expected_damage_taken;
+		int turn_expected_damage_inflicted, turn_expected_damage_taken;
 	};
 
 	int sum_str_int_map(const stats::str_int_map& m);
@@ -94,12 +107,13 @@ namespace statistics
 	void advance_unit(const unit& u);
 
 	config write_stats();
+	void write_stats(config_writer &out);
 	void read_stats(const config& cfg);
 	void fresh_stats();
 	void clear_current_scenario();
 
-	std::vector<std::string> get_categories();
+	void reset_turn_stats(int side);
 	stats calculate_stats(int category, int side);
-}
+} // end namespace statistics
 
 #endif
