@@ -1,29 +1,33 @@
-/* $Id: team.cpp 14883 2006-12-27 13:35:38Z boucman $ */
+/* $Id: team.cpp 24034 2008-02-24 22:58:45Z jhinrichs $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file team.cpp
+//! Team-management, allies, setup at start of scenario.
+
 #include "global.hpp"
 
-#include "game_config.hpp"
 #include "game_events.hpp"
 #include "game_errors.hpp"
+#include "gamestatus.hpp"
 #include "log.hpp"
 #include "network.hpp"
 #include "team.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "serialization/string_utils.hpp"
 
 #include <algorithm>
+#include <cassert>
 #include <cstdlib>
 #include <sstream>
 
@@ -33,6 +37,9 @@ namespace {
 	std::vector<team>* teams = NULL;
 }
 
+//static member initialization
+const int team::default_team_gold = 100;
+
 teams_manager::teams_manager(std::vector<team>& teams_list)
 {
 	teams = &teams_list;
@@ -43,14 +50,6 @@ teams_manager::~teams_manager()
 	teams = NULL;
 }
 
-std::vector<team> teams_manager::clone(std::vector<team>& team_list){
-	std::vector<team> result;
-	for (std::vector<team>::iterator t = team_list.begin(); t != team_list.end(); t++){
-		result.push_back(*t);
-	}
-	return result;
-}
-
 bool teams_manager::is_observer()
 {
 	for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
@@ -72,7 +71,43 @@ void team::target::write(config& cfg) const
 	cfg = criteria;
 }
 
-team::team_info::team_info(const config& cfg)
+team::team_info::team_info(const config& cfg) :
+		name(cfg["name"]),
+		gold(cfg["gold"]),
+		start_gold(),
+		income(cfg["income"]),
+		income_per_village(),
+		can_recruit(),
+		global_recruitment_pattern(),
+		recruitment_pattern(utils::split(cfg["recruitment_pattern"])),
+		enemies(),
+		team_name(cfg["team_name"]),
+		user_team_name(cfg["user_team_name"]),
+		save_id(cfg["save_id"]),
+		current_player(cfg["current_player"]),
+		countdown_time(cfg["countdown_time"]),
+		action_bonus_count(lexical_cast_default<int>(cfg["action_bonus_count"])),
+		flag(cfg["flag"]),
+		flag_icon(cfg["flag_icon"]),
+		description(cfg["description"]),
+		objectives(cfg["objectives"]),
+		objectives_changed(utils::string_bool(cfg["objectives_changed"])),
+		controller(),
+		persistent(),
+		ai_algorithm(cfg["ai_algorithm"]),
+		ai_params(),
+		ai_memory_(),
+		villages_per_scout(),
+		leader_value(0.0),
+		village_value(0.0),
+		targets(),
+		share_maps(false),
+		share_view(false),
+		disallow_observers(utils::string_bool(cfg["disallow_observers"])),
+		allow_player(utils::string_bool(cfg["allow_player"], true)),
+		no_leader(utils::string_bool(cfg["no_leader"])),
+		music(cfg["music"]),
+		colour(cfg["colour"].size() ? cfg["colour"] : cfg["side"])
 {
 	config global_ai_params;
 	const config::child_list& ai_parameters = cfg.get_children("ai");
@@ -89,58 +124,38 @@ team::team_info::team_info(const config& cfg)
 	  ai_memory_.append(**aimem);
 	}
 
+	// at the start of a scenario "start_gold" is not set, we need to take the
+	// value from the gold setting (or fall back to the gold default)
+	if (!cfg["start_gold"].empty())
+		start_gold = cfg["start_gold"];
+	else if (!this->gold.empty())
+		start_gold = this->gold;
+	else
+		start_gold = str_cast(default_team_gold);
 
-	gold = cfg["gold"];
-	income = cfg["income"];
-	name = cfg["name"];
-	team_name = cfg["team_name"];
-	user_team_name = cfg["user_team_name"];
 	if(team_name.empty()) {
 		team_name = cfg["side"];
 	}
-	
-	if(user_team_name.empty()) {
-		user_team_name = team_name;
-	}
 
-	save_id = cfg["save_id"];
 	if(save_id.empty()) {
-		save_id = cfg["description"];
+		save_id = description;
 	}
-	current_player = cfg["current_player"];
 	if (current_player.empty()) {
 		current_player = save_id;
 	}
 
-	countdown_time = cfg["countdown_time"];
-
-	const std::string& bonus = cfg["action_bonus_count"];
-	if(bonus.empty())
-		action_bonus_count = 0;
-	else
-		action_bonus_count = atoi(bonus.c_str());
-
-	colour = lexical_cast_default<int>(cfg["colour"],-1);
-	if(colour == -1)
-		colour = lexical_cast_default<int>(cfg["side"],-1);
-
 	int side = atoi(cfg["side"].c_str());
 
-	std::vector<Uint32> temp_rgb = string2rgb(cfg["team_rgb"]);
-	std::map<int, color_range>::iterator global_rgb = game_config::team_rgb_range.find(side);
+	const std::string temp_rgb_str = cfg["team_rgb"];
+	std::map<std::string, color_range>::iterator global_rgb = game_config::team_rgb_range.find(cfg["side"]);
 
-	if(temp_rgb.size()){
+	if(!temp_rgb_str.empty()){
+		std::vector<Uint32> temp_rgb = string2rgb(temp_rgb_str);
 		team_color_range_[side] = color_range(temp_rgb);
 	}else if(global_rgb != game_config::team_rgb_range.end()){
 		team_color_range_[side] = global_rgb->second;
 	}
 
-	flag = cfg["flag"];
-
-	description = cfg["description"];
-	objectives = cfg["objectives"];
-	objectives_changed = utils::string_bool(cfg["objectives_changed"]);
-
 	const std::string& village_income = cfg["village_gold"];
 	if(village_income.empty())
 		income_per_village = game_config::village_income;
@@ -169,8 +184,6 @@ team::team_info::team_info(const config& cfg)
 	if (persist.empty()) persistent = controller == HUMAN;
 	else persistent = persist == "1";
 
-	ai_algorithm = cfg["ai_algorithm"];
-
 	if(ai_algorithm.empty()) {
 		ai_algorithm = global_ai_params["ai_algorithm"];
 	}
@@ -215,26 +228,25 @@ team::team_info::team_info(const config& cfg)
 	for(std::vector<std::string>::const_iterator i = recruits.begin(); i != recruits.end(); ++i) {
 		can_recruit.insert(*i);
 	}
-
-	std::string recruit_pattern = cfg["recruitment_pattern"];
-	if(recruit_pattern.empty()) {
-		recruit_pattern = global_ai_params["recruitment_pattern"];
+	
+	if(recruitment_pattern.empty()) {
+		recruitment_pattern = 
+			utils::split(global_ai_params["recruitment_pattern"]);
 	}
 
-	recruitment_pattern = utils::split(recruit_pattern);
-
-	//default recruitment pattern is to buy 2 fighters for every 1 archer
+/*	// Default recruitment pattern is to buy 2 fighters for every 1 archer
 	if(recruitment_pattern.empty()) {
 		recruitment_pattern.push_back("fighter");
 		recruitment_pattern.push_back("fighter");
 		recruitment_pattern.push_back("archer");
 	}
-
-	//keep a copy of the initial recruitment_pattern, since it can be changed
-	//on a per-time-of-day or per-turn basis inside [ai] sections
+*/
+	// Keep a copy of the initial recruitment_pattern,
+	// since it can be changed on a per-time-of-day
+	// or per-turn basis inside [ai] sections.
 	global_recruitment_pattern = recruitment_pattern;
 
-	//additional targets
+	// Additional targets
 	config::const_child_itors tgts;
 	for(tgts = cfg.child_range("target"); tgts.first != tgts.second; ++tgts.first) {
 		targets.push_back(target(**tgts.first));
@@ -244,14 +256,13 @@ team::team_info::team_info(const config& cfg)
 		targets.push_back(target(**tgts.first));
 	}
 
-	//share_view and share_maps can't both be enabled, so share_view overrides share_maps.
+	// Share_view and share_maps can't both be enabled,
+	// so share_view overrides share_maps.
 	share_view = utils::string_bool(cfg["share_view"]);
 	share_maps = !share_view && utils::string_bool(cfg["share_maps"],true);
 
 	LOG_NG << "team_info::team_info(...): team_name: " << team_name
 	       << ", share_maps: " << share_maps << ", share_view: " << share_view << ".\n";
-
-	music = cfg["music"];
 }
 
 void team::team_info::write(config& cfg) const
@@ -263,24 +274,24 @@ void team::team_info::write(config& cfg) const
 	cfg["ai_algorithm"] = ai_algorithm;
 
 	cfg["gold"] = gold;
+	cfg["start_gold"] = start_gold;
 	cfg["income"] = income;
 	cfg["name"] = name;
 	cfg["team_name"] = team_name;
-	cfg["user_team_name"] = team_name;
+	cfg["user_team_name"] = user_team_name;
 	cfg["save_id"] = save_id;
 	cfg["current_player"] = current_player;
 	cfg["flag"] = flag;
+	cfg["flag_icon"] = flag_icon;
 	cfg["description"] = description;
 	cfg["objectives"] = objectives;
 	cfg["objectives_changed"] = objectives_changed ? "yes" : "no";
 	cfg["countdown_time"]= countdown_time;
-
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",action_bonus_count);
-	cfg["action_bonus_count"]= buf;
-
-	snprintf(buf,sizeof(buf),"%d",income_per_village);
-	cfg["village_gold"] = buf;
+	cfg["action_bonus_count"]= str_cast(action_bonus_count);
+	cfg["village_gold"] = str_cast(income_per_village);
+	cfg["disallow_observers"] = disallow_observers ? "yes" : "no";
+	cfg["allow_player"] = allow_player ? "yes" : "no";
+	cfg["no_leader"] = no_leader ? "yes" : "no";
 
 	std::stringstream enemies_str;
 	for(std::vector<int>::const_iterator en = enemies.begin(); en != enemies.end(); ++en) {
@@ -296,18 +307,13 @@ void team::team_info::write(config& cfg) const
 	case HUMAN: cfg["controller"] = "human"; break;
 	case NETWORK: cfg["controller"] = "network"; break;
 	case EMPTY: cfg["controller"] = "null"; break;
-	default: wassert(false);
+	default: assert(false); return;
 	}
+
 	cfg["persistent"] = persistent ? "1" : "0";
-
-	snprintf(buf,sizeof(buf),"%d",villages_per_scout);
-	cfg["villages_per_scout"] = buf;
-
-	snprintf(buf,sizeof(buf),"%f",leader_value);
-	cfg["leader_value"] = buf;
-
-	snprintf(buf,sizeof(buf),"%f",village_value);
-	cfg["village_value"] = buf;
+	cfg["villages_per_scout"] = str_cast(villages_per_scout);
+	cfg["leader_value"] = str_cast(leader_value);
+	cfg["village_value"] = str_cast(village_value);
 
 	for(std::vector<target>::const_iterator tg = targets.begin(); tg != targets.end(); ++tg) {
 		tg->write(cfg.add_child("target"));
@@ -343,7 +349,22 @@ void team::team_info::write(config& cfg) const
 	cfg["colour"] = lexical_cast_default<std::string>(colour);
 }
 
-team::team(const config& cfg, int gold) : gold_(gold), auto_shroud_updates_(true), info_(cfg), aggression_(0.0), caution_(0.0)
+team::team(const config& cfg, int gold) : 
+		gold_(gold),
+		villages_(),
+		shroud_(),
+		fog_(),
+		auto_shroud_updates_(true), 
+		info_(cfg), 
+		countdown_time_(0),
+		action_bonus_count_(0),
+		aiparams_(),
+		aggression_(0.0), 
+		caution_(0.0),
+		enemies_(),
+		seen_(),
+		ally_shroud_(),
+		ally_fog_()
 {
 	fog_.set_enabled(cfg["fog"] == "yes");
 	shroud_.set_enabled(cfg["shroud"] == "yes");
@@ -352,14 +373,19 @@ team::team(const config& cfg, int gold) : gold_(gold), auto_shroud_updates_(true
 	LOG_NG << "team::team(...): team_name: " << info_.team_name
 	       << ", shroud: " << uses_shroud() << ", fog: " << uses_fog() << ".\n";
 
-	//gold is the maximum of 'gold' and what is given in the config file
+	// To ensure some mimimum starting gold,
+	// gold is the maximum of 'gold' and what is given in the config file
 	if(info_.gold.empty() == false)
+	{
 		gold_ = maximum(gold,::atoi(info_.gold.c_str()));
+		if (gold_ != ::atoi(info_.gold.c_str()))
+			info_.start_gold = str_cast(gold) + " (" + info_.start_gold + ")";
+	}
 
-	//load in the villages the side controls at the start
+	// Load in the villages the side controls at the start
 	const config::child_list& villages = cfg.get_children("village");
 	for(config::child_list::const_iterator v = villages.begin(); v != villages.end(); ++v) {
-		villages_.insert(gamemap::location(**v));
+		villages_.insert(gamemap::location(**v,game_events::get_state_of_game()));
 	}
 
 	countdown_time_=lexical_cast_default<int>(cfg["countdown_time"],0);
@@ -371,21 +397,17 @@ void team::write(config& cfg) const
 	info_.write(cfg);
 	cfg["shroud"] = uses_shroud() ? "yes" : "no";
 	cfg["fog"] = uses_fog() ? "yes" : "no";
+	cfg["gold"] = str_cast(gold_);
 
-	char buf[50];
-	snprintf(buf,sizeof(buf),"%d",gold_);
-	cfg["gold"] = buf;
-
-	//write village locations
+	// Write village locations
 	for(std::set<gamemap::location>::const_iterator t = villages_.begin(); t != villages_.end(); ++t) {
 		t->write(cfg.add_child("village"));
 	}
 
 	cfg["shroud_data"] = shroud_.write();
 
-	cfg["countdown_time"] = lexical_cast_default<std::string>(countdown_time_);
-	cfg["action_bonus_count"] = lexical_cast_default<std::string>(action_bonus_count_);
-
+	cfg["countdown_time"] = str_cast(countdown_time_);
+	cfg["action_bonus_count"] = str_cast(action_bonus_count_);
 }
 
 bool team::get_village(const gamemap::location& loc)
@@ -401,36 +423,6 @@ void team::lose_village(const gamemap::location& loc)
 	}
 }
 
-void team::clear_villages()
-{
-	villages_.clear();
-}
-
-const std::set<gamemap::location>& team::villages() const
-{
-	return villages_;
-}
-
-bool team::owns_village(const gamemap::location& loc) const
-{
-	return villages_.count(loc) > 0;
-}
-
-int team::gold() const
-{
-	return gold_;
-}
-
-int team::income() const
-{
-	return atoi(info_.income.c_str()) + villages_.size()*info_.income_per_village+game_config::base_income;
-}
-
-void team::new_turn()
-{
-	gold_ += income();
-}
-
 void team::set_time_of_day(int turn, const time_of_day& tod)
 {
 	aiparams_.clear();
@@ -464,8 +456,8 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 		aiparams_.append(*i);
 	}
 
-    //get the recruitment pattern from the matching [ai] section, and fall back
-    //to tge global recruitment pattern otherwise
+    // Get the recruitment pattern from the matching [ai] section,
+    // and fall back to the global recruitment pattern otherwise.
 	info_.recruitment_pattern = utils::split(aiparams_["recruitment_pattern"]);
 	if (info_.recruitment_pattern.empty())
 		info_.recruitment_pattern = info_.global_recruitment_pattern;
@@ -473,58 +465,6 @@ void team::set_time_of_day(int turn, const time_of_day& tod)
 	caution_ = lexical_cast_default<double>(aiparams_["caution"],0.25);
 }
 
-void team::spend_gold(int amount)
-{
-	gold_ -= amount;
-}
-
-void team::set_income(int amount)
-{
-	info_.income = lexical_cast<std::string>(amount);
-}
-
-int team::countdown_time() const
-{
-	return countdown_time_;
-}
-void team::set_countdown_time(int amount)
-{
-	countdown_time_=amount;
-}
-
-int team::action_bonus_count() const
-{
-	return action_bonus_count_;
-}
-void team::set_action_bonus_count(int count)
-{
-	action_bonus_count_=count;
-}
-
-void team::set_current_player(const std::string player){
-	info_.current_player = player;
-}
-
-const std::set<std::string>& team::recruits() const
-{
-	return info_.can_recruit;
-}
-
-std::set<std::string>& team::recruits()
-{
-	return info_.can_recruit;
-}
-
-const std::vector<std::string>& team::recruitment_pattern() const
-{
-	return info_.recruitment_pattern;
-}
-
-const std::string& team::name() const
-{
-	return info_.name;
-}
-
 bool team::calculate_enemies(size_t index) const
 {
 	if(teams == NULL || index >= teams->size()) {
@@ -540,80 +480,40 @@ bool team::calculate_enemies(size_t index) const
 
 bool team::calculate_is_enemy(size_t index) const
 {
-	//we're not enemies of ourselves
+	// We're not enemies of ourselves
 	if(&(*teams)[index] == this) {
 		return false;
 	}
 
-	//if we have a team name, we are friends with anyone who has the same team name
+	// If we have a team name, we are friends
+	// with anyone who has the same team name
 	if(info_.team_name.empty() == false) {
 		return (*teams)[index].info_.team_name != info_.team_name;
 	}
 
-	//if enemies aren't listed, then everyone is an enemy
+	// If enemies aren't listed, then everyone is an enemy
 	if(info_.enemies.empty())
 		return true;
 
 	return std::find(info_.enemies.begin(),info_.enemies.end(),int(index+1)) != info_.enemies.end();
 }
 
-double team::aggression() const
+void team::change_controller(const std::string& controller)
 {
-	return aggression_;
+	team::team_info::CONTROLLER cid;
+	if (controller == "human")
+		cid = team::team_info::HUMAN;
+	else if (controller == "network")
+		cid = team::team_info::NETWORK;
+	else if (controller == "null")
+		cid = team::team_info::EMPTY;
+	else
+		cid = team::team_info::AI;
+
+	info_.controller = cid;
 }
 
-double team::caution() const
-{
-	return caution_;
-}
-
-bool team::is_human() const
-{
-	return info_.controller == team_info::HUMAN;
-}
-
-bool team::is_ai() const
-{
-	return info_.controller == team_info::AI;
-}
-
-bool team::is_network() const
-{
-	return info_.controller == team_info::NETWORK;
-}
-
-bool team::is_empty() const
-{
-	return info_.controller == team_info::EMPTY;
-}
-
-bool team::is_persistent() const
-{
-	return info_.persistent;
-}
-
-void team::make_human()
-{
-	info_.controller = team_info::HUMAN;
-}
-
-void team::make_ai()
-{
-	info_.controller = team_info::AI;
-}
-
-const std::string& team::team_name() const
-{
-	return info_.team_name;
-}
-
-const std::string& team::user_team_name() const
-{
-	return info_.user_team_name;
-}
-
-void team::change_team(const std::string& name,
-					   const std::string& user_name)
+void team::change_team(const std::string& name, const std::string& user_name)
 {
 	info_.team_name = name;
 	if (!user_name.empty())
@@ -625,7 +525,7 @@ void team::change_team(const std::string& name,
 		info_.user_team_name = name;
 	}
 
-	//reset the cache of allies for all teams
+	// Reset the cache of allies for all teams
 	if(teams != NULL) {
 		for(std::vector<team>::const_iterator i = teams->begin(); i != teams->end(); ++i) {
 			i->enemies_.clear();
@@ -633,15 +533,6 @@ void team::change_team(const std::string& name,
 	}
 }
 
-const std::string& team::save_id() const
-{
-	return info_.save_id;
-}
-
-const std::string& team::current_player() const{
-	return info_.current_player;
-}
-
 void team::set_objectives(const t_string& new_objectives, bool silently)
 {
 	info_.objectives = new_objectives;
@@ -649,97 +540,36 @@ void team::set_objectives(const t_string& new_objectives, bool silently)
 		info_.objectives_changed = true;
 }
 
-void team::reset_objectives_changed()
-{
-	info_.objectives_changed = false;
-}
-
-const t_string& team::objectives() const
-{
-	return info_.objectives;
-}
-
-bool team::objectives_changed() const
-{
-	return info_.objectives_changed;
-}
-
-const std::string& team::flag() const
-{
-	return info_.flag;
-}
-
-const std::string& team::ai_algorithm() const
-{
-	return info_.ai_algorithm;
-}
-
-const config& team::ai_parameters() const
-{
-	return aiparams_;
-}
-
-const config& team::ai_memory() const
-{
-       return info_.ai_memory_;
-}
-
 void team::set_ai_memory(const config& ai_mem){
-  //would perhaps be more efficient to allow writing to the memory directly,
-  //but this method comparmentalizes the functionality and protects against
-  //accidentally overwriting the memory
+  // It would perhaps be more efficient to allow writing to the memory directly,
+  // but this method comparmentalizes the functionality and protects against
+  // accidentally overwriting the memory
   info_.ai_memory_=ai_mem;
-  return;
 }
 
-void team::make_network()
+bool team::shrouded(const gamemap::location& loc) const
 {
-	info_.controller = team_info::NETWORK;
+	if(!teams)
+		return shroud_.value(loc.x+1,loc.y+1);
+
+	return shroud_.shared_value(ally_shroud(*teams),loc.x+1,loc.y+1);
 }
 
-double team::leader_value() const
+bool team::fogged(const gamemap::location& loc) const
 {
-	return info_.leader_value;
-}
+	if(shrouded(loc)) return true;
 
-double team::village_value() const
-{
-	return info_.village_value;
-}
+	if(!teams)
+		return fog_.value(loc.x+1,loc.y+1);
 
-int team::villages_per_scout() const
-{
-	return info_.villages_per_scout;
-}
-
-std::vector<team::target>& team::targets()
-{
-	return info_.targets;
-}
-
-bool team::shrouded(int x, int y) const
-{
-	if(!teams || !share_view())
-		return shroud_.value(x+1,y+1);
-
-	return shroud_.shared_value(ally_shroud(*teams),x+1,y+1);
-}
-
-bool team::fogged(int x, int y) const
-{
-	if(shrouded(x,y)) return true;
-
-	if(!teams || !share_view())
-		return fog_.value(x+1,y+1);
-
-	return fog_.shared_value(ally_fog(*teams),x+1,y+1);
+	return fog_.shared_value(ally_fog(*teams),loc.x+1,loc.y+1);
 }
 
 const std::vector<const team::shroud_map*>& team::ally_shroud(const std::vector<team>& teams) const
 {
 	if(ally_shroud_.empty()) {
 		for(size_t i = 0; i < teams.size(); ++i) {
-			if(!is_enemy(i+1)) {
+			if(!is_enemy(i + 1) && (&(teams[i]) == this || teams[i].share_view() || teams[i].share_maps())) {
 				ally_shroud_.push_back(&(teams[i].shroud_));
 			}
 		}
@@ -752,7 +582,7 @@ const std::vector<const team::shroud_map*>& team::ally_fog(const std::vector<tea
 {
 	if(ally_fog_.empty()) {
 		for(size_t i = 0; i < teams.size(); ++i) {
-			if(!is_enemy(i+1)) {
+			if(!is_enemy(i + 1) && (&(teams[i]) == this || teams[i].share_view())) {
 				ally_fog_.push_back(&(teams[i].fog_));
 			}
 		}
@@ -765,22 +595,22 @@ bool team::knows_about_team(size_t index) const
 {
 	const team& t = (*teams)[index];
 
-	//We know about our own team
+	// We know about our own team
 	if(this == &t) return true;
 
-	//If we aren't using shroud or fog, then we know about everyone
+	// If we aren't using shroud or fog, then we know about everyone
 	if(!uses_shroud() && !uses_fog()) return true;
 
-	//We don't know about enemies
+	// We don't know about enemies
 	if(is_enemy(index+1)) return false;
 
-	//We know our allies in multiplayer
+	// We know our allies in multiplayer
 	if(network::nconnections() > 0) return true;
 
-	//We know about allies we're sharing maps with
+	// We know about allies we're sharing maps with
 	if(share_maps() && t.uses_shroud()) return true;
 
-	//We know about allies we're sharing view with
+	// We know about allies we're sharing view with
 	if(share_view() && (t.uses_fog() || t.uses_shroud())) return true;
 
 	return false;
@@ -794,16 +624,6 @@ bool team::copy_ally_shroud()
 	return shroud_.copy_from(ally_shroud(*teams));
 }
 
-const std::string& team::music() const
-{
-	return info_.music;
-}
-
-int team::map_colour_to() const
-{
-	return info_.colour;
-}
-
 int team::nteams()
 {
 	if(teams == NULL) {
@@ -813,19 +633,6 @@ int team::nteams()
 	}
 }
 
-const std::set<gamemap::location> vacant_villages(const std::set<gamemap::location>& villages, const unit_map& units)
-{
-	std::set<gamemap::location> res;
-
-	for(std::set<gamemap::location>::const_iterator i = villages.begin(); i != villages.end(); ++i) {
-		if(units.count(*i) == 0) {
-			res.insert(*i);
-		}
-	}
-
-	return res;
-}
-
 bool is_observer()
 {
 	if(teams == NULL) {
@@ -848,7 +655,7 @@ void validate_side(int side)
 	}
 
 	if(side < 1 || side > int(teams->size())) {
-		throw game::game_error("invalid side(" + lexical_cast_default<std::string>(side) + ") found in unit definition");
+		throw game::game_error("invalid side(" + str_cast(side) + ") found in unit definition");
 	}
 }
 
@@ -883,6 +690,9 @@ void team::shroud_map::place(size_t x, size_t y)
 
 void team::shroud_map::reset()
 {
+	if(enabled_ == false)
+		return;
+
 	for(std::vector<std::vector<bool> >::iterator i = data_.begin(); i != data_.end(); ++i) {
 		std::fill(i->begin(),i->end(),false);
 	}
@@ -973,52 +783,41 @@ bool team::shroud_map::copy_from(const std::vector<const shroud_map*>& maps)
 std::map<int, color_range> team::team_color_range_;
 
 const color_range team::get_side_color_range(int side){
-  size_t index = size_t(get_side_colour_index(side));
-  std::map<int, color_range>::iterator p=team_color_range_.find(index);
-  std::map<int, color_range>::iterator gp=game_config::team_rgb_range.find(index);
+  std::string index = get_side_colour_index(side);
+  std::map<std::string, color_range>::iterator gp=game_config::team_rgb_range.find(index);
 
-  if(p != team_color_range_.end()){
-    return(p->second);
-  }else if(gp != game_config::team_rgb_range.end()){
+  if(gp != game_config::team_rgb_range.end()){
     return(gp->second);
   }
 
-  return(color_range(0x00FF0000,0x00FFFFFF,0x00000000));
+  return(color_range(0x00FF0000,0x00FFFFFF,0x00000000,0x00FF0000));
 }
 
-const Uint32 team::get_side_rgb(int side){
-  return(get_side_color_range(side).mid());
-}
-
-const Uint32 team::get_side_rgb_max(int side){
-  return(get_side_color_range(side).max());
-}
-
-const Uint32 team::get_side_rgb_min(int side){
-  return(get_side_color_range(side).min());
-}
-
-const SDL_Color team::get_side_colour(int side)
+const SDL_Color team::get_minimap_colour(int side)
 {
-	Uint32 rgb=get_side_rgb(side);
-	SDL_Color color={ (0x00FF0000 & rgb)>>16,
-				       (0x0000FF00 & rgb)>>8,
-				       (0x000000FF & rgb),
-	                               0} ;
-	return color;
+	// Note: use mid() instead of rep() unless
+	// high contrast is needed over a map or minimap!
+	return int_to_color(get_side_color_range(side).rep());
 }
 
-int team::get_side_colour_index(int side)
+std::string team::get_side_colour_index(int side)
 {
 	size_t index = size_t(side-1);
+
 	if(teams != NULL && index < teams->size()) {
-		const int side_map = (*teams)[index].map_colour_to();
-		if(side_map >= 1) {
+		const std::string side_map = (*teams)[index].map_colour_to();
+		if(side_map.size()) {
 			return side_map;
 		}
 	}
+	std::stringstream id;
+	id<<side;
+	return id.str();
+}
 
-	return side;
+std::string team::get_side_highlight(int side)
+{
+	return rgb2highlight(get_side_color_range(side+1).mid());
 }
 
 void team::log_recruitable(){
