@@ -1,35 +1,42 @@
-/* $Id: theme.cpp 15917 2007-02-19 15:12:03Z torangan $ */
+/* $Id: theme.cpp 24165 2008-02-29 23:36:38Z jhinrichs $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file theme.cpp
+//!
+
 #include "global.hpp"
 
 #include "config.hpp"
 #include "font.hpp"
+#include "gettext.hpp"
 #include "language.hpp"
 #include "log.hpp"
 #include "sdl_utils.hpp"
 #include "theme.hpp"
 #include "util.hpp"
-#include "wassert.hpp"
 #include "serialization/string_utils.hpp"
+#include "wml_exception.hpp"
 
+#include <cassert>
 #include <cstdlib>
 #include <sstream>
 
+#define DBG_DP LOG_STREAM(debug, display)
 #define LOG_DP LOG_STREAM(info, display)
+#define ERR_DP LOG_STREAM(err, display)
 
 namespace {
-	const SDL_Rect empty_rect = {0,0,0,0};
 	const int XDim = 1024;
 	const int YDim = 768;
 
@@ -37,8 +44,9 @@ namespace {
         const Uint32 DefaultFontRGB = 0x00C8C8C8;
 
 	_rect ref_rect = { 0, 0, 0, 0 };
+}
 
-	size_t compute(std::string expr, size_t ref1, size_t ref2=0 ) {
+static size_t compute(std::string expr, size_t ref1, size_t ref2=0 ) {
 		size_t ref = 0;
 		if (expr[0] == '=') {
 		  ref = ref1;
@@ -50,8 +58,8 @@ namespace {
 		return ref + atoi(expr.c_str());
 	}
 
-	// if x2 or y2 are not specified, use x1 and y1 values
-	_rect read_rect(const config& cfg) {
+	// If x2 or y2 are not specified, use x1 and y1 values
+static _rect read_rect(const config& cfg) {
 		_rect rect = { 0, 0, 0, 0 };
 		const std::vector<std::string> items = utils::split(cfg["rect"].c_str());
 		if(items.size() >= 1)
@@ -73,7 +81,7 @@ namespace {
 		return rect;
 	}
 
-	SDL_Rect read_sdl_rect(const config& cfg) {
+static SDL_Rect read_sdl_rect(const config& cfg) {
 		SDL_Rect sdlrect;
 		const _rect rect = read_rect(cfg);
 		sdlrect.x = rect.x1;
@@ -84,7 +92,7 @@ namespace {
 		return sdlrect;
 	}
 
-	std::string resolve_rect(const std::string& rect_str) {
+static std::string resolve_rect(const std::string& rect_str) {
 		_rect rect = { 0, 0, 0, 0 };
 		std::stringstream resolved;
 		const std::vector<std::string> items = utils::split(rect_str.c_str());
@@ -105,21 +113,23 @@ namespace {
 			resolved << "," << rect.y2;
 		}
 
-		// std::cerr << "Rect " << rect_str << "\t: " << resolved.str() << "\n";
+		// DBG_DP << "Rect " << rect_str << "\t: " << resolved.str() << "\n";
 
 		ref_rect = rect;
 		return resolved.str();
 	}
 
+namespace {
 	config empty_config = config();
+}
 
-	config& find_ref(const std::string& id, config& cfg, bool remove = false) {
+static config& find_ref(const std::string& id, config& cfg, bool remove = false) {
 		for(config::child_map::const_iterator i = cfg.all_children().begin();
 		    i != cfg.all_children().end(); i++) {
 			for (config::child_list::const_iterator j = i->second.begin();
 			     j != i->second.end(); j++) {
 				if ((**j)["id"] == id) {
-					//std::cerr << "Found a " << *(*i).first << "\n";
+					//DBG_DP << "Found a " << *(*i).first << "\n";
 					if (remove) {
 						const config* const res = cfg.find_child((*i).first,"id",id);
 						const size_t index = std::find((*i).second.begin(), (*i).second.end(),
@@ -149,7 +159,7 @@ namespace {
 	}
 #endif
 
-	void expand_partialresolution(config& dst_cfg, const config& top_cfg)
+static void expand_partialresolution(config& dst_cfg, const config& top_cfg)
 	{
 		std::vector<config> res_cfgs_;
 		// resolve all the partialresolutions
@@ -167,7 +177,7 @@ namespace {
 				parent_id = &((*parent)["inherits"]);
 			}
 
-			// add the parent resolution and apply all the modifications of its children
+			// Add the parent resolution and apply all the modifications of its children
 			res_cfgs_.push_back(*parent);
 			while(!parent_stack.empty()) {
 				//override attributes
@@ -207,19 +217,19 @@ namespace {
 				parent_stack.pop_back();
 			}
 		}
-		// add all the resolutions
+		// Add all the resolutions
 		const config::child_list& res_list = top_cfg.get_children("resolution");
 		for(config::child_list::const_iterator j = res_list.begin(); j != res_list.end(); ++j) {
 			dst_cfg.add_child("resolution", (**j));
 		}
-		// add all the resolved resolutions
+		// Add all the resolved resolutions
 		for(std::vector<config>::const_iterator k = res_cfgs_.begin(); k != res_cfgs_.end(); ++k) {
 			dst_cfg.add_child("resolution", (*k));
 		}
 		return;
 	}
 
-	void do_resolve_rects(const config& cfg, config& resolved_config, config* resol_cfg = NULL) {
+static void do_resolve_rects(const config& cfg, config& resolved_config, config* resol_cfg = NULL) {
 
 		// recursively resolve children
 		for(config::all_children_iterator i = cfg.ordered_begin(); i != cfg.ordered_end(); ++i) {
@@ -236,15 +246,15 @@ namespace {
 		// override default reference rect with "ref" parameter if any
 		if (!cfg["ref"].empty()) {
 			if (resol_cfg == NULL) {
-				std::cerr << "Use of ref= outside a [resolution] block\n";
+				ERR_DP << "Use of ref= outside a [resolution] block\n";
 			} else {
-				//std::cerr << ">> Looking for " << cfg["ref"] << "\n";
+				//DBG_DP << ">> Looking for " << cfg["ref"] << "\n";
 				const config ref = find_ref (cfg["ref"], *resol_cfg);
 
 				if (ref["id"].empty()) {
-					std::cerr << "Reference to non-existent rect id \"" << cfg["ref"] << "\"\n";
+					ERR_DP << "Reference to non-existent rect id \"" << cfg["ref"] << "\"\n";
 				} else if (ref["rect"].empty()) {
-					std::cerr << "Reference to id \"" << cfg["ref"] <<
+					ERR_DP << "Reference to id \"" << cfg["ref"] <<
 						"\" which does not have a \"rect\"\n";
 				} else {
 					ref_rect = read_rect(ref);
@@ -256,20 +266,49 @@ namespace {
 			resolved_config.values["rect"] = resolve_rect(cfg["rect"]);
 		}
 	}
-}
 
 theme::object::object() : location_modified_(false), loc_(empty_rect), relative_loc_(empty_rect),
                           last_screen_(empty_rect), xanchor_(object::FIXED), yanchor_(object::FIXED)
-{}
-
-theme::object::object(const config& cfg)
-                   : id_(cfg["id"]),
-					 loc_(read_sdl_rect(cfg)), relative_loc_(empty_rect),
-				     last_screen_(empty_rect),
-                     xanchor_(read_anchor(cfg["xanchor"])),
-					 yanchor_(read_anchor(cfg["yanchor"]))
 {
-	location_modified_ = false;
+}
+
+theme::object::object(const config& cfg) :
+		location_modified_(false), id_(cfg["id"]), loc_(read_sdl_rect(cfg)),
+		relative_loc_(empty_rect), last_screen_(empty_rect),
+		xanchor_(read_anchor(cfg["xanchor"])), yanchor_(read_anchor(cfg["yanchor"]))
+{
+}
+
+theme::tborder::tborder() :
+		size(0.0)
+{
+}
+
+theme::tborder::tborder(const config& cfg) :
+	size(lexical_cast_default<double>(cfg["border_size"], 0.0)),
+
+	background_image(cfg["background_image"]),
+	tile_image(cfg["tile_image"]),
+
+	corner_image_top_left(cfg["corner_image_top_left"]),
+	corner_image_bottom_left(cfg["corner_image_bottom_left"]),
+
+	corner_image_top_right_odd(cfg["corner_image_top_right_odd"]),
+	corner_image_top_right_even(cfg["corner_image_top_right_even"]),
+
+	corner_image_bottom_right_odd(cfg["corner_image_bottom_right_odd"]),
+	corner_image_bottom_right_even(cfg["corner_image_bottom_right_even"]),
+
+	border_image_left(cfg["border_image_left"]),
+	border_image_right(cfg["border_image_right"]),
+
+	border_image_top_odd(cfg["border_image_top_odd"]),
+	border_image_top_even(cfg["border_image_top_even"]),
+
+	border_image_bottom_odd(cfg["border_image_bottom_odd"]),
+	border_image_bottom_even(cfg["border_image_bottom_even"])
+{
+	VALIDATE(size >= 0.0 && size <= 0.5, _("border_size should be between 0.0 and 0.5."));
 }
 
 SDL_Rect& theme::object::location(const SDL_Rect& screen) const
@@ -297,7 +336,7 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 		relative_loc_.w = (loc_.w*screen.w)/XDim;
 		break;
 	default:
-		wassert(false);
+		assert(false);
 	}
 
 	switch(yanchor_) {
@@ -318,7 +357,7 @@ SDL_Rect& theme::object::location(const SDL_Rect& screen) const
 		relative_loc_.h = (loc_.h*screen.h)/YDim;
 		break;
 	default:
-		wassert(false);
+		assert(false);
 	}
 
 	relative_loc_.x = minimum<int>(relative_loc_.x,screen.w);
@@ -459,7 +498,9 @@ theme::menu::menu(const config& cfg) : object(cfg), context_(cfg["is_context_men
 						items_(utils::split(cfg["items"]))
 {}
 
-theme::theme(const config& cfg, const SDL_Rect& screen) {
+theme::theme(const config& cfg, const SDL_Rect& screen) :
+	theme_reset_("theme_reset")
+{
 	config tmp;
 	expand_partialresolution(tmp, cfg);
 	do_resolve_rects(tmp, cfg_);
@@ -498,6 +539,13 @@ bool theme::set_resolution(const SDL_Rect& screen)
 		return false;
 	}
 
+	std::map<std::string,std::string> title_stash;	
+	std::vector<theme::menu>::iterator m;
+	for (m = menus_.begin(); m != menus_.end(); ++m) {
+		if (!m->title().empty() && !m->get_id().empty())
+			title_stash[m->get_id()] = m->title();
+	}
+
 	panels_.clear();
 	labels_.clear();
 	status_.clear();
@@ -506,22 +554,26 @@ bool theme::set_resolution(const SDL_Rect& screen)
 	const config& cfg = **current;
 	add_object(cfg);
 
+	for (m = menus_.begin(); m != menus_.end(); ++m) {
+		if (title_stash.find(m->get_id()) != title_stash.end())
+			m->set_title(title_stash[m->get_id()]); 
+	}
+	
+	theme_reset_.notify_observers();
+	
 	return result;
 }
 
-theme::object& theme::add_object(const config& cfg){
-	static theme::object result;
+void theme::add_object(const config& cfg){
 
 	const config* const main_map_cfg = cfg.child("main_map");
 	if(main_map_cfg != NULL) {
 		main_map_ = object(*main_map_cfg);
-		result = main_map_;
 	}
 
 	const config* const mini_map_cfg = cfg.child("mini_map");
 	if(mini_map_cfg != NULL) {
 		mini_map_ = object(*mini_map_cfg);
-		result = mini_map_;
 	}
 
 	const config* const status_cfg = cfg.child("status");
@@ -537,7 +589,6 @@ theme::object& theme::add_object(const config& cfg){
 		} else {
 			unit_image_ = object();
 		}
-		result = unit_image_;
 	}
 
 	const config::child_list& panel_list = cfg.get_children("panel");
@@ -545,7 +596,6 @@ theme::object& theme::add_object(const config& cfg){
 		panel new_panel(**p);
 		set_object_location(new_panel, (**p)["rect"], (**p)["ref"]);
 		panels_.push_back(new_panel);
-		result = new_panel;
 	}
 
 	const config::child_list& label_list = cfg.get_children("label");
@@ -553,13 +603,12 @@ theme::object& theme::add_object(const config& cfg){
 		label new_label(**lb);
 		set_object_location(new_label, (**lb)["rect"], (**lb)["ref"]);
 		labels_.push_back(new_label);
-		result = new_label;
 	}
 
 	const config::child_list& menu_list = cfg.get_children("menu");
 	for(config::child_list::const_iterator m = menu_list.begin(); m != menu_list.end(); ++m) {
 		menu new_menu(**m);
-		LOG_DP << "adding menu: " << (new_menu.is_context() ? "is context" : "not context") << "\n";
+		DBG_DP << "adding menu: " << (new_menu.is_context() ? "is context" : "not context") << "\n";
 		if(new_menu.is_context())
 			context_ = new_menu;
 		else{
@@ -567,11 +616,15 @@ theme::object& theme::add_object(const config& cfg){
 			menus_.push_back(new_menu);
 		}
 
-		LOG_DP << "done adding menu...\n";
-		result = new_menu;
+		DBG_DP << "done adding menu...\n";
 	}
 
-	return result;
+	const config* const border_cfg = cfg.child("main_map_border");
+	if (border_cfg != NULL) {
+		border_ = tborder(*border_cfg);
+	} else {
+		border_ = tborder();
+	}
 }
 
 void theme::remove_object(std::string id){
@@ -610,8 +663,15 @@ void theme::set_object_location(theme::object& element, std::string rect_str, st
 }
 
 void theme::modify(const config* cfg){
+	std::map<std::string,std::string> title_stash;	
+	std::vector<theme::menu>::iterator m;
+	for (m = menus_.begin(); m != menus_.end(); ++m) {
+		if (!m->title().empty() && !m->get_id().empty())
+			title_stash[m->get_id()] = m->title();
+	}
+
 	{
-		//changes to existing theme objects
+		// changes to existing theme objects
 		const config::child_list& c = cfg->get_children("change");
 		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
 			std::string id = (**j)["id"];
@@ -622,20 +682,24 @@ void theme::modify(const config* cfg){
 			}
 		}
 	}
-	//adding new theme objects
+	// adding new theme objects
 	{
 		const config::child_list& c = cfg->get_children("add");
 		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
 			add_object(**j);
 		}
 	}
-	//removing existent theme objects
+	// removing existent theme objects
 	{
 		const config::child_list& c = cfg->get_children("remove");
 		for(config::child_list::const_iterator j = c.begin(); j != c.end(); ++j) {
 			remove_object((**j)["id"]);
 		}
 	}
+	for (m = menus_.begin(); m != menus_.end(); ++m) {
+		if (title_stash.find(m->get_id()) != title_stash.end())
+			m->set_title(title_stash[m->get_id()]); 
+	}
 }
 
 theme::object& theme::find_element(std::string id){
@@ -691,3 +755,25 @@ std::vector<std::string> theme::get_known_themes(){
         return(names);
 }
 
+theme::menu* theme::refresh_title(const std::string& id, const std::string& new_title){
+	theme::menu* res = NULL;
+
+	for (std::vector<theme::menu>::iterator m = menus_.begin(); m != menus_.end(); ++m){
+		if (m->get_id() == id) { 
+			res = &(*m); 
+			res->set_title(new_title);
+		}
+	}
+
+	return res;
+}
+
+theme::menu* theme::refresh_title2(const std::string& id, const std::string& title_tag){
+	std::string new_title = "";
+
+	config& cfg = find_ref(id, cfg_, false);
+	if (! cfg[title_tag].empty())
+		new_title = cfg[title_tag];
+
+	return refresh_title(id, new_title);
+}
