@@ -1,10 +1,11 @@
-/* $Id: thread.cpp 18614 2007-07-02 16:40:44Z mordante $ */
+/* $Id: thread.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -21,9 +22,8 @@
 #include <vector>
 
 #define ERR_G LOG_STREAM(err, general)
-namespace {
 
-int run_async_operation(void* data)
+static int run_async_operation(void* data)
 {
 	threading::async_operation* const op = reinterpret_cast<threading::async_operation*>(data);
 	op->run();
@@ -45,6 +45,8 @@ int run_async_operation(void* data)
 	return 0;
 }
 
+namespace {
+
 std::vector<SDL_Thread*> detached_threads;
 
 }
