@@ -1,10 +1,11 @@
-/* $Id: thread.hpp 8172 2005-09-06 13:53:14Z ott $ */
+/* $Id: thread.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -63,6 +64,8 @@ public:
 	void join();
 
 	void detach();
+
+	Uint32 get_id() { return SDL_GetThreadID(thread_); }
 private:
 	thread(const thread&);
 	void operator=(const thread&);
@@ -70,6 +73,7 @@ private:
 	SDL_Thread* thread_;
 };
 
+inline Uint32 get_current_thread_id() { return SDL_ThreadID(); }
 // Binary mutexes.
 //
 // Implements an interface to binary mutexes. This class only defines the
@@ -201,7 +205,8 @@ public:
 
 	enum RESULT { COMPLETED, ABORTED };
 
-	async_operation() : aborted_(false), finishedVar_(false) {}
+	async_operation() : 
+		aborted_(false), finished_(), finishedVar_(false), mutex_() {}
 	virtual ~async_operation() {}
 
 	RESULT execute(waiter& wait);
