@@ -1,10 +1,11 @@
-/* $Id: tooltips.cpp 8657 2005-10-28 21:03:07Z ydirson $ */
+/* $Id: tooltips.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -21,9 +22,7 @@
 
 #include <vector>
 
-namespace {
-
-bool rectangles_overlap(const SDL_Rect& a, const SDL_Rect& b)
+static bool rectangles_overlap(const SDL_Rect& a, const SDL_Rect& b)
 {
 	const bool xoverlap = a.x >= b.x && a.x < b.x + b.w ||
 	                      b.x >= a.x && b.x < a.x + a.w;
@@ -34,35 +33,35 @@ bool rectangles_overlap(const SDL_Rect& a, const SDL_Rect& b)
 	return xoverlap && yoverlap;
 }
 
+namespace {
+
 CVideo* video_ = NULL;
 
+static const int font_size = font::SIZE_SMALL;
+#ifdef USE_TINY_GUI
+static const int text_width = 260;
+#else
+static const int text_width = 400;
+#endif
+
 struct tooltip
 {
 	tooltip(const SDL_Rect& r, const std::string& msg) : rect(r), message(msg)
-	{
-		font::text_to_lines(message,60);
-	}
+	{}
 	SDL_Rect rect;
 	std::string message;
 };
 
-static const int font_size = font::SIZE_SMALL;
-
 std::vector<tooltip> tips;
 std::vector<tooltip>::const_iterator current_tooltip = tips.end();
 
 int tooltip_handle = 0;
 
-SDL_Rect current_rect;
 surface current_background = NULL;
 
-SDL_Rect get_text_size(const std::string& msg)
-{
-	SDL_Rect area = {0,0,10000,10000};
-	return font::draw_text(NULL,area,font_size,font::BLACK_COLOUR,msg,0,0);
 }
 
-void clear_tooltip()
+static void clear_tooltip()
 {
 	if(tooltip_handle != 0) {
 		font::remove_floating_label(tooltip_handle);
@@ -70,7 +69,7 @@ void clear_tooltip()
 	}
 }
 
-void show_tooltip(const tooltip& tip)
+static void show_tooltip(const tooltip& tip)
 {
 	if(video_ == NULL) {
 		return;
@@ -80,8 +79,16 @@ void show_tooltip(const tooltip& tip)
 
 	const SDL_Color bgcolour = {0,0,0,128};
 	SDL_Rect area = screen_area();
-	tooltip_handle = font::add_floating_label(tip.message,font_size,font::NORMAL_COLOUR,
-	                                          0,0,0,0,-1,area,font::LEFT_ALIGN,&bgcolour,10);
+
+#ifdef USE_TINY_GUI
+	unsigned int border = 2;
+#else
+	unsigned int border = 10;
+#endif
+
+	const std::string wrapped_message = font::word_wrap_text(tip.message, font_size, text_width);
+	tooltip_handle = font::add_floating_label(wrapped_message,font_size,font::NORMAL_COLOUR,
+	                                          0,0,0,0,-1,area,font::LEFT_ALIGN,&bgcolour,border);
 
 	SDL_Rect rect = font::get_floating_label_rect(tooltip_handle);
 
@@ -102,8 +109,6 @@ void show_tooltip(const tooltip& tip)
 	font::move_floating_label(tooltip_handle,rect.x,rect.y);
 }
 
-}
-
 namespace tooltips {
 
 manager::manager(CVideo& video)
@@ -129,9 +134,11 @@ void clear_tooltips(const SDL_Rect& rect)
 {
 	for(std::vector<tooltip>::iterator i = tips.begin(); i != tips.end(); ) {
 		if(rectangles_overlap(i->rect,rect)) {
+			if (i==current_tooltip) {
+				clear_tooltip();
+			}
 			i = tips.erase(i);
 			current_tooltip = tips.end();
-			clear_tooltip();
 		} else {
 			++i;
 		}
@@ -169,11 +176,4 @@ void process(int mousex, int mousey)
 	current_tooltip = tips.end();
 }
 
-SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
-                   const SDL_Color& colour, const std::string& text,
-                   int x, int y)
-{
-	return font::draw_text(gui, area, size, colour, text, x, y, true);
-}
-
 }
