@@ -1,10 +1,11 @@
-/* $Id: tooltips.hpp 7533 2005-07-20 08:22:37Z ott $ */
+/* $Id: tooltips.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -33,13 +34,6 @@ void clear_tooltips(const SDL_Rect& rect);
 void add_tooltip(const SDL_Rect& rect, const std::string& message);
 void process(int mousex, int mousey);
 
-
-//a function exactly the same as font::draw_text, but will also register
-//a tooltip
-SDL_Rect draw_text(CVideo* gui, const SDL_Rect& area, int size,
-                   const SDL_Color& colour, const std::string& text,
-                   int x, int y);
-
 }
 
 #endif
