@@ -1,17 +1,21 @@
-/* $Id: tstring.cpp 19257 2007-07-31 03:52:34Z sapient $ */
+/* $Id: tstring.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
    Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
-   Copyright (C) 2005 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2005 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file tstring.cpp
+//! Routines for translatable strings.
+
 #include "global.hpp"
 
 #include <sstream>
@@ -38,60 +42,26 @@ namespace {
 
 t_string::walker::walker(const t_string& string) :
 	string_(string.value_),
-	begin_(0)
+	begin_(0),
+	end_(string_.size()),
+	textdomain_(),
+	translatable_(false)
 {
-	if(!string.translatable_) {
-		begin_ = 0;
-		end_ = string_.size();
-		translatable_ = false;
-	} else {
+	if(string.translatable_) {
 		update();
 	}
 }
 
 t_string::walker::walker(const std::string& string) :
 	string_(string),
-	begin_(0)
+	begin_(0),
+	end_(string_.size()),
+	textdomain_(),
+	translatable_(false)
 {
 	update();
 }
 
-void t_string::walker::next()
-{
-	begin_ = end_;
-	update();
-}
-
-bool t_string::walker::eos() const
-{
-	return begin_ == string_.size();
-}
-
-bool t_string::walker::last() const
-{
-	return end_ == string_.size();
-}
-
-bool t_string::walker::translatable() const
-{
-	return translatable_;
-}
-
-const std::string& t_string::walker::textdomain() const
-{
-	return textdomain_;
-}
-
-std::string::const_iterator t_string::walker::begin() const
-{
-	return string_.begin() + begin_;
-}
-
-std::string::const_iterator t_string::walker::end() const
-{
-	return string_.begin() + end_;
-}
-
 void t_string::walker::update()
 {
 	unsigned int id;
@@ -108,7 +78,7 @@ void t_string::walker::update()
 			string_.find(TEXTDOMAIN_SEPARATOR, begin_ + 1);
 
 		if(textdomain_end == std::string::npos || textdomain_end >= string_.size() - 1) {
-			ERR_CF << "Error: invalid string\n";
+			ERR_CF << "Error: invalid string: " << string_ << "\n";
 			begin_ = string_.size();
 			return;
 		}
@@ -125,7 +95,7 @@ void t_string::walker::update()
 	}
 	case ID_TRANSLATABLE_PART:
 		if(begin_ + 3 >= string_.size()) {
-			ERR_CF << "Error: invalid string\n";
+			ERR_CF << "Error: invalid string: " << string_ << "\n";
 			begin_ = string_.size();
 			return;
 		}
@@ -135,7 +105,7 @@ void t_string::walker::update()
 
 		id = string_[begin_ + 1] + string_[begin_ + 2] * 256;
 		if(id >= id_to_textdomain.size()) {
-			ERR_CF << "Error: invalid string\n";
+			ERR_CF << "Error: invalid string: " << string_ << "\n";
 			begin_ = string_.size();
 			return;
 		}
@@ -151,7 +121,7 @@ void t_string::walker::update()
 			end_ = string_.size();
 
 		if(end_ <= begin_ + 1) {
-			ERR_CF << "Error: invalid string\n";
+			ERR_CF << "Error: invalid string: " << string_ << "\n";
 			begin_ = string_.size();
 			return;
 		}
@@ -170,29 +140,34 @@ void t_string::walker::update()
 }
 
 t_string::t_string() :
+	value_(),
+	translated_value_(),
 	translatable_(false),
-	value_()
+	last_untranslatable_(false)
 {
 }
 
 t_string::t_string(const t_string& string) :
-	translatable_(string.translatable_),
-	last_untranslatable_(string.last_untranslatable_),
 	value_(string.value_),
-	translated_value_(string.translated_value_)
+	translated_value_(string.translated_value_),
+	translatable_(string.translatable_),
+	last_untranslatable_(string.last_untranslatable_)
 {
 }
 
 t_string::t_string(const std::string& string) :
+	value_(string),
+	translated_value_(),
 	translatable_(false),
-	value_(string)
+	last_untranslatable_(false)
 {
 }
 
 t_string::t_string(const std::string& string, const std::string& textdomain) :
+	value_(1, ID_TRANSLATABLE_PART),
+	translated_value_(),
 	translatable_(true),
-	last_untranslatable_(false),
-	value_(1, ID_TRANSLATABLE_PART)
+	last_untranslatable_(false)
 {
 	std::map<std::string, unsigned int>::const_iterator idi = textdomain_to_id.find(textdomain);
 	unsigned int id;
@@ -211,8 +186,10 @@ t_string::t_string(const std::string& string, const std::string& textdomain) :
 }
 
 t_string::t_string(const char* string) :
+	value_(string),
+	translated_value_(),
 	translatable_(false),
-	value_(string)
+	last_untranslatable_(false)
 {
 }
 
@@ -302,21 +279,6 @@ t_string& t_string::operator=(const char* string)
 	return *this;
 }
 
-bool t_string::operator==(const t_string& string) const
-{
-	return string.translatable_ == translatable_ && string.value_ == value_;
-}
-
-bool t_string::operator==(const std::string& string) const
-{
-	return !translatable_ && value_ == string;
-}
-
-bool t_string::operator==(const char* string) const
-{
-	return !translatable_ && value_ == string;
-}
-
 t_string t_string::operator+(const t_string& string) const
 {
 	t_string res(*this);
@@ -420,29 +382,6 @@ t_string& t_string::operator+=(const char* string)
 	return *this;
 }
 
-bool t_string::operator!=(const t_string& string) const { return !(*this == string); }
-bool t_string::operator!=(const std::string& string) const { return !(*this == string); }
-bool t_string::operator!=(const char* string) const { return !(*this == string); }
-
-bool t_string::operator<(const t_string& string) const
-{
-	return value_ < string.value_;
-}
-
-bool t_string::empty() const
-{
-	return value_.empty();
-}
-
-std::string::size_type t_string::size() const
-{
-	return str().size();
-}
-
-t_string::operator const std::string&() const
-{
-	return str();
-}
 
 const std::string& t_string::str() const
 {
@@ -465,20 +404,6 @@ const std::string& t_string::str() const
 	return translated_value_;
 }
 
-const char* t_string::c_str() const
-{
-	return str().c_str();
-}
-
-const std::string& t_string::value() const
-{
-	return value_;
-}
-
-void t_string::reset_translation() const
-{
-	translated_value_ = "";
-}
 
 void t_string::add_textdomain(const std::string& name, const std::string& path)
 {
@@ -495,33 +420,3 @@ std::ostream& operator<<(std::ostream& stream, const t_string& string)
 	return stream;
 }
 
-bool operator==(const std::string& a, const t_string& b)
-{
-	return b == a;
-}
-
-bool operator==(const char* a, const t_string& b)
-{
-	return b == a;
-}
-
-bool operator!=(const std::string& a, const t_string& b)
-{
-	return b != a;
-}
-
-bool operator!=(const char* a, const t_string& b)
-{
-	return b != a;
-}
-
-t_string operator+(const std::string& a, const t_string& b)
-{
-	return t_string(a) + b;
-}
-
-t_string operator+(const char*a, const t_string& b)
-{
-	return t_string(a) + b;
-}
-
