@@ -1,4 +1,4 @@
-/* $Id: unit.cpp 24125 2008-02-27 19:28:43Z boucman $ */
+/* $Id: unit.cpp 27191 2008-06-15 09:46:11Z boucman $ */
 /*
    Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -501,7 +501,12 @@ void unit::advance_to(const unit_type* t, bool use_traits, game_state* state)
 	std::string profile;
 	if (type() != NULL)
 	{
-		specific_profile = (cfg_["profile"] != type()->cfg_["profile"]);
+		// Update 2008-02-23 by Shadowmaster: make the test for specific profile take
+		// into account advancing unit's gender and variation to avoid issues like in Elvish Archer+female, which
+		// was considering its profile as "specific" when advancing, as it isn't the same as the
+		// male profile
+		specific_profile = (cfg_["profile"] != type()->get_gender_unit_type(gender_).get_variation(variation_).cfg_["profile"]);
+		
 		if (specific_profile)
 		{
 			profile = cfg_["profile"];
@@ -1576,7 +1581,7 @@ void unit::set_standing(const gamemap::location& loc, bool with_bars)
 	for (unsigned int i = 0; i < 6; i++) {
 		disp->invalidate(arr[i]);
 	}
-	if (preferences::show_standing_animations()) {
+	if (preferences::show_standing_animations()&& !incapacitated()) {
 		start_animation(INT_MAX,loc,choose_animation(*disp,loc,"standing"),with_bars,true,"",0,STATE_STANDING);
 	} else {
 		start_animation(INT_MAX,loc,choose_animation(*disp,loc,"_disabled_"),with_bars,true,"",0,STATE_STANDING);
