@@ -1,4 +1,4 @@
-/* $Id: unit_animation.cpp 24125 2008-02-27 19:28:43Z boucman $ */
+/* $Id: unit_animation.cpp 26046 2008-04-23 20:42:03Z boucman $ */
 /*
    Copyright (C) 2006 - 2008 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -305,12 +305,8 @@ void unit_animation::fill_initial_animations( std::vector<unit_animation> & anim
 	for(itor = animation_base.begin() ; itor != animation_base.end() ; itor++ ) {
 		unit_animation tmp_anim = *itor;
 		// provide all default anims
-		animations.push_back(*itor);
-
-		tmp_anim = *itor;
-		//no event, providing a catch all anim
 		tmp_anim.unit_anim_.remove_frames_after(600);
-		animations.push_back(tmp_anim);
+		animations.push_back(*itor);
 
 		tmp_anim = *itor;
 		tmp_anim.event_ = utils::split("standing");
@@ -746,6 +742,7 @@ int unit_animation::get_begin_time() const
 void unit_animation::start_animation(int start_time,const gamemap::location &src, const gamemap::location &dst, bool cycles, const std::string text, const Uint32 text_color,const bool accelerate)
 {
 		unit_anim_.accelerate = accelerate;
+		new_animation_frame();
 	unit_anim_.start_animation(start_time, src, dst, cycles);
 	if(!text.empty()) {
 		crude_animation crude_build;
