@@ -1,4 +1,4 @@
-/* $Id: unit_animation.hpp 23980 2008-02-20 22:22:22Z boucman $ */
+/* $Id: unit_animation.hpp 24950 2008-03-21 19:23:00Z brunowolff $ */
 /*
    Copyright (C) 2006 - 2008 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -31,7 +31,7 @@ class unit_animation
 		//! Shouldn't be used so only declared.
 		unit_animation();
 	public:
-		typedef enum { MATCH_FAIL=-2 , DEFAULT_ANIM=-1};
+		typedef enum { MATCH_FAIL=-2 , DEFAULT_ANIM=-1} variation_type;
 		typedef enum { HIT, MISS, KILL, INVALID} hit_type;
 		static void fill_initial_animations( std::vector<unit_animation> & animations, const config & cfg);
 		static void add_anims( std::vector<unit_animation> & animations, const config & cfg);
