@@ -1,25 +1,35 @@
-/* $Id$ */
+/* $Id: upload_log.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2005 by Rusty Russell <rusty@rustcorp.com.au>
+   Copyright (C) 2005 - 2008 by Rusty Russell <rusty@rustcorp.com.au>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
+
+//! @file upload_log.cpp
+//! Manage logfiles for uploading as feedback, e.g.\ for champaign-balancing.
+
 #include "global.hpp"
 
 #define GETTEXT_DOMAIN "wesnoth"
 
+#include "config.hpp"
+#include "construct_dialog.hpp"
+#include "display.hpp"
 #include "game_config.hpp"
+#include "game_preferences.hpp"
+#include "gamestatus.hpp"
 #include "gettext.hpp"
 #include "filesystem.hpp"
-#include "preferences.hpp"
 #include "serialization/parser.hpp"
-#include "show_dialog.hpp"
+#include "team.hpp"
+#include "tstring.hpp"
 #include "upload_log.hpp"
 #include "wesconfig.h"
 #include "wml_separators.hpp"
@@ -45,8 +55,9 @@ upload_log::manager::~manager()
 
 static void send_string(TCPsocket sock, const std::string &str)
 {
-	if (SDLNet_TCP_Send(sock, (void *)str.c_str(), str.length())
-		!= (int)str.length()) {
+	if (SDLNet_TCP_Send(sock, const_cast<void*>(static_cast<const void*>(
+			str.c_str())), str.length()) != static_cast<int>(str.length())) {
+
 		throw network::error("");
 	}
 }
@@ -57,7 +68,7 @@ static void send_string(TCPsocket sock, const std::string &str)
 static int upload_logs(void *_ti)
 {
 	TCPsocket sock = NULL;
-	upload_log::thread_info *ti = (upload_log::thread_info *)_ti;
+	upload_log::thread_info *ti = static_cast<upload_log::thread_info*>(_ti);
 
 	const char *header =
 		"POST " TARGET_URL " HTTP/1.1\n"
@@ -116,8 +127,8 @@ upload_log::upload_log(bool enable) : game_(NULL), enabled_(enable)
 {
 	filename_ = next_filename(get_upload_dir(), 100);
 	if (preferences::upload_log() && !thread_.t) {
-		// Thread can outlive us; it uploads everything up to the next
-		// filename, and unsets thread_.t when it's finished.
+		// Thread can outlive us; it uploads everything up to the
+		// next filename, and unsets thread_.t when it's finished.
 		thread_.lastfile = filename_;
 		thread_.t = new threading::thread(upload_logs, &thread_);
 	}
@@ -178,9 +189,9 @@ void upload_log::start(game_state &state, const team &team,
 	if (game_finished(game_))
 		config_.add_child("game", *game_);
 
-
-	if (game_)
-		delete game_;
+	// Start could be called more than once,
+	// so delete game_ to prevent memory leak
+	delete game_;
 	game_ = new config();
 	(*game_)["time"] = lexical_cast<std::string>(SDL_GetTicks() / 1000);
 	(*game_)["campaign"] = state.campaign_define;
@@ -193,15 +204,15 @@ void upload_log::start(game_state &state, const team &team,
 	(*game_)["gold"] = lexical_cast<std::string>(team.gold());
 	(*game_)["num_turns"] = lexical_cast<std::string>(num_turns);
 
-	// We seem to have to walk the map to find some units, and the player's
-	// available_units for the rest.
+	// We seem to have to walk the map to find some units,
+	// and the player's available_units for the rest.
 	for (unit_map::const_iterator un = units.begin(); un != units.end(); ++un){
 		if (un->second.side() == team_number) {
 			all_units.push_back(&un->second);
 		}
 	}
 
-	// FIXME: Assumes first player is "us"; is that valid?
+	//! @todo FIXME: Assumes first player is "us"; is that valid?
 	player_info &player = state.players.begin()->second;
 	for (std::vector<unit>::iterator it = player.available_units.begin();
 		 it != player.available_units.end();
@@ -278,25 +289,18 @@ void upload_log::quit(int turn)
 	add_game_result("quit", turn);
 }
 
+//! Ask user for permission to upload his game-stats.
 void upload_log_dialog::show_beg_dialog(display& disp)
 {
-	std::vector<gui::check_item> options;
-	gui::check_item beg_check = gui::check_item(_("Enable summary uploads"),
-									  preferences::upload_log());
-	beg_check.align = gui::LEFT_ALIGN;
-	options.push_back(beg_check);
-
-	std::string msg = std::string(_("Wesnoth relies on volunteers like yourself for feedback, especially beginners and new players.  Wesnoth keeps summaries of your games: you can help us improve game play by giving permission to send these summaries (anonymously) to wesnoth.org.\n"))
+	std::string msg = std::string(_("Wesnoth relies on volunteers like yourself for feedback, especially beginners and new players. Wesnoth keeps summaries of your games: you can help us improve game play by giving permission to send these summaries (anonymously) to wesnoth.org.\n"))
 		+ " \n`" + _("Summaries are stored here:")
 		+ " \n`~" + get_upload_dir() + "\n \n`"
 		+ _("You can view the results at:") + "\n`~"
 		+ "http://stats.wesnoth.org/?" + preferences::upload_id() + "\n \n";
+	gui::dialog d(disp, _("Help us make Wesnoth better for you!"), msg, gui::OK_ONLY);
 
-
-	gui::show_dialog(disp, NULL,
-					 _("Help us make Wesnoth better for you!"),
-					 msg,
-					 gui::OK_ONLY,
-					 NULL, NULL, "", NULL, 0, NULL, &options);
-	preferences::set_upload_log(options.front().checked);
+	d.add_option(_("Enable summary uploads"),
+		preferences::upload_log(), gui::dialog::BUTTON_CHECKBOX_LEFT);
+	d.show();
+	preferences::set_upload_log(d.option_checked());
 }
