@@ -1,10 +1,11 @@
-/* $Id: button.hpp 15790 2007-02-14 23:55:53Z jhinrichs $ */
+/* $Id: button.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -35,7 +36,8 @@ public:
 	enum SPACE_CONSUMPTION { DEFAULT_SPACE, MINIMUM_SPACE };
 
 	button(CVideo& video, const std::string& label, TYPE type=TYPE_PRESS,
-	       std::string button_image="", SPACE_CONSUMPTION spacing=DEFAULT_SPACE);
+	       std::string button_image="", SPACE_CONSUMPTION spacing=DEFAULT_SPACE,
+		   const bool auto_join=true);
 
 	virtual ~button() {}
 	void set_check(bool check);
@@ -55,6 +57,8 @@ protected:
 	virtual void mouse_up(const SDL_MouseButtonEvent& event);
 	virtual void draw_contents();
 
+	TYPE type_;
+
 private:
 
 	void calculate_size();
@@ -68,8 +72,6 @@ private:
 	enum STATE { UNINIT, NORMAL, ACTIVE, PRESSED, PRESSED_ACTIVE };
 	STATE state_;
 
-	TYPE type_;
-
 	bool pressed_;
 
 	SPACE_CONSUMPTION spacing_;
