@@ -1,10 +1,11 @@
-/* $Id: combo.cpp 11723 2006-05-30 03:00:33Z sapient $ */
+/* $Id: combo.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -18,6 +19,16 @@
 #include "show_dialog.hpp"
 #include "video.hpp"
 
+namespace {
+
+#ifdef USE_TINY_GUI
+	const std::string empty_combo_label = "           ";
+#else
+	const std::string empty_combo_label = "";
+#endif
+
+}
+
 namespace gui {
 
 const int font_size = font::SIZE_SMALL;
@@ -25,7 +36,7 @@ const int horizontal_padding = 10;
 const int vertical_padding = 10;
 
 combo::combo(display& disp, const std::vector<std::string>& items)
-	: button(disp.video(), items.empty() ? "" : items[0]),
+	: button(disp.video(), items.empty() ? empty_combo_label : items[0]),
 	  items_(items), selected_(0), oldSelected_(0), disp_(&disp)
 {
 }
@@ -76,8 +87,8 @@ void combo::process_event()
 	if (!pressed())
 		return;
 	SDL_Rect const &loc = location();
-	set_selected_internal(gui::show_dialog2(*disp_, NULL, "", "", gui::MESSAGE, &items_,
-	                                       NULL, "", NULL, -1, NULL, NULL, loc.x, loc.y + loc.h));
+	set_selected_internal(gui::show_dialog(*disp_, NULL, "", "", gui::MESSAGE, &items_,
+	                                       NULL, "", NULL, -1, NULL, loc.x, loc.y + loc.h));
 }
 
 }
