@@ -1,10 +1,11 @@
-/* $Id: label.hpp 7120 2005-06-04 19:16:06Z ott $ */
+/* $Id: label.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2004 by Philippe Plantier <ayin@anathas.org>
+   Copyright (C) 2004 - 2008 by Philippe Plantier <ayin@anathas.org>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -24,7 +25,7 @@ class label : public widget
 {
 public:
 	label(CVideo& video, const std::string& text, int size=font::SIZE_NORMAL,
-			const SDL_Color& colour=font::NORMAL_COLOUR);
+			const SDL_Color& colour=font::NORMAL_COLOUR, const bool auto_join=true);
 	const std::string& set_text(const std::string& text);
 	const std::string& get_text() const;
 
