@@ -1,10 +1,11 @@
-/* $Id: menu.cpp 14018 2006-10-19 09:15:05Z isaac $ */
+/* $Id: menu.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -15,10 +16,12 @@
 
 #include "widgets/menu.hpp"
 
-#include "language.hpp"
+#include "game_config.hpp"
 #include "font.hpp"
 #include "image.hpp"
+#include "language.hpp"
 #include "marked-up_text.hpp"
+#include "sound.hpp"
 #include "sdl_utils.hpp"
 #include "util.hpp"
 #include "video.hpp"
@@ -99,21 +102,25 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 		return true;
 	}
 
-	const std::string& item1 = row1.fields[column];
-	const std::string& item2 = row2.fields[column];
 	if(alpha_sort_.count(column) == 1) {
+		const std::string& item1 = font::del_tags(row1.fields[column]);
+		const std::string& item2 = font::del_tags(row2.fields[column]);
+
 		std::string::const_iterator begin1 = item1.begin(), end1 = item1.end(),
 		                            begin2 = item2.begin(), end2 = item2.end();
-		while(begin1 != end1 && (font::is_format_char(*begin1) || is_wml_separator(*begin1))) {
+		while(begin1 != end1 && is_wml_separator(*begin1)) {
 			++begin1;
 		}
 
-		while(begin2 != end2 && (font::is_format_char(*begin2) || is_wml_separator(*begin2))) {
+		while(begin2 != end2 && is_wml_separator(*begin2)) {
 			++begin2;
 		}
 
 		return std::lexicographical_compare(begin1,end1,begin2,end2,chars_less_insensitive);
 	} else if(numeric_sort_.count(column) == 1) {
+		const std::string& item1 = font::del_tags(row1.fields[column]);
+		const std::string& item2 = font::del_tags(row2.fields[column]);
+
 		const char* a = item1.c_str();
 		const char* b = item2.c_str();
 
@@ -147,17 +154,17 @@ bool menu::basic_sorter::less(int column, const item& row1, const item& row2) co
 
 menu::menu(CVideo& video, const std::vector<std::string>& items,
            bool click_selects, int max_height, int max_width,
-		   const sorter* sorter_obj, style *menu_style)
-        : scrollarea(video),
+		   const sorter* sorter_obj, style *menu_style, const bool auto_join)
+        : scrollarea(video, auto_join), silent_(false),
           max_height_(max_height), max_width_(max_width), max_items_(-1), item_height_(-1),
 		  heading_height_(-1),
 	  cur_help_(-1,-1), help_string_(-1),
-	  selected_(0), click_selects_(click_selects),
+	  selected_(0), click_selects_(click_selects), out_(false),
 	  previous_button_(true), show_result_(false),
 	  double_clicked_(false),
 	  num_selects_(true),
 	  ignore_next_doubleclick_(false),
-	  last_was_doubleclick_(false),
+	  last_was_doubleclick_(false), use_ellipsis_(false),
 	  sorter_(sorter_obj), sortby_(-1), sortreversed_(false), highlight_heading_(-1)
 {
 	style_ = (menu_style) ? menu_style : &default_style;
@@ -237,7 +244,7 @@ void menu::do_sort()
 	recalculate_pos();
 
 	if(selectid >= 0 && selectid < int(item_pos_.size())) {
-		move_selection(selectid,NO_MOVE_VIEWPORT);
+		move_selection_to(selectid, true, NO_MOVE_VIEWPORT);
 	}
 
 	set_dirty();
@@ -296,16 +303,20 @@ void menu::update_size()
 	    i < i_end; ++i)
 		h += get_item_rect(i).h;
 	h = maximum(h, height());
-	if (max_height_ > 0 && h > (unsigned)max_height_)
+	if (max_height_ > 0 && h > static_cast<unsigned>(max_height_)) {
 		h = max_height_;
+	}
 
+	use_ellipsis_ = false;
 	std::vector<int> const &widths = column_widths();
 	unsigned w = std::accumulate(widths.begin(), widths.end(), 0);
 	if (items_.size() > max_items_onscreen())
 		w += scrollbar_width();
 	w = maximum(w, width());
-	if (max_width_ > 0 && w > (unsigned)max_width_)
+	if (max_width_ > 0 && w > static_cast<unsigned>(max_width_)) {
+		use_ellipsis_ = true;
 		w = max_width_;
+	}
 
 	update_scrollbar_grip_height();
 	set_measurements(w, h);
@@ -380,6 +391,8 @@ void menu::set_heading(const std::vector<std::string>& heading)
 
 void menu::set_items(const std::vector<std::string>& items, bool strip_spaces, bool keep_viewport)
 {
+
+	const bool scrolled_to_max = (has_scrollbar() && get_position() == get_max_position());
 	items_.clear();
 	item_pos_.clear();
 	itemRects_.clear();
@@ -395,6 +408,8 @@ void menu::set_items(const std::vector<std::string>& items, bool strip_spaces, b
 	fill_items(items, strip_spaces);
 	if(!keep_viewport) {
 		set_position(0);
+	} else if(scrolled_to_max) {
+		set_position(get_max_position());
 	}
 
 	update_scrollbar_grip_height();
@@ -416,6 +431,9 @@ void menu::set_max_height(const int new_max_height)
 void menu::set_max_width(const int new_max_width)
 {
 	max_width_ = new_max_width;
+	itemRects_.clear();
+	column_widths_.clear();
+	update_size();
 }
 
 size_t menu::max_items_onscreen() const
@@ -424,7 +442,12 @@ size_t menu::max_items_onscreen() const
 		return size_t(max_items_);
 	}
 
+#ifdef USE_TINY_GUI
+	const size_t max_height = (max_height_ == -1 ? (video().gety()*55)/100 : max_height_) - heading_height();
+#else
 	const size_t max_height = (max_height_ == -1 ? (video().gety()*66)/100 : max_height_) - heading_height();
+#endif
+
 	std::vector<int> heights;
 	size_t n;
 	for(n = 0; n != items_.size(); ++n) {
@@ -450,43 +473,59 @@ void menu::adjust_viewport_to_selection()
 	adjust_position(selected_);
 }
 
+void menu::set_selection_pos(size_t new_selected, bool silent, SELECTION_MOVE_VIEWPORT move_viewport)
+{
+	if (new_selected >= items_.size())
+		return;
+
+	bool changed = false;
+	if (new_selected != selected_) {
+		invalidate_row_pos(selected_);
+		invalidate_row_pos(new_selected);
+		selected_ = new_selected;
+		changed = true;
+	}
+
+	if(move_viewport == MOVE_VIEWPORT) {
+		adjust_viewport_to_selection();
+		if(!silent_ && !silent && changed) {
+			sound::play_UI_sound(game_config::sounds::menu_select);
+		}
+	}
+}
+
 void menu::move_selection_up(size_t dep)
 {
 	set_selection_pos(selected_ > dep ? selected_ - dep : 0);
 }
 
-void menu::reset_selection()
-{
-	set_selection_pos(0);
-}
-
 void menu::move_selection_down(size_t dep)
 {
 	size_t nb_items = items_.size();
 	set_selection_pos(selected_ + dep >= nb_items ? nb_items - 1 : selected_ + dep);
 }
 
-void menu::set_selection_pos(size_t new_selected, SELECTION_MOVE_VIEWPORT move_viewport)
+// private function with control over sound and viewport
+void menu::move_selection_to(size_t id, bool silent, SELECTION_MOVE_VIEWPORT move_viewport)
 {
-	if (new_selected == selected_ || new_selected >= items_.size())
-		return;
-
-	invalidate_row_pos(selected_);
-	invalidate_row_pos(new_selected);
-	selected_ = new_selected;
-
-	if(move_viewport == MOVE_VIEWPORT) {
-		adjust_viewport_to_selection();
+	if(id < item_pos_.size()) {
+		set_selection_pos(item_pos_[id], silent, move_viewport);
 	}
 }
 
-void menu::move_selection(size_t id, SELECTION_MOVE_VIEWPORT move_viewport)
+// public function
+void menu::move_selection(size_t id)
 {
 	if(id < item_pos_.size()) {
-		set_selection_pos(item_pos_[id],move_viewport);
+		set_selection_pos(item_pos_[id], true, MOVE_VIEWPORT);
 	}
 }
 
+void menu::reset_selection()
+{
+	set_selection_pos(0);
+}
+
 void menu::key_press(SDLKey key)
 {
 	if (!click_selects_) {
@@ -521,19 +560,52 @@ void menu::key_press(SDLKey key)
 		set_selection_pos(key - SDLK_1);
 }
 
+bool menu::requires_event_focus(const SDL_Event* event) const
+{
+	if(!focus_ || height() == 0 || hidden()) {
+		return false;
+	}
+	if(event == NULL) {
+		//when event is not specified, signal that focus may be desired later
+		return true;
+	}
+
+	if(event->type == SDL_KEYDOWN) {
+		SDLKey key = event->key.keysym.sym;
+		if (!click_selects_) {
+			switch(key) {
+			case SDLK_UP:
+			case SDLK_DOWN:
+			case SDLK_PAGEUP:
+			case SDLK_PAGEDOWN:
+			case SDLK_HOME:
+			case SDLK_END:
+				return true;
+			default:
+				break;
+			}
+		}
+		if (num_selects_ && key >= SDLK_1 && key <= SDLK_9) {
+			return true;
+		}
+	}
+	//mouse events are processed regardless of focus
+	return false;
+}
+
 void menu::handle_event(const SDL_Event& event)
 {
 	scrollarea::handle_event(event);
-	if (hidden())
+	if (height()==0 || hidden())
 		return;
 
 	if(event.type == SDL_KEYDOWN) {
 		// Only pass key events if we have the focus
-		if (focus())
+		if (focus(&event))
 			key_press(event.key.keysym.sym);
 	} else if(event.type == SDL_MOUSEBUTTONDOWN &&
-	          event.button.button == SDL_BUTTON_LEFT ||
-		  event.type == DOUBLE_CLICK_EVENT) {
+	         (event.button.button == SDL_BUTTON_LEFT || event.button.button == SDL_BUTTON_RIGHT) ||
+	         event.type == DOUBLE_CLICK_EVENT) {
 
 		int x = 0;
 		int y = 0;
@@ -548,7 +620,7 @@ void menu::handle_event(const SDL_Event& event)
 		const int item = hit(x,y);
 		if(item != -1) {
 			set_focus(true);
-			move_selection(item);
+			move_selection_to(item);
 
 			if(click_selects_) {
 				show_result_ = true;
@@ -560,6 +632,9 @@ void menu::handle_event(const SDL_Event& event)
 				} else {
 					double_clicked_ = true;
 					last_was_doubleclick_ = true;
+					if(!silent_) {
+						sound::play_UI_sound(game_config::sounds::button_press);
+					}
 				}
 			} else if (last_was_doubleclick_) {
 				// If we have a double click as the next event, it means
@@ -585,8 +660,14 @@ void menu::handle_event(const SDL_Event& event)
 	} else if(event.type == SDL_MOUSEMOTION) {
 		if(click_selects_) {
 			const int item = hit(event.motion.x,event.motion.y);
-			if (item != -1)
-				move_selection(item);
+			const bool out = (item == -1);
+			if (out_ != out) {
+					out_ = out;
+					invalidate_row_pos(selected_);
+			}
+			if (item != -1) {
+				move_selection_to(item);
+			}
 		}
 
 		const int heading_item = hit_heading(event.motion.x,event.motion.y);
@@ -630,6 +711,17 @@ void menu::scroll(unsigned int)
 	set_dirty();
 }
 
+void menu::set_sorter(sorter *s)
+{
+	if(sortby_ >= 0) {
+		//clear an existing sort
+		sort_by(-1);
+	}
+	sorter_ = s;
+	sortreversed_ = false;
+	sortby_ = -1;
+}
+
 void menu::sort_by(int column)
 {
 	const bool already_sorted = (column == sortby_);
@@ -663,7 +755,7 @@ SDL_Rect menu::style::item_size(const std::string& item) const {
 		const std::string str = *it;
 		if (!str.empty() && str[0] == IMAGE_PREFIX) {
 			const std::string image_name(str.begin()+1,str.end());
-			surface const img = image::get_image(image_name,image::UNSCALED);
+			surface const img = get_item_image(image_name);
 			if(img != NULL) {
 				res.w += img->w;
 				res.h = maximum<int>(img->h, res.h);
@@ -797,11 +889,11 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 			str = *it;
 			if (!str.empty() && str[0] == IMAGE_PREFIX) {
 				const std::string image_name(str.begin()+1,str.end());
-				const surface img = image::get_image(image_name,image::UNSCALED);
-				const int max_width = max_width_ < 0 ? area.w :
-					minimum<int>(max_width_, area.w - xpos);
-				if(img != NULL && (xpos - rect.x) + img->w < max_width
-				   && rect.y + img->h < area.h) {
+				const surface img = style_->get_item_image(image_name);
+				const int remaining_width = max_width_ < 0 ? area.w :
+				minimum<int>(max_width_, ((lang_rtl)? xpos - rect.x : rect.x + rect.w - xpos));
+				if(img != NULL && img->w <= remaining_width
+				&& rect.y + img->h < area.h) {
 					const size_t y = rect.y + (rect.h - img->h)/2;
 					const size_t w = img->w + 5;
 					const size_t x = xpos + ((lang_rtl) ? widths[i] - w : 0);
@@ -812,15 +904,20 @@ void menu::draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type)
 				}
 			} else {
 				column.x = xpos;
-				const std::string to_show = max_width_ > -1 ?
-					font::make_text_ellipsis(str, style_->get_font_size(), loc.w - (xpos - rect.x)) : str;
+				const bool has_wrap = (str.find_first_of("\r\n") != std::string::npos);
+				//prevent ellipsis calculation if there is any line wrapping
+				const std::string to_show =
+					(use_ellipsis_ && !has_wrap) ?
+						font::make_text_ellipsis(str, style_->get_font_size(),
+							loc.w - (xpos - rect.x) - 2*style_->get_thickness(), false)
+						: str;
 				const SDL_Rect& text_size = font::text_area(str,style_->get_font_size());
 				const size_t y = rect.y + (rect.h - text_size.h)/2;
 				font::draw_text(&video(),column,style_->get_font_size(),font::NORMAL_COLOUR,to_show,xpos,y);
 
 				if(type == HEADING_ROW && sortby_ == int(i)) {
 					const surface sort_img = image::get_image(sortreversed_ ? "misc/sort-arrow.png" :
-					                                   "misc/sort-arrow-reverse.png", image::UNSCALED);
+					                                   "misc/sort-arrow-reverse.png");
 					if(sort_img != NULL && sort_img->w <= widths[i] && sort_img->h <= rect.h) {
 						const size_t sort_x = xpos + widths[i] - sort_img->w;
 						const size_t sort_y = rect.y + rect.h/2 - sort_img->h/2;
@@ -845,7 +942,8 @@ void menu::draw_contents()
 	style_->draw_row(*this,0,heading_rect,HEADING_ROW);
 
 	for(size_t i = 0; i != item_pos_.size(); ++i) {
-		style_->draw_row(*this,item_pos_[i],get_item_rect(i),item_pos_[i] == selected_ ? SELECTED_ROW : NORMAL_ROW);
+		style_->draw_row(*this,item_pos_[i],get_item_rect(i),
+			 (!out_ && item_pos_[i] == selected_) ? SELECTED_ROW : NORMAL_ROW);
 	}
 }
 
@@ -868,7 +966,8 @@ void menu::draw()
 				const unsigned int pos = item_pos_[*i];
 				const SDL_Rect& rect = get_item_rect(*i);
 				bg_restore(rect);
-				style_->draw_row(*this,pos,rect,pos == selected_ ? SELECTED_ROW : NORMAL_ROW);
+				style_->draw_row(*this,pos,rect,
+					(!out_ && pos == selected_) ? SELECTED_ROW : NORMAL_ROW);
 				update_rect(rect);
 			}
 		}
@@ -891,6 +990,36 @@ void menu::draw()
 	set_dirty(false);
 }
 
+void menu::wrap_words()
+{
+	int total_width = max_width_ - (style_->get_cell_padding() + (2 * style_->get_thickness()));
+	if(has_scrollbar()) {
+		total_width -= scrollbar_width();
+	}
+	if(total_width <= 0) {
+		return;
+	}
+	std::vector<int> const &widths = column_widths();
+	for(std::vector<item>::iterator i = items_.begin(); i != items_.end(); ++i) {
+		int space_remaining = total_width;
+		for(size_t col = 0; col < i->fields.size() && col < widths.size(); ++col) {
+			std::string &to_wrap = i->fields[col];
+			if (!to_wrap.empty()) {
+				if(widths[col] > space_remaining && to_wrap[0] != IMAGE_PREFIX) {
+					to_wrap = font::word_wrap_text(to_wrap, style_->get_font_size(), space_remaining);
+					break;
+				}
+				space_remaining -= widths[col] + 5;
+			}
+		}
+	}
+	itemRects_.clear();
+	column_widths_.clear();
+	max_items_ = -1; // Force recalculation of the max items.
+	item_height_ = -1; // Force recalculation of the item height.
+	update_size();
+}
+
 int menu::hit(int x, int y) const
 {
 	SDL_Rect const &loc = inner_location();
@@ -908,15 +1037,13 @@ int menu::hit(int x, int y) const
 int menu::hit_column(int x) const
 {
 	std::vector<int> const &widths = column_widths();
-	x -= location().x;
-	for(int j = 0, j_end = widths.size(); j != j_end; ++j) {
-		x -= widths[j];
-		if (x < 0) {
-			return j;
+	int j = -1, j_end = widths.size();
+	for(x -= location().x; x >= 0; x -= widths[j]) {
+		if(++j == j_end) {
+			return -1;
 		}
 	}
-
-	return -1;
+	return j;
 }
 
 std::pair<int,int> menu::hit_cell(int x, int y) const
@@ -938,7 +1065,7 @@ int menu::hit_heading(int x, int y) const
 {
 	const size_t height = heading_height();
 	const SDL_Rect& loc = inner_location();
-	if(y >= loc.y && (size_t)y < loc.y + height) {
+	if(y >= loc.y && static_cast<size_t>(y) < loc.y + height) {
 		return hit_column(x);
 	} else {
 		return -1;
@@ -952,7 +1079,6 @@ SDL_Rect menu::get_item_rect(int item) const
 
 SDL_Rect menu::get_item_rect_internal(size_t item) const
 {
-	const SDL_Rect empty_rect = {0,0,0,0};
 	unsigned int first_item_on_screen = get_position();
 	if (item < first_item_on_screen ||
 	    size_t(item) >= first_item_on_screen + max_items_onscreen()) {
