@@ -1,10 +1,11 @@
-/* $Id: menu.hpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: menu.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003-5 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -20,6 +21,7 @@
 #include <vector>
 
 
+#include "../image.hpp"
 #include "../events.hpp"
 #include "../sdl_utils.hpp"
 
@@ -29,6 +31,9 @@
 
 namespace gui {
 
+class menu;
+extern menu *empty_menu;
+
 class menu : public scrollarea
 {
 public:
@@ -45,7 +50,9 @@ public:
 		virtual SDL_Rect item_size(const std::string& item) const;
 		virtual void draw_row_bg(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 		virtual void draw_row(menu& menu_ref, const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
+		void scale_images(int max_width, int max_height);
 
+		surface get_item_image(const image::locator &i_locator) const;
 		size_t get_font_size() const;
 		size_t get_cell_padding() const;
 		size_t get_thickness() const;
@@ -57,6 +64,7 @@ public:
 
 		int normal_rgb_, selected_rgb_, heading_rgb_;
 		double normal_alpha_, selected_alpha_, heading_alpha_;
+		int max_img_w_, max_img_h_;
 	};
 
 	//image-border selction style
@@ -88,7 +96,7 @@ public:
 		double normal_alpha2_, selected_alpha2_, heading_alpha2_;
 		struct bg_cache
 		{
-			bg_cache() : width(-1), height(-1)
+			bg_cache() : surf(), width(-1), height(-1)
 			{}
 
 			surface surf;
@@ -99,17 +107,16 @@ public:
 	friend class style;
 	friend class imgsel_style;
 	static style &default_style;
-	//static style bluebox_style;
-	static imgsel_style slateborder_style;
+	static style simple_style;
 	static imgsel_style bluebg_style;
 
 	struct item
 	{
-		item() : id(0)
+		item() : fields(), help(), id(0)
 		{}
 
 		item(const std::vector<std::string>& fields, size_t id)
-			: fields(fields), id(id)
+			: fields(fields), help(), id(id)
 		{}
 
 		std::vector<std::string> fields;
@@ -148,12 +155,11 @@ public:
 
 	menu(CVideo& video, const std::vector<std::string>& items,
 	     bool click_selects=false, int max_height=-1, int max_width=-1,
-		 const sorter* sorter_obj=NULL, style *menu_style=NULL);
+		 const sorter* sorter_obj=NULL, style *menu_style=NULL, const bool auto_join=true);
 
 	int selection() const;
 
-	enum SELECTION_MOVE_VIEWPORT { MOVE_VIEWPORT, NO_MOVE_VIEWPORT };
-	void move_selection(size_t id, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
+	void move_selection(size_t id);
 	void reset_selection();
 
 	// allows user to change_item while running (dangerous)
@@ -187,11 +193,15 @@ public:
 
 	void scroll(unsigned int pos);
 
+	//currently, menus do not manage the memory of their sorter
+	//this should be changed to a more object-oriented approach
+	void set_sorter(sorter *s);
 	void sort_by(int column);
+	void wrap_words();
 
 protected:
 	bool item_ends_with_image(const std::string& item) const;
-	void handle_event(const SDL_Event& event);
+	virtual void handle_event(const SDL_Event& event);
 	void set_inner_location(const SDL_Rect& rect);
 
 	bool requires_event_focus(const SDL_Event *event=NULL) const;
@@ -199,6 +209,7 @@ protected:
 	virtual void draw_row(const size_t row_index, const SDL_Rect& rect, ROW_TYPE type);
 
 	style *style_;
+	bool silent_;
 private:
 	size_t max_items_onscreen() const;
 
@@ -226,6 +237,7 @@ private:
 
 	size_t selected_;
 	bool click_selects_;
+	bool out_;
 	bool previous_button_;
 	//std::set<size_t> undrawn_items_;
 
@@ -267,6 +279,9 @@ private:
 	bool ignore_next_doubleclick_;
 	bool last_was_doubleclick_;
 
+	//ellipsis calculation is slightly off, so default to false
+	bool use_ellipsis_;
+
 	const sorter* sorter_;
 	int sortby_;
 	bool sortreversed_;
@@ -281,7 +296,9 @@ private:
 	void assert_pos();
 
 	void update_size();
-	void set_selection_pos(size_t pos, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
+	enum SELECTION_MOVE_VIEWPORT { MOVE_VIEWPORT, NO_MOVE_VIEWPORT };
+	void set_selection_pos(size_t pos, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
+	void move_selection_to(size_t id, bool silent=false, SELECTION_MOVE_VIEWPORT move_viewport=MOVE_VIEWPORT);
 	void move_selection_up(size_t dep);
 	void move_selection_down(size_t dep);
 
