@@ -1,16 +1,20 @@
-/* $Id: scrollarea.cpp 9409 2006-01-03 20:04:50Z jhinrichs $*/
+/* $Id: scrollarea.cpp 23842 2008-02-16 08:47:16Z mordante $*/
 /*
-   Copyright (C) 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2004 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file widgets/scrollarea.cpp
+//!
+
 #include "global.hpp"
 
 #include "widgets/scrollarea.hpp"
@@ -20,8 +24,8 @@
 
 namespace gui {
 
-scrollarea::scrollarea(CVideo &video)
-	: widget(video), scrollbar_(video),
+scrollarea::scrollarea(CVideo &video, const bool auto_join)
+	: widget(video, auto_join), scrollbar_(video),
 	  old_position_(0), recursive_(false), shown_scrollbar_(false),
 	  shown_size_(0), full_size_(0)
 {
@@ -33,6 +37,13 @@ bool scrollarea::has_scrollbar() const
 	return shown_size_ < full_size_ && scrollbar_.is_valid_height(location().h);
 }
 
+handler_vector scrollarea::handler_members()
+{
+	handler_vector h;
+	h.push_back(&scrollbar_);
+	return h;
+}
+
 void scrollarea::update_location(SDL_Rect const &rect)
 {
 	SDL_Rect r = rect;
@@ -152,5 +163,5 @@ void scrollarea::handle_event(const SDL_Event& event)
 	}
 }
 
-}
+} // end namespace gui
 
