@@ -1,16 +1,20 @@
-/* $Id: scrollarea.hpp 9798 2006-01-24 01:17:58Z rusty $ */
+/* $Id: scrollarea.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2004 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file widgets/scrollarea.hpp
+//!
+
 #ifndef SCROLLAREA_HPP_INCLUDED
 #define SCROLLAREA_HPP_INCLUDED
 
@@ -24,14 +28,16 @@ namespace gui {
 class scrollarea : public widget
 {
 public:
-	/// Create a zone with automatic handling of scrollbar.
-	/// \param d the display object
-	/// \param pane the widget where wheel events take place
-	scrollarea(CVideo &video);
+	//! Create a zone with automatic handling of scrollbar.
+//! @todo FIXME: parameterlist ??
+	//- \param d the display object
+	//- \param pane the widget where wheel events take place
+	scrollarea(CVideo &video, bool auto_join=true);
 
 	virtual void hide(bool value = true);
 
 protected:
+	virtual handler_vector handler_members();
 	virtual void update_location(SDL_Rect const &rect);
 	virtual void handle_event(const SDL_Event& event);
 	virtual void process_event();
@@ -61,6 +67,6 @@ private:
 	void test_scrollbar();
 };
 
-}
+} // end namespace gui
 
 #endif
