@@ -1,17 +1,21 @@
-/* $Id: scrollbar.cpp 9798 2006-01-24 01:17:58Z rusty $*/
+/* $Id: scrollbar.cpp 23842 2008-02-16 08:47:16Z mordante $*/
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
-                 2004 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
+   Copyright (C) 2003 by David White <dave@whitevine.net>
+                 2004 - 2008 by Guillaume Melquiond <guillaume.melquiond@gmail.com>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
    See the COPYING file for more details.
 */
 
+//! @file widgets/scrollbar.cpp
+//!
+
 #include "global.hpp"
 
 #include "widgets/scrollbar.hpp"
@@ -45,7 +49,7 @@ scrollbar::scrollbar(CVideo &video)
 	  state_(NORMAL),
 	  grip_position_(0), grip_height_(0), full_height_(0), scroll_rate_(1)
 {
-	static const surface img(image::get_image(scrollbar_mid, image::UNSCALED));
+	static const surface img(image::get_image(scrollbar_mid));
 
 	if (img != NULL) {
 		set_width(img->w);
@@ -54,6 +58,14 @@ scrollbar::scrollbar(CVideo &video)
 	}
 }
 
+handler_vector scrollbar::handler_members()
+{
+	handler_vector h;
+	h.push_back(&uparrow_);
+	h.push_back(&downarrow_);
+	return h;
+}
+
 void scrollbar::update_location(SDL_Rect const &rect)
 {
 	int uh = uparrow_.height(), dh = downarrow_.height();
@@ -119,7 +131,10 @@ void scrollbar::set_shown_size(unsigned h)
 		h = full_height_;
 	if (h == grip_height_)
 		return;
+	bool at_bottom = get_position() == get_max_position() && get_max_position() > 0;
 	grip_height_ = h;
+	if (at_bottom)
+		grip_position_ = get_max_position();
 	set_position(grip_position_);
 	set_dirty(true);
 }
@@ -128,7 +143,10 @@ void scrollbar::set_full_size(unsigned h)
 {
 	if (h == full_height_)
 		return;
+	bool at_bottom = get_position() == get_max_position() && get_max_position() > 0;
 	full_height_ = h;
+	if (at_bottom)
+		grip_position_ = get_max_position();
 	downarrow_.enable(grip_position_ < full_height_ - grip_height_);
 	set_shown_size(grip_height_);
 	set_position(grip_position_);
@@ -189,10 +207,10 @@ SDL_Rect scrollbar::grip_area() const
 	SDL_Rect const &loc = groove_area();
 	if (full_height_ == grip_height_)
 		return loc;
-	int h = (int)loc.h * grip_height_ / full_height_;
+	int h = static_cast<int>(loc.h) * grip_height_ / full_height_;
 	if (h < minimum_grip_height_)
 		h = minimum_grip_height_;
-	int y = loc.y + ((int)loc.h - h) * grip_position_ / (full_height_ - grip_height_);
+	int y = loc.y + (static_cast<int>(loc.h) - h) * grip_position_ / (full_height_ - grip_height_);
 	SDL_Rect res = { loc.x, y, loc.w, h };
 	return res;
 }
@@ -200,15 +218,15 @@ SDL_Rect scrollbar::grip_area() const
 void scrollbar::draw_contents()
 {
 	const surface mid_img(image::get_image(state_ != NORMAL ?
-					scrollbar_mid_hl : scrollbar_mid, image::UNSCALED));
+					scrollbar_mid_hl : scrollbar_mid));
 	const surface bottom_img(image::get_image(state_ != NORMAL ?
-					scrollbar_bottom_hl : scrollbar_bottom, image::UNSCALED));
+					scrollbar_bottom_hl : scrollbar_bottom));
 	const surface top_img(image::get_image(state_ != NORMAL ?
-					scrollbar_top_hl : scrollbar_top, image::UNSCALED));
+					scrollbar_top_hl : scrollbar_top));
 
-	const surface top_grv(image::get_image(groove_top,image::UNSCALED));
-	const surface mid_grv(image::get_image(groove_mid,image::UNSCALED));
-	const surface bottom_grv(image::get_image(groove_bottom,image::UNSCALED));
+	const surface top_grv(image::get_image(groove_top));
+	const surface mid_grv(image::get_image(groove_mid));
+	const surface bottom_grv(image::get_image(groove_bottom));
 
 	if (mid_img == NULL || bottom_img == NULL || top_img == NULL
 	 || top_grv == NULL || bottom_grv == NULL || mid_grv == NULL) {
@@ -219,9 +237,9 @@ void scrollbar::draw_contents()
 	SDL_Rect grip = grip_area();
 	int mid_height = grip.h - top_img->h - bottom_img->h;
 	if (mid_height <= 0) {
-		// for now, minimum size of the middle piece is 1. This should
-		// never really be encountered, and if it is, it's a symptom
-		// of a larger problem, I think.
+		// For now, minimum size of the middle piece is 1.
+		// This should never really be encountered, and if it is,
+		// it's a symptom of a larger problem, I think.
 		mid_height = 1;
 	}
 
@@ -251,12 +269,12 @@ void scrollbar::draw_contents()
 
 	surface const screen = video().getSurface();
 
-	// draw scrollbar "groove"
+	// Draw scrollbar "groove"
 	video().blit_surface(groove.x, groove.y, top_grv);
 	video().blit_surface(groove.x, groove.y + top_grv->h, groove_scaled_);
 	video().blit_surface(groove.x, groove.y + top_grv->h + groove_height, bottom_grv);
 
-	// draw scrollbar "grip"
+	// Draw scrollbar "grip"
 	video().blit_surface(grip.x, grip.y, top_img);
 	video().blit_surface(grip.x, grip.y + top_img->h, mid_scaled_);
 	video().blit_surface(grip.x, grip.y + top_img->h + mid_height, bottom_img);
@@ -298,6 +316,10 @@ void scrollbar::handle_event(const SDL_Event& event)
 				move_position(-grip_height_);
 			else
 				move_position(grip_height_);
+		} else if (on_groove && e.button == SDL_BUTTON_MIDDLE) {
+			int y_dep = e.y - grip.y - grip.h/2;
+			int dep = y_dep * int(full_height_ - grip_height_)/ int(groove.h - grip.h);
+			move_position(dep);
 		}
 		break;
 	}
@@ -309,7 +331,8 @@ void scrollbar::handle_event(const SDL_Event& event)
 			new_state = on_grip ? ACTIVE : NORMAL;
 		} else if (state_ == DRAGGED && groove.h != grip.h) {
 			int y_dep = e.y - grip.y - mousey_on_grip_;
-			int dep = y_dep * int(full_height_ - grip_height_) / (int)(groove.h - grip.h);
+			int dep = y_dep * static_cast<int>(full_height_ - grip_height_) /
+                static_cast<int>(groove.h - grip.h);
 			move_position(dep);
 		}
 		break;
@@ -325,4 +348,5 @@ void scrollbar::handle_event(const SDL_Event& event)
 	state_ = new_state;
 }
 
-}
+} // end namespace gui
+
