@@ -1,10 +1,11 @@
-/* $Id: slider.cpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: slider.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -14,8 +15,10 @@
 #include "global.hpp"
 
 #include "widgets/slider.hpp"
+#include "game_config.hpp"
 #include "font.hpp"
 #include "image.hpp"
+#include "sound.hpp"
 #include "video.hpp"
 
 #include <algorithm>
@@ -30,8 +33,8 @@ namespace {
 namespace gui {
 
 slider::slider(CVideo &video)
-	: widget(video), image_(image::get_image(slider_image, image::UNSCALED)),
-	  highlightedImage_(image::get_image(selected_image, image::UNSCALED)),
+	: widget(video), image_(image::get_image(slider_image)),
+	  highlightedImage_(image::get_image(selected_image)),
 	  min_(-100000), max_(100000), value_(0),
 	  increment_(1), value_change_(false), state_(NORMAL)
 {
@@ -129,7 +132,8 @@ SDL_Rect slider::slider_area() const
 	if (image_.null() || image_->w >= loc.w)
 		return default_value;
 
-	int xpos = loc.x + (value_ - min_) * (int)(loc.w - image_->w) / (max_ - min_);
+	int xpos = loc.x + (value_ - min_) *
+		static_cast<int>(loc.w - image_->w) / (max_ - min_);
 	SDL_Rect res = { xpos, loc.y, image_->w, image_->h };
 	return res;
 }
@@ -168,7 +172,7 @@ void slider::set_slider_position(int x)
 	if (tmp > loc.w - image_->w)
 		tmp = loc.w - image_->w;
 
-	set_value(tmp * (max_ - min_) / (int)(loc.w - image_->w) + min_);
+	set_value(tmp * (max_ - min_) / static_cast<int>(loc.w - image_->w) + min_);
 }
 
 void slider::mouse_motion(const SDL_MouseMotionEvent& event)
@@ -178,7 +182,14 @@ void slider::mouse_motion(const SDL_MouseMotionEvent& event)
 		state_ = on ? ACTIVE : NORMAL;
 	} else if (state_ == CLICKED || state_ == DRAGGED) {
 		state_ = DRAGGED;
+		bool prev_change = value_change_;
+		value_change_ = false;
 		set_slider_position(event.x);
+		if(value_change_) {
+			sound::play_UI_sound(game_config::sounds::slider_adjust);
+		} else {
+			value_change_ = prev_change;
+		}
 	}
 }
 
@@ -188,8 +199,16 @@ void slider::mouse_down(const SDL_MouseButtonEvent& event)
 		return;
 
 	state_ = CLICKED;
+	bool prev_change = value_change_;
+	value_change_ = false;
 	set_focus(true);
 	set_slider_position(event.x);
+	if(value_change_) {
+		sound::play_UI_sound(game_config::sounds::slider_adjust);
+	} else {
+		sound::play_UI_sound(game_config::sounds::button_press);
+		value_change_ = prev_change;
+	}
 }
 
 bool slider::requires_event_focus(const SDL_Event* event) const
@@ -238,8 +257,10 @@ void slider::handle_event(const SDL_Event& event)
 			const SDL_keysym& key = reinterpret_cast<const SDL_KeyboardEvent&>(event).keysym;
 			const int c = key.sym;
 			if(c == SDLK_LEFT) {
+				sound::play_UI_sound(game_config::sounds::slider_adjust);
 				set_value(value_ - increment_);
 			} else if(c == SDLK_RIGHT) {
+				sound::play_UI_sound(game_config::sounds::slider_adjust);
 				set_value(value_ + increment_);
 			}
 		}
@@ -250,4 +271,64 @@ void slider::handle_event(const SDL_Event& event)
 	if (start_state != state_)
 		set_dirty(true);
 }
+
+template<typename T>
+list_slider<T>::list_slider(CVideo &video) :
+	slider(video)
+{
+	set_min(0);
+	set_increment(1);
+	slider::set_value(0);
+}
+
+template<typename T>
+list_slider<T>::list_slider(CVideo &video, const std::vector<T> &items) :
+	slider(video),
+	items_(items)
+{
+	set_min(0);
+	set_increment(1);
+	if(items.size() > 0)
+	{
+		set_max(items.size() - 1);
+	}
+	slider::set_value(0);
+}
+
+template<typename T>
+const T& list_slider<T>::item_selected() const
+{
+	return items_[value()];
+}
+
+template<typename T>
+bool list_slider<T>::select_item(const T& item)
+{
+	for(unsigned i = 0, nb = items_.size(); i < nb; ++i)
+	{
+		if(item == items_[i])
+		{
+			slider::set_value(i);
+			return true;
+		}
+	}
+	return false;
+}
+
+template<typename T>
+void list_slider<T>::set_items(const std::vector<T> &items)
+{
+	items_ = items;
+	if(items.size() > 0)
+	{
+		set_max(items.size() - 1);
+	}
+	slider::set_value(0);
+}
+
+// Force compilation of the following template instantiations
+template class list_slider< double >;
+template class list_slider< int >;
+template class list_slider< std::string >;
+
 } //end namespace gui
