@@ -1,10 +1,11 @@
-/* $Id: slider.hpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: slider.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -69,6 +70,20 @@ private:
 	STATE state_;
 };
 
+
+template<typename T>
+class list_slider : public slider
+{
+	public:
+		list_slider(CVideo &video);
+		list_slider(CVideo &video, const std::vector<T> &items);
+		void set_items(const std::vector<T> &items);
+		bool select_item(const T& item); //use select_item() instead of set_value()
+		const T& item_selected() const; //use item_selected() instead of value()
+	private:
+		std::vector<T> items_;
+};
+
 }
 
 #endif
