@@ -1,10 +1,11 @@
-/* $Id: textbox.cpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: textbox.cpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -29,12 +30,19 @@
 #include <cctype>
 #include <cstring>
 
+#define WRN_DISPLAY LOG_STREAM(warn, display)
+#define DBG_G LOG_STREAM(debug, general)
+
 namespace gui {
 
+#ifdef USE_TINY_GUI
+const int font_size = font::SIZE_TINY;
+#else
 const int font_size = font::SIZE_PLUS;
+#endif
 
-textbox::textbox(CVideo &video, int width, const std::string& text, bool editable, size_t max_size, double alpha, double alpha_focus)
-	   : scrollarea(video), max_size_(max_size), text_(utils::string_to_wstring(text)),
+textbox::textbox(CVideo &video, int width, const std::string& text, bool editable, size_t max_size, double alpha, double alpha_focus, const bool auto_join)
+	   : scrollarea(video, auto_join), max_size_(max_size), text_(utils::string_to_wstring(text)),
 	     cursor_(text_.size()), selstart_(-1), selend_(-1),
 	     grabmouse_(false), text_pos_(0), editable_(editable),
 	     show_cursor_(true), show_cursor_at_(0), text_image_(NULL),
@@ -48,6 +56,10 @@ textbox::textbox(CVideo &video, int width, const std::string& text, bool editabl
 	update_text_cache(true);
 }
 
+textbox::~textbox()
+{
+}
+
 void textbox::set_inner_location(SDL_Rect const &rect)
 {
 	bg_register(rect);
@@ -69,6 +81,7 @@ void textbox::set_text(const std::string& text)
 	selend_ = -1;
 	set_dirty(true);
 	update_text_cache(true);
+	handle_text_changed(text_);
 }
 
 void textbox::append_text(const std::string& text, bool auto_scroll)
@@ -103,6 +116,7 @@ void textbox::append_text(const std::string& text, bool auto_scroll)
 	set_dirty(true);
 	update_text_cache(false);
 	if(auto_scroll && is_at_bottom) scroll_to_bottom();
+	handle_text_changed(text_);
 }
 
 void textbox::clear()
@@ -115,6 +129,7 @@ void textbox::clear()
 	selend_ = -1;
 	set_dirty(true);
 	update_text_cache(true);
+	handle_text_changed(text_);
 }
 
 void textbox::draw_cursor(int pos, CVideo &video) const
@@ -362,7 +377,27 @@ bool textbox::requires_event_focus(const SDL_Event* event) const
 	if(!focus_ || !editable_ || hidden()) {
 		return false;
 	}
-	return (event==NULL || event->type == SDL_KEYDOWN);
+	if(event == NULL) {
+		//when event is not specified, signal that focus may be desired later
+		return true;
+	}
+
+	if(event->type == SDL_KEYDOWN) {
+		SDLKey key = event->key.keysym.sym;
+		switch(key) {
+		case SDLK_UP:
+		case SDLK_DOWN:
+		case SDLK_PAGEUP:
+		case SDLK_PAGEDOWN:
+			//in the future we may need to check for input history or multi-line support
+			//for now, just return false since these events are not handled.
+			return false;
+		default:
+			return true;
+		}
+	}
+	//mouse events are processed regardless of focus
+	return false;
 }
 
 void textbox::handle_event(const SDL_Event& event)
@@ -379,7 +414,7 @@ void textbox::handle_event(const SDL_Event& event)
 	//Sanity check: verify that selection start and end are within text
 	//boundaries
 	if(is_selection() && !(size_t(selstart_) <= text_.size() && size_t(selend_) <= text_.size())) {
-		LOG_STREAM(warn, display) << "out-of-boundary selection\n";
+		WRN_DISPLAY << "out-of-boundary selection\n";
 		selstart_ = selend_ = -1;
 	}
 
@@ -403,7 +438,7 @@ void textbox::handle_event(const SDL_Event& event)
 		int distance = x;
 
 		for(unsigned int i = 1; i < char_x_.size(); ++i) {
-			if((int)yscroll_ + y < char_y_[i]) {
+			if(static_cast<int>(yscroll_) + y < char_y_[i]) {
 				break;
 			}
 
@@ -459,7 +494,7 @@ void textbox::handle_event(const SDL_Event& event)
 	if(c == SDLK_LEFT && cursor_ > 0)
 		--cursor_;
 
-	if(c == SDLK_RIGHT && cursor_ < (int)text_.size())
+	if(c == SDLK_RIGHT && cursor_ < static_cast<int>(text_.size()))
 		++cursor_;
 
 	// ctrl-a, ctrl-e and ctrl-u are readline style shortcuts, even on Macs
@@ -496,10 +531,7 @@ void textbox::handle_event(const SDL_Event& event)
 		if(is_selection()) {
 			erase_selection();
 		} else {
-			if(cursor_ == (int)text_.size()) {
-				text_.resize(text_.size()-1);
-				--cursor_;
-			} else {
+			if(cursor_ < static_cast<int>(text_.size())) {
 				text_.erase(text_.begin()+cursor_);
 			}
 		}
@@ -512,7 +544,7 @@ void textbox::handle_event(const SDL_Event& event)
 	   c == SDLK_DELETE || c == SDLK_BACKSPACE || c == SDLK_END || c == SDLK_HOME ||
 	   c == SDLK_PAGEUP || c == SDLK_PAGEDOWN)) {
 		if(character != 0)
-			LOG_STREAM(info, display) << "Char: " << character << ", c = " << c << "\n";
+			DBG_G << "Char: " << character << ", c = " << c << "\n";
 
 		if(event.key.keysym.mod & copypaste_modifier) {
 			switch(c) {
@@ -576,6 +608,7 @@ void textbox::handle_event(const SDL_Event& event)
 
 	if(changed || old_cursor != cursor_ || old_selstart != selstart_ || old_selend != selend_) {
 		text_image_ = NULL;
+		handle_text_changed(text_);
 	}
 
 	set_dirty(true);
