@@ -1,10 +1,11 @@
-/* $Id: textbox.hpp 15250 2007-01-15 22:42:52Z soliton $ */
+/* $Id: textbox.hpp 23842 2008-02-16 08:47:16Z mordante $ */
 /*
-   Copyright (C) 2003 by David White <davidnwhite@verizon.net>
+   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License.
+   it under the terms of the GNU General Public License version 2
+   or at your option any later version.
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY.
 
@@ -26,7 +27,8 @@ namespace gui {
 class textbox : public scrollarea
 {
 public:
-	textbox(CVideo &video, int width, const std::string& text="", bool editable=true, size_t max_size = 256, double alpha = 0.4, double alpha_focus = 0.2);
+	textbox(CVideo &video, int width, const std::string& text="", bool editable=true, size_t max_size = 256, double alpha = 0.4, double alpha_focus = 0.2, const bool auto_join = true);
+	virtual ~textbox();
 
 	const std::string text() const;
 	void set_text(const std::string& text);
@@ -53,6 +55,8 @@ protected:
 	virtual void scroll(unsigned int pos);
 
 private:
+	virtual void handle_text_changed(const wide_string&) {}
+
 	size_t max_size_;
 
 	wide_string text_;
